/* Tommi Harkanen (C) 2000 */
/* Read the file copyright.txt for details */

%{
#include "typedef.h"
#include "bite.tab.h"
#define MAXVARNAME 34

extern long lineno, printLevel;

/* "dunif"|"dnorm"|"dgamma"|"dbinom"	return DISTR; */

%}

integer		[0-9]+
dreal		([0-9]+"."[0-9]+)
ereal		([0-9]+"."[0-9]+[eE][+-]?[0-9]+)
fname		([a-zA-Z_][a-zA-Z0-9_\.]{0,38}"(")
name		[a-zA-Z_][a-zA-Z0-9_\.]{0,39}
real		{dreal}|{ereal}
nl		\n

%%

#[^\n]*\n	{
		lineno++;
		if (printLevel > 2)
		  bite_printf("Lineno=%ld.\n", lineno);
	}

\"[^\"\n]{1,39}\" {
		long i;
		for (i = 1; i < MAX_STRLEN - 1; i++)
		  yylval.string[i - 1] = yytext[i];
		yylval.string[strlen(yytext) - 2] = '\0';
		return STRING;
	}

[ \t]+		;

\n		{
		lineno++;
		if (printLevel > 2)
		  bite_printf("Lineno=%ld.\n", lineno);
	}

{integer} 	{
		sscanf(yytext, "%ld", &yylval.integer);
		return INTEGER;
	}

{real}		{
		sscanf(yytext, "%lf", &yylval.real);
		return REAL;
	}

"*+"		return MULT2;
"+*"		return MULT2;
"^"		return POW;
"&&"    	return AND;
"||"    	return OR;
">="    	return GE;
"<="    	return LE;
">"	    	return GT;
"<"    		return LT;
"=="    	return EQ;
"!="    	return NE;
"->"    	return FIELD;

"fixle"         return FIXEDLE;
"fixjp"         return FIXEDJP;
"iter_commands" return ITER_COMM;
"init_commands"	return INIT_COMM;
"null"          return NULL_EXP;
"local"		return LOCAL;
"while"		return WHILE;
"switch"	return SWITCH;
"for"		return FOR;
"return"	return RETURN;
"if"		return IF;
"else"		return ELSE;
"echo"		return MYECHO;

"this"    	return THIS;
"I"     	return INDICATOR;
"t0"    	return T0;
"t1"    	return T1;
"points"	return POINTS;
"jumppoint"	return JUMPPOINT;
"minpiecelength"	return MINPL;
"level"		return LEVEL;
"n"		return N_POINTS;
"set"		return SET;
"am"		return AM;
"group"		return GROUP;
"metropolis"	return METROPOLIS;
"gradient"      return GRADIENT;
"parameters"	return PARAMETERS;
"likelihoods"	return LIKELIHOODS;
"multimodvar"   return MULTIMODVAR;
"reverse"	return REVERSE;
"restore"	return RESTORE;
"type"		return TYPE;
"init"		return INIT;
"initials"	return INITIALS;
"in"		return IN;
"prior"		return PRIOR;
"vector"	return VECTOR;
"proposal"	return PROPOSAL;
"epsilon"	return EPSILON;
"wait"		return WAIT;
"noupdate"	return NOUPDATE;
"addlikelihood"	return ADDLIKELIHOOD;
"nolikelihood"	return NOLIKELIHOOD;
"burn"		return BURN;
"monitor"	return MONITOR;
"alpha0"	return ALPHA0;
"beta0"		return BETA0;
"alpha"		return ALPHA;
"beta"		return BETA;
"mu"		return JMU;
"delta"		return DELTA;
"drift"		return DRIFT;
"sigma2"	return SIGMA2;
"mix"		return MIX;
"lgamma"	return LGAMMA;
"lgamma2"	return LGAMMA2;
"lincrgamma"	return LINCRGAMMA;
"ldecrgamma"	return LDECRGAMMA;
"ltimeless"	return LTIMELESS;
"lbrownian"	return LBROWNIAN;
"lgamma_lexis"	return LGAMMA_LEXIS;
"lexis_weight"	return LEXIS_WEIGHT;
"file"		return DFILE;
"response0"	return TRESPONCE;
"response20"	return TRESPONCE2;
"load"		return LOAD;
"cars"		return CARS;
"car"		return CAR;
"label"		return LABEL;
"missing"	return MISSING;
"nomissing"	return NOMISSING;
"restrict"	return RESTRICT;
"internop"	return INTERNOP;
"save"		return SAVE;
"put"		return PUT;
"to"		return TO;
"generate"	return GENERATE;
"model"		return MODEL;
"survival"	return SURVIVAL;
"ttt"		return TTT;
"expectation"	return EXPECTATION;
"printvar"	return PRINTVAR;
"range"		return RANGE;
"variable"	return VARIABLE;
"importance"	return IMPORTANCE;
"weight"	return WEIGHT;
"draw"		return DRAW;
"logscale"     	return LOGSCALE;
"hazard"	return INTENSITY;
"density"	return DENSITY;
"parameter"	return PARAMETER;
"cdf"		return CDF;
"nextpage"	return NEXTPAGE;
"picture"	return PICTURE;
"cut"		return CUT;
"indiv_likel"	return INDIVLIKEL;
"run"		return RUN;

[<<EOF>>]	{
  		bite_eprintf( "EOF detected (line %ld).\n", lineno);
		return RUN;
		}

[-!{}()\[\]\|+/'*;=:~,^\\]	return *yytext;

{fname} 	{ 
		sprintf(yylval.name, "%s", yytext);
		yylval.name[strlen(yytext) - 1] = '\0';
		return FNAME;
		}

{name} 		{
		yylval.name[0] = '\0';
		sprintf(yylval.name, "%s", yytext);
		return NAME;
	}


%%
