%{
#include "typedef.h"
#include "sf3.h"
#include "general3.h"
#include "pipefiles.h"
#include "bit.h"
#include "modelf.h"
#include "load.h"
#include "prior.h"
#include "ftree.h"
#include "ttt.h"
#include "likel.h"
#include "splusplot.h"
#define YYDEBUG 1
/* #include "bite.yy.c" */

/* Version 1.2 of BITE. Made with Lex & Yacc => more flexible - I hope. */

/* Global variables */
extern long max_iter, iter_limit, thinning, skip, drawMod, orientation,
  printLevel, ind, nVars, nextI, frames, cFrame, cPicture, lineno,
  aStack, lStack, maxVars, nextCar, lindex[MAX_STACK_SIZE], level, loadLag,
  fixedIndivVec, findUse, maxWait, annealingLength;
extern double missing, *old, annealingInit;
extern variable *vars;
extern prior param;
extern clusterUpdate cu;
extern gradientUpdate gu;
extern dt d;
extern ct c;
extern pi pred;
extern pi surv;
extern ttt tttplots;
extern mft f;
extern ff model;
extern ft init_comm;
extern ft iter_comm;
extern draw *plots;
extern labelArray labela[MAX_STACK], miss[MAX_STACK];
extern argArray a[MAX_STACK];
extern expectation e;
extern FILE *bite_err;

long njp;
double *jp;
char name[MAX_STACK][MAX_VARNAME];
prior mis = NULL, res;
useVal target;

static void incrLabelArray(labelArray l);
static ft leafR(double x);
static ft leafN(char *name);
static ft leafF(ft ptr, long fieldId);
static ft leafFunct(long fType);
static ft leafMF(char *name);
static ft leafG(long type);
static ft indexLeaf(ft ptr, ft root);
static ft node(long op, ft left, ft right);
static ft arithmeticIf(ft cond, ft left, ft right);
/* static ft triple(long op, ft left, ft right, ft third); */
long checkType(char *name);
void yyerror(const char *msg);
void checkPrior(char *name, prior res);

%}

%token <real> REAL
%token <integer> INTEGER SET PRIOR PROPOSAL ALPHA0 BETA0 ALPHA JMU DFILE TYPE 
	EPSILON TRESPONCE SURVIVAL DENSITY PARAMETER CDF BETA LINCRGAMMA
	MONITOR LOAD CARS CAR LABEL INIT RESTRICT RANGE VARIABLE MULT2
	SAVE PUT TO GENERATE MODEL DRAW LGAMMA INTENSITY MIX RUN INTERNOP
	MISSING NOMISSING T0 T1 POINTS INDICATOR NOLIKELIHOOD TTT
        NEXTPAGE PICTURE EXPECTATION WEIGHT JUMPPOINT LEVEL VECTOR
        IMPORTANCE LTIMELESS INITIALS IN TRESPONCE2 LGAMMA2 AM
        ADDLIKELIHOOD MINPL LDECRGAMMA PRINTVAR LOCAL IF ELSE WHILE FOR RETURN
        GROUP WAIT LOGSCALE NOUPDATE POW NULL_EXP PARAMETERS LIKELIHOODS
        REVERSE RESTORE BURN INIT_COMM ITER_COMM MYECHO N_POINTS SWITCH
        METROPOLIS FIXEDJP FIXEDLE LBROWNIAN DELTA DRIFT SIGMA2 THIS CUT
        INDIVLIKEL GRADIENT MULTIMODVAR LGAMMA_LEXIS LEXIS_WEIGHT
%token <name> NAME FNAME
%token <string> STRING

%left '='
%left '?' ':'
%left OR 
%left AND
%left GT LT GE LE EQ NE
%left '+' '-'
%left '*' '/'
%left MULT2
%left POW
%left UMINUS
%left FIELD '['

%type <ptr> expression index funct funct2 segment statementlist statement

%%

program:
          program item
	| item
;

item:
	  SET VARIABLE NAME ';' {
	    prior last;
	    if (printLevel >= 2)
	      bite_printf("set variable %s ...\n", $3);
	    res = initPrior($3);
	    res->fixed = TRUE;
	    res->update = FALSE;
	    res->ptype = PT_CONST;
	    res->x->x = 0.0;
	    if (param) {
	      last = param;
	      while (last->next)
		last = last->next;
	      last->next = res;
	      res->prev = last;
	    }
	    else
	      param = res;
	  }
	| SET NAME '=' expression ';' {
	    double val = calculate2(-1, $4);
	    if (printLevel >= 2)
	      bite_printf("set %s ...\n", $2);
	    if (!strcmp($2, "pages")) {
	      long j;
	      frames = val;
	      plots = (draw *) malloc((frames + 1) * sizeof(plots));
	      for (j = 1; j <= frames; j++)
		plots[j] = initDraw();
	    } 
	    if (!strcmp($2, "printLevel")) {
	      printLevel = val;
	    }
	    if (!strcmp($2, "loadLag")) {
	      loadLag = val;
	    }
	    if (!strcmp($2, "max_iter")) {
	      max_iter = val;
	    }
	    if (!strcmp($2, "thinning")) {
	      thinning = val;
	    }
	    if (!strcmp($2, "missing")) {
	      missing = val;
	    }
	    if (!strcmp($2, "fixedIndiv")) {
	      fixedIndivVec = val;
	    }
	    if (!strcmp($2, "N")) {
	      long ii, j;
	      ind = val;
	      for (ii = 0; ii < c->storage; ii++)
		c->values[ii] = NULL;
	      /* c->values[ii] = (double *) malloc(ind * sizeof(dum)); */
	      for (j = 0; j < MAX_STACK; j++) {
		labela[j] = (labelArray) malloc(sizeof(jackLabelArray));
		labela[j]->n = 0;
		labela[j]->stor = 1;
		labela[j]->label = (labels *) malloc(1 * sizeof(labela));
		for (ii = 0; ii < labela[j]->stor; ii++)
		  labela[j]->label[ii] = (labels) malloc(sizeof(jackLabel));
	      }
	      for (j = 0; j < MAX_STACK; j++) {
		miss[j] = (labelArray) malloc(sizeof(jackLabelArray));
		miss[j]->n = 0;
		miss[j]->stor = 1;
		miss[j]->label = (labels *) malloc(1 * sizeof(miss));
		for (ii = 0; ii < labela[j]->stor; ii++)
		  miss[j]->label[ii] = (labels) malloc(sizeof(jackLabel));
	      }
	      for (j = 0; j < MAX_STACK; j++) {
		a[j]->stor = MAX_STACK;
		a[j]->ptrArray = (ft *) malloc(MAX_STACK * sizeof(a));
	      }
	    }
	    if (!strcmp($2, "nextI")) {
	      nextI = val;
	    }
	    if (!strcmp($2, "nextCar")) {
	      nextCar = val;
	    }
	    if (!strcmp($2, "missing")) {
	      missing = val;
	    }
	    if (!strcmp($2, "iter_limit")) {
	      iter_limit = val;
	    }
	    if (!strcmp($2, "skip")) {
	      skip = val;
	    }
	    if (!strcmp($2, "drawMod")) {
	      drawMod = val;
	    }
	    if (!strcmp($2, "annealingLength")) {
	      annealingLength = val;
	    }
	    if (!strcmp($2, "annealingInit")) {
	      annealingInit = val;
	    }
	  }
	| SET PRIOR NAME '=' LGAMMA ',' ALPHA0 '=' expression ','
	  BETA0 '=' expression ',' ALPHA '=' expression ','
	  BETA '=' expression ','
          JMU '=' expression ',' MIX '=' REAL ';' {
	    if (printLevel >= 2)
	      bite_printf("set prior %s ...\n", $3);
	    if (nVars == maxVars) {
	      vars = (variable *) realloc(vars, 2 * nVars * sizeof(variable));
	      maxVars = 2 * nVars;
	    }
	    vars[nVars] = initVar($3);
	    vars[nVars]->type = VAR_PRIOR;
	    vars[nVars]->spec = GAMMA;
	    vars[nVars]->x = malloc(sizeof(vValue));
	    vars[nVars]->x->pv = malloc(sizeof(priorVal));
	    vars[nVars]->x->pv->pGa = initPg();
	    vars[nVars]->x->pv->pGa->alpha0 = $9;
	    target.var = vars[nVars];
	    findAndReplaceVarNames(-1, -1, vars[nVars]->x->pv->pGa->alpha0,
				   USE_VAR, target, NULL);
	    vars[nVars]->x->pv->pGa->beta0 = $13;
	    findAndReplaceVarNames(-1, -1, vars[nVars]->x->pv->pGa->beta0,
				   USE_VAR, target, NULL);
	    vars[nVars]->x->pv->pGa->alpha = $17;
	    findAndReplaceVarNames(-1, -1, vars[nVars]->x->pv->pGa->alpha,
				   USE_VAR, target, NULL);
	    vars[nVars]->x->pv->pGa->beta = $21;
	    findAndReplaceVarNames(-1, -1, vars[nVars]->x->pv->pGa->beta,
				   USE_VAR, target, NULL);
	    vars[nVars]->x->pv->pGa->mu = $25;
	    findAndReplaceVarNames(-1, -1, vars[nVars]->x->pv->pGa->mu,
				   USE_VAR, target, NULL);
	    vars[nVars]->x->pv->pGa->mix = $29;
	    nVars++;
	  }
	| SET PRIOR NAME '=' LGAMMA_LEXIS ',' ALPHA0 '=' expression ','
	  BETA0 '=' expression ',' ALPHA '=' expression ',' BETA '=' expression ','
	  LEXIS_WEIGHT '=' '(' {
	    if (printLevel >= 2)
	      bite_printf("set prior %s ...\n", $3);
	    if (nVars == maxVars) {
	      vars = (variable *) realloc(vars, 2 * nVars * sizeof(variable));
	      maxVars = 2 * nVars;
	    }
	    vars[nVars] = initVar($3);
	    vars[nVars]->type = VAR_PRIOR;
	    vars[nVars]->spec = GAMMA_LEXIS;
	    vars[nVars]->x = malloc(sizeof(vValue));
	    vars[nVars]->x->pv = malloc(sizeof(priorVal));
	    vars[nVars]->x->pv->pGL = initPgl();
	    vars[nVars]->x->pv->pGL->alpha0 = $9;
	    target.var = vars[nVars];
	    findAndReplaceVarNames(-1, -1, vars[nVars]->x->pv->pGL->alpha0,
				   USE_VAR, target, NULL);
	    vars[nVars]->x->pv->pGL->beta0 = $13;
	    findAndReplaceVarNames(-1, -1, vars[nVars]->x->pv->pGL->beta0,
				   USE_VAR, target, NULL);
	    vars[nVars]->x->pv->pGL->alpha = $17;
	    findAndReplaceVarNames(-1, -1, vars[nVars]->x->pv->pGL->alpha,
				   USE_VAR, target, NULL);
	    vars[nVars]->x->pv->pGL->beta = $21;
	    findAndReplaceVarNames(-1, -1, vars[nVars]->x->pv->pGL->alpha,
				   USE_VAR, target, NULL);
	    a[aStack]->n = 0;
	  } 
          expressionlist ')' ',' JMU '=' expression ',' MIX '=' REAL ';' {
	    long i;
	    vars[nVars]->x->pv->pGL->lexis_dim = a[aStack]->n;
	    vars[nVars]->x->pv->pGL->lexis_weight = 
	      malloc(a[aStack]->n * sizeof(res));
	    for (i = 0; i < a[aStack]->n; i++) {
	      vars[nVars]->x->pv->pGL->lexis_weight[i] = a[aStack]->ptrArray[i];
	      findAndReplaceVarNames(-1, -1, 
				     vars[nVars]->x->pv->pGL->lexis_weight[i],
				     USE_VAR, target, NULL);
	    }
	    vars[nVars]->x->pv->pGL->mu = $32;
	    findAndReplaceVarNames(-1, -1, vars[nVars]->x->pv->pGL->mu,
				   USE_VAR, target, NULL);
	    vars[nVars]->x->pv->pGL->mix = $36;
	    nVars++;
	  }
	| SET PRIOR NAME '=' LBROWNIAN ',' DELTA '=' expression ','
	  DRIFT '=' expression ',' SIGMA2 '=' expression ','
	  MIX '=' REAL ';' {
	    if (printLevel >= 2)
	      bite_printf("set prior %s ...\n", $3);
	    if (nVars == maxVars) {
	      vars = (variable *) realloc(vars, 2 * nVars * sizeof(variable));
	      maxVars = 2 * nVars;
	    }
	    vars[nVars] = initVar($3);
	    vars[nVars]->type = VAR_PRIOR;
	    vars[nVars]->spec = BROWNIAN;
	    vars[nVars]->x = malloc(sizeof(vValue));
	    vars[nVars]->x->pv = malloc(sizeof(priorVal));
	    vars[nVars]->x->pv->pBM = initPbm();
	    vars[nVars]->x->pv->pBM->delta = $9;
	    target.var = vars[nVars];
	    findAndReplaceVarNames(-1, -1, vars[nVars]->x->pv->pBM->delta,
				   USE_VAR, target, NULL);
	    vars[nVars]->x->pv->pBM->drift = $13;
	    findAndReplaceVarNames(-1, -1, vars[nVars]->x->pv->pBM->drift,
				   USE_VAR, target, NULL);
	    vars[nVars]->x->pv->pBM->sigma2 = $17;
	    findAndReplaceVarNames(-1, -1, vars[nVars]->x->pv->pBM->sigma2,
				   USE_VAR, target, NULL);
	    vars[nVars]->x->pv->pBM->mix = $21;
	    nVars++;
	  }
	| SET PRIOR NAME '=' LGAMMA2 ',' ALPHA0 '=' expression ','
	  BETA0 '=' expression ',' ALPHA '=' expression ',' 
          JMU '=' expression ',' MIX '=' REAL ';' {
	    if (printLevel >= 2)
	      bite_printf("set prior %s ...\n", $3);
	    if (nVars == maxVars) {
	      vars = (variable *) realloc(vars, 2 * nVars * sizeof(variable));
	      maxVars = 2 * nVars;
	    }
	    vars[nVars] = initVar($3);
	    vars[nVars]->type = VAR_PRIOR;
	    vars[nVars]->spec = GAMMA2;
	    vars[nVars]->x = malloc(sizeof(vValue));
	    vars[nVars]->x->pv = malloc(sizeof(priorVal));
	    vars[nVars]->x->pv->pGa = initPg();
	    vars[nVars]->x->pv->pGa->alpha0 = $9;
	    target.var = vars[nVars];
	    findAndReplaceVarNames(-1, -1, vars[nVars]->x->pv->pGa->alpha0,
				   USE_VAR, target, NULL);
	    vars[nVars]->x->pv->pGa->beta0 = $13;
	    findAndReplaceVarNames(-1, -1, vars[nVars]->x->pv->pGa->beta0,
				   USE_VAR, target, NULL);
	    vars[nVars]->x->pv->pGa->alpha = $17;
	    findAndReplaceVarNames(-1, -1, vars[nVars]->x->pv->pGa->alpha,
				   USE_VAR, target, NULL);
	    vars[nVars]->x->pv->pGa->mu = $21;
	    findAndReplaceVarNames(-1, -1, vars[nVars]->x->pv->pGa->mu,
				   USE_VAR, target, NULL);
	    vars[nVars]->x->pv->pGa->mix = $25;
	    nVars++;
	  }
	| SET PRIOR NAME '=' LINCRGAMMA ',' ALPHA0 '=' expression ','
	  BETA0 '=' expression ',' ALPHA '=' expression ',' BETA '=' 
	  expression ',' JMU '=' expression ',' MIX '=' REAL ';' {
	    if (printLevel >= 2)
	      bite_printf("set prior %s ...\n", $3);
	    if (nVars == maxVars) {
	      vars = (variable *) realloc(vars, 2 * nVars * sizeof(variable));
	      maxVars = 2 * nVars;
	    }
	    vars[nVars] = initVar($3);
	    vars[nVars]->type = VAR_PRIOR;
	    vars[nVars]->spec = INCR_GAMMA;
	    vars[nVars]->x = malloc(sizeof(vValue));
	    vars[nVars]->x->pv = malloc(sizeof(priorVal));
	    vars[nVars]->x->pv->pGa = initPg();
	    target.var = vars[nVars];

	    vars[nVars]->x->pv->pGa->alpha0 = $9;
	    findAndReplaceVarNames(-1, -1, vars[nVars]->x->pv->pGa->alpha0,
				   USE_VAR, target, NULL);
	    vars[nVars]->x->pv->pGa->beta0 = $13;
	    findAndReplaceVarNames(-1, -1, vars[nVars]->x->pv->pGa->beta0,
				   USE_VAR, target, NULL);
	    vars[nVars]->x->pv->pGa->alpha = $17;
	    findAndReplaceVarNames(-1, -1, vars[nVars]->x->pv->pGa->alpha,
				   USE_VAR, target, NULL);
	    vars[nVars]->x->pv->pGa->beta = $21;
	    findAndReplaceVarNames(-1, -1, vars[nVars]->x->pv->pGa->beta,
				   USE_VAR, target, NULL);
	    vars[nVars]->x->pv->pGa->mu = $25;
	    findAndReplaceVarNames(-1, -1, vars[nVars]->x->pv->pGa->mu,
				   USE_VAR, target, NULL);
	    vars[nVars]->x->pv->pGa->mix = $29;
	    nVars++;
	  }
	| SET PRIOR NAME '=' LDECRGAMMA ',' ALPHA0 '=' expression ','
	  BETA0 '=' expression ',' JMU '=' expression ',' MIX '=' REAL ';' {
	    if (printLevel >= 2)
	      bite_printf("set prior %s ...\n", $3);
	    if (nVars == maxVars) {
	      vars = (variable *) realloc(vars, 2 * nVars * sizeof(variable));
	      maxVars = 2 * nVars;
	    }
	    vars[nVars] = initVar($3);
	    vars[nVars]->type = VAR_PRIOR;
	    vars[nVars]->spec = DECR_GAMMA;
	    vars[nVars]->x = malloc(sizeof(vValue));
	    vars[nVars]->x->pv = malloc(sizeof(priorVal));
	    vars[nVars]->x->pv->pGa = initPg();
	    target.var = vars[nVars];

	    vars[nVars]->x->pv->pGa->alpha0 = $9;
	    findAndReplaceVarNames(-1, -1, vars[nVars]->x->pv->pGa->alpha0,
				   USE_VAR, target, NULL);
	    vars[nVars]->x->pv->pGa->beta0 = $13;
	    findAndReplaceVarNames(-1, -1, vars[nVars]->x->pv->pGa->beta0,
				   USE_VAR, target, NULL);
	    vars[nVars]->x->pv->pGa->alpha = NULL;
	    vars[nVars]->x->pv->pGa->beta = NULL;
	    vars[nVars]->x->pv->pGa->mu = $17;
	    findAndReplaceVarNames(-1, -1, vars[nVars]->x->pv->pGa->mu,
				   USE_VAR, target, NULL);
	    vars[nVars]->x->pv->pGa->mix = $21;
	    nVars++;
	  }
	| SET PRIOR NAME '=' LTIMELESS ',' {
	    res = initPrior($3);
	  }
          FNAME {
	    a[aStack]->n = 0;
	  }
	  expressionlist ')' ';' {
	    checkPrior($8, res);
	    if (printLevel >= 2)
	      bite_printf("set prior %s ...\n", $3);
	    if (nVars == maxVars) {
	      vars = (variable *) realloc(vars, 2 * nVars * sizeof(variable));
	      maxVars = 2 * nVars;
	    }
	    vars[nVars] = initVar($3);
	    vars[nVars]->type = VAR_PRIOR;
	    vars[nVars]->spec = TIMELESS;
	    vars[nVars]->x = malloc(sizeof(vValue));
	    vars[nVars]->x->pv = malloc(sizeof(priorVal));
	    vars[nVars]->x->pv->pTl = initTl();
	    vars[nVars]->x->pv->pTl->p = res;
	    /*	    res->sample = NULL; */
	    res->ptype = PT_CONST;
	    target.var = vars[nVars];
	    res->transform = NULL;
	    findAndReplaceVarNames2(-1, -1, res, USE_VAR, target);
	    if (!res->fixed && res->type != D_DIRAC)
	      res->x->x = generatePriorValue(-1, res);
	    nVars++;
	  }
	| SET PRIOR NAME ':' {
	    long i, j;
	    i = returnType($3, &j, TRUE);
	    if (i != PT_COV && i != PT_RESP) {
	      /* Create a new scalar parameter. */
	      prior last;
	      if (printLevel >= 2)
	 bite_printf("set prior %s ...\n", $3);
	      res = initPrior($3);
	      if (param) {
		last = param;
		while (last->next)
		  last = last->next;
		last->next = res;
		res->prev = last;
	      }
	      else
		param = res;
	    }
	    else {
	      /* Put a prior for a covariate or response. */
	      if (printLevel >= 2)
	 bite_printf("set prior for data %s ...\n", $3);
	      res = initPrior("");
	    }
	  }
	  expression '~' FNAME {
	    a[aStack]->n = 0;
	  }
	  expressionlist ')' ';' {
	    long ii, jj;
	    ii = returnType($3, &jj, FALSE);
	    /* Define prior distributions */
	    checkPrior($8, res);
	    /* res->sample = NULL; */
	    if (!res->ptype)
	      res->ptype = PT_CONST;
	    target.x = res;
	    res->transform = $6;
	    if (ii != PT_COV && ii != PT_RESP) {
	      findAndReplaceVarNames2(-1, -1, res, USE_PRIOR, target);
	      if (!res->fixed) {
		switch (res->type) {
		case D_MIX:
		case D_DIRICHLET:
		case D_MNORM:
		  generatePriorValue(-1, res);
		  break;
		default:
		  res->x->x = generatePriorValue(-1, res);
		  break;
		}
	      }
	    }
	    else {
	      /* Put a prior for a covariate or response. */
	      long i = 0, j, k, cov = 0, resp = 0, type = NONE;
	      switch (ii) {
	      case PT_COV:
		cov = jj;
		type = NONE;
		if (res->type == D_DIRAC) {
		  long j;
		  switch (k = returnDiracPriorType(res)) {
		  case SCALAR:
		    type = USE_PRIOR;
		    break;
		  case INDIVIDUAL:
		    i = returnType(c->cList[cov][0]->name, &j, FALSE);
		    type = (i == PT_COV) ? USE_COV2 : USE_TABLE;
		    break;
		  default:
		    bite_eprintf( "%s should be either \'indiv\' "
			    "or \'param\'.\n", res->name);
		  }
		}
		else
		  type = USE_COV2; /* was USE_TABLE */
		switch (c->pList[cov]->type) {
		case COV_CNST2:
		  c->pList[cov]->distr.x = initPrior("");
		  copyPrior(res, c->pList[cov]->distr.x);
		  res->fixed = TRUE;
		  if (type != USE_COV2)
		    target.x = c->pList[cov]->distr.x;
		  else
		    target.pvec = c->pList[cov];
		  for (i = 0; i < ind; i++)
		    findAndReplaceVarNames2(i, -1, c->pList[cov]->distr.x,
					    type, target);
		  break;
		case COV_START2:
		  if ($6->i) {
		    k = (long) calculate2(i, $6->i[0]);
		    level++;
		    lindex[level] = k;
		    c->pList[cov]->distr.x2[k] = initPrior("");
		    if (type != USE_COV2)
		      target.x = c->pList[cov]->distr.x2[k];
		    else
		      target.pvec = c->pList[cov];
		    copyPrior(res, c->pList[cov]->distr.x2[k]);
		    res->fixed = TRUE;
		    c->pList[cov]->distr.x2[k]->nolikelihood =
		      res->nolikelihood;
		    for (i = 0; i < ind; i++)
		      findAndReplaceVarNames2(i, -1,
			c->pList[cov]->distr.x2[k], type, target);
		    for (j = 0; j < c->pList[cov]->stor; j++)
		      if (c->pList[cov]->distr.x2[j] && j != k)
			c->pList[cov]->distr.x2[j]->where =
			  c->pList[cov]->distr.x2[k]->where;
		    level--;
		  }
		  break;
		}
		break;
	      case PT_RESP:
		resp = jj;
		j = calculate2(-1, $6->i[0]);
		if (d->pList[resp]->type == CONT) {
		  switch ($6->fieldId) {
		  case T0:
		    d->pList[resp]->distr.pp.t0[j] = initPrior("t0");
		    copyPrior(res, d->pList[resp]->distr.pp.t0[j]);
		    target.x = d->pList[resp]->distr.pp.t0[j];
		    findAndReplaceVarNames2(0, resp, target.x, USE_TABLE,
					    target);
		    break;
		  case T1:
		    d->pList[resp]->distr.pp.t1[j] = initPrior("t1");
		    copyPrior(res, d->pList[resp]->distr.pp.t1[j]);
		    target.x = d->pList[resp]->distr.pp.t1[j];
		    findAndReplaceVarNames2(0, resp, target.x,
					    USE_TABLE, target);
		    break;
		  case POINTS:
		    if ($6->i) {
		      k = (long) calculate2(-1, $6->third->i[0]);
		      if (k < d->pList[resp]->distr.pp.npoints[j]) {
			d->pList[resp]->distr.pp.points[j][k] =
			  initPrior("point");
			copyPrior(res,
				  d->pList[resp]->distr.pp.points[j][k]);
			target.x = d->pList[resp]->distr.pp.points[j][k];
			findAndReplaceVarNames2(0, resp,
						d->pList[resp]->distr.pp.points[j][k],
						USE_TABLE, target);
		      }
		    }
		    break;
		  }
		}
		break;
	      default:
		bite_eprintf( "Unknown type!\n");
		bite_exit(1);
	      }
	    }
	  }
        | WAIT expression INTEGER ';' {
	    prior p = param;
	    long i, j, type = returnType($2->varName, &i, FALSE);
	    if (printLevel >= 2)
	      bite_printf("wait %s ...\n", $2->varName);
	    switch (type) {
	    case  PT_MODELF:
	      for (j = 0; j < f->nFunc[i]; j++)
		f->fList[i][j]->wait = $3;
	      break;
	    default:
	      p = returnPrior($2);
	      p->wait = $3;
	    }
	    if ($3 > maxWait)
	      maxWait = $3;
	  }
        | NOUPDATE expression ';' {
	    prior p = param;
	    if (printLevel >= 2)
	      bite_printf("noupdate %s ...\n", $2->varName);
	    p = returnPrior($2);
	    p->update = FALSE;
	  }
        | INDIVLIKEL expression ':' expression ';' {
	    prior p = param;
	    if (printLevel >= 2)
	      bite_printf("indiv_likel %s ...\n", $2->varName);
	    p = returnPrior($2);
	    p->indiv_likel = $4;
	    findAndReplaceVarNames(0, 0, p->indiv_likel,
				   NONE, target, NULL);
	  }
	| SET VECTOR PRIOR NAME  expression  '~' NAME ';' {
	    long i, n = calculate2(-1, $5);
	    char index[6];
	    prior a, b = NULL, p, last;
	    if (printLevel >= 2)
	      bite_printf("set vector prior %s ...\n", $4);
	    res = initPrior($4);
	    res->ptype = PT_VECTOR;
	    if (param) {
	      last = param;
	      p = param;
	    }
	    else {
	      bite_eprintf( "You must define a scalar parameter first."
		      " (Line %ld.).\n", lineno);
	      bite_exit(1);
	    }
	    while (last->next)
	      last = last->next;
	    while (strcmp(p->name, $7) && p)
	      p = p->next;
	    last->next = res;
	    res->prev = last;
	    res->stor = n;
	    res->vector = malloc(n * sizeof(res));
	    level++;
	    for (i = 0; i < n; i++) {
	      lindex[level] = i;
	      sprintf(index, "%ld", i);
	      a = initPrior(index);
	      res->vector[i] = a;
	      a->up = res;
	      if (p->type == D_MNORM)
		a->x->vec = NULL;
	      copyPrior(p, a);
	      a->x->x = 0.0;
	      a->index = i;
	      /* a->sample = NULL; */
	      if (!a->ptype)
		a->ptype = PT_CONST;
	      target.x = a;
	      a->transform = NULL;
	      findAndReplaceVarNames2(-1, -1, a, USE_PRIOR, target);
	      switch (a->type) {
	      case D_MIX:
	      case D_DIRICHLET:
	      case D_MNORM:
		generatePriorValue(i, a);
		break;
	      default:
		a->x->x = generatePriorValue(i, a);
		break;
	      }
	      if (!i)
		res->x->vec = a;
	      else {
		a->prev = b;
		b->next = a;
	      }
	      b = a;
	    }
	    level--;
	  }
	| PROPOSAL NAME ':' MIX '=' REAL ',' EPSILON '=' REAL ';' {
	    prior ptr = param;
	    long i, j;
	    i = returnType($2, &j, TRUE);
	    if (printLevel >= 2)
	      bite_printf("proposal %s ...\n", $2);
	    if (i != PT_COV && i != PT_RESP) {
	      while (strcmp(ptr->name, $2))
		ptr = ptr->next;
	      if (!ptr) {
		bite_eprintf( "%s: no such parameter.\n", $2);
		bite_exit(1);
	      }
	      ptr->mix = $6;
	      ptr->epsilon = $10;
	    }
	    else {
	      /* Put a prior for a covariate or response. */
	      long cov = 0;
	      switch (i) {
	      case PT_COV:
		cov = j;
		switch (c->pList[cov]->type) {
		case COV_CNST2:
		  if (c->pList[cov]->distr.x) {
		    c->pList[cov]->distr.x->mix = $6;
		    c->pList[cov]->distr.x->epsilon = $10;
		  }
		  else {
		    bite_eprintf( "Prior distribution has not been set up."
			    "(Line %ld)\n", lineno);
		    bite_exit(1);
		  }
		  break;
		default:
		  bite_eprintf( "A proposal can be given either to a parameter "
			  "or to a constant covariate. (Line %ld)\n", lineno);
		  bite_exit(1);
		}
	      }
	    }
	  }
	| ADDLIKELIHOOD NAME expression ';' {
	    prior ptr = param;
	    if (printLevel >= 2)
	      bite_printf("addlikelihood %s;\n", $2);
	    while (strcmp(ptr->name, $2))
	      ptr = ptr->next;
	    if (!ptr) {
	      bite_eprintf( "%s: no such parameter.\n", $2);
	      bite_exit(1);
	    }
	    target.x = NULL;
	    findAndReplaceVarNames(-1, -1, $3, NONE, target,
				   target.x);
	    if ($3->ptype == PT_RESP) {
	      long mark = 0;
	      while (mark < d->n &&
		 strcmp($3->varName, d->dList[mark][0][0]->name))
		mark++;
	      addUses(-1, mark, USE_TABLE, ptr, target, target.x);
	    }
	  }
	| NOLIKELIHOOD expression ';' {
	    prior ptr;
	    if (printLevel >= 2)
	      bite_printf("nolikelihood %s;\n", $2->varName);
	    ptr = returnPrior($2);
	    ptr->nolikelihood = TRUE;
	  }
	| BURN expression INTEGER ';' {
	    prior ptr;
	    if (printLevel >= 2)
	      bite_printf("burn %s;\n", $2->varName);
	    ptr = returnPrior($2);
	    ptr->burn = $3;
	  }
	| RESTRICT expression ':' '(' {
	    if (printLevel >= 2)
	      bite_printf("restrict %s ...\n", $2->varName);
	    a[aStack]->n = 0;
	  }
	  expressionlist ')' ';' {
	    prior ptr = returnPrior($2);
	    target.x = ptr;
	    ptr->restrict1 = a[aStack]->ptrArray[0];
	    findAndReplaceVarNames(-1, -1, ptr->restrict1, USE_PRIOR, target,
				   target.x);
	    ptr->restrict2 = a[aStack]->ptrArray[1];
	    findAndReplaceVarNames(-1, -1, ptr->restrict2, USE_PRIOR, target,
				   target.x);
	    
	  }
	| MISSING expression PRIOR ':' NAME ';' {
	    long i = 0;
	    double code;
	    ft ptr = $2;
/*	    if (printLevel >= 2)
	      bite_printf("missing \'expression\' %s ...\n", $3); */
	    code = calculate2(-1, ptr);
	    rmTree(ptr);
	    while (i < miss[0]->n && miss[0]->label[i]->value != code)
	      i++;
	    if (i == miss[0]->n) {
	      miss[0]->label[i]->value = code;
	      miss[0]->label[i]->label[0] = '\0';
	      strcat(miss[0]->label[i]->label, $5);
	      miss[0]->n++;
	    }
	  }
          | GRADIENT NAME ',' PARAMETERS '(' {
	    if (printLevel >= 2)
	      bite_printf("gradient %s ...\n", $2);
	    a[aStack]->n = 0;
	  }
	  expressionlist ')' {
	    long i;
	    gradientUpdate g = initGU(a[aStack]->n, $2), last = gu;
	    while (last && last->next)
	      last = last->next;
	    if (!last)
	      gu = g;
	    else
	      last->next = g;
	    g->next = NULL;
	    target.x = NULL;
	    for (i = 0; i < g->n; i++) {
	      findAndReplaceVarNames(-1, -1, a[aStack]->ptrArray[i],
				     NONE, target, target.x);
	      if (a[aStack]->ptrArray[i]->ptype != PT_SCALAR &&
		  a[aStack]->ptrArray[i]->ptype != PT_CONST &&
		  a[aStack]->ptrArray[i]->ptype != PT_VECTOR &&
		  a[aStack]->ptrArray[i]->function != F_INDIV) {
		bite_eprintf( "Not a parameter nor "
			"function 'indiv'. (%s, Line %ld.)\n",
			a[aStack]->ptrArray[i]->varName, lineno);
		bite_exit(1);
	      }
	      g->vec[i] = a[aStack]->ptrArray[i];
	    }
	    a[aStack]->n = 0;
	  } ',' LIKELIHOODS '(' expressionlist ')' ';' {
	    long i;
	    gradientUpdate g = gu;
	    while (g->next)
	      g = g->next;
	    g->nl = a[aStack]->n;
	    g->likel = malloc(g->nl * sizeof(g));
	    for (i = 0; i < g->nl; i++) {
	      findAndReplaceVarNames(-1, -1, a[aStack]->ptrArray[i],
				     NONE, target, target.x);
	      g->likel[i] = a[aStack]->ptrArray[i];
	    }
	  }
        | GRADIENT NAME MULTIMODVAR ':' expression ',' expression ','
	  expression ';' {
	    gradientUpdate last = gu;
	    long i = 0;
	    if (printLevel >= 2)
	      bite_printf("gradient %s ...\n", $2);
	    target.x = NULL;
	    findAndReplaceVarNames(-1, -1, $5, NONE, target, target.x);
	    findAndReplaceVarNames(-1, -1, $7, NONE, target, target.x);
	    findAndReplaceVarNames(-1, -1, $9, NONE, target, target.x);
	    if (!($5->ptype == PT_SCALAR ||
		  ($5->ptype == PT_VECTOR && $5->ni == 1))) {
	      bite_eprintf(
		      "Only scalar parameters allowed. (%s, Line %ld.)\n",
		      $5->varName, lineno);
	      bite_exit(1);
	    }
	    while (last && strcmp(last->name, $2))
	      last = last->next;
	    if (last)
	      for (; i < last->n && last->mm_p < 0; i++)
		if (($5->ptype == PT_SCALAR &&
		     last->vec[i]->ptype == PT_SCALAR &&
		     last->vec[i]->var->p == $5->var->p) ||
		    ($5->ptype == PT_VECTOR &&
		     last->vec[i]->ptype == PT_VECTOR &&
		     last->vec[i]->ni > 0 &&
		     last->vec[i]->var->vec == $5->var->vec &&
		     calculate2(i, last->vec[i]->i[0]) ==
		     calculate2(i, $5->i[0]))) {
		  last->mm_p = i;
		  last->mm_mu = calculate2(-1, $7);
		  last->mm_sigma2 = calculate2(-1, $9);
		}
	  }
	| AM NAME '('  {
	    if (printLevel >= 2)
	      bite_printf("AM %s ...\n", $2);
	    a[aStack]->n = 0;
	  }
	  expressionlist ')' {
	    long i;
	    clusterUpdate c = initCU(a[aStack]->n, $2), last = cu;
	    c->type = CLUSTER_RANDOM_WALK;
	    /* if (!strcmp($6, "randomwalk"))
	    else {
	      if (!strcmp($6, "independence"))
		c->type = CLUSTER_INDEPENDENCE;
	      else {
		bite_eprintf( "%s: unknown type for cluster updating.\n",
			$6);
		bite_exit(1);
	      }
	    } */
	    while (last && last->next)
	      last = last->next;
	    if (!last)
	      cu = c;
	    else
	      last->next = c;
	    c->next = NULL;
	    target.x = NULL;
	    for (i = 0; i < c->n; i++) {
	      findAndReplaceVarNames(-1, -1, a[aStack]->ptrArray[i],
				     NONE, target, target.x);
	      if (a[aStack]->ptrArray[i]->ptype != PT_SCALAR &&
		  a[aStack]->ptrArray[i]->ptype != PT_CONST &&
		  a[aStack]->ptrArray[i]->ptype != PT_VECTOR &&
		  a[aStack]->ptrArray[i]->function != F_INDIV) {
		bite_eprintf( "Not a parameter nor "
			"function 'indiv'. (Line %ld.)\n", lineno);
		bite_exit(1);
	      }
	      c->vec[i] = a[aStack]->ptrArray[i];
	    }
	    a[aStack]->n = 0;
	  } '(' expressionlist ')' RANGE NAME '(' expression ','
	  expression ',' expression ')' ';' {
	    long i;
	    prior p = param;
	    clusterUpdate c = cu;
	    while (c->next)
	      c = c->next;
	    initCU2(calculate2(-1, $18), a[aStack]->n, c);
	    target.x = NULL;
	    for (i = 0; i < c->nl; i++) {
	      findAndReplaceVarNames(-1, -1, a[aStack]->ptrArray[i],
				     NONE, target, target.x);
	      if (a[aStack]->ptrArray[i]->ptype != PT_SCALAR &&
		  a[aStack]->ptrArray[i]->ptype != PT_CONST &&
		  a[aStack]->ptrArray[i]->ptype != PT_VECTOR &&
		  a[aStack]->ptrArray[i]->function != F_INDIV) {
		bite_eprintf( "Not a parameter nor "
			"function 'indiv'. (Line %ld.)\n", lineno);
		bite_exit(1);
	      }
	      c->likel[i] = a[aStack]->ptrArray[i];
	    }
	    while (p && strcmp(p->name, $12))
	      p = p->next;
	    if (!p) {
	      bite_eprintf( "%s: no such parameter found.\n", $12);
	      bite_exit(1);
	    }
	    c->nLoop = 1;
	    c->loopParam = malloc(sizeof(p));
	    c->lower = malloc(sizeof(missing));
	    c->upper = malloc(sizeof(missing));
	    c->dim = malloc(sizeof(i));
	    c->loopParam[0] = p;
	    c->lower[0] = $14;
	    c->upper[0] = $16;
	    c->dim[0] = $18;
	  }
	| INIT_COMM expression ';' {
	    init_comm = $2;
	    findAndReplaceVarNames(0, 0, init_comm, NONE, 
				   target, NULL);
	  }
	| ITER_COMM expression ';' {
	    iter_comm = $2;
	    findAndReplaceVarNames(0, 0, iter_comm, NONE, 
				   target, NULL);
	  }
	| GROUP NAME PARAMETERS '=' '('  {
	    if (printLevel >= 2)
	      bite_printf("group %s ...\n", $2);
	    a[aStack]->n = 0;
	  }
	  expressionlist ')' { /* Parameters whose priors are calculated. */
	    long i;
	    clusterUpdate c = initCU(a[aStack]->n, $2), last = cu;
	    c->type = GROUP_UPDATE;
	    while (last && last->next)
	      last = last->next;
	    if (!last)
	      cu = c;
	    else
	      last->next = c;
	    c->next = NULL;
	    target.x = NULL;

	    for (i = 0; i < c->n; i++) {
	      findAndReplaceVarNames(-1, -1, a[aStack]->ptrArray[i],
				     NONE, target, target.x);
	      if (a[aStack]->ptrArray[i]->ptype != PT_SCALAR &&
		  a[aStack]->ptrArray[i]->ptype != PT_CONST &&
		  a[aStack]->ptrArray[i]->ptype != PT_VECTOR &&
		  a[aStack]->ptrArray[i]->function != F_INDIV) {
		bite_eprintf( "Not a parameter nor "
			"function 'indiv'. (Line %ld.)\n", lineno);
		bite_exit(1);
	      }
	      c->vec[i] = a[aStack]->ptrArray[i];
	    }
	    a[aStack]->n = 0;
	  } '=' '(' expressionlist ')' { /* Their new values. */
	    long i;
	    clusterUpdate c = cu;
	    while (c->next)
	      c = c->next;
	    for (i = 0; i < c->n; i++) {
	      if (a[aStack]->n == c->n) {
		/* If Metropolis updating: */
		findAndReplaceVarNames(-1, -1, a[aStack]->ptrArray[i],
				       NONE, target, target.x);
		c->new_val[i] = a[aStack]->ptrArray[i];
	      }
	      else {
		if (a[aStack]->n == 0) {
		  /* Only custom updating. (Metropolis-Hastings.) */
		  c->new_val[i] = c->vec[i];
		}
		else {
		  bite_eprintf( "Incorrect number of expressions on line "
			  "%ld. Should be either 0, or %ld.\n", lineno, c->n);
		}
	      }
	    }
	    a[aStack]->n = 0;
	  } LIKELIHOODS '=' '(' expressionlist ')' { /* Likelihood contributions. */
	    long i;
	    clusterUpdate c = cu;
	    while (c->next)
	      c = c->next;
	    c->nl = a[aStack]->n;
	    c->likel = malloc(c->nl * sizeof(c));
	    for (i = 0; i < c->nl; i++) {
	      findAndReplaceVarNames(-1, -1, a[aStack]->ptrArray[i],
				     NONE, target, target.x);
	      c->likel[i] = a[aStack]->ptrArray[i];
	    }
	    a[aStack]->n = 0;
	  } PROPOSAL '=' '(' expressionlist ')' {
	    /* LOG q(y|x). Includes x->y by inline functions. */
	    long i;
	    clusterUpdate c = cu;
	    while (c->next)
	      c = c->next;
	    c->qn_xy = a[aStack]->n;
	    c->prop_dens_xy = malloc(c->qn_xy * sizeof(c));
	    for (i = 0; i < c->qn_xy; i++) {
	      findAndReplaceVarNames(-1, -1, a[aStack]->ptrArray[i],
				     NONE, target, target.x);
	      c->prop_dens_xy[i] = a[aStack]->ptrArray[i];
	    }
	    a[aStack]->n = 0;
	  } REVERSE '=' '(' expressionlist ')' {
	    /* LOG q(x|y). Includes y->x by inline functions. */
	    long i;
	    clusterUpdate c = cu;
	    while (c->next)
	      c = c->next;
	    c->qn_yx = a[aStack]->n;
	    c->prop_dens_yx = malloc(c->qn_yx * sizeof(c));
	    for (i = 0; i < c->qn_yx; i++) {
	      findAndReplaceVarNames(-1, -1, a[aStack]->ptrArray[i],
				     NONE, target, target.x);
	      c->prop_dens_yx[i] = a[aStack]->ptrArray[i];
	    }
	    a[aStack]->n = 0;
	  } RESTORE '=' '(' expressionlist ')' {
	    /* Inline function(s) for restoring the parameter values
	       in case of acceptance. */
	    long i;
	    clusterUpdate c = cu;
	    while (c->next)
	      c = c->next;
	    c->nclean = a[aStack]->n;
	    c->clean = malloc(c->nclean * sizeof(c));
	    for (i = 0; i < c->nclean; i++) {
	      findAndReplaceVarNames(-1, -1, a[aStack]->ptrArray[i],
				     NONE, target, target.x);
	      c->clean[i] = a[aStack]->ptrArray[i];
	    }
	    a[aStack]->n = 0;
	  } RANGE expressionlist { /* Loop variables. */
	    long i;
	    clusterUpdate c = cu;
	    while (c->next)
	      c = c->next;
	    c->nLoop = a[aStack]->n;
	    c->loopParam = malloc(c->nLoop * sizeof(c));
	    c->dim = malloc(c->nLoop * sizeof(i));
	    c->lower = malloc(c->nLoop * sizeof(missing));
	    c->upper = malloc(c->nLoop * sizeof(missing));
	    for (i = 0; i < c->nLoop; i++) {
	      findAndReplaceVarNames(-1, -1, a[aStack]->ptrArray[i],
				     NONE, target, target.x);
	      if (a[aStack]->ptrArray[i]->ptype != PT_SCALAR) {
		bite_eprintf(
			"Loop parameter %s on line %ld is not a scalar.\n",
			a[aStack]->ptrArray[i]->varName, lineno);
		bite_exit(1);
	      }
	      c->loopParam[i] = a[aStack]->ptrArray[i]->var->p;
	    }
	    a[aStack]->n = 0;
	  } '(' expressionlist ')' ';' { /* Loop limits (3* # of loop var). */
	    long i;
	    clusterUpdate c = cu;
	    while (c->next)
	      c = c->next;
	    target.x = NULL;
	    if (a[aStack]->n != c->nLoop * 3) {
	      bite_eprintf( "The range list of length %ld should have  "
		      "%ld loop arguments. (Line %ld.)\n", c->nLoop,
		      c->nLoop * 3, lineno);
	      bite_exit(1);
	    }
	    for (i = 0; i < a[aStack]->n / 3; i++) {
	      findAndReplaceVarNames(-1, -1, a[aStack]->ptrArray[i],
				     NONE, target, target.x);
	      c->lower[i] = a[aStack]->ptrArray[i*3];
	      c->upper[i] = a[aStack]->ptrArray[i*3+1];
	      c->dim[i] = a[aStack]->ptrArray[i*3+2];
	    }
	  }
	| METROPOLIS NAME PARAMETERS '=' '('  {
	    if (printLevel >= 2)
	      bite_printf("group %s ...\n", $2);
	    a[aStack]->n = 0;
	  }
	  expressionlist ')' { /* Parameters whose priors are calculated. */
	    long i;
	    clusterUpdate c = initCU(a[aStack]->n, $2), last = cu;
	    c->type = GROUP_UPDATE;
	    while (last && last->next)
	      last = last->next;
	    if (!last)
	      cu = c;
	    else
	      last->next = c;
	    c->next = NULL;
	    target.x = NULL;

	    for (i = 0; i < c->n; i++) {
	      findAndReplaceVarNames(-1, -1, a[aStack]->ptrArray[i],
				     NONE, target, target.x);
	      if (a[aStack]->ptrArray[i]->ptype != PT_SCALAR &&
		  a[aStack]->ptrArray[i]->ptype != PT_CONST &&
		  a[aStack]->ptrArray[i]->ptype != PT_VECTOR &&
		  a[aStack]->ptrArray[i]->function != F_INDIV) {
		bite_eprintf( "Not a parameter nor "
			"function 'indiv'. (Line %ld.)\n", lineno);
		bite_exit(1);
	      }
	      c->vec[i] = a[aStack]->ptrArray[i];
	    }
	    a[aStack]->n = 0;
	  } '=' '(' expressionlist ')' { /* Their new values. */
	    long i;
	    clusterUpdate c = cu;
	    while (c->next)
	      c = c->next;
	    for (i = 0; i < c->n; i++) {
	      if (a[aStack]->n == c->n) {
		/* If Metropolis updating: */
		findAndReplaceVarNames(-1, -1, a[aStack]->ptrArray[i],
				       NONE, target, target.x);
		c->new_val[i] = a[aStack]->ptrArray[i];
	      }
	      else {
		if (a[aStack]->n == 0) {
		  /* Only custom updating. (Metropolis-Hastings.) */
		  c->new_val[i] = c->vec[i];
		}
		else {
		  bite_eprintf( "Incorrect number of expressions on line "
			  "%ld. Should be either 0, or %ld.\n", lineno, c->n);
		}
	      }
	    }
	    a[aStack]->n = 0;
	  } LIKELIHOODS '=' '(' expressionlist ')' { /* Likelihood contributions. */
	    long i;
	    clusterUpdate c = cu;
	    while (c->next)
	      c = c->next;
	    c->nl = a[aStack]->n;
	    c->likel = malloc(c->nl * sizeof(c));
	    for (i = 0; i < c->nl; i++) {
	      findAndReplaceVarNames(-1, -1, a[aStack]->ptrArray[i],
				     NONE, target, target.x);
	      c->likel[i] = a[aStack]->ptrArray[i];
	    }
	    a[aStack]->n = 0;
	  } RANGE expressionlist { /* Loop variables. */
	    long i;
	    clusterUpdate c = cu;
	    while (c->next)
	      c = c->next;
	    c->nLoop = a[aStack]->n;
	    c->loopParam = malloc(c->nLoop * sizeof(c));
	    c->dim = malloc(c->nLoop * sizeof(i));
	    c->lower = malloc(c->nLoop * sizeof(missing));
	    c->upper = malloc(c->nLoop * sizeof(missing));
	    for (i = 0; i < c->nLoop; i++) {
	      findAndReplaceVarNames(-1, -1, a[aStack]->ptrArray[i],
				     NONE, target, target.x);
	      if (a[aStack]->ptrArray[i]->ptype != PT_SCALAR) {
		bite_eprintf(
			"Loop parameter %s on line %ld is not a scalar.\n",
			a[aStack]->ptrArray[i]->varName, lineno);
		bite_exit(1);
	      }
	      c->loopParam[i] = a[aStack]->ptrArray[i]->var->p;
	    }
	    a[aStack]->n = 0;
	  } '(' expressionlist ')' ';' { /* Loop limits (3* # of loop var). */
	    long i;
	    clusterUpdate c = cu;
	    while (c->next)
	      c = c->next;
	    target.x = NULL;
	    if (a[aStack]->n != c->nLoop * 3) {
	      bite_eprintf( "The range list of length %ld should have  "
		      "%ld loop arguments. (Line %ld.)\n", c->nLoop,
		      c->nLoop * 3, lineno);
	      bite_exit(1);
	    }
	    for (i = 0; i < a[aStack]->n / 3; i++) {
	      findAndReplaceVarNames(-1, -1, a[aStack]->ptrArray[i],
				     NONE, target, target.x);
	      c->lower[i] = a[aStack]->ptrArray[i*3];
	      c->upper[i] = a[aStack]->ptrArray[i*3+1];
	      c->dim[i] = a[aStack]->ptrArray[i*3+2];
	    }
	  }
	| NOMISSING ';' {
	    miss[0]->n = 0;
	  }
	| SET PRIOR NAME '=' DFILE ';' {
	    long k = 0;
	    prior ptr = param, p;
	    char filename[250];
	    if (printLevel >= 2)
	      bite_printf("set prior %s = file;\n", $3);
	    while (strcmp(ptr->name, $3))
	      ptr = ptr->next;
	    sprintf(filename, "%s.tmp", $3);
	    switch (ptr->ptype) {
	    case PT_SCALAR:
	    case PT_CONST:
	      if (ptr->save == SAVEITER) {
		bite_eprintf( "Cannot save and load in the same time! ");
		bite_eprintf( "(Parameter %s.)\n", ptr->name);
		bite_exit(1);
	      }
	      ptr->save = LOADITER;
	      ptr->file = openFile(filename, 'r');
	      break;
	    case PT_VECTOR:
	      p = ptr->x->vec;
	      ptr->save = LOADITER;
	      ptr->file = openFile(filename, 'r');
	      while (p) {
		sprintf(filename, "%s.tmp", $3);
		/* sprintf(filename, "%s.%ld.tmp", $3, k); */
		if (p->save == SAVEITER) {
		  bite_eprintf( "Cannot save and load in the same time! ");
		  bite_eprintf( "(Parameter %s.)\n", ptr->name);
		  bite_exit(1);
		}
		p->save = LOADITER;
		p->file = ptr->file;
		p = p->next;
		k++;
	      }
	      break;
	    }
	    if (printLevel > 1)
	      bite_printf("Opened file %s.\n", filename);
	  }
	/* | LOAD NAME ',' DFILE '=' STRING ',' TYPE '=' TRESPONCE ','
	  CARS '=' expression ',' CAR '=' expression ';' {
	    bite_eprintf( "Not in use.\n");
	    long k = 0;
	    ft dum;
	    if (printLevel >= 2)
	      bite_printf("load %s ...\n", $2);
	    while (d->dList[k] && strcmp(d->dList[k][0][0]->name, $2))
	      k++;
	    if (!d->dList[k]) {
	      dum = $14;
	      d->nCars[k] = calculate2(-1, dum);
	      rmTree(dum);
	      d->dList[k] = initData(ind, d->nCars[k]);
	      d->n++;
	    }
	    dum = $18;
	    loadData($6, $2, d->n - 1, ind, calculate2(-1, dum) - 1, nextI, d);
	    rmTree(dum);
	  } */
	| LOAD NAME ',' DFILE '=' STRING ',' TYPE '=' NAME ';' {
	    long type = NONE;

	    if (printLevel >= 2)
	      bite_printf("load %s ...\n", $2); 
	    if (!strcmp($10, "response")) {
	      if (d->n >= 1) {
		d->nCars = realloc(d->nCars, (d->n + 1) * sizeof(nextI));
		d->dList = realloc(d->dList, (d->n + 1) * sizeof(d));
		d->pList = realloc(d->pList, (d->n + 1) * sizeof(d));
	      }
	      d->nCars[d->n] = 1;
	      d->dList[d->n] = initData(1, d->nCars[d->n]);
	      d->pList[d->n] = initData2(ind, 1);
	      loadData2($6, $2, d->n, ind);
	      d->n++;
	    }
	    else {
	      long i, l;
	      /* if (!strcmp($10, "constant0"))
		type = COV_CNST; */
	      if (!strcmp($10, "constant"))
		type = COV_CNST2;
	      /* if (!strcmp($10, "startpoints0"))
		type = COV_START; */
	      if (!strcmp($10, "startpoints"))
		type = COV_START2;
	      if (!strcmp($10, "timedependent"))
		type = COV_SF;
	      if (!type) {
		bite_eprintf( "Unknown type %s.\n", $10);
		bite_exit(1);
	      }
	      if (c->n == c->storage) {
		/* Allocate more memory: */
		c->storage *= 2;
		c->cList = realloc(c->cList, c->storage * sizeof( c ) );
		c->pList = realloc(c->pList, c->storage * sizeof( c ) );
		c->nval = realloc(c->nval, c->storage * sizeof( i ) );
		c->values = realloc(c->values, c->storage * sizeof( c ) );
		c->labels = realloc(c->labels, c->storage * sizeof( c ) );
		for (i = c->n; i < c->storage; i++) {
		  c->cList[i] = NULL;
		  c->nval[i] = 0;
		  c->values[i] = NULL;
		  c->labels[i] = NULL;
		}
	      }
	      if (type != COV_CNST2 && type != COV_START2) {
		if (!c->cList[c->n])
		  c->cList[c->n] = initCov(ind);
		c->pList[c->n] = NULL;
	      }
	      else {
		c->cList[c->n] = initCov(1);
		c->pList[c->n] = malloc(sizeof(ParamVector));
	      }
	      loadCov($6, $2, ind, type, nextI, c->cList[c->n]);
	      if (mis && type == COV_CNST) {
		char num[6], dum;
		double x;
		useVal target;
		for (i = 0; i < ind; i++) { /* Find the missing covariates & */
		  /* set the priors up */
		  x = c->cList[c->n][i]->x->x->x;
		  copyPrior(mis, c->cList[c->n][i]->x);
		  c->cList[c->n][i]->x->x->x = x;
		  c->cList[c->n][i]->x->where->val->inUse = 
		    (long *) malloc(f->n * sizeof(i));
		  for (l = 0; l < c->cList[c->n][i]->x->nArgs; l++) {
		    target.x = c->cList[c->n][i]->x;
		    findAndReplaceVarNames(i, -1, c->cList[c->n][i]->x->args[l],
					   USE_PRIOR, target, target.x);
		  }
		  /*		for (l = 0; l < nv; l++)
				addUse(pv[l], c->cList[c->n][i]->x); */

		  if (c->cList[c->n][i]->x->x->x == missing) {
		    c->cList[c->n][i]->x->fixed = FALSE;
		    c->cList[c->n][i]->x->x->x = 
		      generatePriorValue(i, c->cList[c->n][i]->x);
		    /* c->cList[c->n][i]->x->sample = 
		       (double *) malloc((iter_limit - skip) *
		       sizeof(missing)); */
		    c->cList[c->n][i]->x->name = 
		      (char *) malloc(MAX_VARNAME * sizeof(dum));
		    c->cList[c->n][i]->x->name[0] = '\0';
		    strcat(c->cList[c->n][i]->x->name, "missing ");
		    strcat(c->cList[c->n][i]->x->name, $2);
		    strcat(c->cList[c->n][i]->x->name, ", indiv=");
		    intToChar(i + 1, num);
		    strcat(c->cList[c->n][i]->x->name, num);
		  }
		  else
		    c->cList[c->n][i]->x->fixed = TRUE;
		}
	      }
	      c->values[c->n] =
		covLevels(c, $2, ind, &c->nval[c->n], c->values[c->n]);
	      (c->n)++;
	    }
	  }
	| RANGE NAME ':' '(' {
	    a[aStack]->n = 0;
	  }
	  expressionlist ')' ';' {
	    long cov, i, j;
	    j = returnType($2, &cov, FALSE);
	    if (j != PT_COV) {
	      bite_eprintf( "%s is not a covariate (near line %ld).\n",
		      $2, lineno);
	      bite_exit(1);
	    }
	    if (c->nval[cov] < a[aStack]->n) {
	      c->values[cov] =
		realloc(c->values[cov], a[aStack]->n * sizeof(missing));
	      c->nval[cov] = a[aStack]->n;
	    }
	    for (i = 0; i < a[aStack]->n; i++)
	      c->values[cov][i] = calculate2(-1, a[aStack]->ptrArray[i]);
	  }
	| LABEL NAME ':' {
	    if (printLevel >= 2)
	      bite_printf("label %s ...\n", $2);
	    labela[aStack]->n = 0;
	  }
          labellist ';' {
	    long j, k, cov = 0;
	    char dum;
	    while (strcmp($2, c->cList[cov][0]->name))
	      cov++;
	    c->labels[cov] = (char **) malloc(c->nval[cov] * sizeof(c));
	    for (j = 0; j < labela[aStack]->n; j++) {
	      for (k = 0; k < c->nval[cov]; k++)
		if (labela[aStack]->label[j]->value == c->values[cov][k]) {
		  c->labels[cov][k] = (char *)
		    malloc((strlen(labela[aStack]->label[j]->label) + 1) *
			   sizeof(dum));
		  c->labels[cov][k][0] = '\0';
		  strcat(c->labels[cov][k], labela[aStack]->label[j]->label);
		}
	    }
	  }
	| LABEL PICTURE ':' STRING ';' {
	    FILE *titlep;
	    char fname[20];

	    if (printLevel >= 2)
	      bite_printf("label picture ...\n");
	    fname[0] = '\0';
	    sprintf(fname, "title%ld_%ld.tmp", cFrame, cPicture);
	    if ((titlep = bite_fopen(fname, "w")) == NULL) {
	      bite_printf("Unable to open file %s\n", fname);
	      bite_exit(1);
	    }
	    fprintf(titlep, "%s\n", $4);
	    fclose(titlep);
	  }
	| INIT FNAME expression ')' ':' {
	    if (printLevel >= 2)
	      bite_printf("init %s ...\n", $2);
	    labela[aStack]->n = 0;
	  }
          covlist ';' {
	    long j;
	    ft dum;
	    char op = '\0';

	    dum = $3;
	    if (f->n == f->storage) {
	      /* Allocate more memory. */
	      f->storage = f->storage * 2;
	      f->nFunc = realloc(f->nFunc, f->storage * sizeof(j));
	      f->fList = realloc(f->fList, f->storage * sizeof(dum));
	      for (j = f->n; j < f->storage; j++)
		f->fList[j] = NULL;
	    }
	    f->fList[f->n] = makeModel($2, calculate2(-1, dum), -1.0,
				       labela[lStack], c, ind, 
				       &f->nFunc[f->n], fixedIndivVec);
	    rmTree(dum);
	    if (op)
	      for (j = 0; j < f->nFunc[f->n]; j++)
		f->fList[f->n][j]->internOp = op;
	    f->n++;
	  }
        | MINPL NAME '=' expression ':'  {
	    if (printLevel >= 2)
	      bite_printf("minpiecelength %s = ...\n", $2);
	    labela[aStack]->n = 0;
	  }
          covvallist';' {
	    long j, l = 0;
	    double length = calculate2(-1, $4);

	    while (strcmp(f->fList[l][0]->name, $2))
	      l++;
	    for (j = 0; j < f->nFunc[l]; j++)
	      if (testCovEq(f->fList[l][j], labela[lStack]))
		f->fList[l][j]->minPieceLength = length;
	  }
        | PROPOSAL NAME '=' expression ':'  {
	    if (printLevel >= 2)
	      bite_printf("minpiecelength %s = ...\n", $2);
	    labela[aStack]->n = 0;
	  }
          covvallist';' {
	    long j, l = 0;
	    double epsilon = calculate2(-1, $4);

	    while (strcmp(f->fList[l][0]->name, $2))
	      l++;
	    for (j = 0; j < f->nFunc[l]; j++)
	      if (testCovEq(f->fList[l][j], labela[lStack]))
		f->fList[l][j]->epsilon = epsilon;
	  }
        | FIXEDLE NAME ',' DFILE '=' STRING ';' {
	    long j, l = 0, k;
	    double x;
	    FILE *fp;
	    if (!(fp = bite_fopen($6, "r"))) {
	      bite_eprintf( "Couldn't open file %s.\n", $6);
	      bite_exit(1);
	    }
	    while (strcmp(f->fList[l][0]->name, $2))
	      l++;
	    x = readFileD2(fp);
	    for (j = 0; j < f->nFunc[l]; j++) {
	      k = 0;
	      while (x != nextI) {
		f->fList[l][j]->nfixle++;
		f->fList[l][j]->fixle_x = 
		  realloc(f->fList[l][j]->fixle_x, 
			  f->fList[l][j]->nfixle * sizeof(x));
		f->fList[l][j]->fixle_y = 
		  realloc(f->fList[l][j]->fixle_y, 
			  f->fList[l][j]->nfixle * sizeof(x));
		f->fList[l][j]->fixle_x[k] = x;
		x = readFileD2(fp);
		f->fList[l][j]->fixle_y[k] = x;
		k++;
		x = readFileD2(fp);
	      }
	    }
	  }
        | FIXEDJP NAME '=' {
	    if (printLevel >= 2)
	      bite_printf("fixedjp %s = ...\n", $2);
	      a[aStack]->n = 0;
	  }
          '(' expressionlist ')' ':' {
	    long j;
	    njp = a[aStack]->n;
	    jp = malloc(njp * sizeof(missing));
	    for (j = 0; j < njp; j++)
	      jp[j] = calculate2(-1, a[aStack]->ptrArray[j]);
	    labela[aStack]->n = 0;
	  }
          covvallist';' {
	    long j, l = 0, k;
	    
	    while (strcmp(f->fList[l][0]->name, $2))
	      l++;
	    for (j = 0; j < f->nFunc[l]; j++)
	      if (testCovEq(f->fList[l][j], labela[lStack])) {
		f->fList[l][j]->nfixjp = njp;
		f->fList[l][j]->fixjp = malloc(njp * sizeof(missing));
		for (k = 0; k < njp; k++)
		  f->fList[l][j]->fixjp[k] = jp[k];
	      }
	    free(jp);
	  }
	| INTERNOP NAME STRING ';' {
	    long j, l = 0;
	    int op = '\0';
	    if (printLevel >= 2)
	      bite_printf("internop %s = ...\n", $2);
	    if (!strcmp($3, "*"))
	      op = '*';
	    else
	      if (!strcmp($3, "+"))
		op = '+';
	      else
		if (!strcmp($3, "*+") || !strcmp($3, "+*"))
		  op = '_'; /* RS Changes pykala to _ */
		else {
		  bite_eprintf( "%s: no such operator. (Line %ld.)\n",
			  $3, lineno);
		  bite_exit(1);
		}
	    while (strcmp(f->fList[l][0]->name, $2))
	      l++;
	    for (j = 0; j < f->nFunc[l]; j++)
	      f->fList[l][0]->internOp = op;
	  }
        | RESTRICT NAME '=' expression ':'  {
	    if (printLevel >= 2)
	      bite_printf("restrict %s = ...\n", $2);
	    labela[aStack]->n = 0;
	  }
          covvallist';' {
	    long j, l = 0;
	    double length = calculate2(-1, $4);

	    while (strcmp(f->fList[l][0]->name, $2))
	      l++;
	    for (j = 0; j < f->nFunc[l]; j++)
	      if (testCovEq(f->fList[l][j], labela[lStack]))
		f->fList[l][j]->restrict0 = length;
	  }
	| SAVE expression ';' {
	    long i, j = 0, k = 0, l, cov = 0, mark = 0, found = FALSE;
	    char filename[250], str[20], *name;
	    if (printLevel >= 2)
	      bite_printf("save %s ...\n", $2->varName);
	    /* Save the value(s) of a parameter to a file. */
	    while (j < f->n && strcmp(f->fList[j][0]->name, $2->varName))
	      j++;
	    filename[0] = '\0';
	    if (j < f->n) { /* A model function. */
	      found = TRUE;
	      for (k = 0; k < f->nFunc[j]; k++) {
		if (f->fList[j][k]->save == LOADITER) {
		  bite_eprintf( "Cannot save and load in the same time!");
		  bite_eprintf( " (Function %s.)\n", f->fList[j][k]->name);
		  bite_exit(1);
		}
		f->fList[j][k]->save = SAVEITER;
		sprintf(filename, "%s.tmp", $2->varName);
		/*		if ((f->fList[j][k]->fp = bite_fopen(filename, "w")) == NULL) {
		  bite_eprintf( "Cannot open file %s.\n", filename);
		  bite_exit(1);
		} */
		f->fList[j][k]->file = openFile(filename, 'w');
		l = 0;
		sprintf(filename, "# %s(", $2->varName);
		writeFileS(f->fList[j][k]->file, filename);
		while (f->fList[j][k]->cov[l][0]) {
		  if (l) {
		    sprintf(filename, ", ");
		    writeFileS(f->fList[j][k]->file, filename);
		  }
		  doubleToChar(f->fList[j][k]->covVal[l], str);
	          sprintf(filename, "%s=%s", f->fList[j][k]->cov[l], str);
		  writeFileS(f->fList[j][k]->file, filename);
	          l++;
		}
		sprintf(filename, ")\n");
		writeFileS(f->fList[j][k]->file, filename);
	      }
	    }
	    if (!found) { /* A scalar parameter */
	      prior ptr = param, p;
	      while (ptr && strcmp(ptr->name, $2->varName))
		ptr = ptr->next;
	      if (ptr) {
		found = TRUE;
		sprintf(filename, "%s.tmp", $2->varName);
		switch (ptr->ptype) {
		case PT_CONST:
		case PT_SCALAR:
		  if (ptr->save == LOADITER) {
		    bite_eprintf( "Cannot save and load in the same time! ");
		    bite_eprintf( "(Parameter %s.)\n", ptr->name);
		    bite_exit(1);
		  }
		  ptr->save = SAVEITER;
		  ptr->file = openFile(filename, 'w');
		  break;
		case PT_VECTOR:
		  p = ptr->x->vec;
		  ptr->save = SAVEITER;
		  ptr->file = openFile(filename, 'w');
		  while (p) {
		    sprintf(filename, "%s.tmp", $2->varName);
		    /* sprintf(filename, "%s.%ld.tmp", $2->varName, k); */
		    if (p->save == LOADITER) {
		      bite_eprintf( "Cannot save and load in the same ");
		      bite_eprintf( "time! (Parameter %s.)\n", p->name);
		      bite_exit(1);
		    }
		    p->save = SAVEITER;
		    p->file = ptr->file;
		    p = p->next;
		    k++;
		  }
		  break;
		}
	      }
	    }
	    if (!found) {
	      while (cov < c->n && strcmp($2->varName, c->cList[cov][0]->name))
		cov++;
	      if (cov < c->n) { /* A covariate. */
		prior dum = NULL;
		found = TRUE;
		name = c->cList[cov][0]->name;
		for (i = 0; i < ind; i++) {
		  filename[0] = '\0';
		  switch (c->cList[cov][i]->type) {
		  case COV_CNST:
		    dum = c->cList[cov][i]->x;
		    break;
		  case COV_SF:
		    bite_eprintf( "%s: Timedependent ",
			    c->cList[cov][0]->name);
		    bite_eprintf( "covariates cannot be random.\n");
		    bite_exit(1);
		    break;
		  case COV_START:
		    if ($2->i) {
		      j = 1;
		      dum = c->cList[cov][i]->x->x->vec;
		      k = (long) calculate2(i, $2->i[0]);
		      if (dum) {
			while (dum && j < k) {
			  dum = dum->next;
			  j++;
			}
		      }
		    }
		    break;
		  }
		  if (dum) {
		    if (dum->save == LOADITER) {
		      bite_eprintf( "Cannot save and load in the same");
		      bite_eprintf( " time! (Covariate %s.)\n", name);
		      bite_exit(1);
		    }
		    dum->save = SAVEITER;
		    sprintf(filename, "%s.", name);
		    /* sprintf(filename, "%s.%ld.", name, i + 1); */
		    if (c->cList[cov][i]->type == COV_START)
		      sprintf(filename, "%s%ld.", filename, k);
		    strcat(filename, "tmp");
		    dum->file = openFile(filename, 'w');
		  }
		}
	      }
	    }
	    if (!found) {
	      while (mark < d->n &&
		     strcmp($2->varName, d->dList[mark][0][0]->name))
		mark++;
	      if (mark < d->n) {
		prior dum = NULL;
		found = TRUE;
		j = calculate2(-1, $2->i[0]);
		name = d->dList[mark][0][j]->name;
		for (i = 0; i < ind; i++) {
		  if (d->dList[mark][i][j]->type == CONT) {
		    switch ($2->fieldId) {
		    case T0:
		      dum = d->dList[mark][i][j]->pp->t0;
		      break;
		    case T1:
		      dum = d->dList[mark][i][j]->pp->t1;
		      break;
		    case POINTS:
		      if ($2->i) {
			dum = d->dList[mark][i][j]->pp->vector;
			k = (long) calculate2(i, $2->third->i[0]) - 1;
			j = 0;
			if (dum) {
			  while (dum && j < k) {
			    dum = dum->next;
			    j++;
			  }
			}
		      }
		      break;
		    }
		    if (dum) {
		      if (dum->save == LOADITER) {
			bite_eprintf( "Cannot save and load in the same ");
			bite_eprintf( "time! (Responce %s.)\n", name);
			bite_exit(1);
		      }
		      dum->save = SAVEITER;
		      sprintf(filename, "%s.%ld.%s.", name, j + 1, dum->name);
		      /* sprintf(filename, "%s.%ld.%s.%ld.", name, j + 1, 
			      dum->name, i + 1); */
		      if ($2->fieldId == POINTS)
			sprintf(filename, "%s%ld.", filename, k);
		      strcat(filename, "tmp");
		      dum->file = openFile(filename, 'w');
		    }
		  }
		}
	      }
	    }
	    if (!found) {
	      bite_eprintf( "%s: What is that?\n", $2->varName);
	      bite_exit(1);
	    }
	  }
	| INITIALS expression IN STRING ';' {
	    long i, j, type, idx, k;
	    prior ptr = param, p;
	    double dumd;
	    char filename[250];
	    FILE *fp;
	    if (printLevel >= 2)
	      bite_printf("initials %s ...\n", $2->varName);
	    /* Load initial values of parameters from a file. */
	    sprintf(filename, "%s", $4);
	    if ((fp = bite_fopen(filename, "r")) == NULL) {
	      bite_printf("Unable to open file %s\n", filename);
	      bite_exit(1);
	    }
	    type = returnType($2->varName, &idx, FALSE);
	    switch (type) {
	    case PT_MODELF:
	      for (i = 0; i < f->nFunc[idx]; i++)
		readSf2(fp, f->fList[idx][i]->f->f, -999.0);
	      /* bite_eprintf( "Model functions not implemented yet.\n");
	      bite_exit(1); */
	      break;
	    case PT_SCALAR:
	      while (ptr && strcmp(ptr->name, $2->varName))
		ptr = ptr->next;
	      if (ptr->ptype == PT_VECTOR) {
		p = ptr->x->vec;
		while (p) {
		  fscanf(fp, "%lf", &dumd);
		  p->x->x = (dumd == missing) ? p->x->x : dumd;
		  p = p->next;
		}
	      }
	      else {
		fscanf(fp, "%lf", &dumd);
		ptr->x->x = (dumd == missing) ? ptr->x->x : dumd;
	      }
	      break;
	    case PT_COV:
	      for (i = 0; i < ind; i++) {
		switch (c->pList[idx]->type) {
		case COV_CNST2:
		  fscanf(fp, "%lf", &dumd);
		  if (dumd != missing)
		    c->pList[idx]->val.x[i] = dumd;
		  break;
		case COV_SF:
		  bite_eprintf( "Timedependent not implemented yet.\n");
		  bite_exit(1);
		  break;
		case COV_START2:
		  j = dumd = 0;
		  while (j < c->pList[idx]->n2[i] && dumd != nextI) {
		    fscanf(fp, "%lf", &dumd);
		    if (dumd != missing && dumd != nextI)
		      c->pList[idx]->val.x2[i][j] = dumd;
		    j++;
		  }
		  break;
		}
	      }
	      break;
		/* filename[0] = '\0';
		switch (c->cList[idx][i]->type) {
		case COV_CNST:
		  fscanf(fp, "%lf", &dumd);
		  if (dumd != missing)
		    c->cList[idx][i]->x->x->x = dumd;
		  break;
		case COV_SF:
		  bite_eprintf( "Timedependent not implemented yet.\n");
		  bite_exit(1);
		  break;
		case COV_START:
		  if ($2->i) {
		    j = 1;
		    dum = c->cList[idx][i]->x->x->vec;
		    k = (long) calculate2(i, $2->i[0]);
		    if (dum) {
		      while (dum && j < k) {
			dum = dum->next;
			j++;
		      }
		      fscanf(fp, "%lf", &dumd);
		      if (dumd != missing)
			dum->x->x = dumd;
		    }
		  }
		  break;
		}
	      }
	      break; */
	      /* case PT_RESP:
	      j = calculate2(-1, $2->i[0]);
	      name = d->dList[idx][0][j]->name;
	      for (i = 0; i < ind; i++) {
		if (d->dList[idx][i][j]->type == CONT) {
		  switch ($2->fieldId) {
		  case T0:
		    fscanf(fp, "%lf", &d->dList[idx][i][j]->pp->t0->x->x);
		    break;
		  case T1:
		    fscanf(fp, "%lf", &d->dList[idx][i][j]->pp->t1->x->x);
		    break;
		  case POINTS:
		    if ($2->i) {
		      dum = d->dList[idx][i][j]->pp->vector;
		      k = (long) calculate2(i, $2->third->i[0]) - 1;
		      j = 0;
		      if (dum) {
			while (dum && j < k) {
			  dum = dum->next;
			  j++;
			}
			fscanf(fp, "%lf", &dum->x->x);
		      }
		    }
		    break;
		  }
		}
	      } 
	      break; */
	    case PT_RESP:
	      j = ($2->ni) ? calculate2(-1, $2->i[0]) : 0;
	      for (i = 0; i < ind; i++) {
		if (d->pList[idx]->n2[i] > j) {
		  switch ($2->fieldId) {
		  case T0:
		    if (!d->pList[idx]->fixed.pp.t0[i][j])
		      fscanf(fp, "%lf", &d->pList[idx]->val.pp.t0[i][j]);
		    else
		      fscanf(fp, "%lf", &dumd);
		    break;
		  case T1:
		    if (!d->pList[idx]->fixed.pp.t1[i][j])
		      fscanf(fp, "%lf", &d->pList[idx]->val.pp.t1[i][j]);
		    else
		      fscanf(fp, "%lf", &dumd);
		    break;
		  case POINTS:
		    if ($2->i) {
		      k = (long) calculate2(i, $2->third->i[0]) - 1;
		      if (d->pList[idx]->val.pp.npoints[i][j] > k &&
			  !d->pList[idx]->fixed.pp.points[i][j][k])
			fscanf(fp, "%lf",
			       &d->pList[idx]->val.pp.points[i][j][k]);
		      else
			fscanf(fp, "%lf", &dumd);
		    }
		    break;
		  }
		}
		else
		  fscanf(fp, "%lf", &dumd);
	      }
	      break;
	    }
	  }
	| PUT NAME TO NAME ':' {
	    if (printLevel >= 2)
	      bite_printf("put %s ...\n", $2);
	    labela[aStack]->n = 0;
	  }
          covvallist ';' {
	    long j = 5, l = 0, m = 0;

	    while (strcmp(f->fList[l][0]->name, $4))
	      l++;
	    while (m < nVars && strcmp($2, vars[m]->name))
	      m++; 
	    for (j = 0; j < f->nFunc[l]; j++) {
	      if (testCovEq(f->fList[l][j], labela[lStack])) {
		use ptr;/* Put a variable to a model function. */
		switch (vars[m]->type) {
		case VAR_PRIOR:
		  f->fList[l][j]->prior = vars[m]->spec;
		  f->fList[l][j]->pVal = vars[m]->x->pv;
		  if (vars[m]->spec == TIMELESS) {
		    f->fList[l][j]->type = SF_TIMELESS;
		    f->fList[l][j]->f->p =
		      copyPrior(vars[m]->x->pv->pTl->p, f->fList[l][j]->f->p);
		  }
		  else
		    f->fList[l][j]->type = SF_ORIG;
		}
		ptr = addUse2(vars[m]->where);
		if (!vars[m]->where)
		  vars[m]->where = ptr;
		ptr->type = USE_MF;
		ptr->val->f = f->fList[l][j];
		if (m >= 0 && vars[m]->spec == TIMELESS) {
		  target.f = f->fList[l][j];
		  findAndReplaceVarNames2(-1, -1, f->fList[l][j]->f->p,
					  USE_MF, target);
		}
		if (m >= 0 && vars[m]->spec == GAMMA_LEXIS) {
		  if (vars[m]->x->pv->pGL->lexis_dim != 
		      f->fList[l][0]->neighbour_dim) {
		    bite_eprintf(
			    "Dimension of the lexis gamma prior does not match the number of covariates of the model function (line %ld).\n",
			    lineno);
		    bite_exit(1);
		  }
		  /* if (!f->fList[l][j]->neighbour_dim) {
		    f->fList[l][j]->neighbour_dim =
		      vars[nVars]->x->pv->pGL->lexis_dim;
		    f->fList[l][j]->neighbour_prev = 
		      malloc(vars[nVars]->x->pv->pGL->lexis_dim * sizeof(f));
		    f->fList[l][j]->neighbour_next = 
		      malloc(vars[nVars]->x->pv->pGL->lexis_dim * sizeof(f));
		    for (k = 0; k < vars[nVars]->x->pv->pGL->lexis_dim; k++) {
		      f->fList[l][j]->neighbour_prev[k] = NULL;
		      f->fList[l][j]->neighbour_next[k] = NULL;
		    }
		  }
		  f->fList[l][j]->neighbour_prev[0] = prev;
		  if (prev)
		    prev->neighbour_prev[0] = f->fList[l][j];
		    prev = f->fList[l][j]; */
		}
	      }
	    }
	  }
	| PUT NAME TO expression ';' {
	    long i = 0, j, k, cov = 0, resp = 0;
	    prior ptr = param, dum;
	    if (printLevel >= 2)
	      bite_printf("put %s ...\n", $2);
	    /* Put a prior distribution to a covariate. */
	    if (!$4->varName) {
	      bite_eprintf( "\'put %s\': expression too complicated.\n", $2);
	      bite_exit(1);
	    }
	    while (ptr && strcmp(ptr->name, $2))
	      ptr = ptr->next;
	    if (!ptr) {
	      bite_eprintf( "%s: parameter not found.\n", $2);
	      bite_exit(1);
	    }
	    while (cov < c->n && strcmp($4->varName, c->cList[cov][0]->name))
	      cov++;
	    if (cov < c->n) {
	      if (!c->pList[cov]) {
		for (i = 0; i < ind; i++) {
		  switch (c->cList[cov][i]->type) {
		  case COV_CNST:
		    copyPrior(ptr, c->cList[cov][i]->x);
		    target.x = c->cList[cov][i]->x;
		    findAndReplaceVarNames2(i, -1, c->cList[cov][i]->x,
					    USE_PRIOR, target);
		    break;
		  case COV_SF:
		    bite_eprintf( "%s: Timedependent covariates cannot "
			    "be random.\n", c->cList[cov][0]->name);
		    bite_exit(1);
		    break;
		  case COV_START:
		    if ($4->i) {
		      j = 0;
		      dum = c->cList[cov][i]->x->x->vec;
		      k = (long) calculate2(i, $4->i[0]);
		      while (dum && j < k) {
			dum = dum->next;
			j++;
		      }
		      if (dum) {
			copyPrior(ptr, dum);
			c->cList[cov][i]->x->nolikelihood = ptr->nolikelihood;
			target.x = c->cList[cov][i]->x;
			findAndReplaceVarNames2(i, -1, dum, USE_PRIOR, target);
		      }
		    }
		    break;
		  }
		}
	      }
	      else {
		long type = NONE;
		if (ptr->type == D_DIRAC) {
		  long j;
		  switch (k = returnDiracPriorType(ptr)) {
		  case SCALAR:
		    type = USE_PRIOR;
		    break;
		  case INDIVIDUAL:
		    i = returnType(c->cList[cov][0]->name, &j, FALSE);
		    type = (i == PT_COV) ? USE_COV2 : USE_TABLE;
		    break;
		  default:
		    bite_eprintf( "%s should be either \'indiv\' "
			    "or \'param\'.\n", ptr->name);
		  }
		}
		else
		  type = USE_COV2; /* was USE_TABLE */
		switch (c->pList[cov]->type) {
		case COV_CNST2:
		  c->pList[cov]->distr.x = initPrior("");
		  copyPrior(ptr, c->pList[cov]->distr.x);
		  ptr->fixed = TRUE;
		  if (type != USE_COV2)
		    target.x = c->pList[cov]->distr.x;
		  else
		    target.pvec = c->pList[cov];
		  for (i = 0; i < ind; i++)
		    findAndReplaceVarNames2(i, -1, c->pList[cov]->distr.x,
					    type, target);
		  break;
		case COV_START2:
		  if ($4->i) {
		    k = (long) calculate2(i, $4->i[0]);
		    level++;
		    lindex[level] = k;
		    c->pList[cov]->distr.x2[k] = initPrior("");
		    if (type != USE_COV2)
		      target.x = c->pList[cov]->distr.x2[k];
		    else
		      target.pvec = c->pList[cov];
		    copyPrior(ptr, c->pList[cov]->distr.x2[k]);
		    ptr->fixed = TRUE;
		    c->pList[cov]->distr.x2[k]->nolikelihood =
		      ptr->nolikelihood;
		    for (i = 0; i < ind; i++)
		      findAndReplaceVarNames2(i, -1,
			c->pList[cov]->distr.x2[k], type, target);
		    for (j = 0; j < c->pList[cov]->stor; j++)
		      if (c->pList[cov]->distr.x2[j] && j != k)
			c->pList[cov]->distr.x2[j]->where =
			  c->pList[cov]->distr.x2[k]->where;
		    level--;
		  }
		  break;
		}
	      }
	    }
	    else {
	      while (resp < d->n &&
		     strcmp($4->varName, d->dList[resp][0][0]->name))
		resp++;
	      if (resp < d->n) {
		j = calculate2(-1, $4->i[0]);
		if (d->pList[resp]) {
		  if (d->pList[resp]->type == CONT) {
		    switch ($4->fieldId) {
		    case T0:
		      d->pList[resp]->distr.pp.t0[j] = initPrior("t0");
		      copyPrior(ptr, d->pList[resp]->distr.pp.t0[j]);
		      target.x = d->pList[resp]->distr.pp.t0[j];
		      findAndReplaceVarNames2(0, resp, target.x, USE_TABLE,
					      target);
		      break;
		    case T1:
		      d->pList[resp]->distr.pp.t1[j] = initPrior("t1");
		      copyPrior(ptr, d->pList[resp]->distr.pp.t1[j]);
		      target.x = d->pList[resp]->distr.pp.t1[j];
		      findAndReplaceVarNames2(0, resp, target.x,
					      USE_TABLE, target);
		      break;
		    case POINTS:
		      if ($4->i) {
			k = (long) calculate2(i, $4->third->i[0]);
			if (k < d->pList[resp]->distr.pp.npoints[j]) {
			  d->pList[resp]->distr.pp.points[j][k] =
			    initPrior("point");
			  copyPrior(ptr,
				    d->pList[resp]->distr.pp.points[j][k]);
			  target.x = d->pList[resp]->distr.pp.points[j][k];
			  findAndReplaceVarNames2(0, resp,
				  d->pList[resp]->distr.pp.points[j][k],
				  USE_TABLE, target);
			}
		      }
		      break;
		    }
		  }
		}
		else {
		  for (i = 0; i < ind; i++) {
		    /*		  for (j = 0; j < d->nCars[resp]; j++) { */
		    if (d->dList[resp][i][j]->type == CONT) {
		      switch ($4->fieldId) {
		      case T0:
			copyPrior(ptr, d->dList[resp][i][j]->pp->t0);
			target.x = d->dList[resp][i][j]->pp->t0;
			findAndReplaceVarNames2(i, resp, target.x, USE_TABLE,
						target);
			break;
		      case T1:
			copyPrior(ptr, d->dList[resp][i][j]->pp->t1);
			target.x = d->dList[resp][i][j]->pp->t1;
			findAndReplaceVarNames2(i, resp, target.x,
						USE_TABLE, target);
			break;
		      case POINTS:
			if ($4->i) {
			  dum = d->dList[resp][i][j]->pp->vector;
			  k = (long) calculate2(i, $4->third->i[0]);
			  j = 0;
			  if (dum) {
			    while (dum && j < k) {
			      dum = dum->next;
			      j++;
			    }
			    if (dum) {
			      copyPrior(ptr, dum);
			      findAndReplaceVarNames2(i, resp, dum, USE_TABLE,
						      target);
			    }
			  }
			}
			break;
		      }
		    }
		  }
		}
	      }
	      else {
		bite_eprintf( "%s: covariate or ", $4->varName);
		bite_eprintf( "responce not found.\n");
		bite_exit(1);
	      }
	    }
	  }
	| PUT DFILE TO FNAME {
	    if (printLevel >= 2)
	      bite_printf("put file to %s();\n", $4);
	    labela[aStack]->n = 0;
	  }
          covvallist ')' ';' {
	    long k = 0, l = 0;
	    char /*dum[20],*/ filename[250];

	    while (strcmp(f->fList[l][0]->name, $4))
	      l++;
	    filename[0] = '\0';
	    strcat(filename, $4);
	    strcat(filename, ".tmp");
	    for (k = 0; k < f->nFunc[l]; k++) {
	      /* if (testCovEq(f->fList[l][k], labela[lStack])) {
		if (f->fList[l][k]->save == SAVEITER) {
		  bite_eprintf( "Cannot save and load in the same time!");
		  bite_eprintf( " (Function %s.)\n", f->fList[l][k]->name);
		  bite_exit(1);
		}
		f->fList[l][k]->save = LOADITER;
		filename[0] = '\0';
		strcat(filename, $4);
		m = 0;
		while (f->fList[l][k]->cov[m][0]) {
		  strcat(filename, ".");
		  strcat(filename, f->fList[l][k]->cov[m]);
		  strcat(filename, "=");
		  doubleToChar(f->fList[l][k]->covVal[m], dum);
		  strcat(filename, dum);
		  m++;
		}
		strcat(filename, ".tmp");
		if ((f->fList[l][k]->fp = bite_fopen(filename, "r")) == NULL) {
		  bite_eprintf( "Cannot open file %s\n", filename);
		  bite_exit(1);
		}
		f->fList[l][k]->file = openFile(filename, 'r');
	      } */
	      f->fList[l][k]->save = LOADITER;
	      f->fList[l][k]->file = openFile(filename, 'r');
	      if (printLevel > 1)
	 bite_printf("Opened file %s.\n", filename);
	    }
	  }
	| GENERATE ';' {
	    long j, k, l;
	    if (printLevel >= 2)
	      bite_printf("generate;\n");
	    for (l = 0; l < f->n; l++) {
	      for (j = 0; j < f->nFunc[l]; j++) {
	      /* for (j = 0; j < f->nFunc[l][0]->neighbour_dim; j++) { */
		if (f->fList[l][j]->prior == -1) {
		  long i = 0;
		  bite_eprintf( "No prior for model function %s(",
			  f->fList[l][0]->name);
		  while (f->fList[l][j]->cov[i][0]) {
		    bite_eprintf( "%s=%g,", f->fList[l][j]->cov[i],
			    f->fList[l][j]->covVal[i]);
		    i++;
		  }
		  bite_eprintf( ")\n");
		  bite_exit(1);
		}
		if (f->fList[l][0]->name[0] &&
		    f->fList[l][0]->type == SF_ORIG) {
		  if (f->fList[l][j]->prior != JHEIKK) {
		    if (f->fList[l][j]->fixjp ||
			f->fList[l][j]->prior == BROWNIAN) {
		      f->fList[l][j]->f->f = 
			poiProcess(0.0, f->fList[l][j]->length, 0, 0, NULL);
		      if (f->fList[l][j]->fixjp)
			for (k = 0; k < f->fList[l][j]->nfixjp; k++)
			  addNode(f->fList[l][j]->f->f, k + 1,
				  f->fList[l][j]->fixjp[k], 1);
		      else {
			double
			  delta = calculate2(-1, f->fList[l][j]->pVal->pBM->delta),
			  drift = calculate2(-1, f->fList[l][j]->pVal->pBM->drift),
			  sigma2 = calculate2(-1, f->fList[l][j]->pVal->pBM->sigma2),
			  x1 = 0, y1 = 0, x2 = 0, y2 = 0;
			long k1 = 0;
			for (k = 0; k < f->fList[l][j]->length / delta - 1; k++) {
			  if (k1 < f->fList[l][j]->nfixle &&
			      k * delta <= x2 && x2 < (k + 1) * delta) {
			    x1 = x2;
			    y1 = y2;
			    x2 = f->fList[l][j]->fixle_x[k1];
			    y2 = f->fList[l][j]->fixle_y[k1];
			    k1++;
			  }
			  addNode(f->fList[l][j]->f->f, k + 1, (k + 1) * delta,
				  (k * delta < x2) ?
				  ((k+1) * delta - x1) / (x2-x1) * (y2-y1) + y1:
				  ((k * delta <= x2 && x2 < (k + 1) * delta) ?
				   y2 :
				   f->fList[l][j]->f->f->levels[k] +
				   drift * delta + ran_norm() *
				   sqrt(sigma2 * delta)));
			}
		      }
		    }
		    else {
		      if (f->fList[l][j]->prior != GAMMA_LEXIS) {
			f->fList[l][j]->f->f = 
			  poiProcess(0.0, f->fList[l][j]->length,
				     calculate2(-1, f->fList[l][j]->pVal->pGa->mu),
				     f->fList[l][j]->minPieceLength, NULL);
		      }
		      else {
			f->fList[l][j]->f->f = 
			  poiProcess(0.0, f->fList[l][j]->length,
				     calculate2(-1, f->fList[l][j]->pVal->pGL->mu),
				     f->fList[l][j]->minPieceLength, NULL);
		      }
		    }
#ifdef HAMMAS
		    if (f->fList[l][j]->f->f->n == 2) {
		      addNode(f->fList[l][j]->f->f, 1, 1,
			      (f->fList[l][j]->f->f->levels[0] +
			       f->fList[l][j]->f->f->levels[1]) / 2);
		      addNode(f->fList[l][j]->f->f, 2, 2,
			      (f->fList[l][j]->f->f->levels[1] +
			       f->fList[l][j]->f->f->levels[2]) / 2);
		    }
		    else {
		      if (f->fList[l][j]->f->f->n > 2) {
			if (f->fList[l][j]->f->f->jumpPoints[1] > 2) {
			  addNode(f->fList[l][j]->f->f, 1, 1,
				  (f->fList[l][j]->f->f->levels[0] +
				   f->fList[l][j]->f->f->levels[1]) / 2);
			  addNode(f->fList[l][j]->f->f, 2, 2,
				  (f->fList[l][j]->f->f->levels[1] +
				   f->fList[l][j]->f->f->levels[2]) / 2);
			}
			else {
			}
		      }
		    }
#endif
		  }
		  else { /* Create initial values for a simple function and */
		    /* generating points of the Arjas-Heikkinen prior. */
		    if (f->fList[l][j]->fixjp) {
		      f->fList[l][j]->f->f = 
			poiProcess(0.0, f->fList[l][j]->length, 0, 0, NULL);
		      for (k = 0; k < f->fList[l][j]->nfixjp; k++)
			addNode(f->fList[l][j]->f->f, k + 1,
				f->fList[l][j]->fixjp[k], 1);
		    }
		    else
		      f->fList[l][j]->f->f = 
			poiProcess(0.0, f->fList[l][j]->length,
				   f->fList[l][j]->pVal->pJ->lambdaXi,
				   f->fList[l][j]->minPieceLength, NULL);
		    if (f->fList[l][j]->f->f->n == 2)
		      addNode(f->fList[l][j]->f->f, 1, 
			      ran_uni(0.0, f->fList[l][j]->length), 0.0);
		    for (k = 1; k < f->fList[l][j]->f->f->n - 1; k++)
		      f->fList[l][j]->f->f->genPoints[k - 1] = 
			f->fList[l][j]->f->f->jumpPoints[k];
		    for (k = 1; k < f->fList[l][j]->f->f->n - 2; k++)
		      f->fList[l][j]->f->f->jumpPoints[k] = 
			(f->fList[l][j]->f->f->genPoints[k - 1] +
			 f->fList[l][j]->f->f->genPoints[k]) / 2.0;
		    delNode(f->fList[l][j]->f->f, f->fList[l][j]->f->f->n - 2);
		  }
		  switch (f->fList[l][j]->prior) {
		  case GAMMA:
		    levelsG(f->fList[l][j]->f->f, 
			    calculate2(-1, f->fList[l][j]->pVal->pGa->alpha0),
			    calculate2(-1, f->fList[l][j]->pVal->pGa->beta0),
			    calculate2(-1, f->fList[l][j]->pVal->pGa->alpha),
			    calculate2(-1, f->fList[l][j]->pVal->pGa->beta),
			    f->fList[l][j]->restrict0);
		    break;
		  case GAMMA2:
		    levelsG2(f->fList[l][j]->f->f, 
			    calculate2(-1, f->fList[l][j]->pVal->pGa->alpha0),
			    calculate2(-1, f->fList[l][j]->pVal->pGa->beta0),
			    calculate2(-1, f->fList[l][j]->pVal->pGa->alpha),
			    f->fList[l][j]->restrict0);
		    break;
		  case JHEIKK:
		    for (k = 0; k < f->fList[l][j]->f->f->n - 1; k++)
		      f->fList[l][j]->f->f->levels[k] = 
			exp(f->fList[l][j]->pVal->pJ->mu + ran_norm() *
			    sqrt(f->fList[l][j]->pVal->pJ->sigma2));
		    break;
		  case INCR_GAMMA:
		    levelsIG(f->fList[l][j]->f->f, 
			    calculate2(-1, f->fList[l][j]->pVal->pGa->alpha0),
			    calculate2(-1, f->fList[l][j]->pVal->pGa->beta0),
			    calculate2(-1, f->fList[l][j]->pVal->pGa->alpha),
			    calculate2(-1, f->fList[l][j]->pVal->pGa->beta),
			    f->fList[l][j]->restrict0);
		    break;
		  case DECR_GAMMA:
		    levelsDG(f->fList[l][j]->f->f, 
			    calculate2(-1, f->fList[l][j]->pVal->pGa->alpha0),
			    calculate2(-1, f->fList[l][j]->pVal->pGa->beta0),
			    f->fList[l][j]->restrict0);
		    break;
		  case GAMMA_LEXIS:
		    {
		      double *alpha_vec = 
			malloc(f->fList[l][j]->pVal->pGL->lexis_dim * sizeof(missing));
		      long k;
		      for (k = 0; k<f->fList[l][j]->pVal->pGL->lexis_dim; k++)
			alpha_vec[k] = calculate2(-1, f->fList[l][j]->pVal->pGL->lexis_weight[k]);
		      levelsGL(f->fList[l][j],
			       f->fList[l][j]->f->f, 
			       calculate2(-1, f->fList[l][j]->pVal->pGL->alpha0),
			       calculate2(-1, f->fList[l][j]->pVal->pGL->beta0),
			       calculate2(-1, f->fList[l][j]->pVal->pGL->alpha),
			       calculate2(-1, f->fList[l][j]->pVal->pGL->beta),
			       alpha_vec);
		    }
		    break;
		  }
		}
	      }
	    }
	    generateCovs();
	    generateResp();
	  }
	| MODEL NAME '=' expression ';' {
	    long j, k = 0;
	    char dum;

	    if (printLevel >= 2)
	      bite_printf("model %s ...\n", $2);
	    model->n = d->n;
	    while (k < d->n && strcmp($2, d->dList[k][0][0]->name))
	      k++;
	    if (k == d->n) {
	      bite_eprintf( "Responce %s not found.\n", $2);
	      bite_exit(1);
	    }
	    if (!model->ftList[k]) {
	      model->ftList[k] = (ft *) malloc( (1) * sizeof(model) );
	      model->ftList[k][0] = $4;
	      for (j = 0; j < ind; j++) {
		/* model->ftList[k][j] = initFNode();
		copyFt($4, model->ftList[k][j]); */
		findCovsAndMfs(j, k, c, f, model->ftList[k][0]);
	      }
	      /* findCovsAndMfs(j, k, c, f, model->ftList[k][ind]); */
	    }
	    model->name[k] = (char *) malloc((MAX_VARNAME + 1) * sizeof(dum));
	    model->name[k][0] = '\0';
	    strcat(model->name[k], $2);
	  }
	| MONITOR FNAME {
	    if (printLevel >= 2)
	      bite_printf("monitor %s ...\n", $2);
	    labela[aStack]->n = 0;
	  }
	  covvallist ')' ':' INTEGER ';' {
	    long l = 0, j;
	    while (strcmp(f->fList[l][0]->name, $2))
	      l++;
	    for (j = 0; j < f->nFunc[l]; j++)
	      if (testCovEq(f->fList[l][j], labela[lStack]))
		f->fList[l][j]->monitor = $7;
	  }
	| SURVIVAL NAME ':' expression TO expression ';' {
	    long l;
	    double val;
	    pi next;
	    if (!surv) {
	      surv = (pi) malloc(sizeof(jackPi));
	      surv->next = NULL;
	    }
	    next = surv;
	    while (next->next)
	      next = next->next; /* :-/ HA HA! */
	    next->next = (pi) malloc(sizeof(jackPi)); 
	    /* Add a new survival fn. */
	    next->next->n = 0;
	    next->next->mark = 0;
	    next->next->bot = malloc(sizeof(next));
	    next->next->top = malloc(sizeof(next));
	    next->next->sample = (double ***) malloc(sizeof(next));
	    while (strcmp($2, model->name[next->next->mark]))
	      next->next->mark++;
	    next->next->bot[0] = malloc(ind * sizeof(next));
	    next->next->bot[0][0] = $4;
	    findAndReplaceVarNames(0, next->next->mark, next->next->bot[0][0],
				   NONE, target, NULL);
	    for (l = 1; l < ind; l++) {
	      next->next->bot[0][l] = initFNode();
	      copyFt($4, next->next->bot[0][l]);
	      findAndReplaceVarNames(l, next->next->mark,
				     next->next->bot[0][l], NONE, target,
				     NULL);
	    }
	    next->next->top[0] = malloc(ind * sizeof(next));
	    next->next->top[0][0] = $6;
	    next->next->sample[0] = (double **) malloc(ind * sizeof(next));
	    next->next->sample[0][0] = (double *)
	      malloc(2 * GP * sizeof(val));
	    findAndReplaceVarNames(0, next->next->mark, next->next->top[0][0],
				   NONE, target, NULL);
	    for (l = 1; l < ind; l++) {
	      next->next->sample[0][l] = (double *)
		malloc(2 * GP * sizeof(val));
	      next->next->top[0][l] = initFNode();
	      copyFt(next->next->top[0][0], next->next->top[0][l]);
	      findAndReplaceVarNames(l, next->next->mark,
				     next->next->top[0][l], NONE, target,
				     NULL);
	    }
	    next->next->n = 1;
	  }
	| TTT NAME ':' {
	    labela[aStack]->n = 0;
	  }
	  covlist ';' {
	    initTTT($2);
	  }
        | EXPECTATION NAME ':' expression ';' {
	    long i, j, n = 1;
	    expectation ee = (expectation) malloc(sizeof(Expectation)),
	      last = e;
	    if (printLevel >= 2)
	      bite_printf("expectation %s ...\n", $2);
	    ee->type = returnNodeType($4, &n);
	    if (ee->type > MODELF) {
	      bite_eprintf( "You cannot put both model functions and ");
	      bite_eprintf( "individual parameters to the same function.");
	      bite_eprintf( "(Line %ld.)\n", lineno);
	      bite_exit(1);
	    }
	    while (last && last->next) /* Find the last element */
	      last = last->next;
	    if (!last) /* If no expectations set before... */
	      e = ee;
	    else
	      last->next = ee; /* ... otherwise make 'ee' be the last one. */
	    ee->next = NULL;   /* Some settings. */
	    ee->printVar = FALSE;
	    ee->saveVal = FALSE;
	    ee->from[0] = '\0';
	    ee->to[0] = '\0';
	    sprintf(ee->name, "%s", $2);
	    switch (ee->type) {
	    case NONE:
	      bite_eprintf( "It is no use to calculate expectations of ");
	      bite_eprintf( "a constant. (Line %ld.)\n", lineno);
	      bite_exit(1);
	      break;
	    case SCALAR:
	      ee->pe = (PostExp *) malloc(n * sizeof(PostExp));
	      ee->n = n;
	      selectOneMF2(NULL, TRUE);
	      for (i = 0; i < ee->n; i++) {
		ee->pe[i].f = (ft *) malloc(sizeof(ee));
		ee->pe[i].f[0] = initFNode();
		copyFt($4, ee->pe[i].f[0]);
		findAndReplaceVarNames(0, 0, ee->pe[i].f[0], NONE, 
				       target, NULL);
		selectOneMF2(ee->pe[i].f[0], FALSE);
	      }
	      break;
	    case INDIVIDUAL:
	      ee->n = ind;
	      ee->pe = (PostExp *) malloc(ind * sizeof(PostExp));
	      for (i = 0; i < ee->n; i++) {
		ee->pe[i].f = (ft *) malloc(sizeof(ee));
		ee->pe[i].f[0] = $4;
	      }
	      findAndReplaceVarNames(0, 0, $4, NONE, target, NULL);
	      break;
	    case MODELF:
	      ee->pe = (PostExp *) malloc(n * sizeof(PostExp));
	      ee->n = n;
	      selectOneMF(0, NULL, TRUE);
	      for (i = 0; i < ee->n; i++) {
		ee->pe[i].f = (ft *) malloc(sizeof(ee));
		ee->pe[i].f[0] = initFNode();
		copyFt($4, ee->pe[i].f[0]);
		findAndReplaceVarNames(0, 0, ee->pe[i].f[0], NONE, 
				       target, NULL);
		selectOneMF(i, ee->pe[i].f[0], FALSE);
	      }
	      /* findAndReplaceVarNames(0, 0, ee->pe[0].f[0], NONE, 
				     target, NULL);
	      ee->pe[i].f[0] = initFNode();
		 copyFt($4, ee->pe[i].f[0]);
		 selectOneMF(ee->pe[i].f[0], FALSE);
	      ee->pe[i].f[0] = $4;
	      findAndReplaceVarNames(0, 0, ee->pe[i].f[0], NONE, 
				     target, NULL);
	      } */
	      break;
	    }
	    for (i = 0; i < ee->n; i++) {
	      ee->pe[i].n = 1;
	      ee->pe[i].mean = (double *) malloc(sizeof(missing));
	      ee->pe[i].variance = (double *) malloc(sizeof(missing));
	      ee->pe[i].weight = NULL;
	      ee->pe[i].val = NULL;
	      ee->pe[i].counter = 0;
	      for (j = 0; j < ee->pe[i].n; j++) {
		ee->pe[i].mean[j] = 0;
		ee->pe[i].variance[j] = 0;
	      }
	    }
	  }
        | EXPECTATION NAME ':' expression RANGE NAME '(' expression ','
	  expression ',' expression ')' ';' {
	    long i, j, n = 0;
	    prior p = param;
	    expectation ee = (expectation) malloc(sizeof(Expectation)),
	      last = e;
	    /* ft dum = $4; */


	    if (printLevel >= 2)
	      bite_printf("expectation %s ...\n", $2);
	    ee->type = returnNodeType($4, &n);
	    if (ee->type > MODELF) {
	      bite_eprintf( "You cannot put both model functions and ");
	      bite_eprintf( "individual parameters to the same function.");
	      bite_eprintf( "(Line %ld.)\n", lineno);
	      bite_exit(1);
	    }
	    while (last && last->next) /* Find the last element */
	      last = last->next;
	    if (!last) /* If no expectations set before... */
	      e = ee;
	    else
	      last->next = ee; /* ... otherwise make 'ee' be the last node. */
	    ee->from[0] = '\0';
	    ee->to[0] = '\0';
	    ee->printVar = FALSE;
	    ee->saveVal = FALSE;
	    ee->next = NULL;   /* Some settings. */
	    sprintf(ee->name, "%s", $2);
	    switch (ee->type) {
	    case NONE:
	      bite_eprintf( "It is no use to calculate expectations of ");
	      bite_eprintf( "a constant. (Line %ld.)\n", lineno);
	      bite_exit(1);
	      break;
	    case SCALAR:
	      ee->pe = (PostExp *) malloc(sizeof(PostExp));
	      ee->n = (n) ? n : 1;
	      selectOneMF2(NULL, TRUE);
	      for (i = 0; i < ee->n; i++) {
		ee->pe[i].f = (ft *) malloc(sizeof(ee));
		ee->pe[i].f[0] = initFNode();
		copyFt($4, ee->pe[i].f[0]);
		findAndReplaceVarNames(0, 0, ee->pe[i].f[0], NONE, 
				       target, NULL);
		selectOneMF2(ee->pe[i].f[0], FALSE);
	      }
	      break;
	    case INDIVIDUAL:
	      ee->n = ind;
	      ee->pe = (PostExp *) malloc(ind * sizeof(PostExp));
	      for (i = 0; i < ee->n; i++) {
		ee->pe[i].f = (ft *) malloc(sizeof(ee));
		ee->pe[i].f[0] = $4;
	      }
	      findAndReplaceVarNames(0, 0, $4, NONE, target, NULL);
	      break;
	    case MODELF:
	      ee->pe = (PostExp *) malloc(n * sizeof(PostExp));
	      ee->n = n;
	      selectOneMF(0, NULL, TRUE);
	      for (i = 0; i < ee->n; i++) {
		ee->pe[i].f = (ft *) malloc(sizeof(ee));
		/* ee->pe[i].f[0] = initFNode();
		copyFt($4, ee->pe[i].f[0]); */
		ee->pe[i].f[0] = initFNode();
		copyFt($4, ee->pe[i].f[0]);
		findAndReplaceVarNames(0, 0, ee->pe[i].f[0], NONE, 
				       target, NULL);
		selectOneMF(i, ee->pe[i].f[0], FALSE);
	      }
	      /*findAndReplaceVarNames(0, 0, ee->pe[0].f[0], NONE, 
				     target, NULL);*/
	      break;
	    }
	    while (p && strcmp(p->name, $6))
	      p = p->next;
	    if (!p) {
	      bite_eprintf( "%s: no such variable found.\n", $6);
	      bite_exit(1);
	    }
	    for (i = 0; i < ee->n; i++) {
	      ee->pe[i].n = calculate2(-1, $12);
	      ee->pe[i].loopParam = p;
	      ee->pe[i].lower = $8;
	      ee->pe[i].upper = $10;
	      ee->pe[i].mean = malloc(ee->pe[i].n * sizeof(missing));
	      ee->pe[i].variance = malloc(ee->pe[i].n * sizeof(missing));
	      ee->pe[i].weight = NULL;
	      ee->pe[i].val = NULL;
	      ee->pe[i].counter = 0;
	      for (j = 0; j < ee->pe[i].n; j++) {
		ee->pe[i].mean[j] = 0;
		ee->pe[i].variance[j] = 0;
	      }
	    }
	    findAndReplaceVarNames(0, 0, $8, NONE, target, NULL);
	    findAndReplaceVarNames(0, 0, $10, NONE, target, NULL);
	  }
	| PRINTVAR EXPECTATION NAME ';' {
	    expectation eptr = e;

	    if (printLevel >= 2)
	      bite_printf("printvar expectation %s;\n", $3);
	    while (eptr && strcmp($3, eptr->name) && eptr->next) 
	      eptr = eptr->next; /* Find the correct expectation */
	    if (eptr)
	      eptr->printVar = TRUE;
	    else {
	      bite_eprintf( "%s: no such expectation found.\n", $3);
	      bite_exit(1);
	    }
	  }
        | SAVE EXPECTATION NAME ';' {
	    long i, j, k;
	    expectation eptr = e;
	    FILE *fp;
	    char fileName[40];

	    if (printLevel >= 2)
	      bite_printf("store expectation %s;\n", $3);
	    while (eptr && strcmp($3, eptr->name) && eptr->next) 
	      eptr = eptr->next; /* Find the correct expectation */
	    if (eptr)
	      eptr->saveVal = TRUE;
	    else {
	      bite_eprintf( "%s: no such expectation found.\n", $3);
	      bite_exit(1);
	    }
	    /* Initialize the files: */
	    if (eptr->pe[0].n == 1) { /* A scalar expectation */
	      sprintf(fileName, "%s.sav", eptr->name);
	      fp = bite_fopen(fileName, "w");
//	        fprintf(fp, "");
	      fclose(fp);
	    }
	    else { /* A vector valued expectation. */
	      for (i = 0; i < eptr->n; i++) {
		sprintf(fileName, "%s.%ld.sav", eptr->name, i);
		fp = bite_fopen(fileName, "w");
//		fprintf(fp, "");
		fclose(fp);
	      }
	    }
	    for (j = 0; j < eptr->n; j++) {
	      eptr->pe[j].val = malloc(eptr->pe[j].n * sizeof(f));
	      for (k = 0; k < eptr->pe[j].n; k++)
		eptr->pe[j].val[k] = malloc(EXP_VAL_STOR * sizeof(missing));
	    }
	  }
	| WEIGHT EXPECTATION NAME ':' expression ';' {
	    long i;
	    expectation eptr = e;

	    if (printLevel >= 2)
	      bite_printf("weight expectation %s ...\n", $3);
	    while (eptr && strcmp($3, eptr->name) && eptr->next) 
	      eptr = eptr->next; /* Find the correct expectation */
	    
	    switch (eptr->type) {
	    case NONE:
	    case SCALAR:
	    case INDIVIDUAL:
	      for (i = 0; i < eptr->n; i++) {
		eptr->pe[i].weight = $5;
	      }
	      findAndReplaceVarNames(0, 0, $5, NONE, target, NULL);
	      break;
	    case MODELF:
	      selectOneMF(0, NULL, TRUE);
	      for (i = 0; i < eptr->n; i++) {
		/* eptr->pe[i].weight = initFNode();
		copyFt($5, eptr->pe[i].weight);
		selectOneMF(eptr->pe[i].weight, FALSE);
		findAndReplaceVarNames(0, 0, eptr->pe[i].weight, NONE, 
				       target, NULL); */
		eptr->pe[i].weight = $5;
		selectOneMF(i, eptr->pe[i].weight, FALSE);
	      }
	      findAndReplaceVarNames(0, 0, eptr->pe[0].weight, NONE, 
				     target, NULL);
	      break;
	    }
	  }
	| IMPORTANCE EXPECTATION NAME ':' NAME TO NAME ';' {
	    expectation eptr = e;

	    if (printLevel >= 2)
	      bite_printf("importance expectation %s ...\n", $3);
	    while (eptr && strcmp($3, eptr->name) && eptr->next) 
	      eptr = eptr->next; /* Find the correct expectation */

	    sprintf(eptr->from, "%s", $5);
	    sprintf(eptr->to, "%s", $7);
	  }
	| LOGSCALE ';' {
	    plots[cFrame]->logScale[cPicture] = TRUE;
	  }
	| DRAW FNAME {
	    if (printLevel >= 2)
	      bite_printf("draw %s ...\n", $2);
	    labela[aStack]->n = 0;
	  }
	  covvallist ')' ':' '(' INTEGER ',' INTEGER ',' expression ')' ';' {
	    FILE *meanp;
	    char fname[15];
	    long i, j = 3, l = 0, m, data = 0,
	      riskSet = 0, drawType = NONE;
	    double ci;

	    if (cPicture == 0)
	      cFrame++;
	    cPicture++;
	    if (!findUse) {
	      long i, j;
	      for (i = 0; i < f->n; i++)
		for (j = 0; j < f->nFunc[i]; j++)
		  findInd(ind, model, f->fList[i][j], f);
	      findUse = TRUE;
	    }
	    drawType = DRAW_MF;
	    while (strcmp(f->fList[l][0]->name, $2))
	      l++;
	    data = $8;
	    riskSet =$10;
	    ci = calculate2(-1, $12);
	    rmTree($12);
	    /* Find the correct model function: */
	    for (j = 0; j < f->nFunc[l]; j++) {
	      if (testCovEq(f->fList[l][j], labela[lStack])) {
		long length, cov;
		plots[cFrame]->f[cPicture] = f->fList[l][j];
		plots[cFrame]->mean[cPicture] = 
		  (double *) malloc(GP * sizeof(ci));
		for (i=0; i<GP; i++)
		  plots[cFrame]->mean[cPicture][i] = 0.0;
		plots[cFrame]->data[cPicture] = data;
		plots[cFrame]->riskSet[cPicture] = riskSet;
		riskSetSize(cFrame, cPicture);
		if (0.0 < ci && ci < 100.0) {
		  plots[cFrame]->bpr[cPicture] = 1.0 - ci / 100.0;
		  plots[cFrame]->median[cPicture] = 
		    initMedian((iter_limit - skip) / thinning);
		}
		else
		  plots[cFrame]->bpr[cPicture] = 1.0;
		sprintf(fname, "title%ld_%ld.tmp", cFrame, cPicture);
		if ((meanp = bite_fopen(fname, "w")) == NULL) {
		  bite_printf("Unable to open file %s\n", fname);
		  bite_exit(1);
		}
		fprintf(meanp, "%s(", f->fList[l][j]->name);
		length = strlen(f->fList[l][j]->name) + 1;
		if (f->fList[l][j]->cov[0][0]) {
		  m = 0;
		  while (f->fList[l][j]->cov[m][0]) {
		    if (m) {
		      fprintf(meanp, ", ");
		      length += 2;
		    }
		    if (length > 20) {
		      fprintf(meanp, "\\n");
		      length = 0;
		    }
		    cov = 0;
		    while (strcmp(c->cList[cov][0]->name, 
				  f->fList[l][j]->cov[m]))
		      cov++;
		    if (c->labels[cov]) {
		      long val = 0;
		      while (c->values[cov][val] != f->fList[l][j]->covVal[m])
			val++;
		      fprintf(meanp, "%s=%s", f->fList[l][j]->cov[m], 
			      c->labels[cov][val]);
		      length += strlen(f->fList[l][j]->cov[m]) +
			strlen(c->labels[cov][val]) + 1;
		    }
		    else {
		      fprintf(meanp, "%s=%g", f->fList[l][j]->cov[m], 
			      f->fList[l][j]->covVal[m]);
		      length += strlen(f->fList[l][j]->cov[m]) +
			(long) ((f->fList[l][j]->covVal[m] >= 0.0) ?
			       log(f->fList[l][j]->covVal[m] + 1) :
			       log(-f->fList[l][j]->covVal[m] + 1) / 
			       log(10.0)) + 1;
		    }
		    m++;
		  }
		}
		fprintf(meanp, ")\n");
		fclose(meanp);
		if (plots[cFrame]->nPic < cPicture)
		  plots[cFrame]->nPic = cPicture;
	      }
	    }
	    if (cPicture >= 11) {
	      bite_printf("Too many plots on a page (line %ld)\n", lineno);
	      bite_exit(1);
	    }
	    plots[cFrame]->drawType[cPicture] = drawType;
	  }
	| DRAW INTENSITY '(' NAME ',' INTEGER ')' ':' '(' INTEGER ','
	  INTEGER ',' expression ')' ';' {
	    FILE *meanp;
	    char fname[15];
	    long k = 0, l = 0, data = 0, drawType = NONE;
	    double ci;

	    if (cPicture == 0)
	      cFrame++;
	    cPicture++;
	    if (!findUse) {
	      long i, j;
	      for (i = 0; i < f->n; i++)
		for (j = 0; j < f->nFunc[i]; j++)
		  findInd(ind, model, f->fList[i][j], f);
	      findUse = TRUE;
	    }
	    if (printLevel >= 2)
	      bite_printf("draw intensity (%s, %ld) ...\n", $4, $6);
	    drawType = DRAW_INTENSITY;
	    while (strcmp(model->name[l], $4))
	      l++;
	    k = $6;
	    data = $10;
	    ci = calculate2(-1, $14);
	    rmTree($14);
	    plots[cFrame]->tree[cPicture] = model->ftList[l][k - 1];
	    plots[cFrame]->dest[cPicture] = model->ftList[l][ind];
	    plots[cFrame]->mean[cPicture] = (double *) malloc(GP * sizeof(ci));
	    plots[cFrame]->data[cPicture] = data;
	    plots[cFrame]->riskSet[cPicture] = FALSE;
	    if (data)
	      writeRecord(d->nCars[l], d->dList[l][k - 1], cFrame,
			  cPicture, 0.0);
	    if (ci > 0.0 && ci < 100.0) {
	      plots[cFrame]->bpr[cPicture] = 1.0 - ci / 100.0;
	      plots[cFrame]->median[cPicture] =
		initMedian((iter_limit - skip) / thinning);
	    }
	    else
	      plots[cFrame]->bpr[cPicture] = 1.0;
	    if (plots[cFrame]->nPic < cPicture)
	      plots[cFrame]->nPic = cPicture;
	    sprintf(fname, "title%ld_%ld.tmp", cFrame, cPicture);
	    if ((meanp = bite_fopen(fname, "w")) == NULL) {
	      bite_printf("Unable to open file %s\n", fname);
	      bite_exit(1);
	    }
	    fprintf(meanp, "Intensity %s, individual %ld\n", model->name[l], k);
	    fclose(meanp);
	    if (cPicture >= 11) {
	      bite_printf("Too many plots on a page (line %ld)\n", lineno);
	      bite_exit(1);
	    }
	    plots[cFrame]->drawType[cPicture] = drawType;
	  }
	| DRAW DENSITY PARAMETER NAME ';' {
	    char title[100] = "Posterior density estimate of ";
	    prior ptr = param;

	    if (cPicture == 0)
	      cFrame++;
	    cPicture++;
	    if (printLevel >= 2)
	      bite_printf("draw density parameter %s : ...\n", $4);
	    while (strcmp(ptr->name, $4))
	      ptr = ptr->next;
	    plots[cFrame]->pv[cPicture] = ptr;
	    plots[cFrame]->drawType[cPicture] = DRAW_PARAM;
	    strcat(title, ptr->name);
	    if (plots[cFrame]->nPic < cPicture)
	      plots[cFrame]->nPic = cPicture;
	    writeTitleFile(cFrame, cPicture, title);
/*	    if (cmds[i]->values[1][0] == 'd') */
	      plots[cFrame]->data[cPicture] = -1;
/*	    else
	      plots[cFrame]->data[cPicture] = -2; */
	    if (cPicture >= 11) {
	      bite_printf("Too many plots on a page (line %ld)\n", lineno);
	      bite_exit(1);
	    }
	    /*	    plots[cFrame]->pv[cPicture]->sample = 
	      (double *) malloc((iter_limit - skip) * sizeof(dummy)); */
	  }
	| DRAW CDF '(' NAME ',' INTEGER ')' ';' {
	    FILE *meanp;
	    char fname[15];
	    long k = 0, l = 0, drawType = NONE;
	    double ci;

	    if (cPicture == 0)
	      cFrame++;
	    cPicture++;
	    if (!findUse) {
	      long i, j;
	      for (i = 0; i < f->n; i++)
		for (j = 0; j < f->nFunc[i]; j++)
		  findInd(ind, model, f->fList[i][j], f);
	      findUse = TRUE;
	    }
	    if (printLevel >= 2)
	      bite_printf("draw cdf (%s, %ld) ...\n", $4, $6);
	    drawType = DRAW_CDF;
	    while (strcmp(model->name[l], $4))
	      l++;
	    k = $6;
	    plots[cFrame]->tree[cPicture] = model->ftList[l][k - 1];
	    plots[cFrame]->dest[cPicture] = model->ftList[l][ind];
	    plots[cFrame]->mean[cPicture] = (double *) malloc(GP * sizeof(ci));
	    plots[cFrame]->data[cPicture] = NONE;
	    plots[cFrame]->riskSet[cPicture] = NONE;
	    if (plots[cFrame]->nPic < cPicture)
	      plots[cFrame]->nPic = cPicture;
	    fname[0] = '\0';
	    sprintf(fname, "title%ld_%ld.tmp", cFrame, cPicture);
	    if ((meanp = bite_fopen(fname, "w")) == NULL) {
	      bite_printf("Unable to open file %s\n", fname);
	      bite_exit(1);
	    }
	    fprintf(meanp, "Estimated cdf of %s, individual %ld\n",
		    model->name[l], k);
	    fclose(meanp);
	    if (cPicture >= 11) {
	      bite_printf("Too many plots on a page (line %ld)\n", lineno);
	      bite_exit(1);
	    }
	    plots[cFrame]->drawType[cPicture] = drawType;
	  }
        | NEXTPAGE ';' {
	    cPicture = 0;
	    if (cFrame >= frames) {
	      bite_printf("Too many pages (line %ld)\n", lineno);
	      bite_exit(1);
	    }
	  }
	| RUN ';' {
	    frames = cFrame;
	    return(0);
	  }
;

expression:
	  expression OR expression { $$ = node(OR, $1, $3); }
        | expression AND expression { $$ = node(AND, $1, $3); }
        | expression '+' expression { $$ = node('+', $1, $3); }
        | expression '-' expression { $$ = node('-', $1, $3); }
        | expression '*' expression { $$ = node('*', $1, $3); }
        | expression '/' expression { $$ = node('/', $1, $3); }
        | expression MULT2 expression { $$ = node('_', $1, $3); }  /* RS Changes pykala to _ */
        | expression POW expression { $$ = node('^', $1, $3); }
        | expression '?' expression ':' expression
            { $$ = arithmeticIf($1, $3, $5); }
        | expression LT expression { $$ = node(LT, $1, $3); }
        | expression GT expression { $$ = node(GT, $1, $3); }
        | expression GE expression { $$ = node(GE, $1, $3); }
        | expression LE expression { $$ = node(LE, $1, $3); }
        | expression EQ expression { $$ = node(EQ, $1, $3); }
        | expression NE expression { $$ = node(NE, $1, $3); }
        | '-' expression %prec UMINUS { 
	    $$ = node('-', $2, NULL); 
	  }
        | '(' expression ')' { $$ = $2; }
	| expression index {
	    if ($1->third) {
	      indexLeaf($1->third, $2);
	      $$ = $1;
	    }
	    else
	      $$ = indexLeaf($1, $2);
	  }
	| NULL_EXP { $$ = NULL; }
	| funct
	| expression FIELD T0 { $$ = leafF($1, T0); }
	| expression FIELD T1 { $$ = leafF($1, T1); }
	| expression FIELD POINTS { $$ = leafF($1, POINTS); }
	| expression FIELD JUMPPOINT { $$ = leafF($1, JUMPPOINT); }
	| expression FIELD N_POINTS { $$ = leafF($1, N_POINTS); }
	| expression FIELD LEVEL { $$ = leafF($1, LEVEL); }
	| expression FIELD INTENSITY { $$ = leafF($1, INTENSITY); }
        | INTEGER { $$ = leafR((double) $1); }
        | REAL  { $$ = leafR($1); }
        | NAME { $$ = leafN($1); }
        | STRING {
	    $$ = leafN($1);
	    $$->ptype = PT_STRING;
	    $$->var->str = $$->varName;
	  }
        | segment {
	  /* ft root = $1; */
	    $$ = $1;
	  }
;

segment:
	  '{'
	  localvariables statementlist '}' {
	    ft root, left;
	    char dum;
	    int i;
	    root = initFNode();
	    $$ = root;
	    $$->ptype = PT_SEMICOL;
	    $$->nArgs = labela[lStack]->n;
	    if (labela[lStack]->n) {
	      $$->args = malloc(labela[lStack]->n * sizeof($$));
	      for (i = 0; i < labela[lStack]->n; i++) {
		$$->args[i] = initFNode();
		$$->args[i]->varName =
		  malloc((strlen(labela[lStack]->label[i]->label) + 1) *
			 sizeof(dum));
		sprintf($$->args[i]->varName, "%s",
			labela[lStack]->label[i]->label);
	      }
	    }
	    left = $$->left = $3;
	  }
;

localvariables:
	  LOCAL {
	    labela[lStack]->n = 0;
	  }
          covlist ';' {
	  }
;

statementlist:
	  statementlist statement {
	    $$ = initFNode();
	    $$->left = $1;
	    $$->right = $2;
	    $$->ptype = PT_SEMICOL;
	  }
        | statement {
	    $$ = $1;
	  }
;

statement:
	  '{' statementlist '}' {
	    $$ = $2;
	  }
	| MYECHO {
	    aStack++;
	    a[aStack]->n = 0;
	  }
	  expressionlist ';' {
	    int i;
            $$ = initFNode();
	    $$->nArgs = a[aStack]->n;
	    $$->args = malloc(a[aStack]->n * sizeof(param));
	    for (i = 0; i < a[aStack]->n; i++) {
	      $$->args[i] = a[aStack]->ptrArray[i];
	      if (a[aStack]->ptrArray[i]->ptype != PT_STRING)
		findAndReplaceVarNames(-1, -1, a[aStack]->ptrArray[i],
				       NONE, target, NULL);
	    }
	    $$->ptype = PT_ECHO;
	    $$->left = initFNode(); /* This is not good. */
	    aStack--;
	  }
	| expression '=' expression ';' {
            $$ = initFNode();
	    $$->left = $1;
	    $$->right = $3;
	    $$->ptype = PT_SET;
	  }
        | IF '(' expression ')' statement ELSE statement {
	    $$ = initFNode();
	    $$->left = $3;
	    $$->right = $5;
	    $$->third = $7;
	    $$->ptype = PT_IF;
	  }
        | IF '(' expression ')' statement {
	    $$ = initFNode();
	    $$->left = $3;
	    $$->right = $5;
	    $$->third = NULL;
	    $$->ptype = PT_IF;
	  }
	| WHILE '(' expression ')' statement {
            $$ = initFNode();
	    $$->left = $3;
	    $$->right = $5;
	    $$->ptype = PT_WHILE;
	  }
        | SWITCH {
	    aStack++;
	    a[aStack]->n = 0;
	  }
	  '(' expressionlist ')' ';' {
	    long i;
	    if (a[aStack]->n % 2 != 0 || (a[aStack]->n - 1) / 2 < 0) {
	      bite_eprintf( "Incorrect number of arguments in \'switch\'. "
		      "(Line %ld.)\n", lineno);
	      bite_exit(1);
	    }
	    $$ = initFNode();
	    target.x = NULL;
	    for (i = 0; i < a[aStack]->n; i++)
	      findAndReplaceVarNames(-1, -1, a[aStack]->ptrArray[i],
				     NONE, target, target.x);
	    /* The variable for choosing between actions */
	    $$->left = a[aStack]->ptrArray[0];
	    $$->nArgs = a[aStack]->n - 1;
	    $$->args = malloc((a[aStack]->n - 1) * sizeof(c));
	    for (i = 1; i < a[aStack]->n; i++)
	      $$->args[i-1] = a[aStack]->ptrArray[i];
	    $$->ptype = PT_SWITCH;
	    aStack--;
	  }
        | FOR '(' statement ',' expression ',' statement ')' statement {
            $$ = initFNode();
	    $$->nArgs = 3;
	    $$->args = malloc(3 * sizeof($$));
	    $$->args[0] = $3;
	    $$->args[1] = $5;
	    $$->args[2] = $7;
	    $$->left = $9;
	    $$->ptype = PT_FOR;
	  }
        | CUT {
	    aStack++;
	    a[aStack]->n = 0;
	  }
	  expressionlist ';' {
	    int i;
	    if (a[aStack]->n < 3) {
	      bite_eprintf( "Too few arguments. (Line %ld.)\n", lineno);
	      bite_exit(1);
	    }
            $$ = initFNode();
	    $$->nArgs = a[aStack]->n;
	    $$->args = malloc(a[aStack]->n * sizeof(param));
	    for (i = 0; i < a[aStack]->n; i++) {
	      $$->args[i] = a[aStack]->ptrArray[i];
	      if (a[aStack]->ptrArray[i]->ptype != PT_STRING)
		findAndReplaceVarNames(-1, -1, a[aStack]->ptrArray[i],
				       NONE, target, NULL);
	    }
	    if ($$->args[0]->ptype == PT_SCALAR || $$->args[0]->ptype == PT_VECTOR) {
	      bite_eprintf(
		      "First argument should be a parameter. (Line %ld.)\n", lineno);
	      bite_exit(1);
	    }
	    $$->ptype = PT_CUT;
	    $$->left = initFNode(); /* This is not good. */
	    aStack--;
	  }
        | RETURN expression ';' {
            $$ = initFNode();
            $$->left = $2;
	    $$->ptype = PT_RETURN;
	  }
;

funct:
	  FNAME {
	    aStack++;
	    lStack++;
	    sprintf(name[aStack], "%s", $1);
	    if (aStack == MAX_STACK) {
	      bite_eprintf( "Stack overflow. (Line %ld.)\n", lineno);
	      bite_exit(1);
	    }
	    a[aStack]->n = 0;
	    labela[lStack]->n = 0;
	  }
	  funct2 ')' {
	    $$ = $3;
	    aStack--;
	    lStack--;
	  }
;

funct2:
	  expression '|' covlist {
	    long type = checkType(name[aStack]), j;
	    switch (type) {
	    case F_SUM:
	    case F_PROD:
	      break;
	    default:
	      bite_eprintf( "Only \'sum\' or \'prod\' can have this ");
	      bite_eprintf( "kind of arguments. (Line %ld.)\n", lineno);
	      bite_exit(1);
	      break;
	    }
	    $$ = leafG(type);
	    /* $$->f = findMf2(labela[lStack], f); */
	    if (!$$->f)
	      $$->f = makeModel("", 0, -1.0, labela[lStack], c, ind, 
				&$$->storageF, FALSE);
	    else {
	      long i=0;
	      while ($$->f != f->fList[i])
		i++;
	      $$->storageF = f->nFunc[i];
	    }
	    $$->fc = malloc($$->storageF * sizeof(j));
	    for (j = 0; j < $$->storageF; j++)
	      $$->fc[j] = j;
	    /* $$->choice = initSf(0.0, 0.0, NULL);
	    for (j = 1; j < $$->storageF; j++)
	      addNode($$->choice, j, j, j); */
	    $$->args[0] = $1;
	  }
	| expressionlist {
	    long type = checkType(name[aStack]), error = NONE;
	    switch (type) {
	    case F_EXP:
	      error = (a[aStack]->n != 1) ? 1 : 0;
	      break;
	    case F_LOG:
	      error = (a[aStack]->n != 1) ? 1 : 0;
	      break;
	    case F_MIN:
	      error = (a[aStack]->n < 1) ? 1 : 0;
	      break;
	    case F_MAX:
	      error = (a[aStack]->n < 1) ? 1 : 0;
	      break;
	    case F_VALUE:
	      error = (a[aStack]->n != 2) ? 1 : 0;
	      break;
	    case F_INT:
	      error = (a[aStack]->n != 3) ? 1 : 0;
	      break;
	    case F_LOGLIKE:
	      break;
	    case F_INVINT:
	      error = (a[aStack]->n != 2) ? 1 : 0;
	      break;
	    case D_DUNIF:
	    case D_BIN:
	    case D_NORM:
	    case D_GAMMA:
	    case D_UNIF:
	    case D_POISSON:
	    case D_DIRAC:
	    case D_MNORM:
	      /* $$ = leafR(0.0);
	      $$->ptype = PT_SCALAR;
	      $$->var->p = initPrior("copy this");
	      checkPrior(name[aStack], $$->var->p); */
	      error = 0;
	      break;
	    default:
	      $$ = leafMF(name[aStack]);
	      break;
	    }
	    if (type && !error)
	      /* if (type && type >= F_LOG && !error) */
	      $$ = leafFunct(type);
	    if (error) {
	      bite_eprintf( "Wrong number of arguments. (Line %ld.)\n",
		      lineno);
	      bite_exit(1);
	    }
	  } 
        | covvallist {
	    long modelf = 0, n = 0;
	    ft ptr;
	    while (modelf < f->n && strcmp(name[aStack], 
					   f->fList[modelf][0]->name))
	      modelf++;
	    if (modelf < f->n) {
	      ptr = leafMF(name[aStack]);
	      ptr->ptype = PT_MODELF;
	      ptr->storageF = 0;
	      ptr->f = f->fList[modelf];
	      for (n = 0; n < f->nFunc[modelf]; n++) {
		if (cmpCovs(f->fList[modelf][n], labela[lStack], 0)) {
		  /*if (!ptr->choice)
		    ptr->choice = initSf(0.0, (double) n, NULL);
		  else
		    addNode(ptr->choice, ptr->choice->n, 
			    (double) ptr->choice->n, n); */
		  if (!ptr->fc)
		    ptr->fc = malloc(f->nFunc[modelf] * sizeof(missing));
		  ptr->fc[ptr->storageF] = n;
		  ptr->var->m = f->fList[modelf][n];
		  ptr->storageF++;
		}
	      }
	      /* if (ptr->choice->n == 1) {
		ptr->var->m = ptr->f[(long) ptr->choice->levels[0]];
		delSf(ptr->choice);
		ptr->choice = NULL;
		ptr->storageF = 0;
		ptr->f = NULL;
	      } */
	    }
	    else {
	      bite_eprintf(
		      "%s, no such model function found! (Line %ld.)\n", 
		      name[aStack], lineno);
	      bite_exit(1);
	    }
	    $$ = ptr;
	  }
;

expressionlist:
          expressionlist ',' expression	{
	    if ($3) {
	      if (a[aStack]->n == a[aStack]->stor) {
		(a[aStack]->stor)++;
		a[aStack]->ptrArray =
		  realloc(a[aStack]->ptrArray,
			  a[aStack]->stor * sizeof(a));
	      }
	      a[aStack]->ptrArray[a[aStack]->n] = $3;
	      (a[aStack]->n)++;
	    }
	    else {
	      bite_eprintf( "No null allowed in expressionlist with more "
		      "than 1 element. (Line %ld)\n", lineno);
	      bite_exit(1);
	    }
	  }
        | expression {
	    if ($1) {
	      a[aStack]->ptrArray[a[aStack]->n] = $1;
	      (a[aStack]->n)++;
	    }
	  }
;

index:
	  '[' expression ']' {
	    $$ = $2;
	  }
;

covlist:
          covlist ',' NAME {
	    labela[lStack]->label[labela[lStack]->n]->label[0] = '\0';
	    strcat(labela[lStack]->label[labela[lStack]->n]->label, $3);
	    incrLabelArray(labela[lStack]);
	  }
        | NAME {
	    labela[lStack]->label[labela[lStack]->n]->label[0] = '\0';
	    strcat(labela[lStack]->label[labela[lStack]->n]->label, $1);
	    incrLabelArray(labela[lStack]);
	  }
	|
;

covvallist:
          covvallist ',' NAME '=' expression {
	    ft dum;
	    dum = $5;
	    labela[lStack]->label[labela[lStack]->n]->value =
	      calculate2(-1, dum);
	    labela[lStack]->label[labela[lStack]->n]->label[0] = '\0';
	    strcat(labela[lStack]->label[labela[lStack]->n]->label, $3);
	    rmTree(dum);
	    incrLabelArray(labela[lStack]);
	  }
        | NAME '=' expression {
	    ft dum;
	    dum = $3;
	    labela[lStack]->label[labela[lStack]->n]->value =
	      calculate2(-1, dum);
	    labela[lStack]->label[labela[lStack]->n]->label[0] = '\0';
	    strcat(labela[lStack]->label[labela[lStack]->n]->label, $1);
	    rmTree(dum);
	    incrLabelArray(labela[lStack]);
	  }
	|
;

labellist:
          labellist ',' expression '=' STRING {
	    ft dum;
	    dum = $3;
	    labela[lStack]->label[labela[lStack]->n]->value =
	      calculate2(-1, dum);
	    labela[lStack]->label[labela[lStack]->n]->label[0] = '\0';
	    strcat(labela[lStack]->label[labela[lStack]->n]->label, $5);
	    rmTree(dum);
	    incrLabelArray(labela[lStack]);
	  }
        | expression '=' STRING	{
	    ft dum;
	    dum = $1;
	    labela[lStack]->label[labela[lStack]->n]->value =
	      calculate2(-1, dum);
	    labela[lStack]->label[labela[lStack]->n]->label[0] = '\0';
	    strcat(labela[lStack]->label[labela[lStack]->n]->label, $3);
	    rmTree(dum);
	    incrLabelArray(labela[lStack]);
	  }
;

%%

/* #include "epils12.c" */ 

void yyerror(const char *msg)
{
  bite_eprintf( "%s near line %ld.\n", msg, lineno);
  bite_exit(1);
}

long yywrap()
{
  return 0;
}

static void incrLabelArray(labelArray l)
{
  long ii;

  l->n++;
  if (l->n == l->stor) {
    l->label = realloc(l->label, 2 * l->stor * sizeof(jackLabel));
    for (ii = l->stor; ii < l->stor * 2; ii++)
      l->label[ii] = malloc(sizeof(jackLabel));
    l->stor = l->stor * 2;
  }
}
static ft leafR(double x)
{
  ft ptr = initFNode();

  ptr->ptype = PT_CONST;
  ptr->var->x = x;
  return ptr;
}

static ft leafN(char *name)
{
  ft ptr = initFNode();
  char dum;

/*  p = findPrior(name, param); */
  ptr->varName = (char *) malloc((strlen(name) + 1) * sizeof(dum));
  ptr->varName[0] = '\0';
  strcat(ptr->varName, name);
  return ptr;
}

static ft leafFunct(long fType) /* The leaf is a function. */
{
  long i;
  ft ptr = initFNode();

  ptr->function = fType;
  ptr->nArgs = a[aStack]->n;
  if (a[aStack]->n && !ptr->args)
    ptr->args = malloc(a[aStack]->n * sizeof(a));
  for (i = 0; i < a[aStack]->n; i++)
    ptr->args[i] = a[aStack]->ptrArray[i];
  return ptr;
}

static ft leafF(ft ptr, long fieldId)
{
  ptr->fieldId = fieldId;
  ptr->third = initFNode();
  return ptr;
}

static ft leafMF(char *name)
{
  long i;
  ft ptr = initFNode();
  char dum;
  prior p;

  ptr->varName = (char *) malloc((MAX_VARNAME + 2) * sizeof(dum));
  ptr->varName[0] = '\0';
  strcat(ptr->varName, name);
  ptr->nArgs = a[aStack]->n;
  if (a[aStack]->n) {
    ptr->args = (ft *) malloc(a[aStack]->n * sizeof(p));
    for (i = 0; i < a[aStack]->n; i++)
      ptr->args[i] = a[aStack]->ptrArray[i];
    /*  ptr->args[0] = initFNode();
	copyFt(root, ptr->args[0]); */
  }

  return ptr;
}

static ft leafG(long type)
{
  ft ptr = initFNode();
  char dum;
  prior p;

  ptr->varName = (char *) malloc(2 * sizeof(dum));
  ptr->varName[0] = '\0';
  ptr->function = type;
  ptr->nArgs = 1;
  ptr->args = (ft *) malloc(sizeof(p));

  return ptr;
}

static ft indexLeaf(ft ptr, ft root)
{
/*  ptr->varName = malloc((MAX_VARNAME + 2) * sizeof(dum));
  ptr->varName[0] = '\0';
  strcat(ptr->varName, name);
  ptr->ni = 0; */
  if (!ptr->i)
    ptr->i = (ft *) malloc(MAX_ARGS * sizeof(ptr));
  ptr->i[ptr->ni] = root;
  ptr->ni++;

  return ptr;
}

static ft node(long op, ft left, ft right)
{
  ft ptr = initFNode();

  ptr->op = op;
  ptr->left = left;
  ptr->right = right;
  return ptr;
}

static ft arithmeticIf(ft cond, ft left, ft right)
{
  ft ptr = initFNode();

  ptr->op = (long) '?';
  ptr->third = cond;
  ptr->left = left;
  ptr->right = right;
  return ptr;
}

/* static ft triple(long op, ft left, ft right, ft third)
{
  ft ptr = initFNode();

  ptr->op = op;
  ptr->left = left;
  ptr->right = right;
  ptr->third = third;
  return ptr;
} */

void checkPrior(char *name, prior res)
{
  long error = TRUE;

  if (!strcmp(name, "dirac") ||
      !strcmp(name, "poisson")) {
    error = FALSE;
    if (!strcmp(name, "dirac"))
      res->type = D_DIRAC;
    if (!strcmp(name, "poisson"))
      res->type = D_POISSON;
    res->args = malloc(sizeof(res));
    res->nArgs = 1;
    res->args[0] = a[aStack]->ptrArray[0];
  }
  if (!strcmp(name, "dunif") ||
      (!strcmp(name, "normal") && a[aStack]->n == 2) ||
      !strcmp(name, "gamma") || !strcmp(name, "binomial") ||
      !strcmp(name, "unif") || !strcmp(name, "laplace")) {
    error = FALSE;
    if (!strcmp(name, "dunif"))
      res->type = D_DUNIF;
    if (!strcmp(name, "binomial"))
      res->type = D_BIN;
    if (!strcmp(name, "normal"))
      res->type = D_NORM;
    if (!strcmp(name, "gamma"))
      res->type = D_GAMMA;
    if (!strcmp(name, "unif"))
      res->type = D_UNIF;
    if (!strcmp(name, "laplace"))
      res->type = D_LAPLACE;
    res->args = malloc(2 * sizeof(res));
    res->nArgs = 2;
    res->args[0] = a[aStack]->ptrArray[0];
    res->args[1] = a[aStack]->ptrArray[1];
  }
  if (!strcmp(name, "normal") && a[aStack]->n > 2) {
    long i;
    char dum[8];
    prior p1, p2;
    error = FALSE;
    res->type = D_MNORM;
    res->ptype = PT_VECTOR;
    res->args = malloc(a[aStack]->n * sizeof(res));
    res->nArgs = a[aStack]->n;
    for (i = 0; i < a[aStack]->n; i++)
      res->args[i] = a[aStack]->ptrArray[i];
    i = 0;
    sprintf(dum, "%ld", i);
    res->vector = malloc(res->nArgs * sizeof(res));
    res->vector[0] = res->x->vec = p1 = initPrior(dum);
    res->vector[0]->up = res;
    p1->ptype = PT_CONST;
    do {
      i++;
      sprintf(dum, "%ld", i);
      res->vector[i] = p2 = initPrior(dum);
      p2->up = res;
      p2->ptype = PT_CONST;
      p1->next = p2;
      p2->prev = p1;
      p1 = p2;
    } while (i * i + i < res->nArgs);
    res->stor = res->nArgs;
  }
  if (!strcmp(name, "mix")) {
    long i;
    error = FALSE;
    res->type = D_MIX;
    res->ptype = PT_MIX;
    res->args = malloc(a[aStack]->n * sizeof(res));
    res->nArgs = a[aStack]->n;
    for (i = 0; i < a[aStack]->n; i++)
      res->args[i] = a[aStack]->ptrArray[i];
  }
  if (!strcmp(name, "student")) {
    long i;
    if (a[aStack]->n != 3) {
      bite_eprintf( "Incorrect number of arguments near line %ld.\n",
	      lineno);
      bite_exit(1);
    }
    error = FALSE;
    res->type = D_STUDENT;
    res->args = malloc(a[aStack]->n * sizeof(res));
    res->nArgs = a[aStack]->n;
    for (i = 0; i < a[aStack]->n; i++)
      res->args[i] = a[aStack]->ptrArray[i];
  }
  if (!strcmp(name, "dirichlet")) {
    long i;
    char dum[8];
    prior p1, p2;
    error = FALSE;
    res->type = D_DIRICHLET;
    res->ptype = PT_VECTOR;
    res->args = malloc(a[aStack]->n * sizeof(res));
    res->nArgs = a[aStack]->n;
    for (i = 0; i < a[aStack]->n; i++)
      res->args[i] = a[aStack]->ptrArray[i];
    i = 0;
    sprintf(dum, "%ld", i);
    res->stor = res->nArgs;
    res->vector = malloc(res->nArgs * sizeof(res));
    res->vector[0] = res->x->vec = p1 = initPrior(dum);
    res->vector[0]->up = res;
    p1->ptype = PT_CONST;
    do {
      i++;
      sprintf(dum, "%ld", i);
      res->vector[i] = p2 = initPrior(dum);
      p2->up = res;
      p2->ptype = PT_CONST;
      p1->next = p2;
      p2->prev = p1;
      p1 = p2;
    } while (i < res->nArgs - 1);
  }
  if (!strcmp(name, "discrete")) {
    long i;
    error = FALSE;
    res->type = D_DISCR;
    res->args = malloc(a[aStack]->n * sizeof(res));
    res->nArgs = a[aStack]->n;
    for (i = 0; i < a[aStack]->n; i++)
      res->args[i] = a[aStack]->ptrArray[i];
    if (a[aStack]->n % 2) {
      bite_eprintf( "Incorrect number of arguments near line %ld.\n",
	      lineno);
      bite_exit(1);
    }
  }
  if (error) {
    bite_eprintf( "%s: unknown distribution or incorrect "
	    "number of arguments (%ld).\n", name, res->nArgs);
    bite_exit(1);
  }
}

long checkType(char *name)
{
  if (!strcmp(name, "dirac"))
    return(D_DIRAC);
  if (!strcmp(name, "unif"))
    return(D_UNIF);
  if (!strcmp(name, "norm"))
    return(D_NORM);
  if (!strcmp(name, "gamma"))
    return(D_GAMMA);
  if (!strcmp(name, "student"))
    return(D_STUDENT);
  if (!strcmp(name, "dunif"))
    return(D_DUNIF);
  if (!strcmp(name, "binomial"))
    return(D_BIN);
  if (!strcmp(name, "laplace"))
    return(D_LAPLACE);
  if (!strcmp(name, "poisson"))
    return(D_POISSON);
  if (!strcmp(name, "d_unif"))
    return(DD_UNIF);
  if (!strcmp(name, "d_norm"))
    return(DD_NORM);
  if (!strcmp(name, "d_gamma"))
    return(DD_GAMMA);
  if (!strcmp(name, "student"))
    return(DD_STUDENT);
  if (!strcmp(name, "d_binomial"))
    return(DD_BIN);
  if (!strcmp(name, "mix"))
    return(D_MIX);
  if (!strcmp(name, "exp"))
    return(F_EXP);
  if (!strcmp(name, "log"))
    return(F_LOG);
  if (!strcmp(name, "min"))
    return(F_MIN);
  if (!strcmp(name, "max"))
    return(F_MAX);
  if (!strcmp(name, "gammaf"))
    return(F_GAMMA);
  if (!strcmp(name, "value"))
    return(F_VALUE);
  if (!strcmp(name, "integral"))
    return(F_INT);
  if (!strcmp(name, "sum"))
    return(F_SUM);
  if (!strcmp(name, "prod"))
    return(F_PROD);
  if (!strcmp(name, "loglikelihood"))
    return(F_LOGLIKE);
  if (!strcmp(name, "logcontrib"))
    return(F_LOGCONTR);
  if (!strcmp(name, "logprior"))
    return(F_LOGPRIOR);
  if (!strcmp(name, "invintegral"))
    return(F_INVINT);
  if (!strcmp(name, "indiv"))
    return(F_INDIV);
  if (!strcmp(name, "index"))
    return(F_INDEX);
  if (!strcmp(name, "old"))
    return(F_OLD);
  if (!strcmp(name, "pppoint"))
    return(F_PPPOINT);
  if (!strcmp(name, "int"))
    return(F_FLOOR);
  if (!strcmp(name, "addpoint"))
    return(F_ADDP);
  if (!strcmp(name, "delpoint"))
    return(F_DELP);
  if (!strcmp(name, "addinterval"))
    return(F_ADDI);
  if (!strcmp(name, "delinterval"))
    return(F_DELI);
  if (!strcmp(name, "cut"))
    return(F_CUT);
  if (!strcmp(name, "probit"))
    return(F_PROBIT);
  if (!strcmp(name, "this"))
    return(F_THIS);
  return(0);
}

/* void run()
{
  simulation();
} */
