/* Tommi Harkanen (C) 2000 */
/* Read the file copyright.txt for details */

#ifndef FTREE
#define FTREE
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include "typedef.h"
#include "sf3.h"
#include "general3.h"
#include "bit.h"
#include "ftree.h"
#include "load.h"
#include "prior.h"
#include "likel.h"
#include "modelf.h"
#include "bite.tab.h"

/* To avoid compiler warning "warning: implicit declaration of function 'lgamma'": */
double (lgamma)(double);

/* This module contains functions to build a formula tree given the model & 
   to calculate the intensity given the stepwise constant functions. */

extern long nVars, printLevel, lineno, ind, iter, lindex[MAX_STACK_SIZE],
  level, index0, lStack;
extern variable *vars;
extern double *old;
extern mft f;
extern prior param;
extern dt d;
extern ct c;
extern ff model;
extern labelArray labela[MAX_STACK], miss[MAX_STACK];

static long indiv_func = FALSE;

/* long printLevel = 1;

void main()
{
  long n;
  ft root;
  char string[80], *formula[MAX_COMMANDS];

  scanf("%s", string);
  for (n = 0; n < MAX_COMMANDS; n++)
    formula[n] = NULL;
  split2(string, formula, &n);
  root = makeTree(0, n, formula);
  bite_printf("%ld\n", depth(root));

  inorder(root);
  bite_printf(" = %f\n", calculate2(ind, root));
} */

ff initFF(long marks)
{
  long i;
  ff res;

  res = (ff) malloc(sizeof(jackfF));
  res->n = 0;
  res->storage = marks;
  res->name = (char **) malloc(marks * sizeof(res));
  res->ftList = (ft **) malloc(marks * sizeof(res));
 for (i = 0; i < marks; i++) {
    res->name[i] = NULL;
    res->ftList[i] = NULL;
  }

  return(res);
}

void split2(char line[80], char *words[MAX_COMMANDS], long *i)
{
  long j = 0, k;
  char dum;

  *i = 0;
  while (line[j] != '\0' && line[j] != '#' && line[j] != '\n') {
    words[*i] = (char *) malloc( 40 * sizeof( dum ) );
    while (line[j] == ' ' || line[j] == '\t')
      j++;
    k = 0;
    while (aritmeticOp(line[j]) || specialChar(line[j])) {
      words[*i][0] = line[j++];
      (*i)++;
      words[*i] = (char *) malloc( 40 * sizeof( dum ) );
    }
    while (line[j] == ' ' || line[j] == '\t')
      j++;
    while (line[j] != '\0' && line[j] != '#' && line[j] != ' ' && 
	   line[j] != '\n' && line[j] != '\t' && !aritmeticOp(line[j]) &&
	   !specialChar(line[j])) {
      words[*i][k++] = line[j++];
    }
    if (k > 0) {
      words[*i][k] = '\0';
      (*i)++;
    }
  }
  words[*i] = (char *) malloc( 40 * sizeof( dum ) );
  words[*i][0] = '\0';
}

void copyFt(ft from, ft to)
{
  long i;

#ifdef DEBUG
  to->lineno = from->lineno;
#endif
  to->lineno = from->lineno;
  to->op = from->op;
  to->function = from->function; /* These 5 fields are for scalar formula */
  if ((to->nArgs = from->nArgs) != 0) {       /* trees */
    to->args = (ft *) malloc(from->nArgs * sizeof(to));
    for (i = 0; i < from->nArgs; i++) {
      to->args[i] = initFNode();
      copyFt(from->args[i], to->args[i]);
    }
  }
  if ((to->ni = from->ni) != 0) { /* These for indices */
    to->i = (ft *) malloc(from->ni * sizeof(to));
    for (i = 0; i < from->ni; i++) {
      to->i[i] = initFNode();
      copyFt(from->i[i], to->i[i]);
    }
  }
  to->ptype = from->ptype; /* What kind of node? */
  to->fieldId = from->fieldId; /* Field, e.g. JUMPPOINT. */
  to->varName = from->varName;
  to->var = (pType *) malloc(sizeof(pType));
  switch (from->ptype) {
  case PT_SCALAR:
    to->var->p = from->var->p;
    break;
  case PT_VECTOR:
    to->var->vec = from->var->vec;
    break;
  case PT_PPROCESS:
    to->var->pp = from->var->pp;
    break;
  case PT_SF:
    to->var->f = from->var->f;
    break;
  case PT_RESP:
    to->var->resp = from->var->resp;
    break;
  case PT_CONST:
    to->var->x = from->var->x;
    break;
  case PT_MODELF:
    to->var->m = from->var->m;
    break;
  }
/*  to->var = from->var;
  to->x = from->x; */

  to->storageF = from->storageF; /* and the rest for simple function trees. */
  if (from->choice) {
    if (!to->choice) {
      to->choice = malloc(ind * sizeof(to));
      for (i = 0; i < ind; i++)
	to->choice[i] = NULL;
    }
    for (i = 0; i < ind; i++)
      to->choice[i] = copySf(from->choice[i], to->choice[i]);
  }
  to->f = from->f;
  to->storageG = 0;
/*  to->sp = 0; */
  to->g = NULL;
  to->start = NULL;
  to->stop = NULL;
  /*  to->factor = NULL; */

  if (from->left) {
    to->left = initFNode();
    copyFt(from->left, to->left);
  }
  if (from->right) {
    to->right = initFNode();
    copyFt(from->right, to->right);
  }
  if (from->third) {
    to->third = initFNode();
    copyFt(from->third, to->third);
  }
}

prior copyPrior(prior from, prior to)
{
  char dum;
  prior ptr1, ptr2;

  if (from) {
    if (!to)
      to = initPrior(from->name);
    else {
      if (!to->name) {
	to->name = (char *) malloc((strlen(from->name) + 2) * sizeof(dum));
	strcat(to->name, from->name);
      }
    }
    to->nolikelihood = from->nolikelihood;
    to->update = from->update;
    to->wait = from->wait;
    to->burn = from->burn;
    to->mix = from->mix;
    to->epsilon = from->epsilon;
    /*    if (from->transform) {
      to->transform = initFNode();
      copyFt(from->transform, to->transform);
    }
    if (from->restrict1) {
      to->restrict1 = initFNode();
      copyFt(from->restrict1, to->restrict1);
    }
    if (from->restrict2) {
      to->restrict2 = initFNode();
      copyFt(from->restrict2, to->restrict2);
    } */
    to->transform = from->transform;
    to->restrict1 = from->restrict1;
    to->restrict2 = from->restrict2;
    to->type = from->type;
    to->nArgs = from->nArgs;
      /*      to->args = (argArray) malloc(sizeof(jackArgs)); */
    to->args = from->args;
      /* to->args->n = from->args->n;
      to->args->ptrArray = (ft *) malloc(from->args->n * sizeof(to));
      for (i = 0; i < from->args->n; i++) {
      	to->args->ptrArray[i] = initFNode();
	copyFt(from->args->ptrArray[i], to->args->ptrArray[i]);
      } */
    to->where = from->where;
    if (to->ptype == NONE) {
      switch (from->ptype) {
      case PT_CONST:
	to->x->x = from->x->x;
	break;
      case PT_SCALAR:
	to->x->p = from->x->p;
	break;
      case PT_VECTOR:
	to->x->vec = copyPrior(from->x->vec, to->x->vec);
	if (from->x->vec) {
	  ptr1 = from->x->vec->next;
	  ptr2 = to->x->vec;
	  while (ptr1) {
	    ptr2->next = copyPrior(ptr1, ptr2->next);
	    ptr2->next->prev = ptr2;
	    ptr1 = ptr1->next;
	    ptr2 = ptr2->next;
	  }
	}
	break;
      case PT_PPROCESS:
	to->x->pp = (pProcess) malloc(sizeof(jackPp));
	copyPrior(from->x->pp->t0, to->x->pp->t0);
	copyPrior(from->x->pp->t1, to->x->pp->t1);
	copyPrior(from->x->pp->vector, to->x->pp->vector);
	break;
      case PT_SF:
	bite_eprintf( "Simple function part of \'copyPrior\' has not ");
	bite_eprintf( "been implemented yet.\n");
	break;
      }
    }
    to->ptype = from->ptype;
/*  to->ngt = from->ngt;
  to->gt = from->gt;
  to->nlt = from->nlt;
  to->lt = from->lt;
  to->next = from->next; */

    return to;
  }
  else
    return (prior) NULL;
}

void rmPrior(prior ptr)
{
  long i;
  prior p, next;

  free(ptr->name);
  rmTree(ptr->transform);
  rmTree(ptr->restrict1);
  rmTree(ptr->restrict2);
  for (i = 0; i < ptr->nArgs; i++)
    rmTree(ptr->args[i]);
  free(ptr->args);
  if (ptr->ptype == NONE) {
    switch (ptr->ptype) {
    case PT_CONST:
      break;
    case PT_SCALAR:
      break;
    case PT_VECTOR:
      p = ptr->x->vec;
      while (p) {
	next = p->next;
	rmPrior(p);
	p = next;
      }
      break;
    case PT_PPROCESS:
      bite_eprintf( "Point process parts of \'rmPrior\' have not been implemented yet.\n");
      break;
    case PT_SF:
      bite_eprintf( "Simple function part of \'rmPrior\' has not been implemented yet.\n");
      break;
    }
  }
  free(ptr->x);
  free(ptr);
}

/* void findLikelForArgs(ft tree, long type, useVal target)
{
   When giving a prior distribution with 'set prior' command, this
     function will check the parameters of the arguments.
  long i, n;
  use list;
  prior ptr, list;

  if (tree && tree->ptype) {
    switch (tree->ptype) {
    case PT_SCALAR:
      ptr = tree->var->p;
      break;
    case PT_VECTOR:
      ptr = tree->var->vec;
      break;
    case PT_PPROCESS:
      switch (tree->fieldId) {
      case T0:
	ptr = tree->var->pp->t0;
	break;
      case T1:
	ptr = tree->var->pp->t1;
	break;
      case POINTS:
	i = 1;
	n = calculate2(ind, tree->third->i[0]);
	list = tree->var->pp->vector;
	while (i < n && list && list->next) {
	  list = list->next;
	  i++;
	}
	ptr = tree->var->pp->vector;
	break;
      }
      break;
    case PT_RESP:
      
      break;
    case PT_SF:
      ptr = NULL;
      break;
    }
    if (type && !(ptr->where && 
		  (ptr->where->val->x == target.x || ptr->where->next && 
		   ptr->where->next->val->x == target.x))) {
      list = addUse2(ptr->where);  Add 'use' only if the argument 
				     parameter
      if (!ptr->where)  hasn't been added before for this parameter
	ptr->where = list;  and the parameter is not used itself.
      list->type = type;
      switch (type) {
      case USE_PRIOR:
	list->val->x = target.x;
	break;
      case USE_VAR:
	list->val->var = target.var;
	break;
      }
    }
  }
  if (tree->left)
    findLikelForArgs(tree->left, type, target);
  if (tree->right)
    findLikelForArgs(tree->right, type, target);
} */

void addUses(long i, long mark, long type, prior arg, useVal target, prior p)
{
long ii, jj, match = 0, add = FALSE;
use ptr;

/*  if (!(type == USE_PRIOR && target.x->nolikelihood) && type) { */
if (i >= 0 && mark < 0 && type == USE_PRIOR)
type = USE_INDPARAM;
if (p && p->fixed && i < 0 && type != USE_MF)
type = NONE;
if ((p && !p->nolikelihood && type) || (!p && type == USE_TABLE)) {
if (arg->where) {
if (arg->where->type == type)
match = 1;
if (arg->where->next && arg->where->next->type == type)
match += 2;
}

switch (type) {
case USE_PRIOR:
if (!(((match & 1) == 1 && arg->where->val->x == target.x) ||
((match & 2) == 2 && arg->where->next->val->x == target.x)))
add = TRUE;
break;
case USE_MF:
if (!arg->where || arg->where->type != USE_VAR)
add = TRUE;
break;
case USE_TABLE:
if (!(((match & 1) == 1 && arg->where->val->inUse[mark]) ||
((match & 2) == 2 && arg->where->next->val->inUse[mark])))
add = TRUE;
break;
case USE_TABLE2:
if (!(((match & 1) == 1 && 
checkBit(i % LONGLEN,
arg->where->val->inUse2[mark][i / LONGLEN])) ||
((match & 2) == 2 &&
checkBit(i % LONGLEN,
arg->where->next->val->inUse2[mark][i / LONGLEN]))))
add = TRUE;
break;
case USE_VAR:
if (!(((match & 1) == 1 && arg->where->val->var == target.var) ||
((match & 2) == 2 && arg->where->next->val->var == target.var)))
add = TRUE;
break;
case USE_INDPARAM:
if (!(((match & 1) == 1 && arg->where->val->indParam.i == i
&& arg->where->val->indParam.p == target.x) ||
((match & 2) == 2 && arg->where->next->val->indParam.i == i
&& arg->where->next->val->indParam.p == target.x)))
add = TRUE;
break;
case USE_COV2:
if (!(((match & 1) == 1 && arg->where->val->pvec == target.pvec) ||
((match & 2) == 2 && arg->where->next->val->pvec == target.pvec)))
add = TRUE;
break;
}

if (add) {
if ((type == USE_TABLE && !match) || type != USE_TABLE)
ptr = addUse2(arg->where);
else {
if ((match & 1) == 1)
ptr = arg->where;
else
ptr = arg->where->next;
}
ptr->type = type;

switch (type) {
case USE_PRIOR:
ptr->val->x = target.x;
break;
case USE_MF:
ptr->val->f = target.f;
break;
case USE_TABLE:
if (!match) {
ptr->val->inUse = (long *) malloc(d->n * sizeof(jj));
for (jj = 0; jj < d->n; jj++)
ptr->val->inUse[jj] = FALSE;
}
ptr->val->inUse[mark] = TRUE;
break;
case USE_TABLE2:
if (!match) {
ptr->val->inUse2 = (long **) malloc(d->n * sizeof(jj));
ptr->val->inUse2[0] = (long *) malloc((ind / LONGLEN + 1) *
sizeof(jj));
for (ii = 0; ii < MAX_MARKS; ii++)
for (jj = 0; jj < i / LONGLEN + 1; jj++)
ptr->val->inUse2[ii][jj] = 0;
}
putBitTo1(i % LONGLEN, 
&ptr->val->inUse2[(mark < 0) ? 0 : mark][i / LONGLEN]);
break;
case USE_VAR:
ptr->val->var = target.var;
break;
case USE_INDPARAM:
ptr->val->indParam.i = i;
ptr->val->indParam.p = target.x;
break;
case USE_COV2:
ptr->val->pvec = target.pvec;
break;
}

if (!arg->where)
arg->where = ptr;
}
}
}

void findAndReplaceVarNames(long ind, long mark, ft tree, long type,
			    useVal target, prior p)
{ /* If an argument of a scalar formula is not numeric, then it is either a
     hierarchical parameter, a covariate, a responce, or a model function. */
  long i, ii = FALSE, varIdx, car;
  prior pp, ptr = param, list;

  if (tree->function == F_INDIV)
    indiv_func = ii = TRUE;
  for (i = 0; i < tree->ni; i++)
    findAndReplaceVarNames(ind, mark, tree->i[i], type, target, p);
  if (ii == TRUE)
    indiv_func = ii = FALSE;
  /* if (ptr && tree->varName)
     If not an element of a vector valued parameter, then */
  if (tree->third)
    findAndReplaceVarNames(ind, mark, tree->third, type, target, p);
  if (tree->left) {
    long j;
    prior p2 = param, newP;
    if (tree->nArgs && tree->ptype == PT_SEMICOL) {
      while (p2 && p2->next)
	p2 = p2->next;
      for (j = 0; j < tree->nArgs; j++) {
	newP = initPrior(tree->args[j]->varName);
	newP->fixed = TRUE;
	newP->ptype = PT_CONST;
	if (param) {
	  p2->next = newP;
	  newP->prev = p2;
	  p2 = p2->next;
	}
	else
	  param = p2 = newP;
      }
    }
    findAndReplaceVarNames(ind, mark, tree->left, type, target, p);
    if (tree->nArgs && tree->ptype == PT_SEMICOL) {
      for (j = 0; j < tree->nArgs; j++) {
	p2 = p2->prev;
	rmPrior(p2->next);
	p2->next = NULL;
      }
    }
  }
  if (tree->ptype != PT_SEMICOL) {
    for (i = 0; i < tree->nArgs; i++)
      findAndReplaceVarNames(ind, mark, tree->args[i], type, target, p);
  }
  if (tree->right)
    findAndReplaceVarNames(ind, mark, tree->right, type, target, p);

  if (tree->varName && tree->ptype != PT_STRING) {
    long i = 0, j, n = 0, varType = returnType(tree->varName, &varIdx, FALSE);
    switch (varType) {
    case PT_SCALAR:
      /* Find the last parameter of the node name: */
      while (ptr && ptr->next)
	ptr = ptr->next;
      while (ptr && strcmp(ptr->name, tree->varName))
	ptr = ptr->prev;
      if (ptr) {
	if (tree->ni) { /* A vector */
	  long n = 0;
	  if (type) {
	    if (ind >= 0)
	      n = calculate2(ind, tree->i[0]);
	    else {
	      if (type == USE_MF)
		n = calculate2(findInd2(target.f), tree->i[0]);
	      else {
		pp = ptr->x->vec;
		while (pp) {
		  addUses(ind, mark, type, pp, target, p);
		  pp = pp->next;
		}
	      }
	    }
	  }
	  tree->ptype = PT_VECTOR;
	  tree->var->vec = ptr;
	  pp = ptr->x->vec;
	  while (pp && i < n) {
	    pp = pp->next;
	    i++;
	  }
	  if (!pp) {
	    bite_eprintf( "Vector '%s` is not long enough: "
		    "couldn't find the %ld'th element.\n", ptr->name, n);
	    bite_exit(1);
	  }
	  /* if (n > ptr->stor) {
	     bite_eprintf( "Vector '%s` is not long enough: "
	     "couldn't find the %ld'th element.\n", ptr->name, n);
	     bite_exit(1);
	     }
	     pp = ptr->vector[n]; */
	  if (type != USE_VAR) {
	    if (ind >= 0 || n > 0)
	      addUses(ind, mark, type, pp, target, p);
	    else
	      addUses(ind, mark, type, ptr, target, p);
	  }
	}
	else {/* A scalar or a vector */
	  if (ptr->ptype == PT_VECTOR) {
	    tree->ptype = PT_VECTOR;
	    tree->var->vec = ptr->x->vec;
	  }
	  else {
	    tree->ptype = PT_SCALAR; /* Hierarchical parameter */
	    tree->var->p = ptr;
	  }
	  addUses(ind, mark, type, ptr, target, ptr);
	}
      }
      if (tree->ptype == PT_SCALAR)
	for (i = 0; i < tree->var->p->nArgs; i++)
	  findAndReplaceVarNames(ind, mark, tree->var->p->args[i],
				 type, target, p);
      break;
    case PT_COV:
      if (ind < 0 && !indiv_func) {
	if (type == USE_PRIOR)
	  target.x->fixed = TRUE;
	if (p && p->type != D_DIRAC)
	  p->fixed = TRUE;
      }
      /* while (i < c->n && strcmp(c->cList[i][0]->name, tree->varName))
	 i++; */
      i = varIdx;
      /* if (i < c->n) { else a covariate */
      tree->ptype = PT_COV;
      switch (c->cList[i][0]->type) {
      case COV_CNST:
	tree->var->cov = c->cList[i];
	if (ind >= 0)
	  addUses(ind, mark, type, c->cList[i][ind]->x, target, p);
	break;
      case COV_CNST2:
	tree->ptype = PT_COV2;
	tree->var->pvec = c->pList[i];
	if (c->pList[i]->distr.x)
	  addUses(ind, mark, type, c->pList[i]->distr.x, target, p);
	break;
      case COV_START2:
	tree->ptype = PT_COV2;
	tree->var->pvec = c->pList[i];
	if (tree->i[0] && ind >= 0) {
	  n = calculate2(ind, tree->i[0]);
	  if (c->pList[i]->stor > n && c->pList[i]->distr.x2[n])
	    addUses(ind, mark, type, c->pList[i]->distr.x2[n], target, p);
	}
	break;
      case COV_SF:
	tree->var->cov = c->cList[i] /* [ind]->x->x->f */ ;
	break;
      case COV_START:
	tree->var->cov = c->cList[i] /* [ind]->x->x->vec */ ;
	if (tree->i[0] && ind >= 0) {
	  j = 0;
	  n = calculate2(ind, tree->i[0]);
	  list = c->cList[i][ind]->x->x->vec;
	  while (j < n && list && list->next) {
	    list = list->next;
	    j++;
	  }
	  if (j == n)
	    addUses(ind, mark, type, list, target, p);
	}
	break;
      }
      break;
    case PT_RESP:
      if (ind < 0 && !indiv_func) {
	if (type == USE_PRIOR)
	  target.x->fixed = TRUE;
	if (p && p->type != D_DIRAC)
	  p->fixed = TRUE;
      }
      /* else { */
      car = 0;
      if (tree->i)
	car = calculate2(ind, tree->i[0]);
      /* i = 0;
	 while (i < d->n && strcmp(d->dList[i][0][0]->name, tree->varName))
	 i++; */
      i = varIdx;
      /* if (i < d->n) { else a responce */
      if (car >= d->nCars[i]) {
	bite_eprintf( "%s: not so many observation intervals.\n",
		tree->varName);
	bite_exit(1);
      }
      if (d->pList[i]) {
	tree->ptype = PT_RESP2;
	tree->var->pvec = d->pList[i];
      }
      else {
	tree->ptype = PT_RESP;
	tree->var->resp = d->dList[i];
      }
      switch (tree->fieldId) {
      case T0:
	if (ind >= 0) {
	  if (d->pList[i] && !d->pList[i]->distr.pp.t0[car])
	    d->pList[i]->distr.pp.t0[car] = initPrior("t0");
	  addUses(ind, mark, type,
		  (d->pList[i]) ? d->pList[i]->distr.pp.t0[car] :
		  d->dList[i][ind][car]->pp->t0,
		  target, p);
	}
	break;
      case T1:
	if (ind >= 0) {
	  if (d->pList[i] && !d->pList[i]->distr.pp.t1[car])
	    d->pList[i]->distr.pp.t1[car] = initPrior("t1");
	  addUses(ind, mark, type,
		  (d->pList[i]) ? d->pList[i]->distr.pp.t1[car] :
		  d->dList[i][ind][car]->pp->t1,
		  target, p);
	}
	break;
      case POINTS:
	j = 0;
	n = calculate2(ind, tree->third->i[0]);
	if (ind >= 0) {
	  if (d->pList[i]) {
	    if (!d->pList[i]->distr.pp.points[car][n])
	      d->pList[i]->distr.pp.points[car][n] = initPrior("point");
	    addUses(ind, mark, type,
		    d->pList[i]->distr.pp.points[car][n], target, p);
	  }
	  else {
	    list = d->dList[i][ind][car]->pp->vector;
	    while (j < n && list && list->next) {
	      list = list->next;
	      j++;
	    }
	    if (j == n)
	      addUses(ind, mark, type, list, target, p);
	  }
	}
	break;
      }
      break;
    case PT_MODELF:
      /* else { or a model function. */
      /* tree->ptype = PT_MODELF;
	 i = 0;
	 while (i < f->n && strcmp(tree->varName, f->fList[i][0]->name))
	 i++;
	 if (i == f->n && tree->varName[0] && iter >= 0) {
	 bite_eprintf( "%s: what is this?\n", tree->varName);
	 bite_exit(1);
	 } */
      /* for (j = 0;
	 j < f->nFunc[i] && !testCovEq(f->fList[i][j], labela[lStack]);
	 j++);
	 if (j < f->nFunc[i]) {
	 }
	 else {
	 bite_eprintf( "Model function instance ");
	 bite_eprintf( "%s(", f->fList[i][0]->name);
	 if (f->fList[i][0]->cov[0][0]) {
	 int ii = 0;
	 while (f->fList[i][0]->cov[ii][0]) {
	 if (ii)
	 bite_eprintf( ", ");
	 bite_eprintf( "%s == %g",
	 f->fList[i][0]->cov[ii], f->fList[i][0]->covVal[ii]);
	 i++;
	 }
	 }
	 bite_eprintf( ") not found.\n");
	 } */
      break;
    }
  }
}

void findAndReplaceVarNames2(long i, long mark, prior param, long type,
			     useVal target)
{
  long j;

  if (param->transform)
    findAndReplaceVarNames(i, mark, param->transform, NONE, target, param);
  if (param->restrict1)
    findAndReplaceVarNames(i, mark, param->restrict1, type, target, param);
  if (param->restrict2)
    findAndReplaceVarNames(i, mark, param->restrict2, type, target, param);
  for (j = 0; j < param->nArgs; j++)
    findAndReplaceVarNames(i, mark, param->args[j], type,
			   target, param);
/*  printParam(param); */
}

void replaceVarName(ft tree, long n_vars, char *name_vec[])
{
  /* Search for nodes and replace the scalar variables with their current values.
   */
}
  
void rmTree(ft root)
{
  if (root) {
    long i;
    for (i = 0; i < root->nArgs; i++)
      rmTree(root->args[i]);
    free(root->args);
    /*    free(root->jpIndex); */
    rmTree(root->left);
    rmTree(root->right);
    rmTree(root->third);
    for (i = 0; i < root->storageG; i++)
      freeSf(root->g[i]);
    free(root->g);
    free(root);
  }
}

double findCValue(long ind, char *cov, double time, ct c)
{
  long i = 0;

  while (strcmp(cov, c->cList[i][0]->name))
    i++;
  switch (c->cList[i][0]->type) {
  case COV_CNST:
    return c->cList[i][ind]->x->x->x;
    break;
  case COV_CNST2:
    return covValue(ind, 0, c->pList[i]);
    break;
  default:
    return value(c->cList[i][ind]->x->x->f, time);
  }
  return 0.0;
}

void makeChoice(long i, char **covs, double *values, ct c, mft f, 
		char *name, ft root)
{
  long k, j = 0;

  if (!root->choice) {
    root->choice = malloc(ind * sizeof(root));
    for (k = 0; k < ind; k++)
      root->choice[k] = NULL;
  }
  if (!root->choice[i]) {
    root->choice[i] = initSf(0.0, 0.0, NULL);
/*    root->choice->jumpPoints[1] = 0.0; */
  }
  else {
    root->choice[i]->n = 2;
    root->choice[i]->jumpPoints[0] = 0.0;
    root->choice[i]->jumpPoints[1] = 0.0;
/*    root->choice->levels[0] = 0.0; */
  }
  while (covs[j][0]) {
    if (findCov(c, covs[j])[i]->type == COV_SF)
      opSf('+', root->choice[i], findCov(c, covs[j])[i]->x->x->f, FALSE);
    j++;
  }
  for (j = 0; j < root->choice[i]->n - 1; j++) {
    k = 0;
    while (covs[k][0]) {
      values[k] = findCValue(i, covs[k], 
			     (root->choice[i]->jumpPoints[j] + 
			      root->choice[i]->jumpPoints[j + 1]) / 2.0, c);
      k++;
    }
    root->choice[i]->levels[j] = findMf(f, name, k, covs, values);
    /* x = findMf(f, name, k, covs, values);
    k = 0;
    while (root->f[k] != x)
      k++;
    root->choice[i]->levels[j] = (double) k; */
  }
}

void checkCovs(ft root, long mark, long ind, ct c, mft f)
{
  if (root->function && root->nArgs)
    checkCovs(root->args[0], mark, ind, c, f);
  if (root->f) {
    long j;
    double values[MAX_COV];
    if (!root->choice) {
      /* No time dependent covariates */
      j = 0;
      while (root->f[0]->cov[j][0]) {
	values[j] = findCValue(ind, root->f[0]->cov[j], 0.0, c);
	j++;
      }
      /*root->f[0] = findMf(f, root->f[0]->name, j, root->f[0]->cov, values);*/
      if (root->f[root->fc[ind]]->nIndiv < 0) {
	putBitTo0(ind % LONGLEN,
		  &root->f[root->fc[ind]]->inUse[mark][ind / LONGLEN]);
	root->fc[ind] = findMf(f, root->f[0]->name, j, root->f[0]->cov, values);
	putBitTo1(ind % LONGLEN,
		  &root->f[root->fc[ind]]->inUse[mark][ind / LONGLEN]);
      }
    }
    else {
      /* At least one time dependent covariate */
      makeChoice(ind, root->f[0]->cov, values, c, f, root->f[0]->name, root);
    }
  }
  else {
    if (root->left)
      checkCovs(root->left, mark, ind, c, f);
    if (root->right)
      checkCovs(root->right, mark, ind, c, f);
  }
}

long isTreeRandom(long i, ft root)
{
  long res = 0, c1 = 0, i1 = 0, c2 = 0, i2 = 0;
  prior p = NULL, elem1, elem2;

  if (root) {
    res +=isTreeRandom(i, root->left);
    res +=isTreeRandom(i, root->right);
    res +=isTreeRandom(i, root->third);
    if (root->i)
      i1 = calculate2(i, root->i[0]);
    if (root->third && root->third->i)
      i2 = calculate2(i, root->third->i[0]);
    for (i = 0; i < root->ni; i++)
      res +=isTreeRandom(i, root->i[i]);
    for (i = 0; i < root->nArgs; i++)
      res +=isTreeRandom(i, root->args[i]);
    switch (root->ptype) {
    case PT_SCALAR:
      p = root->var->p;
      break;
    case PT_VECTOR:
      elem1 = root->var->vec;
      while (elem1 && c1 < i1) {
	elem1 = elem1->next;
	c1++;
      }
      p = elem1;
      break;
    case PT_SF:
      break;
    case PT_RESP:
      switch (root->fieldId) {
      case T0:
	p = root->var->resp[i][i1]->pp->t0;
	break;
      case T1:
	p = root->var->resp[i][i1]->pp->t1;
	break;
      case POINTS:
	elem2 = root->var->resp[i][i1]->pp->vector;
	while (elem2 && c2 < i2) {
	  elem2 = elem2->next;
	  c2++;
	}
	p = elem2;
	break;
      }
      break;
    case PT_COV:
      switch (root->var->cov[i]->type) {
      case COV_CNST:
	p = root->var->cov[i]->x;
	break;
      case COV_SF:
	break;
      case COV_START:
	if (root->ni) {
	  long j = 0;
	  prior list = root->var->cov[i]->x->x->vec;
	  while (j < i1 && list && list->next) {
	    list = list->next;
	    j++;
	  }
	  if (j == i1)
	    p = list;
	}
	break;
      }
      break;
    case PT_MODELF:
      /* switch (root->fieldId) {
      case JUMPPOINT:
	p = root->var->m->f->jumpPoints[i2];
	break;
      case LEVEL:
	p = root->var->m->f->levels[i2];
	break;
      } */
      break;
    default:
      break;
    }
    res += p->ptype != PT_CONST && p->ptype && !p->fixed;
  }
  return res;
}

/* For "Lex/Yacc" intensity model: */

void findCovsAndMfs(long ind2, long mark, ct c, mft f, ft root)
{
  if (root) {
    long i = 0, j, jj, ii, k = 0;
    double values[MAX_COV];
    char **covs;
    use ptr;

    findCovsAndMfs(ind2, mark, c, f, root->left);
    if (root->function && root->nArgs)
      findCovsAndMfs(ind2, mark, c, f, root->args[0]);
    if (root->varName && root->varName[0]) {
      while (i < f->n && strcmp(f->fList[i][0]->name, root->varName))
	i++;
      if (i == f->n) {
	bite_eprintf( "%s: no such model function.\n", root->varName);
	bite_exit(1);
      }
      covs = f->fList[i][0]->cov;
      j = 0;
      k = 0;
      while (covs[j][0]) {
	if (findCov(c, covs[j])[0]->type == COV_SF)
	  k = 1;
	jj = returnType(covs[j], &ii, FALSE);
	if (c->pList[ii]->type == COV_CNST2 && c->pList[ii]->distr.x) {
	  /* If the covariate has been given a prior then add the mark to
	     the likelihood. */
	  if (c->pList[ii]->distr.x->where &&
	      c->pList[ii]->distr.x->where->type == USE_TABLE)
	    ptr = c->pList[ii]->distr.x->where;
	  else {
	    ptr = addUse2(c->pList[ii]->distr.x->where);
	    ptr->type = USE_TABLE;
	    ptr->val->inUse = (long *) malloc(d->n * sizeof(j));
	    for (jj = 0; jj < d->n; jj++)
	      ptr->val->inUse[jj] = FALSE;
	  }
	  if (!c->pList[ii]->distr.x->where)
	    c->pList[ii]->distr.x->where = ptr;
	  ptr->val->inUse[mark] = TRUE;
	}
	j++;
      }

      jj = returnType(root->args[0]->varName, &ii, FALSE);
      if (!jj) {
	bite_eprintf( "Cannot find %s.\n", root->args[0]->varName);
	bite_exit(1);
      }
      if (c->pList[ii])
	root->start2 = c->pList[ii];
      else
	root->start = c->cList[ii][ind2]->x;
	
      if (root->nArgs == 2 && root->args[1]->varName) {
	jj = returnType(root->args[1]->varName, &ii, FALSE);
	if (c->pList[ii])
	  root->stop2 = c->pList[ii];
	else
	  root->stop = c->cList[ii][ind2]->x;
      }

      if (root->start) {
	if (root->start->where && root->start->where->type == USE_TABLE)
	  ptr = root->start->where;
	else {
	  ptr = addUse2(root->start->where);
	  ptr->type = USE_TABLE;
	  ptr->val->inUse = (long *) malloc(d->n * sizeof(j));
	  for (jj = 0; jj < d->n; jj++)
	    ptr->val->inUse[jj] = FALSE;
	}
	if (!root->start->where)
	  root->start->where = ptr;
	ptr->val->inUse[mark] = TRUE;
      }
      else {
	ii = 0;
	while (ii < root->start2->stor && !root->start2->distr.x2[ii])
	  ii++;
	if (ii < root->start2->stor) {
	  if (root->start2->distr.x2[ii]->where &&
	      root->start2->distr.x2[ii]->where->type == USE_TABLE)
	    ptr = root->start2->distr.x2[ii]->where;
	  else {
	    ptr = addUse2(root->start2->distr.x2[ii]->where);
	    ptr->type = USE_TABLE;
	    ptr->val->inUse = (long *) malloc(d->n * sizeof(j));
	    for (jj = 0; jj < d->n; jj++)
	      ptr->val->inUse[jj] = FALSE;
	  }
	  if (!root->start2->distr.x2[ii]->where)
	    root->start2->distr.x2[ii]->where = ptr;
	  ptr->val->inUse[mark] = TRUE;
	}
      }

      if (root->stop) {
	ptr = addUse2(root->stop->where);
	ptr->type = USE_TABLE;
	ptr->val->inUse = (long *) malloc(d->n * sizeof(j));
	for (jj = 0; jj < d->n; jj++)
	  ptr->val->inUse[jj] = FALSE;
	ptr->val->inUse[mark] = TRUE;
	if (!root->stop->where)
	  root->stop->where = ptr;
      }

      if (!k && ind2 < ind) {
	/* No time dependent covariates */
	j = 0;
	while (covs[j][0]) {
	  values[j] = findCValue(ind2, covs[j], 0.0, c);
	  j++;
	}
	/* root->f = (mf *) malloc(sizeof(values));
	root->f[0] = findMf(f, root->varName, j, covs, values);
	root->storageF = 1; */
	if (!root->f) {
	  root->f = f->fList[i];
	  root->fc = malloc(ind * sizeof(i));
	}
	root->fc[ind2] = findMfO(f, root->varName, j, covs, values);
	root->storageF = f->nFunc[i];
      }
      else {
	/* At least one time dependent covariate */
	root->f = f->fList[i];
	root->storageF = f->nFunc[i];
	if (ind2 < ind)
	  makeChoice(ind2, covs, values, c, f, root->varName, root);
      }
    }
    findCovsAndMfs(ind2, mark, c, f, root->right);
  }
}

ft initFNode()
{
  ft res;

  res = malloc( sizeof( jackFt ) );
  res->op = '\0';

#ifdef DEBUG
  res->lineno = lineno;
#endif
  res->lineno = lineno;
  res->function = NONE; /* These 4 fields are for scalar formula trees */
  res->nArgs = 0;
  res->args = NULL;
  res->varName = NULL;
  res->ptype = NONE;
  res->var = (pType *) malloc(sizeof(pType));
/*  res->x = 0.0; */

  res->storageF = 0; /* and the rest for simple function trees. */
  res->ni = 0;
  res->i = NULL;
  res->fieldId = NONE;
  /*  res->index = -1;
  res->jpIndex = NULL; */
  res->choice = NULL;
  res->f = NULL;
  res->fc = NULL;
  res->storageG = 0;
/*  res->sp = 0; */
  res->g = NULL;
  res->start = NULL;
  res->stop = NULL;
  res->start2 = NULL;
  res->stop2 = NULL;
  /*  res->factor = NULL; */

  res->left = NULL;
  res->right = NULL;
  res->third = NULL;

  return(res);
}

long checkFt(mft f, long i, ft ptr, mf target)
{
  long res = 0, j, k;

  if (ptr) {
    res += checkFt(f, i, ptr->left, target);
    if (ptr->function && ptr->nArgs)
      res += checkFt(f, i, ptr->args[0], target);
    if (!ptr->op) {
/*      if (!ptr->choice) {
	if (ptr->f[0] == target) 
	  res++;
      }
      else {
	while (strcmp(f->fList[i][0]->name, ptr->f[0]->name))
	  i++;
	for (j = 0; j < f->nFunc[i]; j++)
	  if (ptr->f[j] == target) {
	    for (k = 0; k < ptr->choice->n - 1; k++)
	      if (ptr->choice->levels[k] == j)
		res++;
	  }
      } */
      if (ptr->fc)
	res += (ptr->f[ptr->fc[i]] == target) ? 1 : 0;
      else {
	for (j = 0; j < ptr->storageF; j++)
	  if (ptr->f[j] == target) {
	    if (!ptr->choice)
	      res++;
	    else
	      for (k = 0; k < ptr->choice[i]->n - 1; k++)
		if (ptr->choice[i]->levels[k] == (double) j)
		  res++;
	  }
      }
    }
    res += checkFt(f, i, ptr->right, target);
  }

  return(res);
}

long checkFtCov(mft f, ft ptr, char *cov)
{
  long res = 0, i = 0, i2 = 0;

  if (ptr) {
    res += checkFtCov(f, ptr->left, cov);
    if (!ptr->op) {
      while (strcmp(f->fList[i][0]->name, ptr->f[0]->name))
	i++;
      while (strcmp(cov, f->fList[i][0]->cov[i2]) &&
	     f->fList[i][0]->cov[i2][0])
	i2++;
      if (f->fList[i][0]->cov[i2][0]) 
	res++;
    }
    res += checkFtCov(f, ptr->right, cov);
  }

  return(res);
}

void findInd(long ind, ff model, mf target, mft f)
{
  long mark, i;

  if (target->name[0]) {	/* Is this a true model function? */
    long *v = malloc(d->n * sizeof(i)), n;
    for (i = 0; i < ind; i++) {
      n = 0;
      for (mark = 0; mark < model->n; mark++) {
#ifndef B_FACTOR
	if (checkFt(f, i, model->ftList[mark][0], target)) {
	  v[mark] = 1;
	  n++;
	  if (target->nIndiv < 0)
	    putBitTo1(i % LONGLEN, &target->inUse[mark][i / LONGLEN]);
	}
	else {
#endif
	  v[mark] = 0;
	  if (target->nIndiv < 0)
	    putBitTo0(i % LONGLEN, &target->inUse[mark][i / LONGLEN]);
	}
      }
      if (target->nIndiv >= 0 && n) {
	for (mark = 0; mark < d->n; mark++) {
	  if (target->nIndiv % LONGLEN == 0)
	    target->inUse[mark] = realloc(target->inUse[mark],
				     (target->nIndiv / LONGLEN + 1) *
				     sizeof(ind));
	  if (v[mark])
	    putBitTo1(target->nIndiv % LONGLEN,
		      &target->inUse[mark][target->nIndiv / LONGLEN]);
	  else
	    putBitTo0(target->nIndiv % LONGLEN,
		      &target->inUse[mark][target->nIndiv / LONGLEN]);
	}
	target->indivVec = realloc(target->indivVec,
				    (target->nIndiv+1) * sizeof(n));
	target->indivVec[target->nIndiv] = i;
	target->nIndiv++;
      }
    }
    free(v);
  }
  else { /* Or one only used in calculating expectations? */
    for (i = 0; i < ind; i++) {
      if (cmpCovs(target, NULL, i))
	putBitTo1(i % LONGLEN, &target->inUse[0][i / LONGLEN]);
      else
	putBitTo0(i % LONGLEN, &target->inUse[0][i / LONGLEN]);
    }
  }
}

void inorder(long i, ft ptr)
{
  long j, ii, mark, cov;
  prior p;
  mf target;

  if (ptr) {
    if (ptr->left != NULL && ptr->right != NULL)
      bite_printf("(");
    inorder(i, ptr->left);
    if (ptr->op) {
      switch (ptr->op) {
      case LT:
	bite_printf(" < ");
	break;
      case GT:
	bite_printf(" > ");
	break;	 
      case LE:	 
	bite_printf(" <= ");
	break;	 
      case GE:	 
	bite_printf(" >= ");
	break;	 
      case EQ:	 
	bite_printf(" == ");
	break;	 
      case NE:	 
	bite_printf(" != ");
	break;	 
      case AND:	 
	bite_printf(" && ");
	break;
      case OR:
	bite_printf(" || ");
	break;
      default:
	bite_printf(" %c ", (int) ptr->op);
      }
    }
    if (ptr->f) {
      ii = ptr->fc[i];
      printMf2(ptr->f[ii]);
      if (ptr->start || ptr->start2) {
	bite_printf(" s.p. ");
	bite_printf("%g", ((ptr->start) ? ptr->start->x->vec->x->x :
		      covValue(i, 0, ptr->start2)));
	p = (ptr->start) ? ptr->start->x->vec : NULL;
	if (ptr->start) {
	  for (p = ptr->start->x->vec; p; p = p->next)
	    bite_printf("%c%g", (p != ptr->start->x->vec) ? ',' : ' ',
		   ptr->start->x->vec->x->x);
	}
	else {
	  for (j = 1; j < ptr->start2->n2[j]; j++)
	    bite_printf("%c%g", (j) ? ',' : ' ', covValue(i, j, ptr->start2));
	}
	bite_printf(" == ");
	if (!ptr->g) {
	  if (!ptr->choice) {
	    switch (ptr->f[0]->type) {
	    case SF_ORIG:
	      printSf(ptr->f[0]->f->f);
	      break;
	    case SF_TIMELESS:
	      bite_printf("%g", paramValue(i, ptr->f[0]->f->p));
	      break;
	    }
	  }
	  else {
	    long i;
	    for (i = 0; i < ptr->choice[i]->n - 1; i++) {
	      ptr->g[1] = cutSf(ptr->f[(long)ptr->choice[i]->levels[i]]->f->f,
				ptr->choice[i]->jumpPoints[i] - 
				((ptr->start) ? ptr->start->x->vec->x->x :
				 covValue(i, 0, ptr->start2)),
				ptr->choice[i]->jumpPoints[i + 1] - 
				((ptr->start) ? ptr->start->x->vec->x->x :
				 covValue(i, 0, ptr->start2)),
				ptr->g[1]);
	      bite_printf("%ld: %g - %g, ", i, ptr->choice[i]->jumpPoints[i], 
		     ((ptr->start) ? ptr->start->x->vec->x->x :
		      covValue(i, 0, ptr->start2)));
	      opSf('+', ptr->g[0], ptr->g[1], FALSE);
	    }
	    printSf(ptr->g[0]);
	  }
	}
	else
	  printSf(ptr->g[0]);
      }
      else
	if (ptr->f[ii]->name[0])
	  printSf(ptr->f[ii]->f->f);
    }
    if (ptr->function) {
      long mark = 0;
      sf lambda;
      switch (ptr->function) {
      case F_LOG:
	bite_printf("log(");
	inorder(i, ptr->args[0]);
	bite_printf(")");
	break;
      case F_EXP:
	bite_printf("exp(");
	inorder(i, ptr->args[0]);
	bite_printf(")");
	break;
      case F_MIN:
	bite_printf("min(");
	inorder(i, ptr->args[0]);
	bite_printf(",");
	inorder(i, ptr->args[1]);
	bite_printf(")");
	break;
      case F_MAX:
	bite_printf("max(");
	inorder(i, ptr->args[0]);
	bite_printf(",");
	inorder(i, ptr->args[1]);
	bite_printf(")");
      case F_VALUE:
	switch (ptr->args[0]->ptype) {
	case PT_RESP:
	case PT_RESP2:
	  mark = 0;
	  while (mark < d->n && ptr->args[0]->var->resp != d->dList[mark])
	    mark++;
	  bite_printf("value(%s,", d->dList[mark][0][0]->name);
	  inorder(i, ptr->args[1]);
	  if (ptr->args[0]->fieldId == INTENSITY) {
	    putSfToFt2(i, mark, NULL, -INFTY, INFTY, 
		       model->ftList[mark][0], model->ftList[mark][0], 
		       d, f);
	    /*lambda = calculate(i, model->ftList[mark][0]);
	      res = value(lambda, calculate2(i, ptr->args[1])); */
	    inorder(i, model->ftList[mark][0]);
	  }
	  else {
	    bite_eprintf( "Only intensity available (%s).\n", 
		    ptr->args[0]->varName);
	    bite_exit(1);
	  }
	  break;
	case PT_MODELF:
	  target = (ptr->fc) ? ptr->f[i] : ptr->args[0]->var->m;
	  bite_printf("value of ");
	  printMf2(target);
	  bite_printf(" at ");
	  inorder(i, ptr->args[1]);
	  break;
	default:
	  bite_eprintf( "No such data type for \"value\" (%s).\n", 
		  ptr->args[0]->varName);
	  bite_exit(1);
	}
	break;
      case F_LOGLIKE:
	switch (ptr->args[0]->ptype) {
	case PT_RESP2:
	  mark = 0;
	  while (d->pList[mark] != ptr->args[0]->var->pvec)
	    mark++;
	case PT_RESP:
	  inorder(i, ptr->args[0]);
	  j = printLevel;
	  printLevel = 4;
	  bite_printf(" == %g",
		 indLogL(i, (ptr->args[0]->ptype == PT_RESP) ?
			 ptr->args[0]->var->resp[0][0]->mark : mark, NULL,
			 calculate2(i, ptr->args[1]),
			 calculate2(i, ptr->args[2]), d, model));
	  printLevel = j;
	  inorder(i, ptr->args[1]);
	  inorder(i, ptr->args[2]);
	  break;
	default:
	  bite_eprintf( "No such parameter type for ");
	  bite_eprintf( "\"loglikelihood\" on line %ld.\n", lineno);
	  bite_exit(1);
	}
	break;
      case F_SUM:
	if (ptr->f)
	  target = ptr->f[i];
	else
	  target = findFt(ptr, NULL, f)->var->m;
	bite_printf("sum of ");
	for (j = 0; j < ind; j++)
	  if (checkBit(j % LONGLEN, target->inUse[0][j / LONGLEN])) {
	    bite_printf("\n%ld:", j);
	    inorder(j, ptr->args[0]);
	  }
	break;
      case F_PROD:
	target = findFt(ptr, NULL, f)->var->m;
	bite_printf("prod of ");
	inorder(0, ptr->args[0]);
	for (j = 0; j < ind; j++)
	  if (checkBit(j % LONGLEN, target->inUse[0][j / LONGLEN]))
	    bite_printf("\n%ld,", j);
	break;
      case F_INT:
	switch (ptr->args[0]->ptype) {
	case PT_RESP2:
	  while (d->pList[mark] != ptr->args[0]->var->pvec)
	    mark++;
	  if (ptr->args[0]->fieldId == INTENSITY) {
	    putSfToFt2(i, mark, NULL, -INFTY, INFTY, 
		       model->ftList[mark][0], model->ftList[mark][0],
		       d, f);
	    lambda = calculate(i, model->ftList[mark][0]);
	    bite_printf("Integral of ");
	    inorder(i, model->ftList[mark][0]); 
	    inorder(i, ptr->args[0]);
	    bite_printf(" on ");
	    inorder(i, ptr->args[1]);
	    inorder(i, ptr->args[2]);
	    bite_printf(" equals ");
	    printSf(lambda);
	    bite_printf(" == %g ", intSf(lambda, calculate2(i, ptr->args[1]), 
			  calculate2(i, ptr->args[2])));
	  }
	  else {
	    bite_eprintf( "Only hazard available (%s).\n", 
		    ptr->args[0]->varName);
	    bite_exit(1);
	  }
	  break;
	case PT_MODELF:
	  target = (ptr->fc) ? ptr->f[i] : ptr->args[0]->var->m;
	  bite_printf("integral of ");
	  printMf2(target);
	  bite_printf(" on ");
	  inorder(i, ptr->args[1]);
	  inorder(i, ptr->args[2]);
	  break;
	default:
	  bite_eprintf( "No such data type for \"integral\" (%s).\n", 
		  ptr->args[0]->varName);
	  bite_exit(1);
	}
	break;
      case F_INVINT:
	switch (ptr->args[0]->ptype) {
	case PT_RESP:
	  break;
	case PT_MODELF:
	  break;
	}
      }
    }
    else {
      if (ptr->var) {
	long i1[2] = {0, 0}, ii, c1 = 0, i2 = 0, c2 = 0;
	prior elem1, elem2;
	for (ii = 0; ii < ptr->ni; ii++)
	  i1[ii] = calculate2(i, ptr->i[ii]);
	if (ptr->third && ptr->third->i)
	  i2 = calculate2(i, ptr->third->i[0]);
	switch (ptr->ptype) {
	case PT_CONST:
	  bite_printf("const==%g ", ptr->var->x);
	  break;
	case PT_SCALAR:
	  bite_printf("scalar==%g ", paramValue(i, ptr->var->p));
	  break;
	case PT_VECTOR:
	  ii = 0;
	  elem1 = ptr->var->vec->x->vec;
	  while (ii < ptr->ni) {
	    c1 = 0;
	    while (elem1 && c1 < i1[ii]) {
	      elem1 = elem1->next;
	      c1++;
	    }
	    elem1 = (elem1->ptype == PT_VECTOR) ?
	      elem1->x->vec : elem1;
	    ii++;
	  }
	  bite_printf("%s[%ld]==%g ", ptr->var->vec->name, i1[ii],
		 paramValue(i, elem1));
	  break;
	case PT_SF:
	  break;
	case PT_RESP:
	  if (i >= 0) {
	    switch (ptr->fieldId) {
	    case T0:
	      bite_printf("%s[%ld][%ld]->t0==%g ", ptr->var->resp[0][0]->name,
		     i, i1[0],
		     paramValue(i, ptr->var->resp[i][i1[0]]->pp->t0));
	      break;
	    case T1:
	      bite_printf("%s[%ld][%ld]->t1==%g ", ptr->var->resp[0][0]->name,
		     i, i1[0],
		     paramValue(i, ptr->var->resp[i][i1[0]]->pp->t1));
	      break;
	    case POINTS:
	      elem2 = ptr->var->resp[i][i1[0]]->pp->vector;
	      while (elem2 && c2 < i2) {
		elem2 = elem2->next;
		c2++;
	      }
	      bite_printf("%s[%ld][%ld]->points[%ld]==%g ",
		     ptr->var->resp[0][0]->name, i, i1[0], i2,
		     (elem2) ? paramValue(i, elem2) : -INFTY);
	      break;
	    }
	  }
	  break;
	case PT_RESP2:
	  mark = 0;
	  while (d->pList[mark] != ptr->var->pvec)
	    mark++;
	  if (i >= 0) {
	    bite_printf("%s[%ld][%ld]->", d->dList[mark][0][0]->name, i, i1[0]);
	    switch (ptr->fieldId) {
	    case T0: bite_printf("t0");
	      break;
	    case T1: bite_printf("t1");
	      break;
	    case POINTS: bite_printf("points[%ld]", i2);
	      break;
	    case INTENSITY: bite_printf("intensity");
	      break;
	    }
	    bite_printf("==%g ", respValue(i, i1[0], i2, ptr->fieldId,
				      ptr->var->pvec));
	  }
	  break;
	case PT_COV:
	  if (i >= 0) {
	    switch (ptr->var->cov[0]->type) {
	    case COV_CNST:
	      bite_printf("%s[%ld]==%g ", ptr->var->cov[i]->name, i,
		     paramValue(i, ptr->var->cov[i]->x));
	      break;
	    case COV_SF:
	      break;
	    case COV_START:
	      if (ptr->ni) {
		long j = 0;
		prior list = ptr->var->cov[i]->x->x->vec;
		while (j < i1[0] && list && list->next) {
		  list = list->next;
		  j++;
		}
		if (j == i1[0])
		  bite_printf("%s[%ld][%ld]==%g ", ptr->var->cov[i]->name, i,
			 i1[0], paramValue(i, list));
	      }
	      break;
	    }
	  }
	  else
	    bite_printf("0.0");
	  break;
	case PT_COV2:
	  cov = 0;
	  while (c->pList[cov] != ptr->var->pvec)
	    cov++;
	  bite_printf("%s[%ld][%ld]==%g ", c->cList[cov][0]->name, i, i1[0],
		 covValue(i, i1[0], ptr->var->pvec));
	  break;
	case PT_MODELF:
	  target = (ptr->fc) ? ptr->f[i] : ptr->var->m;
	  printMf2(target);
	  switch (target->type) {
	  case SF_ORIG:
	    switch (ptr->fieldId) {
	    case JUMPPOINT:
	      bite_printf("->jumpPoint[%ld]==%g ", i2,
		     target->f->f->jumpPoints[i2]);
	      break;
	    case LEVEL:
	      bite_printf("->levels[%ld]==%g ", i2, target->f->f->levels[i2]);
	      break;
	    }
	    break;
	  case SF_TIMELESS:
	    switch (ptr->fieldId) {
	    case JUMPPOINT:
	      bite_printf("->jumpPoint[%ld]==%g ", i2, INFTY);
	      break;
	    case LEVEL:
	      bite_printf("->levels[%ld]==%g ", i2, paramValue(i, target->f->p));
	      break;
	    }
	    break;
	  }
	  break;
	}
      }
    }
    inorder(i, ptr->right);
    if (ptr->left != NULL && ptr->right != NULL)
      bite_printf(")");
  }
}

long depth(ft root)
{
  long max = 10, i, j, k, div, height = 0;
  ft ptr;

  if (root) {
    for (i = 0; i < exp(log(2.0) * max); i++) {
      ptr = root;
      j = exp(log(2.0) * (max - 1));
      k = 0;
      for (div = 0; div < max; div++) {
	if (i / j && ptr->left) {
	  j /= 2;
	  k++;
	  ptr = ptr->left;
	}
	else if (!(i / j) && ptr->right) {
	  j /= 2;
	  k++;
	  ptr = ptr->right;
	}
      }
      if (k > height)
	height = k;
    }
  }
  return(height);
}

double covValue(long i, long j, paramVector pv)
{
  switch (pv->type) {
  case COV_CNST2:
    return (pv->distr.x && pv->distr.x->type == D_DIRAC &&
	    !checkBit(i % LONGLEN, pv->fixed.x[i / LONGLEN])) ?
      calculate2(i, pv->distr.x->args[0]) : pv->val.x[i];
    break;
  case COV_START2:
    return (pv->distr.x2[j] && pv->distr.x2[j]->type == D_DIRAC &&
	    !checkBit(j % LONGLEN, pv->fixed.x2[i][j / LONGLEN])) ?
      calculate2(i, pv->distr.x2[j]->args[0]) : pv->val.x2[i][j];
    break;
  }
  return 0.0;
}

double respValue(long i, long car, long j, long type, paramVector pv)
{
  switch (type) {
  case T0:
    return (pv->distr.pp.t0[car] && pv->distr.pp.t0[car]->type == D_DIRAC &&
	    !pv->fixed.pp.t0[i][car]) ?
      calculate2(i, pv->distr.pp.t0[car]->args[0]) : pv->val.pp.t0[i][car];
    break;
  case T1:
    return (pv->distr.pp.t1[car] && pv->distr.pp.t1[car]->type == D_DIRAC &&
	    !pv->fixed.pp.t1[i][car]) ?
      calculate2(i, pv->distr.pp.t1[car]->args[0]) : pv->val.pp.t1[i][car];
    break;
  case POINTS:
    return (pv->distr.pp.points[car][j] &&
	    pv->distr.pp.points[car][j]->type == D_DIRAC &&
	    pv->val.pp.npoints[i][car] &&
	    !checkBit(j % LONGLEN,
		      pv->fixed.pp.points[i][car][j / LONGLEN])) ?
      calculate2(i, pv->distr.pp.points[car][j]->args[0]) :
	((pv->val.pp.npoints[i][car] > j) ? pv->val.pp.points[i][car][j] :
	 -INFTY);
    break;
  }
  return 0.0;
}

void putFt(long ind, ft ptr1, ft ptr2, sf target, double bot, double top)
{
  if (ptr1) {
    putFt(ind, ptr1->left, ptr2->left, target, bot, top);
    if (ptr1->function) {
      long i, n = 1;
      if (n > ptr2->storageG) {
	sf *dum;
	dum = (sf *) malloc(n * sizeof(ptr1));
	for (i = 0; i < ptr2->storageG; i++)
	  dum[i] = ptr2->g[i];
	for (i = ptr2->storageG; i < n; i++)
	  dum[i] = NULL;
	free(ptr2->g);
	ptr2->g = dum;
	ptr2->storageG = n;
      }
      if (ptr1->nArgs)
	putFt(ind, ptr1->args[0], ptr2->args[0], target, bot, top);
    }
    if (ptr1->f) {
      prior ptr, ptrstop;
      long i, n;
      if (!ptr2->g) {
	ptr2->storageG = 10;
	ptr2->g = (sf *) malloc(10 * sizeof(ptr1));
	for (i = 0; i < 10; i++)
	  ptr2->g[i] = NULL;
      }
      ptr2->start = ptr1->start;
      ptr2->stop = ptr1->stop;
      ptr2->f = ptr1->f;
      ptr2->choice = ptr1->choice;
      if (ptr1->start) {
	ptr = ptr1->start->x->vec;
	n = 0; /* Length of start point vector. */
	while (ptr) {
	  ptr = ptr->next;
	  n++;
	}
      }
      else
	n = ptr1->start2->n2[ind];
      if (n > ptr2->storageG) {
	sf *dum;
	dum = (sf *) malloc(n * sizeof(ptr1));
	for (i = 0; i < ptr2->storageG; i++)
	  dum[i] = ptr2->g[i];
	for (i = ptr2->storageG; i < n; i++)
	  dum[i] = NULL;
	free(ptr2->g);
	ptr2->g = dum;
	ptr2->storageG = n;
      }
      if (!ptr1->choice) {
	ptr = (ptr1->start) ? ptr1->start->x->vec : NULL;
	ptrstop = (ptr1->stop) ? ptr1->stop->x->vec : NULL;
	i = 0;
	if (ptr || (ptr1->start2 && n > 0)) {
	  while (ptr || (ptr1->start2 && i < n)) {
	    /*	for (i = 0; i < ptr1->start->n; i++) { */
	    if (!ptr2->g[i])
	      ptr2->g[i] = initSf(0.0, 0.0, NULL);
	    /*	  bite_printf("%s: ", ptr1->f[0]->name);
		  printSf(ptr1->f[0]->f); */
	    switch (ptr1->f[ptr1->fc[ind]]->type) {
	    case SF_ORIG:
	      copySf(ptr1->f[ptr1->fc[ind]]->f->f, ptr2->g[i]);
	      move2Sf(ptr2->g[i], (ptr) ? paramValue(ind, ptr) :
		      covValue(ind, i, ptr1->start2));
	      break;
	    case SF_TIMELESS:
	      ptr2->g[i]->jumpPoints[0] = -INFTY;
	      ptr2->g[i]->n = 1;
	      ptr2->g[i]->levels[0] =
		paramValue(ind, ptr1->f[ptr1->fc[ind]]->f->p);
	      addNode(ptr2->g[i], 1, INFTY, 0);
	      break;
	    }
	    /*	  printSf(ptr2->g[i]); */
	    /*	  if (ptr1->op == '\0' && !ptr1->choice && ptr1->f[0]->f == target) */
	    if (ptr1->op == '\0')
	      cut2Sf(ptr2->g[0], bot, top);
	    if (ptrstop) {
	      cut2Sf(ptr2->g[i], -INFTY, ptrstop->x->x);
	      ptrstop = ptrstop->next;
	    }
	    ptr = (ptr) ? ptr->next : NULL;
	    i++;
	  }
	  /*	  if (!ptr2->f)
	    ptr2->f = initSf(0.0, 0.0, NULL);
	  copySf(ptr2->g[0], ptr1->f */
	}
	else { /* Put a null simple function. */
	  if (!ptr2->g[0])
	    ptr2->g[0] = initSf(0.0, 0.0, NULL);
	  ptr2->g[0]->n = 1;
	} 
      }
      else {
	if (!ptr2->g[0])
	  ptr2->g[0] = initSf(0.0, 0.0, NULL);
	if (!ptr2->g[1])
	  ptr2->g[1] = initSf(0.0, 0.0, NULL);
/*	bite_printf("%s choice: ", ptr1->f[0]->name);
	printSf(ptr1->choice); */
/*	for (i = 0; i < ptr1->storageF; i++) {
	  bite_printf("%ld: ", i);
	  printSf(ptr1->f[i]->f);
	} */
	ptr2->choice = ptr1->choice;
	ptr2->f = ptr1->f;
      }
/*      ptr2->sp = ptr1->start->n; */
    }
    putFt(ind, ptr1->right, ptr2->right, target, bot, top);
  }
}

ft findFt(ft ptr, mf target, mft f)
{
  long j;
  ft res = NULL, dum = NULL;

  if (!ptr->f && ptr->ptype != PT_MODELF) {
    if (ptr->left && (dum = findFt(ptr->left, target, f)))
      res = dum;
    if (!dum && ptr->right && (dum = findFt(ptr->right, target, f)))
      res = dum;
    if (!dum && ptr->nArgs && (dum = findFt(ptr->args[0], target, f)))
      res = dum;
  }
  else {
    for (j = 0; j < ptr->nArgs; j++)
      if ((dum = findFt(ptr->args[j], target, f)))
	res = dum;
    if (target) {
      for (j = 0; j < ptr->storageF; j++)
	if (ptr->f[j] == target)
	  res = ptr;
    }
    else /* If no target specified, return the node of an model */
      if (ptr->ptype == PT_MODELF || ptr->storageF) /* function argument. */
	res = ptr;
  }
  return(res);
}

void putSfToFt(long ind, long mark, mf target, long updJ, long updL, 
	       ft aim, ft tree, dt d, mft f)
{
  if ((updJ < 0 && updL < 0) || 
      (target && target->prior == JHEIKK && updJ > 0))
    putFt(ind, tree, aim, NULL, -INFTY, INFTY);
  else {
    if (target) {
      long j, below, above;
      double prec = 0.0, bot = -INFTY, top = INFTY;
      ft ptr;
      prior ptrlast;
      for (j = 0; j < d->nCars[mark]; j++)
	if (d->dList[mark][ind][j]->dData && 
	    d->dList[mark][ind][j]->precision > prec)
	  prec = d->dList[mark][ind][j]->precision;
      ptr = findFt(tree, target, f);
      if (updJ > 0) { /* Determine which intervals are necessary for */
	below = updJ - 1; /* calculating the likelihood. */
	above = updJ + 1;
      }
      else {
	below = updL;
	above = updL + 1;
      }
      if (target->prior == JHEIKK && updJ > 0) { /* Arjas-Heikkinen prior. */
	below = updJ - 1;
	above = updJ + 2;
      }
      switch (target->type) {
      case SF_ORIG:
	if (below < 0) /* Check that the endpoints of the interval are OK. */
	  below = 0;
	if (above > target->f->f->n - 1)
	  above = target->f->f->n - 1;
	ptrlast = ptr->start->x->vec;
	while (ptrlast)
	  ptrlast = ptrlast->next;
	bot = target->f->f->jumpPoints[below] + 
	  paramValue(ind, ptr->start->x->vec);
	top = target->f->f->jumpPoints[above] + paramValue(ind, ptrlast);
	if (prec) { /* If the observations are not directly a point */
	  bot -= prec; /* process, (e.g. the epilepsy data) we must widen */
	  top += 2 * prec; /* the interval. */
	}
	break;
      case SF_TIMELESS:
	bot = -INFTY;
	top = INFTY;
	break;
      }
      putFt(ind, tree, aim, NULL, bot, top);
    }
    else
      putFt(ind, tree, aim, NULL, -INFTY, INFTY);
  }
}

long returnNodeType(ft root, long *n)
{
  long type = NONE, typeV[4] = { NONE, NONE, NONE, NONE }, arg;
  long i, j, k, isInd = FALSE, isMF = FALSE, isS = FALSE;
  prior ptr = param;
  useVal target;

  if (root->ptype != PT_SEMICOL) {
    if (root->left) {
      typeV[1] = returnNodeType(root->left, n);
      if (root->right)
	typeV[2] = returnNodeType(root->right, n);
      for (i = 1; i <= 2; i++) {
	if (typeV[i] == INDIVIDUAL)
	  isInd = TRUE;
	if (typeV[i] == MODELF)
	  isMF = TRUE;
	if (typeV[i])
	  isS = TRUE;
      }
    }
    else { /* A leaf */
      if (root->function != F_INDIV) {
	for (i = 0; i < root->nArgs; i++) {
	  arg = returnNodeType(root->args[i], n);
	  if (arg == INDIVIDUAL)
	    isInd = TRUE;
	  if (arg == MODELF)
	    isMF = TRUE;
	  if (arg)
	    isS = TRUE;
	}
      }
      else
	isS = TRUE;
      i = returnType(root->varName, &j, FALSE);
      switch (i) {
      case PT_SCALAR:
	isS = TRUE;
	findAndReplaceVarNames(0, 0, root, NONE, target, NULL);
	k = 0;
	while (k < j) {
	  ptr = ptr->next;
	  k++;
	}
	break;
      case PT_COV:
      case PT_RESP:
      case PT_COV2:
      case PT_RESP2:
	isInd = TRUE;
	break;
      case PT_MODELF:
	isMF = TRUE;
	break;
      }
      if (isS && !isInd && !isMF) {
	if (ptr->ptype == PT_VECTOR) {
	  prior p = ptr;
	  i = root->ni;
	  while (i && p->ptype == PT_VECTOR) {
	    i--;
	    p = p->x->vec;
	  }
	  /*	ptr = param;
		i = 0;
		while (i < j) {
		ptr = ptr->next;
		i++;
		} */
	  if (p->ptype == PT_VECTOR) {
	    *n = 0;
	    p = p->x->vec;
	    while (p) {
	      p = p->next;
	      (*n)++;
	    }
	  }
	  else {
	    if (*n == 0)
	      *n = 1;
	  }
	}
      }
      if (isMF) {
	if (root->storageF) {
	  isMF = TRUE;
	  *n = (root->choice) ? root->choice[i]->n : root->storageF;
	}
	else {
	  if (*n == 0)
	    *n = 1;
	}
      }
      /*    if (!(root->ptype == PT_CONST && root->ptype != 0) || root->function) {
	    while (ptr && strcmp(ptr->name, root->varName))
	    ptr = ptr->next;
	    if (ptr) {
	    isS = TRUE;
	    if (ptr->ptype == PT_VECTOR) {
	    prior p = ptr->x->vec;
	    *n = 0;
	    while (p) {
	    p = p->next;
	    (*n)++;
	    }
	    }
	    }
	    else {
	    while (cov < c->n && strcmp(root->varName, c->cList[cov][0]->name))
	    cov++;
	    if (cov < c->n)
	    isInd = TRUE;
	    else {
	    while (resp < d->n &&
	    strcmp(root->varName, d->dList[resp][0][0]->name))
	    resp++;
	    if (resp < d->n)
	    isInd = TRUE;
	    else {
	    while (modelf < f->n &&
	    strcmp(root->varName, f->fList[modelf][0]->name))
	    modelf++;
	    if (modelf < f->n)
	    isS = TRUE;
	    if (root->storageF) {
	    isMF = TRUE;
	    *n = root->choice[i]->n;
	    }
	    }
	    }
	    }
	    } */
      if (root->function == F_SUM || root->function == F_PROD) {
	isMF = TRUE;
	*n = root->storageF;
	/* *n = root->choice[i]->n; */
	isInd = FALSE;
      }
      if (root->ptype == PT_SEMICOL)
	isMF = isInd = FALSE;
    }
  }
  else
    isS = TRUE;
  /* Determine the type of the node: */
  if (isInd && isMF)
    type = INDIVIDUAL + MODELF;
  else {
    if (isInd)
      type = INDIVIDUAL;
    else {
      if (isMF)
	type = MODELF;
      else
	if (isS)
	  type = SCALAR;
    }
  }
  return type;
}

long returnNodeType2(ft root)
{
  long type = NONE, type2,
    i, j;

  if (root->left) {
    type = returnNodeType2(root->left);
    if (root->right) {
      type2 = returnNodeType2(root->right);
      if ((type == INDIVIDUAL && type2 == MODELF) ||
	  (type2 == INDIVIDUAL && type == MODELF) || type > 4 || type2 > 4)
	type = INDIVIDUAL + MODELF;
      else
        type = (type > type2) ? type : type2;
    }
  }
  else { /* A leaf */
    i = returnType(root->varName, &j, FALSE);
    switch (i) {
    case PT_SCALAR:
      type = SCALAR;
      break;
    case PT_COV:
    case PT_RESP:
    case PT_COV2:
    case PT_RESP2:
      type = INDIVIDUAL;
      break;
    case PT_MODELF:
      type = MODELF;
      break;
    }
  }
  return type;
}

long returnDiracPriorType(prior p)
{
  long type = NONE;

  if (p->type == D_DIRAC)
    type = returnNodeType2(p->args[0]);
  return type;
}

/* The following is used in 'expectation' (especially for 'PT_MODELF', but
   also for parameter vectors 'PT_VECTOR'). */
  
void selectOneMF(long i, ft root, long init)
{
  static long n;
  long j;
  /*  prior p; */

  if (init) {
    n = 0;
  }
  else {
    if (root->left) {
      selectOneMF(i, root->left, FALSE);
      if (root->right)
	selectOneMF(i, root->right, FALSE);
    }
    else {
      if (root->storageF) {
	root->ptype = PT_MODELF;
	if (root->choice) {
	  root->var->m = root->f[(long) root->choice[i]->levels[n]];
	  root->storageF = 0;
	  delSf(root->choice[i]);
	  root->choice[i] = NULL;
	  root->f = NULL;
	  findInd(ind, model, root->var->m, f);
	}
	else {
	  if (i < root->storageF) {
	    root->var->m = root->f[(root->fc) ? root->fc[i] : i];
	    findInd(ind, model, root->f[(root->fc) ? root->fc[i] : i], f);
	  }
	}
	n++;
      }
      for (j = 0; j < root->nArgs; j++)
	selectOneMF(i, root->args[j], FALSE);
    }
  }
}

void selectOneMF2(ft root, long init)
     /* This one for vectors. */
{
  static long n;
  long i;
  prior p;

  if (init) {
    n = 0;
  }
  else {
    if (root->left) {
      selectOneMF2(root->left, FALSE);
      if (root->right)
	selectOneMF2(root->right, FALSE);
    }
    else {
      if (root->ptype == PT_VECTOR && !root->ni) {
	/* p = root->var->vec;
	for (i = 0; i < n; i++)
	  p = p->next; */
	p = root->var->vec->up->vector[n];
	root->var->p = p;
	root->ptype = PT_SCALAR;
	n++;
      }
      for (i = 0; i < root->nArgs; i++)
	selectOneMF2(root->args[i], FALSE);
    }
  }
}

void putSfToFt2(long ind, long mark, mf target, double below, double above,
	        ft aim, ft tree, dt d, mft f)
{
  long k, j;
  double prec = 0.0, bot, top;
  ft ptr;
  prior ptrlast;

  if (target) {
    for (j = 0; j < d->nCars[mark]; j++)
      if (!d->pList[mark] && d->dList[mark][ind][j]->dData && 
	  d->dList[mark][ind][j]->precision > prec)
	prec = d->dList[mark][ind][j]->precision;
    ptr = findFt(tree, target, f);
    if (ptr->start && ptr->start->x->vec) {
      ptrlast = ptr->start->x->vec;
      while (ptrlast && ptrlast->next)
	ptrlast = ptrlast->next;
      bot = below + paramValue(ind, ptr->start->x->vec);
      top = above + paramValue(ind, ptrlast);
      if (prec) {   /* If the observations are not directly a point process, */
	bot -= prec; /* (e.g. the epilepsy data) we must widen the interval. */
	top += 2 * prec;
      }
    }
    else {
      if (ptr->start2) {
	long noDiscr3 = TRUE;
	bot = below + covValue(ind, 0, ptr->start2);
	top = above + covValue(ind, ptr->start2->n2[ind] - 1, ptr->start2);
	for (j = 0; j < d->nCars[mark]; j++) {
	  if (!d->pList[mark] && d->dList[mark][ind][j]->type == DISCR3) {
	    noDiscr3 = FALSE;
	    if (d->dList[mark][ind][j]->end > bot) {
	      k = d->dList[mark][ind][j]->nObs - 1;
	      while (k && d->dList[mark][ind][j]->times[k - 1] > bot)
		k--;
	      if (d->dList[mark][ind][j]->nObs < 2 ||
		  d->dList[mark][ind][j]->times[0] > bot)
		bot = d->dList[mark][ind][j]->begin;
	      else
		bot = d->dList[mark][ind][j]->times[k];
	    }
	    if (d->dList[mark][ind][j]->begin < top) {
	      k = 0;
	      while (k < d->dList[mark][ind][j]->nObs - 2 &&
		     d->dList[mark][ind][j]->times[k] < top)
		k++;
	      if (d->dList[mark][ind][j]->nObs < 2 ||
		  d->dList[mark][ind][j]->times[d->dList[mark][ind][j]->nObs - 2] < top)
		top = d->dList[mark][ind][j]->end;
	      else
		top = d->dList[mark][ind][j]->times[k];
	    }
	  }
	}
	if (prec && noDiscr3) {
	  bot -= prec;
	  top += 2 * prec;
	}
      }
      else {
	bot = 0.0;
	top = 0.0;
      }
    }
    putFt(ind, tree, aim, NULL, bot, top);
  }
  else
    putFt(ind, tree, aim, NULL, below, above);
}

sf calculate(long ind, ft ptr)
  /* Calculate the value of a simple function formula tree. */
{
  sf left, right;
  mf ff;
  prior ptrsp;

  if (ptr->left) {
    long i;
    left = calculate(ind, ptr->left);
    right = calculate(ind, ptr->right);
    if (!ptr->g) {
      ptr->storageG = 10;
      ptr->g = (sf *) malloc(10 * sizeof(ptr));
      for (i = 0; i < 10; i++)
	ptr->g[i] = NULL;
    }
    switch (ptr->op) {
    case '*':
      ptr->g[0] = multSf(left, right, ptr->g[0]);
      break;
    case '+':
      ptr->g[0] = addSf(left, right, ptr->g[0]);
      break;
    case '_': /* RS Changes pykala to _ */
      ptr->g[0] = multSf2(left, right, ptr->g[0]);
      break;
    }
  }
  else {
    if (ptr->function) {
      long i;
      ptr->g[0] = calculate(ind, ptr->args[0]);
      for (i = 0; i < ptr->g[0]->n; i++)
	switch (ptr->function) {
	case F_EXP:
	  ptr->g[0]->levels[i] = exp(ptr->g[0]->levels[i]);
	  break;
	default:
	  bite_eprintf( "Unknown function, line %ld.\n", ptr->lineno);
	  bite_exit(1);
	}
    }
    else {
      long i;
      if (!ptr->choice) {
	if (ptr->start) {
	  ptrsp = ptr->start->x->vec;
	  if (ptrsp) {
	    if (ptrsp && ptrsp->next) {
	      ptrsp = ptrsp->next;
	      i = 1;
	      while (ptrsp) {
		/*    for (i = 1; i < ptr->sp; i++) */
		opSf(ptr->f[0]->internOp, ptr->g[0], ptr->g[i], FALSE);
		ptrsp = ptrsp->next;
		i++;
	      }
	    }
	  }
	  else {
	    ptr->g[0]->n = 1;
	  }
	}
	else {
	  ff = (ptr->fc) ? ptr->f[ptr->fc[ind]] : ptr->f[0];
	  if (ptr->start2->n2[ind])
	    for (i = 1; i < ptr->start2->n2[ind]; i++)
	      opSf(ff->internOp, ptr->g[0], ptr->g[i], FALSE);
	  else
	    ptr->g[0]->n = 1;
	}
      }
      else {
	if (ptr->choice) {
	  /*      bite_printf("\n");
		  printSf(ptr->choice[i]); */
	  ptr->g[1] = cutSf(ptr->f[(long)ptr->choice[ind]->levels[0]]->f->f,
			    ptr->choice[ind]->jumpPoints[0] - 
			    ((ptr->start) ? ptr->start->x->vec->x->x :
			     ptr->start2->val.x2[ind][0]),
			    ptr->choice[ind]->jumpPoints[1] - 
			    ((ptr->start) ? ptr->start->x->vec->x->x :
			     ptr->start2->val.x2[ind][0]), ptr->g[1]);
	  copySf(ptr->g[1], ptr->g[0]);
	  /*      printSf(ptr->f[(long)ptr->choice[i]->levels[0]]->f);
		  printSf(ptr->g[0]); */
	  for (i = 1; i < ptr->choice[ind]->n - 1; i++) {
	    ptr->g[1] = cutSf(ptr->f[(long)ptr->choice[ind]->levels[i]]->f->f,
			      ptr->choice[ind]->jumpPoints[i] - 
			      ((ptr->start) ? ptr->start->x->vec->x->x :
			       covValue(ind, 0, ptr->start2)),
			      ptr->choice[ind]->jumpPoints[i + 1] - 
			      ((ptr->start) ? ptr->start->x->vec->x->x :
			       covValue(ind, 0, ptr->start2)), ptr->g[1]);
	    /*	bite_printf("%ld %ld\n", i, (long)ptr->choice[i]->levels[i]);
		printSf(ptr->f[(long)ptr->choice[i]->levels[i]]->f);
		printSf(ptr->g[1]); */
	    opSf('+', ptr->g[0], ptr->g[1], FALSE);
	    /*	printSf(ptr->g[0]);
		bite_printf(""); */
	  }
	  move2Sf(ptr->g[0], (ptr->start) ? ptr->start->x->vec->x->x :
		  covValue(ind, 0, ptr->start2));
	}
	/*      else {
		if (ptr->factor) {
		if (ptr->factor->x->x->f)
		copySf(ptr->factor->x->x->f, ptr->g[0]);
		else {
		ptr->g[0]->n = 2;
		ptr->g[0]->jumpPoints[0] = -1.0e15;
		ptr->g[0]->jumpPoints[1] = 1.0e15;
		ptr->g[0]->levels[0] = ptr->factor->x->x->x;
		}
		}
		} */
      }
    }
  }
  return(ptr->g[0]); 
}

double paramValue(long ind, prior p)
{
  double res = 0.0;

  level++;
  lindex[level] = p->index;
  switch (p->ptype) {
  case PT_CONST:
    if (p->type == D_DIRAC && !p->fixed)
      res = calculate2(ind, p->args[0]);
    else
      res = p->x->x;
    break;
  case PT_SCALAR:
    res = paramValue(ind, p->x->p);
    break;
  case PT_MIX:
    res = p->x->mix.x;
    break;
  case PT_VECTOR:
    res = paramValue(ind, p->x->vec);
    break;
  default:
    bite_eprintf( "%s: parameter is not scalar or a vector.\n", p->name);
    bite_exit(1);
  }
  level--;
  return res;
}

double logPrior(long i, ft ptr, ft ptr1, ft ptr2)
{
  double res = -INFTY;
  mf target;

  switch (ptr->ptype) {
  case PT_SCALAR:
    res = returnPriorLogDensity(i, ptr->var->p,
				paramValue(i, ptr->var->p));
    break;
  case PT_VECTOR:
    {
      long n, idx = 0;
      prior pp = ptr->var->vec;
      while (!pp->type) {
	pp = pp->x->vec;
	if (idx == ptr->ni && !ptr->var->vec->type) {
	  bite_eprintf( "Not enough indices for logprior(%s).\n",
		  ptr->varName);
	  bite_exit(1);
	}
	n = (ptr->ni) ? calculate2(i, ptr->i[idx]) : 0;
	level++;
	lindex[level] = n;
	/* j = 0;
	   while (j++ < n && pp)
	   pp = pp->next; */
	pp = pp->up->vector[n];
	idx++;
      }
      res = returnPriorLogDensity(i, pp, paramValue(i, pp));
      level -= idx;
    }
    break;
  case PT_RESP2:
    {
      long mark = 0;
      while (d->pList[mark] != ptr->var->pvec)
	mark++;
      res = indLogL(i, mark, NULL,
		    calculate2(i, ptr1),
		    calculate2(i, ptr2), d, model);
    }
    break;
  case PT_COV2:
    switch (ptr->var->pvec->type) {
    case COV_CNST2:
      res =
	returnPriorLogDensity(i,
			      ptr->var->pvec->distr.x,
			      ptr->var->pvec->val.x[i]);
      break;
    case COV_START2:
      {
	long m = calculate2(i, ptr->i[1]);
	level++;
	lindex[level] = m;
	if (m >= ptr->var->pvec->n2[i]) {
	  bite_eprintf( "%ld: Not so many start points "
		  "for individual %ld.\n", m, i);
	  bite_exit(1);
	}
	res =
	  returnPriorLogDensity(i,
				ptr->var->pvec->distr.x2[m],
				ptr->var->pvec->val.x2[i][m]);
	level--;
      }
      break;
    }
    break;
  case PT_MODELF:
    if (ptr->storageF > 1 &&
	(i < 0 || i >= ptr->storageF)) {
      bite_eprintf( "The model function list %s(...) does "
	      "not have element %ld. Only %ld elements available. "
	      "(Line %ld.)\n",
	      ptr->varName, i, ptr->storageF,
	      ptr->lineno);
      bite_exit(1);
    }
    target = (ptr->fc && ptr->storageF > 1) ?
      ptr->f[i] : ptr->var->m;
    res = returnMfLogPriorDensity(i, target);
    break;
  default:
    bite_eprintf( "No such parameter type for ");
    bite_eprintf( "\"logprior\" on line %ld.\n", lineno);
    bite_exit(1);
  }
  return res;
}

double logContrib(long i, ft ptr)
{
  /*long n, m = 0, idx, mark, cov; */
  double res = -INFTY;
  prior p;
  /*use use_list = NULL;*/

  switch (ptr->ptype) {
  case PT_SCALAR:
    if (ptr->ni)
      p = ptr->var->p->vector[(long) calculate2(-1, ptr->i[0])];
    else
      p = ptr->var->p;
    res = logLikel3(i, p->where, p->indiv_likel);
    break;
  case PT_VECTOR:
    {
      long n, idx = 0;
      prior pp = ptr->var->vec;
      while (!pp->type) {
	pp = pp->x->vec;
	if (idx == ptr->ni) {
	  bite_eprintf( "Not enough indices for logprior(%s).\n",
		  ptr->varName);
	  bite_exit(1);
	}
	n = calculate2(i, ptr->i[idx]);
	level++;
	lindex[level] = n;
	/* j = 0;
	   while (j++ < n && pp)
	   pp = pp->next; */
	pp = pp->up->vector[n];
	idx++;
      }
      res = logLikel3(i, pp->where, pp->indiv_likel);
      level -= idx;
    }
    break;
  case PT_RESP2:
    res = logLikel3(i, ptr->var->p->where,
		    ptr->var->p->indiv_likel);
    break;
  case PT_COV2:
    switch (ptr->var->pvec->type) {
    case COV_CNST2:
      res = logLikel3(i, ptr->var->pvec->distr.x->where,
		      ptr->var->pvec->distr.x->indiv_likel);
      break;
    case COV_START2:
      {
	long m = calculate2(i, ptr->i[0]);
	level++;
	lindex[level] = m;
	if (m >= ptr->var->pvec->n2[i]) {
	  bite_eprintf( "%ld: Not so many start points for individual %ld.\n", m, i);
	  bite_exit(1);
	}
	res = logLikel3(i, ptr->var->pvec->distr.x2[m]->where,
			ptr->var->pvec->distr.x2[m]->indiv_likel);
	level--;
      }
      break;
    }
    break;
  default:
    bite_eprintf( "No such parameter type for ");
    bite_eprintf( "\"logprior\" on line %ld.\n", lineno);
    bite_exit(1);
  }
  return res;
}

void setValue(long i, ft ptr, double value)
{
  long n, m = 0;
  mf target;
  use use_list = NULL;

  /* bite_printf("Set %s = %g;\n", ptr->left->varName, x); */
  if (ptr->function == F_INDIV) {
    long cars;
    n = calculate2(i, ptr->args[1]);
#ifdef DEBUG
    bite_printf("=%g; ", value);
#endif
    switch (ptr->args[0]->ptype) {
    case PT_COV2:
      switch (ptr->args[0]->var->pvec->type) {
      case COV_CNST2:
	ptr->args[0]->var->pvec->val.x[n] = value;
	use_list = 
	  (ptr->args[0]->var->pvec->distr.x &&
	   ptr->args[0]->var->pvec->distr.x->where) ?
	  ptr->args[0]->var->pvec->distr.x->where : NULL;
	break;
      case COV_START2:
	m = calculate2(n, ptr->args[0]->i[0]);
	if (m >= ptr->args[0]->var->pvec->n2[n]) {
	  bite_eprintf( "%ld: Not so many start points for individual %ld.\n", m, n);
	  bite_exit(1);
	}
	use_list = (ptr->args[0]->var->pvec->distr.x2[m]) ? 
	  ptr->args[0]->var->pvec->distr.x2[m]->where : NULL;
	ptr->args[0]->var->pvec->val.x2[n][m] = value;
	break;
      }
      break;
    case PT_RESP2:
      cars = calculate2(n, ptr->args[0]->i[0]);
      switch (ptr->args[0]->fieldId) {
      case T0:
	ptr->args[0]->var->pvec->val.pp.t0[n][cars] = value;
	use_list = (ptr->args[0]->var->pvec->distr.pp.t0[m]) ? 
	  ptr->args[0]->var->pvec->distr.pp.t0[m]->where : NULL;
	break;
      case T1:
	ptr->args[0]->var->pvec->val.pp.t1[n][cars] = value;
	use_list = (ptr->args[0]->var->pvec->distr.pp.t1[m]) ? 
	  ptr->args[0]->var->pvec->distr.pp.t1[m]->where : NULL;
	break;
      case POINTS:
	m = calculate2(n, ptr->args[0]->i[1]);
	if (m >= ptr->args[0]->var->pvec->n2[n]) {
	  bite_eprintf( "%ld: Not so many event times on the observation interval %ld for individual %ld.\n",
		  m, cars, n);
	  bite_exit(1);
	}
	use_list = (ptr->args[0]->var->pvec->distr.pp.points[cars][m]) ? 
	  ptr->args[0]->var->pvec->distr.pp.points[cars][m]->where : NULL;
	ptr->args[0]->var->pvec->val.pp.points[n][cars][m] = value;
	break;
      }
      break;
    case PT_MODELF:
      if (ptr->args[0]->storageF > 1 &&
	  (n < 0 || n >= ptr->args[0]->storageF)) {
	bite_eprintf( "The model function list %s(...) does "
		"not have element %ld. Only %ld elements available. "
		"(Line %ld.)\n",
		ptr->args[0]->varName, n,
		ptr->args[0]->storageF,
		ptr->args[0]->lineno);
	bite_exit(1);
      }
      target = (ptr->args[0]->fc &&
		ptr->args[0]->storageF > 1) ?
	ptr->args[0]->f[n] : ptr->args[0]->var->m;
      if (target->type == SF_ORIG) {
	long idx = 0;
	switch (ptr->args[0]->fieldId) {
	case LEVEL:
	  idx = calculate2(i, ptr->args[0]->third->i[0]);
	  if (idx < 0 || idx >= target->f->f->n - 1) {
	    bite_eprintf( "The model function %s does "
		    "not have %ld\'th level. Only %ld elements available. "
		    "(Line %ld.)\n",
		    target->name, idx, target->f->f->n - 1,
		    ptr->args[0]->lineno);
	    bite_exit(1);
	  }
	  target->f->f->levels[idx] = value;
	  break;
	case JUMPPOINT:
	  idx = calculate2(i, ptr->args[0]->third->i[0]);
	  target->f->f->jumpPoints[idx] = value;
	  break;
	case N_POINTS:
	  break;
	case NONE: /* Copy the simple function. */
	  if (ptr->right->function == F_INDIV)
	    copySf((ptr->right->args[0]->fc &&
		    ptr->right->args[0]->storageF > 1) ?
		   ptr->right->args[0]->f[(long) calculate2(i, ptr->right->args[1])]->f->f
		   : ptr->right->args[0]->var->m->f->f, target->f->f);
	  else
	    copySf(ptr->right->var->m->f->f, target->f->f);
	  break;
	}
      }
      else {
	bite_eprintf( "Incorrect model function types for "
		"assignment %s=%s;\n", ptr->args[0]->varName,
		ptr->right->varName);
	bite_exit(1);
      }
      break;
    default:
      use_list = NULL;
    }
  }
  else {
    n = 0;
#ifdef DEBUG
    bite_printf("%s=%g; ", ptr->var->p->name, value);
#endif
    switch (ptr->ptype) {
    case PT_SCALAR:
      use_list = ptr->var->p->where;
      ptr->var->p->x->x = value;
      break;
    case PT_VECTOR:
      {
	long n = calculate2(i, ptr->i[0]);
	prior p;
	if (n < 0 || n >= ptr->var->vec->stor) {
	  bite_eprintf(
		  "Vector %s does not have element %ld (line %ld).\n",
		  ptr->varName, n, ptr->lineno);
	  bite_exit(1);
	}
	p = ptr->var->vec->vector[n];
	/* prior p = ptr->var->vec->x->vec;
	   j = 0;
	   while (j++ < n && p)
	   p = p->next; */
	use_list = p->where;
	p->x->x = value;
      }
      break;
    case PT_MODELF:
      if (ptr->var->m->type == SF_ORIG &&
	  !ptr->fieldId) {
	/* Copy the whole simple function: */
	if (ptr->right->function == F_INDIV)
	  copySf((ptr->right->args[0]->fc &&
		  ptr->right->args[0]->storageF > 1) ?
		 ptr->right->args[0]->f[(long) calculate2(i, ptr->right->args[1])]->f->f
		 : ptr->right->args[0]->var->m->f->f, ptr->var->m->f->f);
	else
	  copySf(ptr->right->var->m->f->f, ptr->var->m->f->f);
      }
      else {
	/* Set the value of a component of a simple function: */
	mf target;
	if (ptr->storageF > 1) {
	  if (i < 0 || i >= ptr->storageF) {
	    bite_eprintf( "The model function list %s(...) does "
		    "not have element %ld. Only %ld elements available. "
		    "(Line %ld.)\n",
		    ptr->varName, i, ptr->storageF,
		    ptr->lineno);
	    bite_exit(1);
	  }
	  target = ptr->f[i];
	}
	else
	  target = ptr->var->m;
	if (target->type == SF_ORIG) {
	  long idx = 0;
	  switch (ptr->fieldId) {
	  case LEVEL:
	    idx = calculate2(i, ptr->third->i[0]);
	    target->f->f->levels[idx] = value;
	    break;
	  case JUMPPOINT:
	    idx = calculate2(i, ptr->third->i[0]);
	    target->f->f->jumpPoints[idx] = value;
	    break;
	  case N_POINTS:
	    break;
	  }
	}
	else {
	  bite_eprintf( "Incorrect model function types for "
		  "assignment %s=%s;\n", ptr->varName,
		  ptr->right->varName);
	  bite_exit(1);
	}
      }
      use_list = NULL;
      break;
    default:
      use_list = NULL;
      break;
    }
  }
  if (use_list) {
    long mark;
    /* If the covariate is used in a hazard function, then */
    use u = use_list;
    while (u) {
      switch (u->type) {
      case USE_TABLE:
	for (mark = 0; mark < d->n; mark++)
	  if (u->val->inUse[mark])
	    checkCovs(model->ftList[mark][0], mark, n, c, f);
	break;
      case USE_INDPARAM:
	if (u->val->indParam.p->where &&
	    u->val->indParam.p->where->type == USE_TABLE) {
	  for (mark = 0; mark < d->n; mark++)
	    if (u->val->indParam.p->where->val->inUse[mark])
	      checkCovs(model->ftList[mark][0], mark,
			u->val->indParam.i, c, f);
	}
	break;
      }
      u = u->next;
    }
  }
}

double calculate2(long i, ft ptr)
     /* Calculate the value of a scalar formula tree. */
{
  long j, n; /* , idx, mark, cov; */
  double res = 0.0, x;
  mf target;
  useVal targetU;

  if (ptr) {
#ifdef DEBUG
    bite_printf(" ");
#endif
    if (ptr->left) {
      if (ptr->op  != NONE)
	res = calculate2(i, ptr->left);
#ifdef DEBUG
      bite_printf("[=%g] ", res);
      if ((long) ptr->op < 256)
	bite_printf("%c ", (char) ptr->op);
      else
	bite_printf("%ld ", ptr->op);
#endif
      switch (ptr->op) {
      case (long) '*':
	res *= calculate2(i, ptr->right);
	break;
      case (long) '+':
	res += calculate2(i, ptr->right);
	break;
      case (long) '/':
	res /= calculate2(i, ptr->right);
	break;
      case (long) '-':
	if (ptr->right)
	  res -= calculate2(i, ptr->right);
	else
	  res = -res;
	break;
      case (long) '^':
	res = pow(res, calculate2(i, ptr->right));
	break;
      case (long) '?':
	if (calculate2(i, ptr->third))
	  res = calculate2(i, ptr->left);
	else
	  res = calculate2(i, ptr->right);
	break;
      case LT:
	res = (res < calculate2(i, ptr->right)) ? 1.0 : 0.0;
	break;
      case GT:
	res = (res > calculate2(i, ptr->right)) ? 1.0 : 0.0;
	break;
      case LE:
	res = (res <= calculate2(i, ptr->right)) ? 1.0 : 0.0;
	break;
      case GE:
	res = (res >= calculate2(i, ptr->right)) ? 1.0 : 0.0;
	break;
      case EQ:
	res = (res == calculate2(i, ptr->right)) ? 1.0 : 0.0;
	break;
      case NE:
	res = (res != calculate2(i, ptr->right)) ? 1.0 : 0.0;
	break;
      case AND:
	res = (res && calculate2(i, ptr->right)) ? 1.0 : 0.0;
	break;
      case OR:
	res = (res || calculate2(i, ptr->right)) ? 1.0 : 0.0;
	break;
      default:
	/* bite_eprintf( "Unknown operator %ld.\n", ptr->op);
	   bite_exit(1); */
	switch (ptr->ptype) {
	case PT_SEMICOL:
	  if (ptr->right) {
	    res = calculate2(i, ptr->left);
	    if (!res)
	      res = calculate2(i, ptr->right);
	    else
	      calculate2(i, ptr->right);
	  }
	  else { /* An inline function */
	    int j;
	    prior p = param, newP;
	    if (ptr->nArgs) {
	      while (p && p->next)
		p = p->next;
	      for (j = 0; j < ptr->nArgs; j++) {
		newP = initPrior(ptr->args[j]->varName);
		newP->fixed = TRUE;
		newP->ptype = PT_CONST;
		if (param) {
		  p->next = newP;
		  newP->prev = p;
		  p = p->next;
		}
		else
		  param = p = newP;
	      }
	      findAndReplaceVarNames(0, 0, ptr->left, NONE, targetU, NULL);
	    }
	    res = calculate2(i, ptr->left);
	    if (ptr->nArgs) {
	      for (j = 0; j < ptr->nArgs; j++) {
		p = p->prev;
		rmPrior(p->next);
		p->next = NULL;
	      }
	    }
	  }
	  break;
	case PT_SET:
	  setValue(i, ptr->left,
		   (ptr->right->ptype == PT_MODELF && !ptr->right->fieldId) ?
		   0.0 : calculate2(i, ptr->right));
	  break;
	case PT_IF:
#ifdef DEBUG
	  bite_printf("If (");
#endif
	  if (calculate2(i, ptr->left)) {
#ifdef DEBUG
	    bite_printf(") ");
#endif
	    calculate2(i, ptr->right);
#ifdef DEBUG
	    bite_printf("\n");
#endif
	  }
	  else {
#ifdef DEBUG
	    bite_printf(") ");
#endif
	    calculate2(i, ptr->third);
#ifdef DEBUG
	    bite_printf("\n");
#endif
	  }
	  break;
	case PT_WHILE:
	  while (calculate2(i, ptr->left))
	    calculate2(i, ptr->right);
	  break;
	case PT_SWITCH:
	  j = (long) calculate2(i, ptr->left);
	  if (0 < j && j <= ptr->nArgs)
	    res = calculate2(i, ptr->args[j - 1]);
	  break;
	case PT_FOR:
#ifdef DEBUG
	  bite_printf("For ...\n");
#endif
	  for (calculate2(i, ptr->args[0]); calculate2(i, ptr->args[1]);
	       calculate2(i, ptr->args[2]))
	    calculate2(i, ptr->left);
	  break;
	case PT_RETURN:
	  res = calculate2(i, ptr->left);
	  break;
	case PT_ECHO:
	  {
	    int ii = 0;
	    for (; ii < ptr->nArgs; ii++) {
	      if (ptr->args[ii]->ptype == PT_STRING)
		bite_printf("%s ", ptr->args[ii]->var->str);
	      else
		bite_printf("%g ", calculate2(i, ptr->args[ii]));
	    }
	    bite_printf("\n");
	    /* fflush(NULL); */
	  }
	  break;
	case PT_CUT:
	  {
	    int ii = 2;
	    prior p;
	    if (ptr->args[0]->ptype == SCALAR)
	      p = ptr->args[0]->var->p;
	    else
	      p = ptr->args[0]->var->vec->vector[(long) calculate2(i, ptr->args[0]->i[0])];
	    for (; ii < ptr->nArgs; ii++) {
	      copyFt(ptr->args[1], p->cut);
	      if (ptr->args[ii]->ptype == PT_STRING)
		bite_printf("%s ", ptr->args[ii]->var->str);
	      else
		bite_printf("%g ", calculate2(i, ptr->args[ii]));
	    }
	    bite_printf("\n");
	  }
	  break;
	}
	break;
      }
    }
    else {
      if (ptr->function) {
	long mark = 0, car = 0;
	paramVector r;
	double x1;
	sf lambda;
#ifdef DEBUG
	bite_printf("Function %ld ( ", ptr->function);
#endif
	switch (ptr->function) {
	case F_THIS:
	  res = lindex[level - (long) calculate2(i, ptr->args[0])];
	  break;
	case D_DIRAC:
	  res = calculate2(i, ptr->args[0]);
	  break;
	case D_UNIF:
	  res = ran_uni(calculate2(i, ptr->args[0]),
			calculate2(i, ptr->args[1]));
	  break;
	case D_NORM:
	  res = calculate2(i, ptr->args[0]) +
	    ran_norm() * sqrt(calculate2(i, ptr->args[1]));
	  break;
	case D_GAMMA:
	  res = ran_gamma(calculate2(i, ptr->args[0]),
			  calculate2(i, ptr->args[1]));
	  break;
	case D_STUDENT:
	  res = ran_student(calculate2(i, ptr->args[0]),
			    calculate2(i, ptr->args[1]),
			    calculate2(i, ptr->args[2]));
	  break;
	case D_DUNIF:
	  res = (long) ran_uni(calculate2(i, ptr->args[0]),
			       calculate2(i, ptr->args[1])+1);
	  break;
	case D_DISCR:
	  {
	    int i = -1;
	    double sum = 0.0, a = ran_uni(0.0, 1.0);
	    do {
	      i += 2;
	    } while (i < ptr->nArgs &&
		     (sum += calculate2(ind, ptr->args[i])) && sum < a);
	    if (i < ptr->nArgs)
	      res = calculate2(ind, ptr->args[i - 1]);
	    else
	      bite_eprintf( "Generating \'discrete\', but the probabilities do not seem to sum up to one!\n");
	  }
	  break;
	case DD_BIN:
	case DD_DISCR:
	case DD_UNIF:
	case DD_NORM:
	case DD_GAMMA:
	case DD_STUDENT:
	  res = 
	    returnPriorLogDensity2(i, calculate2(i, ptr->args[0]),
				   ptr->function - 30, ptr->nArgs - 1,
				   &(ptr->args[1]));
	  break;
	case F_LOG:
	  res = log(calculate2(i, ptr->args[0]));
	  break;
	case F_EXP:
	  res = exp(calculate2(i, ptr->args[0]));
	  break;
	case F_PROBIT:
	  res = probit(calculate2(i, ptr->args[0]),
		       calculate2(i, ptr->args[1]),
		       calculate2(i, ptr->args[2]));
	  break;
	case F_FLOOR:
	  x = calculate2(i, ptr->args[0]);
	  res = (x < -0xfffffffl) ? -0xfffffffl :
	    (x < 0xfffffffl) ? (long) x : 0xfffffffl;
	  break;
	case F_MIN:
	  {
	    int ii = 0;
	    res = INFTY;
	    for (; ii < ptr->nArgs; ii++)
	      res = ((x1 = calculate2(i, ptr->args[ii])) < res) ? x1 : res;
	  }
	  break;
	case F_MAX:
	  {
	    int ii = 0;
	    res = -INFTY;
	    for (; ii < ptr->nArgs; ii++)
	      res = ((x1 = calculate2(i, ptr->args[ii])) > res) ? x1 : res;
	  }
	  break;
	case F_GAMMA:
	  res = lgamma(calculate2(i, ptr->args[0]));
	  break;
	case F_VALUE:
	  switch (ptr->args[0]->ptype) {
	  case PT_RESP:
	    while (mark < d->n && ptr->args[0]->var->resp != d->dList[mark])
	      mark++;
	    bite_eprintf( "Missing code at ftree.c:2914.\n");
	    break;
	  case PT_RESP2:
	    while (d->pList[mark] != ptr->args[0]->var->pvec)
	      mark++;
	    if (ptr->args[0]->fieldId == INTENSITY) {
	      /* putSfToFt2(i, mark, NULL, -INFTY, INFTY, 
		 model->ftList[mark][ind], model->ftList[mark][i], 
		 d, f);
		 lambda = calculate(i, model->ftList[mark][ind]); */
	      putSfToFt2(i, mark, NULL, -INFTY, INFTY, 
			 model->ftList[mark][0], model->ftList[mark][0], 
			 d, f);
	      lambda = calculate(i, model->ftList[mark][0]);
	      res = value(lambda, calculate2(i, ptr->args[1]));
	    }
	    else {
	      bite_eprintf( "Only intensity available (%s).\n", 
		      ptr->args[0]->varName);
	      bite_exit(1);
	    }
	    break;
	  case PT_MODELF:
	    switch (ptr->args[0]->var->m->type) {
	    case SF_ORIG:
	      res = value((ptr->args[0]->storageF < 2 || !ptr->args[0]->fc) ?
			  ptr->args[0]->var->m->f->f :
			  ptr->args[0]->f[ptr->args[0]->fc[i]]->f->f,
			  calculate2(i, ptr->args[1]));
	      break;
	    case SF_TIMELESS:
	      res = paramValue(i, (ptr->args[0]->storageF < 2 || !ptr->args[0]->fc) ?
			       ptr->args[0]->var->m->f->p :
			       ptr->args[0]->f[ptr->args[0]->fc[i]]->f->p);
	      break;
	    }
	    break;
	  default:
	    bite_eprintf( "No such data type for \"value\" (%s).\n", 
		    ptr->args[0]->varName);
	    bite_exit(1);
	  }
	  break;
	case F_LOGLIKE:
	  switch (ptr->args[0]->ptype) {
	  case PT_RESP:
	    res = indLogL(i, ptr->args[0]->var->resp[0][0]->mark, NULL,
			  calculate2(i, ptr->args[1]),
			  calculate2(i, ptr->args[2]), d, model);
	    break;
	  case PT_RESP2:
	    {
	      long mark = 0;
	      while (d->pList[mark] != ptr->args[0]->var->pvec)
		mark++;
	      if (i < 0 || i >= ind) {
		bite_eprintf( "Individual %ld not found, can be at most %ld. Line %ld.\n",
			i, ind - 1, ptr->lineno);
		bite_exit(1);
	      }
	      res = indLogL(i, mark, NULL,
			    calculate2(i, ptr->args[1]),
			    calculate2(i, ptr->args[2]), d, model);
	    }
	    break;
	  default:
	    /* bite_eprintf( "No such parameter type for ");
	       bite_eprintf( "\"loglikelihood\" on line %ld.\n", lineno);
	       bite_exit(1); */
	    {
	      long i, mark;
	      for (mark = 0; mark < d->n; mark++)
		for (i = 0; i < ind; i++)
		  res += indLogL(i, mark, NULL, -INFTY, INFTY, d, model);
	    }
	  }
	  break;
	case F_PPPOINT:
	  while (d->pList[mark] != ptr->args[0]->var->pvec)
	    mark++;
	  if (ptr->args[0]->fieldId == INTENSITY) {
	    putSfToFt2(i, mark, NULL, -INFTY, INFTY, 
		       model->ftList[mark][0], model->ftList[mark][0],
		       d, f);
	    lambda = calculate(i, model->ftList[mark][0]);
	    res = ran_poiPr(lambda, calculate2(i, ptr->args[1]),
			    (ptr->args[2]) ? calculate2(i, ptr->args[2]) :
			    -INFTY);
	  }
	  else {
	    bite_eprintf( "Only hazard available (%s).\n", 
		    ptr->args[0]->varName);
	    bite_exit(1);
	  }
	  break;
	case F_LOGPRIOR:
	  if (ptr->nArgs == 1)
	    res = logPrior(i, ptr->args[0], NULL, NULL);
	  if (ptr->nArgs == 3)
	    res = logPrior(i, ptr->args[0], ptr->args[1], ptr->args[2]);
	  break;
	case F_LOGCONTR:
	  if (ptr->nArgs == 1)
	    res = logContrib(i, ptr->args[0]);
	  if (ptr->nArgs == 3)
	    res = logContrib(i, ptr->args[0]);
	  break;
	case F_SUM:
	  res = 0.0;
	  if (ptr->f)
	    target = ptr->f[i];
	  else
	    target = findFt(ptr, NULL, f)->var->m;
	  if (ptr->args[0]->ptype == PT_RESP &&
	      ptr->args[0]->fieldId == INTENSITY) {
	    while (mark < d->n && ptr->args[0]->var->resp != d->dList[mark])
	      mark++;
	    if (!ptr->ptype) {
	      ptr->ptype = PT_SF;
	      ptr->var->f = initSf(0.0, 0.0, NULL);
	      ptr->storageG = 1;
	      ptr->g = (sf *) malloc(1 * sizeof(ptr));
	      for (i = 0; i < 1; i++)
		ptr->g[i] = NULL;
	    }
	    ptr->var->f->n = 1; /* Reset the target */
	    for (j = 0; j < ind; j++) {
	      if (checkBit(j % LONGLEN, target->inUse[0][j / LONGLEN])) {
		putSfToFt2(j, mark, NULL, -INFTY, INFTY, 
			   model->ftList[mark][ind], model->ftList[mark][j], 
			   d, f);
		lambda = calculate(i, model->ftList[mark][ind]);
		bite_printf("Indiv==%ld. ", j);
		printSf(lambda);
		ptr->g[0] = addSf(ptr->var->f, lambda, ptr->g[0]);
		copySf(ptr->g[0], ptr->var->f);
		printSf(ptr->var->f);
	      }
	    }
	  }
	  else {
	    for (j = 0; j < ind; j++)
	      if (!(j % LONGLEN) && !target->inUse[mark][j / LONGLEN])
		j += LONGLEN - 1;
	      else
		if (checkBit(j % LONGLEN, target->inUse[0][j / LONGLEN]))
		  res += calculate2(j, ptr->args[0]);
	  }
	  break;
	case F_PROD:
	  res = 1.0;
	  target = findFt(ptr, NULL, f)->var->m;
	  for (j = 0; j < ind; j++)
	    if (checkBit(j % LONGLEN, target->inUse[0][j / LONGLEN]))
	      res *= calculate2(j, ptr->args[0]);
	  break;
	case F_INT:
	  switch (ptr->args[0]->ptype) {
	  case PT_RESP:
	    while (mark < d->n && ptr->args[0]->var->resp != d->dList[mark])
	      mark++;
	    if (ptr->args[0]->fieldId == INTENSITY) {
	      putSfToFt2(i, mark, NULL, -INFTY, INFTY, 
			 model->ftList[mark][ind], model->ftList[mark][i], 
			 d, f);
	      lambda = calculate(i, model->ftList[mark][ind]);
	      res = intSf(lambda, calculate2(i, ptr->args[1]), 
			  calculate2(i, ptr->args[2]));
	    }
	    else {
	      bite_eprintf( "Only hazard available (%s).\n", 
		      ptr->args[0]->varName);
	      bite_exit(1);
	    }
	    break;
	  case PT_RESP2:
	    while (d->pList[mark] != ptr->args[0]->var->pvec)
	      mark++;
	    if (ptr->args[0]->fieldId == INTENSITY) {
	      putSfToFt2(i, mark, NULL, -INFTY, INFTY, 
			 model->ftList[mark][0], model->ftList[mark][0],
			 d, f);
	      lambda = calculate(i, model->ftList[mark][0]);
	      res = intSf(lambda, calculate2(i, ptr->args[1]), 
			  calculate2(i, ptr->args[2]));
	    }
	    else {
	      bite_eprintf( "Only hazard available (%s).\n", 
		      ptr->args[0]->varName);
	      bite_exit(1);
	    }
	    break;
	  case PT_MODELF:
	    switch (ptr->args[0]->var->m->type) {
	    case SF_ORIG:
	      res = intSf((ptr->args[0]->storageF < 2 || !ptr->args[0]->fc) ?
			  ptr->args[0]->var->m->f->f :
			  ptr->args[0]->f[ptr->args[0]->fc[i]]->f->f,
			  calculate2(i, ptr->args[1]),
			  calculate2(i, ptr->args[2]));
	      break;
	    case SF_TIMELESS:
	      res = paramValue(i, (ptr->args[0]->storageF < 2 || !ptr->args[0]->fc) ?
			       ptr->args[0]->var->m->f->p :
			       ptr->args[0]->f[ptr->args[0]->fc[i]]->f->p) *
		(calculate2(i, ptr->args[2]) - calculate2(i, ptr->args[1]));
	      break;
	      /* Commented 28.1.2004:
		 case SF_ORIG:
		 res =
		 intSf(ptr->args[0]->var->m->f->f, calculate2(i, ptr->args[1]),
		 calculate2(i, ptr->args[2]));
		 break;
		 case SF_TIMELESS:
		 res = paramValue(i, ptr->args[0]->var->m->f->p) *
		 (calculate2(i, ptr->args[2]) - calculate2(i, ptr->args[1]));
		 break;*/
	    }
	    break;
	  default:
	    bite_eprintf( "No such data type for \"integral\" (%s).\n", 
		    ptr->args[0]->varName);
	    bite_exit(1);
	  }
	  break;
	case F_INVINT:
	  switch (ptr->args[0]->ptype) {
	  case PT_RESP:
	    while (mark < d->n && ptr->args[0]->var->resp != d->dList[mark])
	      mark++;
	    if (ptr->args[0]->fieldId == INTENSITY) {
	      putSfToFt2(i, mark, NULL, -INFTY, INFTY,
			 model->ftList[mark][ind], model->ftList[mark][i], 
			 d, f);
	      lambda = calculate(i, model->ftList[mark][ind]);
	      res = invIntSf(lambda, calculate2(i, ptr->args[1]));
	    }
	    else {
	      bite_eprintf( "Only hazard available (%s).\n", 
		      ptr->args[0]->varName);
	      bite_exit(1);
	    }
	    break;
	  case PT_MODELF:
	    break;
	  default:
	    if (ptr->function == F_SUM)
	      res = invIntSf(ptr->var->f, calculate2(i, ptr->args[1]));
	    else {
	      bite_eprintf( "No such data type for \"integral\" (%s).\n", 
		      ptr->args[0]->varName);
	      bite_exit(1);
	    }
	  }
	  break;
	case F_INDIV:
	  res = calculate2(-1, ptr->args[1]);
	  if (res < 0 || res >= ind) {
	    bite_eprintf(
		    "%g: no such individual (line %ld).\n", res, ptr->lineno);
	    bite_exit(1);
	  }
	  res = calculate2(res, ptr->args[0]);
	  break;
	case F_INDEX:
	  /* if (!sscanf(ptr->var->x->name, "%lf", &res)) */
	  /* res = index0; */
	  if (index0 < 0)
	    bite_eprintf( "Warning: \'index(0)\'<0!\n");
	  j = calculate2(-1, ptr->args[0]);
	  res = lindex[(j > level) ? 0 : level - j];
	  break;
	case F_OLD:
	  res = old[(long) calculate2(-1, ptr->args[0])];
	  break;
	case F_ADDP:
	  if (i < 0) {
	    bite_eprintf( "Individual not specified, line %ld", ptr->lineno);
	    bite_exit(1);
	  }
	  /* Addition and deletion of points is likely to work only if a
	     prior distribution is _not_ defined for any of the points. */
	  switch (ptr->args[0]->ptype) {
	  case PT_COV2:
	    r = ptr->args[0]->var->pvec;
	    if (r->type == COV_START2) {
	      x = calculate2(i, ptr->args[1]);
	      for (n = 0; n < r->n2[i] && r->val.x2[i][n] > x; n++);
	      r->n2[i]++;
	      if (r->n2[i] == r->stor) {
		r->stor++;
		r->distr.x2 = realloc(r->distr.x2, r->stor * sizeof(c));
		r->distr.x2[r->stor - 1] = NULL;
		for (j = 0; j < ind; j++) {
		  r->val.x2[j] = realloc(r->val.x2[j], r->stor * sizeof(x1));
		  r->fixed.x2[j] = realloc(r->fixed.x2[j],
					   (r->stor / LONGLEN + 1) * sizeof(i));
		}
	      }
	      for (j = r->n2[i] - 1; j > n; j--)
		r->val.x2[i][j] = r->val.x2[i][j - 1];
	      r->val.x2[i][n] = x;
	      res = 1;
	      /* This may need to be changed: */
	      putBitTo1(r->n2[i] % LONGLEN, &r->fixed.x2[i][r->n2[i] / LONGLEN]);
	    }
	    break;
	  case PT_RESP2:
	    r = ptr->args[0]->var->pvec;
	    car = (ptr->args[0]->third && ptr->args[0]->third->ni) ?
	      calculate2(i, ptr->args[0]->third->i[0]) : 0;
	    if (0 <= car && car < r->n2[i]) {
	      x = calculate2(i, ptr->args[1]);
	      for (n = 0; n < r->val.pp.npoints[i][car] &&
		     r->val.pp.points[i][car][n] > x; n++);
	      if (n >= r->val.pp.npoints[i][car]) {
		r->val.pp.points[i][car] =
		  realloc(r->val.pp.points[i][car], (n + 1) * sizeof(x1));
		r->fixed.pp.points[i][car] =
		  realloc(r->fixed.pp.points[i][car],
			  (n / LONGLEN + 1) * sizeof(i));
	      }
	      for (j = r->val.pp.npoints[i][car] - 1; j > n; j--)
		r->val.pp.points[i][car][j] = r->val.pp.points[i][car][j - 1];
	      r->val.pp.points[i][car][n] = x;
	      res = 1;
	      putBitTo1(i % LONGLEN,
			&r->fixed.pp.points[i][car][n / LONGLEN]);
	      r->val.pp.npoints[i][car]++;
	    }
	    break;
	  }
	  break;
	case F_DELP:
	  if (i < 0) {
	    bite_eprintf( "Individual not specified, line %ld", ptr->lineno);
	    bite_exit(1);
	  }
	  switch (ptr->args[0]->ptype) {
	  case PT_COV2:
	    r = ptr->args[0]->var->pvec;
	    if (r->type == COV_START2) {
	      x = calculate2(i, ptr->args[1]);
	      for (n = 0; n < r->n2[i] && r->val.x2[i][n] != x; n++);
	      if (n < r->n2[i]) { /* If startpoint x was found: */
		r->n2[i]--;
		for (j = n; j < r->n2[i]; j++)
		  r->val.x2[i][j] = r->val.x2[i][j + 1];
		res = 1;
	      }
	    }
	    break;
	  case PT_RESP2:
	    r = ptr->args[0]->var->pvec;
	    car = (ptr->args[0]->third && ptr->args[0]->third->ni) ?
	      calculate2(i, ptr->args[0]->third->i[0]) : 0;
	    if (0 <= car && car < r->n2[i]) {
	      x = calculate2(i, ptr->args[1]);
	      for (n = 0; n < r->val.pp.npoints[i][car] &&
		     r->val.pp.points[i][car][n] != x; n++);
	      for (j = r->val.pp.npoints[i][car] - 1; j > n; j--)
		r->val.pp.points[i][car][j - 1] = r->val.pp.points[i][car][j];
	      res = 1;
	      r->val.pp.npoints[i][car]++;
	    }
	    break;
	  }
	  break;
	case F_ADDI:
	  bite_eprintf( "Not implemented, line %ld", ptr->lineno);
	  bite_exit(1);
	  break;
	case F_DELI:
	  bite_eprintf( "Not implemented, line %ld", ptr->lineno);
	  bite_exit(1);
	  break;
#ifdef DEBUG
	  bite_printf(") ");
#endif
	}
      }
      else {
	if (ptr->var) {
	  long i1[2] = {0, 0}, ii, i2 = 0, c2 = 0; // c1 = 0
	  prior elem1, elem2;
	  if (ptr->ni)
	    i1[0] = calculate2(i, ptr->i[0]);
	  if (ptr->third && ptr->third->i)
	    i2 = calculate2(i, ptr->third->i[0]);
	  switch (ptr->ptype) {
	  case PT_CONST:
	    res = ptr->var->x;
#ifdef DEBUG
	    bite_printf("%g", ptr->var->x);
#endif
	    break;
	  case PT_SCALAR:
	    res = paramValue(i, ptr->var->p);
#ifdef DEBUG
	    printName(ptr->var->p);
#endif
	    break;
	  case PT_VECTOR:
#ifdef DEBUG
	    bite_printf("%s", ptr->var->vec->name);
#endif
	    /*index0 = ptr->var->vec->x->vec->index;*/
	    ii = 0;
	    elem1 = ptr->var->vec->x->vec;
	    while (ii < ptr->ni) {
//	      c1 = 0;
	      i1[ii] = calculate2(i, ptr->i[ii]);
	      /* while (elem1 && c1 < i1[ii]) {
		 elem1 = elem1->next;
		 c1++;
		 } */
#ifdef DEBUG
	      bite_printf("[%ld]", i1[ii]);
	      fflush(NULL);
#endif
	      if (elem1->up->stor <= i1[ii] || i1[ii] < 0) {
		bite_eprintf(
			"%ld\'th index of %s is out of range (line %ld).\n",
			i1[ii], ptr->varName, ptr->lineno);
		bite_exit(1);
	      }
	      elem1 = elem1->up->vector[i1[ii]];
	      elem1 = (elem1->ptype == PT_VECTOR) ?
		elem1->x->vec : elem1;
	      ii++;
	    }
	    res = paramValue(i, elem1);
#ifdef DEBUG
	    printName(elem1);
#endif
	    break;
	  case PT_SF:
	    break;
	  case PT_RESP:
	    if (i >= 0) {
	      switch (ptr->fieldId) {
	      case T0:
		res = paramValue(i, ptr->var->resp[i][i1[0]]->pp->t0);
		break;
	      case T1:
		res = paramValue(i, ptr->var->resp[i][i1[0]]->pp->t1);
		break;
	      case POINTS:
		elem2 = ptr->var->resp[i][i1[0]]->pp->vector;
		while (elem2 && c2 < i2) {
		  elem2 = elem2->next;
		  c2++;
		}
		res = (elem2) ? paramValue(i, elem2) : -INFTY;
		break;
	      }
	    }
	    break;
	  case PT_RESP2:
	    if (i >= 0) {
	      res = respValue(i, i1[0], i2, ptr->fieldId, ptr->var->pvec);
#ifdef DEBUG
	      bite_printf("%s ", ptr->var->pvec->name);
#endif
	      /* switch (ptr->fieldId) {
		 case T0:
		 res = ptr->var->pvec->val.pp.t0[i][i1[0]];
		 break;
		 case T1:
		 res = ptr->var->pvec->val.pp.t1[i][i1[0]];
		 break;
		 case POINTS:
		 res = (ptr->var->pvec->val.pp.npoints[i][i1[0]] > i2) ?
		 ptr->var->pvec->val.pp.points[i][i1[0]][i2] : INFTY;
		 break;
		 } */
	    }
	    break;
	  case PT_COV:
	    if (i >= 0) {
	      switch (ptr->var->cov[0]->type) {
	      case COV_CNST:
		res = paramValue(i, ptr->var->cov[i]->x);
		break;
	      case COV_SF:
		break;
	      case COV_START:
		if (ptr->ni) {
		  long j = 0;
		  prior list = ptr->var->cov[i]->x->x->vec;
		  while (j < i1[0] && list && list->next) {
		    list = list->next;
		    j++;
		  }
		  if (j == i1[0])
		    res = paramValue(i, list);
		}
		break;
	      }
	    }
	    else
	      res = 0.0;
	    break;
	  case PT_COV2:
	    res = covValue(i, i1[0], ptr->var->pvec);
#ifdef DEBUG
	    bite_printf("%s ", ptr->var->pvec->name);
#endif
	    /* switch (ptr->var->pvec->type) {
	       case COV_CNST2:
	       res = ptr->var->pvec->val.x[i];
	       break;
	       case COV_START2:
	       res = (ptr->var->pvec->n2[i] > i1[0]) ?
	       ptr->var->pvec->val.x2[i][i1[0]] : 0.0;
	       break;
	       } */
	    break;
	  case PT_MODELF:
	    if (ptr->storageF > 1 && (i < 0 || i >= ptr->storageF)) {
	      bite_eprintf( "The model function list %s(...) does "
		      "not have element %ld. Only %ld elements available. "
		      "(Line %ld.)\n",
		      ptr->varName, i, ptr->storageF,
		      ptr->lineno);
	      bite_exit(1);
	    }
	    target = (ptr->fc && ptr->storageF > 1) ? ptr->f[i] : ptr->var->m;
	    switch (target->type) {
	    case SF_ORIG:
	      switch (ptr->fieldId) {
	      case JUMPPOINT:
		res = target->f->f->jumpPoints[i2];
		break;
	      case LEVEL:
		res = target->f->f->levels[i2];
		break;
	      case N_POINTS:
		res = target->f->f->n - 2;
		break;
	      }
	      break;
	    case SF_TIMELESS:
	      switch (ptr->fieldId) {
	      case JUMPPOINT:
		res = INFTY;
		break;
	      case LEVEL:
		res = paramValue(i, target->f->p);
		break;
	      case N_POINTS:
		res = 0;
		break;
	      }
	      break;
	    }
	    break;
	  }
	}
      }
    }
#ifdef DEBUG
    bite_printf(" ");
#endif
  }
#ifdef DEBUG
  bite_printf("return(%g)\n", res);
#endif
  return res;
}

void returnSupport(long ind, prior node, double *bot, double *top)
{
  switch (node->type) {
  case D_DUNIF:
    *bot = calculate2(ind, node->args[0]);
    *top = calculate2(ind, node->args[1]);
    break;
  case D_UNIF:
    *bot = calculate2(ind, node->args[0]);
    *top = calculate2(ind, node->args[1]);
    break;
  case D_BIN:
    *bot = 0.0;
    *top = calculate2(ind, node->args[0]);
    break;
  case D_NORM:
    *bot = -INFTY;
    *top = INFTY;
    break;
  case D_MNORM:
    *bot = -INFTY;
    *top = INFTY;
    break;
  case D_GAMMA:
    *bot = 0.0;
    *top = INFTY;
    break;
  case D_LAPLACE:
    *bot = -INFTY;
    *top = INFTY;
    break;
  case D_STUDENT:
    *bot = -INFTY;
    *top = INFTY;
    break;
  case D_DIRAC:
    *bot = calculate2(ind, node->args[0]);
    *top = calculate2(ind, node->args[0]);
    break;
  case D_DISCR:
    *bot = calculate2(ind, node->args[0]);
    *top = calculate2(ind, node->args[node->nArgs - 2]);
    break;
  case D_POISSON:
    *bot = 0.0;
    *top = INFTY;
    break;
  }
}

double generatePriorValue(long ind, prior node)
{
  long i, trys = 0, found = FALSE, restricted = FALSE;
  double p, res = 0.0, bot = -INFTY, top = INFTY, y, pbot, ptop;

  level++;
  lindex[level] = node->index;
  if (node->restrict1 && node->restrict2) {
    bot = calculate2(ind, node->restrict1);
    top = calculate2(ind, node->restrict2);
    returnSupport(ind, node, &pbot, &ptop);
    if (bot > pbot || top < ptop)
      restricted = TRUE;
  }

  do {
    switch (node->type) {
    case D_DUNIF:
      res = ran_uni(calculate2(ind, node->args[0]),
		    calculate2(ind, node->args[1]) + 1.0);
      res = (long) res;
      break;
    case D_UNIF:
      res = ran_uni(calculate2(ind, node->args[0]),
		    calculate2(ind, node->args[1]));
      res = res;
      break;
    case D_BIN:
      res = 0.0;
      p = calculate2(ind, node->args[1]);
      for (i = 0; i < (long) calculate2(ind, node->args[0]); i++)
	if (ran_uni(0.0, 1.0) < p)
	  res += 1.0;
      break;
    case D_NORM:
      res = calculate2(ind, node->args[0]) + ran_norm() * 
	sqrt(calculate2(ind, node->args[1]));
      break;
    case D_MNORM:
      node->x->vec->x->x = calculate2(ind, node->args[0]) + ran_norm() * 
	sqrt(calculate2(ind, node->args[2]));
      node->x->vec->next->x->x = calculate2(ind, node->args[1]) +
	ran_norm() * sqrt(calculate2(ind, node->args[5]));
      break;
    case D_GAMMA:
      res = ran_gamma(calculate2(ind, node->args[0]), 
		      calculate2(ind, node->args[1]));
      break;
    case D_LAPLACE:
      p = calculate2(ind, node->args[1]);
      res = calculate2(ind, node->args[0]) +
	ran_gamma(1.0, 1.0 / p) -
	ran_gamma(1.0, 1.0 / p);
      break;
    case D_STUDENT:
      res = ran_student(calculate2(ind, node->args[0]), 
			calculate2(ind, node->args[1]), 
			calculate2(ind, node->args[2]));
      break;
    case D_DIRICHLET:
      {
	int i;
	double sum = 0, *a = malloc(node->nArgs * sizeof(sum)),
	  *x = malloc(node->nArgs * sizeof(sum));
	prior ptr = node->x->vec;
	for (i = 0; i < node->nArgs; i++) {
	  a[i] = calculate2(ind, node->args[i]);
	  sum += x[i] = ran_gamma(a[i], 1.0);
	}
	for (i = 0; i < node->nArgs; i++) {
	  ptr->x->x = x[i] / sum;
	  ptr = ptr->next;
	}
	free(a);
	free(x);
      }
      break;
    case D_DISCR:
      {
	int i = -1;
	double sum = 0.0, a = ran_uni(0.0, 1.0);
	do {
	  i += 2;
	} while (i < node->nArgs &&
		 (sum += calculate2(ind, node->args[i])) && sum < a);
	if (i < node->nArgs)
	  res = calculate2(ind, node->args[i - 1]);
	else
	  bite_eprintf( "Generating \'discrete\', but the probabilities "
		  "do not seem to sum up to one!\n");
      }
      break;
    case D_POISSON:
      {
	long a = (long)calculate2(ind, node->args[0]);
	if (a < 50)
	  res = ran_poi(a);
	else
	  res = (long) (a + ran_norm() * sqrt(a) + 0.5);
      }
      break;
    case D_MULTIN:
      {
	/* int i;
	double sum = 0, *a = malloc(node->nArgs * sizeof(sum)),
	  *x = malloc(node->nArgs * sizeof(sum));
	prior ptr = node->x->vec;
	for (i = 0; i < node->nArgs; i++) {
	  a[i] = calculate2(ind, node->args[i]);
	  sum += x[i] = ran_gamma(a[i], 1.0);
	}
	for (i = 0; i < node->nArgs; i++) {
	  ptr->x->x = x[i] / sum;
	  ptr = ptr->next;
	}
	free(a);
	free(x); */
      }
      break;
    case D_DIRAC:
      res = calculate2(ind, node->args[0]);
      break;
    case D_MIX:
      {
	long i = 0;
	double x = ran_uni(0.0, 1.0), cw = 0.0, w;
	while (i < node->nArgs - 1 &&
	       cw + (w = calculate2(ind, node->args[i + 1])) < x) {
	  i += 2;
	  cw += w;
	}
	node->x->mix.type = i / 2;
	node->x->mix.x = res =
	  generatePriorValue(ind, node->args[i]->var->p);
      }
      break;
    default:
      if (node->update) {
	bite_eprintf( "%ld: no such distribution.\n", node->type);
	bite_exit(1);
      }
    }
/*    node->x->x = res; */
    trys++;
    if (restricted) {
      if (bot <= res && res < top)
	found = TRUE;
    }
  } while (restricted && !found && trys < 10);
    
  if (restricted && !found) {
    if (bot < top) {
      /*      for (i = 0; i <= 10; i++) {
	y = exp(returnPriorLogDensity(ind, node, bot + (i / 10.0) * (top - bot)));
	if (y > max)
	  max = y;
	  }*/
      res = ran_uni(bot, top);
      for (trys = 1; trys < 100; trys++) {
	y = (ran_uni(0.0, 1.0) > 0.5) ? ran_uni(bot, top) :
	  ran_uni(res - (top - bot) / 20, res + (top - bot) / 20);
	if (LOG(ran_uni(0.0, 1.0)) < returnPriorLogDensity(ind, node, y) -
	    returnPriorLogDensity(ind, node, res)) {
	  res = y;
	  /*bite_printf("%g ", y);*/
	}
      }
      /*bite_printf(": %g\n", res);*/
      /*while (trys < 200 && !found) {
	trys++;
	res = ran_uni(bot, top);
	if (ran_uni(0.0, max) < exp(returnPriorLogDensity(ind, node, res)))
	  found = TRUE;
      }
      if (!found) {
	long i = 0;
	bite_eprintf( "Proposal not good for %s. Parameters are ",
		node->name);
	while (i < node->nArgs) {
	  bite_eprintf( "%g, ", calculate2(ind, node->args[i]));
	  i += 1;
	}
	bite_eprintf( "\n");
	res = ran_uni(bot, top);
      }*/
    }
    else
      res = bot;
  }

  level--;

  return res;
}

/*double returnPriorDensity(long ind, prior node, double x)
{
  long n, k;
  double res = 1.0, p, a, b, bot, top;

  switch (node->type) {
  case D_DUNIF:
    bot = calculate2(ind, node->args[0]);
    top = calculate2(ind, node->args[1]);
    res = (x >= bot && x <= top) ? 1.0 / (1.0 + top - bot) : 0.0;
    break;
  case D_UNIF:
    bot = calculate2(ind, node->args[0]);
    top = calculate2(ind, node->args[1]);
    res = (x >= bot && x <= top) ? 1.0 / ((top > bot) ? top - bot : INFTY) 
      : 0.0;
    break;
  case D_BIN:
    n = (long)calculate2(ind, node->args[0]);
    k = (long)x;
    p = calculate2(ind, node->args[1]);
    res = factorial(n) / (factorial(k) * factorial(n - k)) *
      exp((double)k * LOG(p)) * exp((double)(n - k) * LOG(1.0 - p));
    break;
  case D_NORM:
    a = calculate2(ind, node->args[0]);
    b = calculate2(ind, node->args[1]);
    res = 1 / (sqrt(2 * PI * b)) * exp(-0.5 * pow(x - a, 2.0) / b);
    break;
  case D_MNORM:
      double x[2], mu[2], a[4], b[4];
      x[0] = paramValue(ind, node->x->vec);
      x[1] = paramValue(ind, node->x->vec->next);
      mu[0] = calculate2(ind, node->args[0]);
      mu[1] = calculate2(ind, node->args[1]);
      a[0] = calculate2(ind, node->args[2]);
      a[1] = calculate2(ind, node->args[3]);
      a[2] = calculate2(ind, node->args[4]);
      a[3] = calculate2(ind, node->args[5]);
      invMatrix(4, a, b);
      a[0] = (x[0] - mu[0]) * b[0] + (x[1] - mu[1]) * b[2];
      a[1] = (x[0] - mu[0]) * b[1] + (x[1] - mu[1]) * b[3];
      res = (x[0] - mu[0]) * a[0] + (x[1] - mu[1]) * a[1];
      res = exp(-0.5 * res) / (2 * PI) * determinant(4, b);
    }
    break;
  case D_GAMMA:
    a = calculate2(ind, node->args[0]);
    b = calculate2(ind, node->args[1]);
    res = (x > 0) ? dens_gamma(x, a, b) * exp(a * log(b)) / exp(lgamma(a)) : 0.0;
    break;
  case D_DIRICHLET:
    {
      int i;
      double sum = 0.0, a;
      prior ptr = node->x->vec;
      for (i = 0; i < node->nArgs; i++) {
	sum += a = calculate2(ind, node->args[i]);
	res *= pow(paramValue(ind, ptr), a - 1.0) / exp(lgamma(a));
      }
      res *= exp(lgamma(sum));
    }
    break;
  case D_DISCR:
    {
      int i = 0;
      while (i < node->nArgs && calculate2(ind, node->args[i]) != x)
	i += 2;
      if (i < node->nArgs)
	res = calculate2(ind, node->args[i + 1]);
      else
	bite_eprintf( "Calculating \'discrete\' density, but value %g "
		"couldn\'t be found!\n", x);
    }
    break;
  case D_MIX:
    res = returnPriorDensity(ind, node->args[2 * node->x->mix.type]->var->p,
			     node->x->mix.x) *
      calculate2(ind, node->args[2 * node->x->mix.type + 1]);
    break;
  default:
    break;
  }

  if (node->restrict1 && node->restrict2)
    res = (x >= calculate2(ind, node->restrict1) &&
	   x <= calculate2(ind, node->restrict2)) ? res : 0.0;

  return res;
} */

double returnPriorLogDensity2(long ind, double x, long type, long nArgs,
			      ft* args)
{
long n, k;
double res = 0.0, p, a, b, c, bot, top;

switch (type) {
case D_DUNIF:
bot = calculate2(ind, args[0]);
top = calculate2(ind, args[1]);
res = (x >= bot && x <= top) ? LOG(1.0 / (1.0 + top - bot)) : -INFTY;
break;
case D_UNIF:
bot = calculate2(ind, args[0]);
top = calculate2(ind, args[1]);
res = (x >= bot && x <= top) ?
LOG(1.0 / ((top > bot) ? top - bot : INFTY)) : -INFTY;
break;
case D_BIN:
n = (long)calculate2(ind, args[0]);
k = (long)x;
p = calculate2(ind, args[1]);
res = (0 <= k && k <= n) ? 
lgamma(n + 1) - (lgamma(k + 1) + lgamma(n - k + 1)) +
(double)k * LOG(p) + (double)(n - k) * LOG(1.0 - p) : -INFTY;
/* res = LOG(factorial(n) / (factorial(k) * factorial(n - k))) +
   (double)k * LOG(p) + (double)(n - k) * LOG(1.0 - p);
   if (res < -0.5 * INFTY)
   bite_printf("%ld, %ld, %f\n", n, k, p); */
break;
case D_NORM:
a = calculate2(ind, args[0]);
b = calculate2(ind, args[1]);
res = - (LOG(2 * PI * b) + pow(x - a, 2.0) / b) / 2.0 ;
break;
case D_GAMMA:
a = calculate2(ind, args[0]);
b = calculate2(ind, args[1]);
res = (x > 0) ?
LOG(dens_gamma(x, a, b)) + a * LOG(b) - lgamma(a) : -INFTY;
break;
case D_LAPLACE:
a = calculate2(ind, args[0]);
b = calculate2(ind, args[1]);
res = - LOG(2*b) - fabs(x - a) / b;
break;
case D_STUDENT:
a = calculate2(ind, args[0]);
b = calculate2(ind, args[1]);
c = calculate2(ind, args[2]);
res = lgamma((c+1.0)/2.0) - lgamma(c/2.0) -
LOG(c*PI)/2.0 - LOG(sqrt(b)) -
(c+1.0)/2.0 * LOG(1.0 + (pow(x-a,2.0))/b/c);
break;
case D_DISCR:
{
  int i = 0;
  while (i < nArgs && calculate2(ind, args[i]) != x)
    i += 2;
  if (i < nArgs)
    res = LOG(calculate2(ind, args[i + 1]));
  else
    bite_eprintf( "Calculating \'discrete\' density, but value %g "
	    "couldn\'t be found!\n", x);
}
 break;
 case D_POISSON:
   a = calculate2(ind, args[0]);
   k = (long)x;
   res = (double) k * LOG(a) - a - lgamma(1.0 + (double) k);
   break;
 default:
   break;
}

 return res;
}

double returnPriorLogDensity(long ind, prior node, double x)
{
  double res = 0.0;

  if (!node->type && node->up)
    node = node->up;
  level++;
  lindex[level] = node->index;
  switch (node->type) {
  case D_DUNIF:
  case D_UNIF:
  case D_BIN:
  case D_NORM:
  case D_GAMMA:
  case D_STUDENT:
  case D_DISCR:
  case D_LAPLACE:
  case D_POISSON:
    res = returnPriorLogDensity2(ind, x, node->type, node->nArgs, node->args);
    break;
  case D_MNORM:
    { /* Only 2-dim multinormal! */
      double x[2], mu[2], a[4], b[4];
      x[0] = paramValue(ind, node->x->vec); /* Value */
      x[1] = paramValue(ind, node->x->vec->next);
      mu[0] = calculate2(ind, node->args[0]); /* Mean */
      mu[1] = calculate2(ind, node->args[1]);
      a[0] = calculate2(ind, node->args[2]); /* Variance */
      a[1] = calculate2(ind, node->args[3]);
      a[2] = calculate2(ind, node->args[4]);
      a[3] = calculate2(ind, node->args[5]);
      invMatrix(4, a, b);  /* Invert the variance matrix a -> b */
      a[0] = (x[0] - mu[0]) * b[0] + (x[1] - mu[1]) * b[2]; /* Matrix */
      a[1] = (x[0] - mu[0]) * b[1] + (x[1] - mu[1]) * b[3]; /* multiplica. */
      res = (x[0] - mu[0]) * a[0] + (x[1] - mu[1]) * a[1];
      res = -0.5 * res - LOG((2 * PI) * determinant(4, b)); /* Density */
    }
    break;
  case D_DIRICHLET:
    {
      int i;
      double sum = 0.0, a;
      prior ptr = node->x->vec;
      for (i = 0; i < node->nArgs; i++) {
        sum += a = calculate2(ind, node->args[i]);
        res += (a - 1.0) * paramValue(ind, ptr) - lgamma(a);
	ptr = ptr->next;
      }
      res += lgamma(sum);
    }
    break;
  case D_MIX:
    res = returnPriorLogDensity(ind,
				node->args[2 * node->x->mix.type]->var->p,
				node->x->mix.x) +
      LOG(calculate2(ind, node->args[2 * node->x->mix.type + 1]));
    break;
  default:
    break;
  }

  if (node->restrict1 && node->restrict2)
    res = (x >= calculate2(ind, node->restrict1) &&
	   x <= calculate2(ind, node->restrict2)) ? res : -INFTY;
  /*  if (res <= 0.0)
    bite_printf("%s:%g ", node->name, x); */
  level--;

  return res;
}

/* double returnPriorLogDensity(long ind, prior node, double x)
{
  long n, k;
  double res = 0.0, p, a, b, bot, top;

  switch (node->type) {
  case D_DUNIF:
    bot = calculate2(ind, node->args[0]);
    top = calculate2(ind, node->args[1]);
    res = (x >= bot && x <= top) ? LOG(1.0 / (1.0 + top - bot)) : 0.0;
    break;
  case D_UNIF:
    bot = calculate2(ind, node->args[0]);
    top = calculate2(ind, node->args[1]);
    res = (x >= bot && x <= top) ?
      LOG(1.0 / ((top > bot) ? top - bot : INFTY)) : 0.0;
    break;
  case D_BIN:
    n = (long)calculate2(ind, node->args[0]);
    k = (long)x;
    p = calculate2(ind, node->args[1]);
    res = LOG(factorial(n) / (factorial(k) * factorial(n - k))) +
      (double)k * log(p) + (double)(n - k) * log(1.0 - p);
    break;
  case D_NORM:
    a = calculate2(ind, node->args[0]);
    b = calculate2(ind, node->args[1]);
    res = - (LOG(2 * PI * b) + pow(x - a, 2.0) / b) / 2.0 ;
    break;
  case D_MNORM:
    { Only 2-dim multinormal!
      double x[2], mu[2], a[4], b[4];
      x[0] = paramValue(ind, node->x->vec); Value
      x[1] = paramValue(ind, node->x->vec->next);
      mu[0] = calculate2(ind, node->args[0]); Mean
      mu[1] = calculate2(ind, node->args[1]);
      a[0] = calculate2(ind, node->args[2]); Variance
      a[1] = calculate2(ind, node->args[3]);
      a[2] = calculate2(ind, node->args[4]);
      a[3] = calculate2(ind, node->args[5]);
      invMatrix(4, a, b);  Invert the variance matrix a -> b
      a[0] = (x[0] - mu[0]) * b[0] + (x[1] - mu[1]) * b[2]; Matrix
      a[1] = (x[0] - mu[0]) * b[1] + (x[1] - mu[1]) * b[3]; multiplica.
      res = (x[0] - mu[0]) * a[0] + (x[1] - mu[1]) * a[1];
      res = -0.5 * res - LOG((2 * PI) * determinant(4, b)); Density
    }
    break;
  case D_GAMMA:
    a = calculate2(ind, node->args[0]);
    b = calculate2(ind, node->args[1]);
    res = (x > 0) ?
      LOG(dens_gamma(x, a, b)) + a * LOG(b) - LOG(gamma(a)) : 0.0;
    break;
  case D_DIRICHLET:
    {
      int i;
      double sum = 0.0, a;
      prior ptr = node->x->vec;
      for (i = 0; i < node->nArgs; i++) {
        sum += a = calculate2(ind, node->args[i]);
        res += (a - 1.0) * paramValue(ind, ptr) - LOG(gamma(a));
      }
      res += LOG(gamma(sum));
    }
    break;
  case D_DISCR:
    {
      int i = 0;
      while (i < node->nArgs && calculate2(ind, node->args[i]) != x)
        i += 2;
      if (i < node->nArgs)
        res = LOG(calculate2(ind, node->args[i + 1]));
      else
        bite_eprintf( "Calculating \'discrete\' density, but value %g "
                "couldn\'t be found!\n", x);
    }
    break;
  case D_MIX:
    res = LOG(returnPriorDensity(ind,
                                 node->args[2 * node->x->mix.type]->var->p,
                                 node->x->mix.x) *
              calculate2(ind, node->args[2 * node->x->mix.type + 1]));
    break;
  default:
    break;
  }

  if (node->restrict1 && node->restrict2)
    res = (x >= calculate2(ind, node->restrict1) &&
           x <= calculate2(ind, node->restrict2)) ? res : 0.0;

  return res;
} */

double returnMfPriorDensity(long ind, mf f)
{
  double res = 1.0;

  switch (f->prior) {
  case GAMMA:
    {
      long i;
      double alpha0, beta0, alpha, mu;

      alpha0 = calculate2(ind, f->pVal->pGa->alpha0);
      beta0 = calculate2(ind, f->pVal->pGa->beta0);
      alpha = calculate2(ind, f->pVal->pGa->alpha);
      mu = calculate2(ind, f->pVal->pGa->mu);
      if (mu)
	res = exp(-f->length * mu) * pow(mu, f->f->f->n - 2);
      res *= pow(beta0, alpha0) / exp(lgamma(alpha0)) * 
	dens_gamma(f->f->f->levels[0], alpha0, beta0);
      for (i = 1; i < f->f->f->n - 1; i++)
	res *= pow(alpha / f->f->f->levels[i - 1], alpha) / exp(lgamma(alpha)) *
	  dens_gamma(f->f->f->levels[i], alpha,
		     alpha / f->f->f->levels[i - 1]);
    }
  break;
  case GAMMA2:
    {
      long i;
      double a, 
	alpha0 = calculate2(ind, f->pVal->pGa->alpha0) /
	(f->f->f->jumpPoints[1] - f->f->f->jumpPoints[0]),
	beta0 = calculate2(ind, f->pVal->pGa->beta0) /
	(f->f->f->jumpPoints[1] - f->f->f->jumpPoints[0]),
	alpha = calculate2(ind, f->pVal->pGa->alpha),
	mu = calculate2(ind, f->pVal->pGa->mu);
      if (mu)
	res = exp(-f->length * mu) * pow(mu, f->f->f->n - 2);
      res *= pow(beta0, alpha0) / exp(lgamma(alpha0)) * 
	dens_gamma(f->f->f->levels[0], alpha0, beta0);
      for (i = 1; i < f->f->f->n - ((mu) ? 0 : 1); i++) {
	a = alpha / (f->f->f->jumpPoints[i + 1] - f->f->f->jumpPoints[i]) + 1;
	res *= pow(a / f->f->f->levels[i - 1], a) / exp(lgamma(a)) *
	  dens_gamma(f->f->f->levels[i], a, a / f->f->f->levels[i - 1]);
      }
    }
  break;
  case BROWNIAN:
    {
      long i;
      double delta = calculate2(-1, f->pVal->pBM->delta),
	drift = calculate2(-1, f->pVal->pBM->drift),
	sigma2 = calculate2(-1, f->pVal->pBM->sigma2);
      res = 1.0;
      for (i = 1; i < f->f->f->n - 1; i++)
	res *= dens_norm(f->f->f->levels[i] - f->f->f->levels[i - 1],
			 drift * delta, sigma2);
    }
  break;
  case GAMMA_LEXIS:
    {
      long i, j, k = 0;
      double sum_aver, w, 
	alpha0 = calculate2(ind, f->pVal->pGL->alpha0),
	beta0 = calculate2(ind, f->pVal->pGL->beta0),
	alpha = calculate2(ind, f->pVal->pGL->alpha),
	mu = calculate2(ind, f->pVal->pGL->mu),
	*lexis_weight = malloc(f->pVal->pGL->lexis_dim * sizeof(w));
      for (k = 0; k<f->pVal->pGL->lexis_dim; k++)
	lexis_weight[k] = calculate2(-1, f->pVal->pGL->lexis_weight[k]);
      if (mu)
	res = exp(-f->length * mu) * pow(mu, f->f->f->n - 2);
      k = 0;
      for (j = 0; j < f->neighbour_dim; j++) {
	if (f->neighbour_prev[j])
	  k++;
      }
      if (k) {
	/* if (f->restrict0 > 0.0)
	   f->f->f->levels[0] = f->restrict0; */
	for (i = ((f->restrict0 > 0) ? 1 : 0); i < f->f->f->n - 1; i++) {
	  sum_aver = (i) ? f->f->f->levels[i - 1] : 0.0;
	  w = (i) ? 1.0 : 0.0;
	  for (j = 0; j < f->neighbour_dim; j++) {
	    if (f->neighbour_prev[j]) {
	      w += lexis_weight[j];
	      sum_aver += 
		intSf(f->neighbour_prev[j]->f->f,
		      f->f->f->jumpPoints[i], f->f->f->jumpPoints[i + 1]) /
		(f->f->f->jumpPoints[i + 1] - f->f->f->jumpPoints[i]) * lexis_weight[j];
	    }
	  }
	  res *= pow(alpha / (sum_aver / w), alpha) / exp(lgamma(alpha)) *
	    dens_gamma(f->f->f->levels[i], alpha, alpha / (sum_aver / w));
	}
      }
      else {
	/* The base model function: no previous model functions,
	   so apply the Arjas-Gasbarra prior. */
	if (f->restrict0 < 0.0) {
	  res *= pow(beta0, alpha0) / exp(lgamma(alpha0)) *
	    dens_gamma(f->f->f->levels[0], alpha0, beta0);
	}
	if (f->restrict0 == 0.0)
	  res *= 1;
	for (i = ((f->restrict0 < 0.0) ? 1 : 2); i < f->f->f->n - 1; i++) {
	  res *= pow(alpha / f->f->f->levels[i - 1], alpha) / exp(lgamma(alpha)) *
	    dens_gamma(f->f->f->levels[i], alpha, alpha / f->f->f->levels[i - 1]);
	}
      }
      free(lexis_weight);
    }
  break;
  default:
    if (f->type == SF_TIMELESS) {
      if (f->f->p->type != D_DIRAC)
	res = exp(returnPriorLogDensity(ind, f->f->p,
					paramValue(ind, f->f->p)));
      else
	res = logLikel2(ind, f, -INFTY, INFTY, d, model);
    }
  }
  return res;
}


double returnLexisMfLogPriorDensity(long ind, mf f, 
				    long updL, double int_start, double int_stop,
				    double alpha0, double beta0, double alpha,
				    double beta, double *lexis_weight,
				    long children)
{
  long i, j, k = 0, updLa = -1, updLb = -1;
  double w, res = 0.0, sum_aver;

  if (updL < 0) {
    /* Seek the jump points updLa and updLb, which cover interval (start, stop): */
    updLa = 0;
    while (updLa + 1 < f->f->f->n && f->f->f->jumpPoints[updLa + 1] < int_start)
      updLa++;
    updLb = updLa;
    while (f->f->f->jumpPoints[updLb] <= int_stop && updLb + 2 < f->f->f->n)
      updLb++;
    /* printSf(f->f->f); */
    if (printLevel >=3)
      bite_printf("updL=%ld, interval=(%g, %g), updLa=%ld <= i <= updLb=%ld.\n", updL, int_start, int_stop, updLa, updLb);
    for (i = updLa; i <= updLb; i++)
      /* Loop over the levels: */
      res += returnLexisMfLogPriorDensity(ind, f, 
					  i, int_start, int_stop,
					  alpha0, beta0, alpha, beta, lexis_weight,
					  children);
  } else {
    /* Calculate the prior density for level updL: */
    for (j = 0; j < f->neighbour_dim; j++) {
      if (f->neighbour_prev[j])
	k++;
    }
    if (k) {
      /* If there are previous neighbours: */
      if (f->restrict0 >= 0.0)
	f->f->f->levels[0] = f->restrict0;
      w = (updL) ? 1.0 : 0.0;
      sum_aver = (updL) ? f->f->f->levels[updL - 1] : 0.0;
      /* if (updL) {
	sum_aver += f->f->f->levels[updL - 1];
	k++;
	} */
      if (printLevel >=3)
	bite_printf("base: w=%g, sum_aver=%g\n", w, sum_aver);
      for (j = 0; j < f->neighbour_dim; j++) {
	if (f->neighbour_prev[j]) {
	  w += lexis_weight[j];
	  if (printLevel >=3) {
	    bite_printf("\tdim=%ld, lexis_weight=%g, interval=(%g, %g)\n",
			j, lexis_weight[j], f->f->f->jumpPoints[updL], f->f->f->jumpPoints[updL + 1]);
	    printSf(f->neighbour_prev[j]->f->f);
	    bite_printf("\tintSf=%f\n",
			intSf(f->neighbour_prev[j]->f->f,
			      f->f->f->jumpPoints[updL], f->f->f->jumpPoints[updL + 1]));
	  }
	  sum_aver += 
	    intSf(f->neighbour_prev[j]->f->f,
		  f->f->f->jumpPoints[updL], f->f->f->jumpPoints[updL + 1]) /
	    (f->f->f->jumpPoints[updL + 1] - f->f->f->jumpPoints[updL]) * lexis_weight[j];
	  /* k++; */
	}
      }
      if (printLevel >=3)
	bite_printf("w=%g, sum_aver=%g\n", w, sum_aver);
      res += log_dens_gamma(f->f->f->levels[updL], alpha * beta,
			    alpha / (sum_aver / (w)));
    } else {
      /* No previous neighbours, then Arjas-Gasbarra (1994): */
      if (updL == 0) {
	if (f->restrict0 < 0)
	  res += /* LOG(beta0) * alpha0 - lgamma(alpha0) + */
	    log_dens_gamma(f->f->f->levels[0], alpha0, beta0);
      } else {
	res += log_dens_gamma(f->f->f->levels[updL], alpha * beta,
			      alpha / f->f->f->levels[updL - 1]);
      }
    }
    if (children) {
      /* Calculate the prior densities of the child nodes, which
	 depend on 'f', if needed: */
      for (j = 0; j < f->neighbour_dim; j++) {
	if (f->neighbour_next[j])
	  res += returnLexisMfLogPriorDensity(ind, f->neighbour_next[j], -1,
					      f->f->f->jumpPoints[updL],
					      f->f->f->jumpPoints[updL + 1],
					      alpha0, beta0, alpha, beta, lexis_weight,
					      FALSE);
      }
    }
  }
  return(res);
}

double returnMfLogPriorDensity(long ind, mf f)
{
  long i, j, k = 0;
  double w, alpha0, beta0, alpha, beta, mu, res = 0.0, sum_aver;
  double *lexis_weight = NULL;

  switch (f->prior) {
  case GAMMA:
    alpha0 = calculate2(ind, f->pVal->pGa->alpha0);
    beta0 = calculate2(ind, f->pVal->pGa->beta0);
    alpha = calculate2(ind, f->pVal->pGa->alpha);
    beta = calculate2(ind, f->pVal->pGa->beta);
    mu = calculate2(ind, f->pVal->pGa->mu);
    if (mu)
      res = -f->length * mu + LOG(mu) * f->f->f->n - 2;
    if (f->restrict0 < 0)
      res += LOG(beta0) * alpha0 - lgamma(alpha0) + 
	LOG(dens_gamma(f->f->f->levels[0], alpha0, beta0));
    for (i = 1; i < f->f->f->n - 1; i++) {
      res += LOG(alpha / f->f->f->levels[i - 1]) * alpha * beta -
	lgamma(alpha * beta) +
	LOG(dens_gamma(f->f->f->levels[i], alpha * beta,
		       alpha / f->f->f->levels[i - 1]));
    }
    break;
  case GAMMA_LEXIS:
    alpha0 = calculate2(ind, f->pVal->pGL->alpha0);
    beta0 = calculate2(ind, f->pVal->pGL->beta0);
    alpha = calculate2(ind, f->pVal->pGL->alpha);
    beta = calculate2(ind, f->pVal->pGL->beta);
    lexis_weight = 
      malloc(f->pVal->pGL->lexis_dim * sizeof(alpha0));
    for (i = 0; i < f->pVal->pGL->lexis_dim; i++)
      lexis_weight[i] = calculate2(ind, f->pVal->pGL->lexis_weight[i]);
    mu = calculate2(ind, f->pVal->pGL->mu);
    if (mu)
      res = -f->length * mu + LOG(mu) * (f->f->f->n - 2);
    for (j = 0; j < f->neighbour_dim; j++) {
      if (f->neighbour_prev[j])
	k++;
    }
    if (k) {
      /* if (f->restrict0 > 0.0)
	 f->f->f->levels[0] = f->restrict0; */
      for (i = ((f->restrict0 > 0) ? 1 : 0); i < f->f->f->n - 1; i++) {
	w = (i) ? 1.0 : 0.0;
	sum_aver = (i) ? f->f->f->levels[i - 1] : 0.0;
	/* if (i) {
	  sum_aver += f->f->f->levels[i - 1];
	  k++;
	  } */
	for (j = 0; j < f->neighbour_dim; j++) {
	  if (f->neighbour_prev[j]) {
	    w += lexis_weight[j];
	    sum_aver += 
	      intSf(f->neighbour_prev[j]->f->f,
		    f->f->f->jumpPoints[i], f->f->f->jumpPoints[i + 1]) /
	      (f->f->f->jumpPoints[i + 1] - f->f->f->jumpPoints[i]) * lexis_weight[j];
	    /* k++; */
	  }
	}
	res += LOG(alpha / (sum_aver / (w))) * alpha * beta -
	  lgamma(alpha * beta) +
	  LOG(dens_gamma(f->f->f->levels[i], alpha * beta,
			 alpha / (sum_aver / (w))));
      }
    }
    else {
      if (f->restrict0 < 0)
	res += LOG(beta0) * alpha0 - lgamma(alpha0) + 
	  LOG(dens_gamma(f->f->f->levels[0], alpha0, beta0));
      for (i = 1; i < f->f->f->n - 1; i++) {
	res += LOG(alpha / f->f->f->levels[i - 1]) * alpha * beta -
	  lgamma(alpha * beta) +
	  LOG(dens_gamma(f->f->f->levels[i], alpha * beta,
			 alpha / f->f->f->levels[i - 1]));
      }
      /* for (i = 0; i < f->f->f->n - 1; i++) {
	res += LOG(beta0) * alpha0 - lgamma(alpha0) + 
	  LOG(dens_gamma(f->f->f->levels[0], alpha0, beta0));
	  } */
    }
    free(lexis_weight);
    break;
  case INCR_GAMMA:
    alpha0 = calculate2(ind, f->pVal->pGa->alpha0);
    beta0 = calculate2(ind, f->pVal->pGa->beta0);
    alpha = calculate2(ind, f->pVal->pGa->alpha);
    beta = calculate2(ind, f->pVal->pGa->beta);
    mu = calculate2(ind, f->pVal->pGa->mu);
    if (mu)
      res = -f->length * mu + LOG(mu) * f->f->f->n - 2;
    if (f->restrict0 < 0)
      res += LOG(beta0) * alpha0 - lgamma(alpha0) + 
	LOG(dens_gamma(f->f->f->levels[0], alpha0, beta0));
    for (i = 1; i < f->f->f->n - 1; i++)
      res += LOG(beta) * alpha - lgamma(alpha) +
	LOG(dens_gamma(f->f->f->levels[i] -
		       f->f->f->levels[i - 1], alpha, beta));
    break;
  case BROWNIAN:
    {
      long i;
      double delta = calculate2(-1, f->pVal->pBM->delta),
	drift = calculate2(-1, f->pVal->pBM->drift),
	sigma2 = calculate2(-1, f->pVal->pBM->sigma2);
      res = 0.0;
      for (i = 1; i < f->f->f->n - 1; i++)
	res += LOG(dens_norm(f->f->f->levels[i] - f->f->f->levels[i - 1],
			     drift * delta, sigma2));
    }
  break;
  default:
    res = 0.0;
    /* bite_eprintf( "Not implemented.\n");
       bite_exit(1); */
  }
  return res;
}

void printParam(long ind, prior ptr)
{
  long i;
  prior list;

  bite_printf("Parameter %s: fixed %ld, distribution %ld.\n", ptr->name, ptr->fixed,
	 ptr->type);
  bite_printf("Value (%ld): ", ptr->ptype);
  switch(ptr->ptype) {
  case PT_SCALAR:
  case PT_CONST:
    bite_printf("%f ", paramValue(ind, ptr));
    bite_printf("(transformed %f).\n", calculate2(ind, ptr->transform));
    break;
  case PT_VECTOR:
    list = ptr->x->vec;
    while (list) {
      bite_printf("%f, ", paramValue(ind, list));
      list = list->next;
    }
    bite_printf("\n");
    break;
  case PT_PPROCESS:
    bite_printf("t0: %f, t1: %f, points: ", paramValue(ind, ptr->x->pp->t0),
	   paramValue(ind, ptr->x->pp->t1));
    list = ptr->x->pp->vector;
    while (list) {
      bite_printf("%f, ", paramValue(ind, list));
      list = list->next;
    }
    bite_printf("\n");
    break;
  }
  if (ptr->restrict1)
    bite_printf("\tRestrict: (%f, %f)\n", calculate2(ind, ptr->restrict1),
	   calculate2(ind, ptr->restrict2));
  if (ptr->args) {
    bite_printf("\tArguments: ");
    for (i = 0; i < ptr->nArgs; i++)
      bite_printf("%f, ", calculate2(ind, ptr->args[i]));
    bite_printf("\n");
  }
  printUse(ptr->where);
  if (!ptr->prev)
    bite_printf("\tFirst.\n");
  if (!ptr->prev)
    bite_printf("\tLast.\n");
}

prior returnPrior(ft ptr)
{
  long ii, jj, k;
  prior res = NULL;

  ii = returnType(ptr->varName, &jj, FALSE);
  switch (ii) {
  case PT_SCALAR:
    res = param;
    for (k = 0; k < jj; k++)
      res = res->next;
    if (ptr->ni)
      res = res->vector[(long) calculate2(-1, ptr->i[0])];
    break;
  case PT_COV:
    switch (c->pList[jj]->type) {
    case COV_CNST2:
      res = c->pList[jj]->distr.x;
      break;
    case COV_START2:
      res = c->pList[jj]->distr.x2[(long) calculate2(-1, ptr->i[0])];
      break;
    }
    break;
  case PT_RESP:
    switch (ptr->fieldId) {
    case T0:
      res = d->pList[jj]->distr.pp.t0[(long) calculate2(-1, ptr->i[0])];
      break;
    case T1:
      res = d->pList[jj]->distr.pp.t1[(long) calculate2(-1, ptr->i[0])];
      break;
    case POINTS:
      res = d->pList[jj]->distr.pp.points[(long) calculate2(-1, ptr->i[0])]
	[(long) calculate2(-1, ptr->third->i[0])];
    }
    break;
  default:
    bite_eprintf( "Unknown type!\n");
    bite_exit(1);
  }
  return(res);
}

long returnType(char *name, long *i, long set_prior)
{
  long j = 0, res = NONE;
  prior ptr = param;

  if (name) {
    while (ptr && strcmp(name, ptr->name)) {
      ptr = ptr->next;
      j++;
    }
    if (ptr) {
      res = PT_SCALAR;
      *i = j;
    }
    j = 0;
    while (j < c->n && strcmp(name, c->cList[j][0]->name))
      j++;
    if (j < c->n) {
      res = PT_COV;
      *i = j;
    }
    j = 0;
    while (j < d->n && strcmp(name, d->dList[j][0][0]->name))
      j++;
    if (j < d->n) {
      res = PT_RESP;
      *i = j;
    }
    j = 0;
    while (j < f->n && strcmp(name, f->fList[j][0]->name))
      j++;
    if (j < f->n){
      res = PT_MODELF;
      *i = j;
    }
    if (!res && name[0] && !set_prior) {
      /* if (iter >= 0) { */
      bite_eprintf( "%s: what is that?\n", name);
      bite_exit(1);
      /* }
      else {
	res = PT_SCALAR;
	*i = 0;
      } */
    }
  }
  return res;
}


void generateCovs()
{
  long i, j, cov;
  prior ptr;

  for (cov = 0; cov < c->n; cov++) {
    for (i = 0; i < ind; i++) {
      switch (c->cList[cov][0]->type) {
      case COV_CNST:
	if (!c->cList[cov][i]->x->fixed && c->cList[cov][i]->x->type !=
	    D_DIRAC && c->cList[cov][i]->x->update)
	  c->cList[cov][i]->x->x->x =
	    generatePriorValue(i, c->cList[cov][i]->x);
	break;
      case COV_CNST2:
	if (c->pList[cov]->distr.x &&
	    !checkBit(i % LONGLEN, c->pList[cov]->fixed.x[i / LONGLEN])
	    && c->pList[cov]->distr.x->type != D_DIRAC
	    && c->pList[cov]->distr.x->update)
	  c->pList[cov]->val.x[i] =
	    generatePriorValue(i, c->pList[cov]->distr.x);
	break;
      case COV_START2:
	for (j = 0; j < c->pList[cov]->n2[i]; j++)
	  if (c->pList[cov]->distr.x2 &&
	      c->pList[cov]->distr.x2[j] &&
	      !checkBit(j % LONGLEN,
			c->pList[cov]->fixed.x2[i][j / LONGLEN])
	      && c->pList[cov]->distr.x2[j]->type != D_DIRAC
	      && c->pList[cov]->distr.x2[j]->update)
	    c->pList[cov]->val.x2[i][j] =
	      generatePriorValue(i, c->pList[cov]->distr.x2[j]);
	break;
      case COV_SF:
	break;
      case COV_START:
	ptr = c->cList[cov][i]->x->x->vec;
	while (ptr) {
	  if (!ptr->fixed && ptr->type != D_DIRAC)
	    ptr->x->x = generatePriorValue(i, ptr);
	  ptr = ptr->next;
	}
	break;
      }
    }
  }
}

void generateResp()
{
  long mark, i, car, j;
  prior ptr;

  for (mark = 0; mark < d->n; mark++)
    for (i = 0; i < ind; i++)
      for (car = 0; car < d->nCars[mark]; car++) {
	if (d->pList[mark]) {
	  if (d->pList[mark]->distr.pp.t0[car] &&
	      !d->pList[mark]->fixed.pp.t0[i][car] &&
	      d->pList[mark]->distr.pp.t0[car]->type != D_DIRAC &&
	      d->pList[mark]->distr.pp.t0[car]->update)
	    d->pList[mark]->val.pp.t0[i][car] = 
	      generatePriorValue(i, d->pList[mark]->distr.pp.t0[car]);
	  if (d->pList[mark]->distr.pp.t1[car] &&
	      !d->pList[mark]->fixed.pp.t1[i][car] &&
	      d->pList[mark]->distr.pp.t1[car]->type != D_DIRAC &&
	      d->pList[mark]->distr.pp.t1[car]->update)
	    d->pList[mark]->val.pp.t1[i][car] = 
	      generatePriorValue(i, d->pList[mark]->distr.pp.t1[car]);
	  for (j = 0; j < d->pList[mark]->val.pp.npoints[i][car]; j++)
	    if (d->pList[mark]->distr.pp.points[car][j] &&
		!checkBit(j % LONGLEN,
			  d->pList[mark]->fixed.pp.points[i][car][j /
								 LONGLEN]) &&
		d->pList[mark]->distr.pp.points[car][j]->type != D_DIRAC &&
		d->pList[mark]->distr.pp.points[car][j]->update)
	      d->pList[mark]->val.pp.points[i][car][j] =
		generatePriorValue(i, d->pList[mark]->distr.pp.points[car][j]);
	}
	else {
	  if (d->dList[mark][i][car]->type == CONT) {
	    if (!d->dList[mark][i][car]->pp->t0->fixed &&
		d->dList[mark][i][car]->pp->t0->type != D_DIRAC)
	      d->dList[mark][i][car]->pp->t0->x->x = 
		generatePriorValue(i, d->dList[mark][i][car]->pp->t0);
	    if (!d->dList[mark][i][car]->pp->t1->fixed &&
		d->dList[mark][i][car]->pp->t1->type != D_DIRAC)
	      d->dList[mark][i][car]->pp->t1->x->x = 
		generatePriorValue(i, d->dList[mark][i][car]->pp->t1);
	    ptr = d->dList[mark][i][car]->pp->vector;
	    while (ptr) {
	      if (!ptr->fixed && ptr->type != D_DIRAC)
		ptr->x->x = generatePriorValue(i, ptr);
	      ptr = ptr->next;
	    }
	  }
	}
      }
}

void writeLogFileParam(prior param)
{
  /* The scalar and vector parameters: */
  prior ptr = param;

  while (ptr) {
    if (ptr->ptype == PT_VECTOR && !ptr->type) {
      writeLogFileParam(ptr->x->vec);
      bite_printf("\n");
    }
    else {
      if (!ptr->fixed) {
	bite_printf("Parameter ");
	printName(ptr);
	bite_printf("\n");
	printUse(ptr->where);
      }
    }
    ptr = ptr->next;
  }
}

void writeLogFileVar()
{
  int i;

  for (i = 0; i < nVars; i++) {
    bite_printf("Model function prior %s.\n", vars[i]->name);
    printUse(vars[i]->where);
  }
}

void writeLogFileCov()
{
  long cov = 0, i;

  for (; cov < c->n; cov++) {
    switch (c->pList[cov]->type) {
    case COV_CNST2:
      if (c->pList[cov]->distr.x) {
	bite_printf("Covariate %s:\n", c->pList[cov]->name);
	printUse(c->pList[cov]->distr.x->where);
      }
      break;
    case COV_START2:
      for (i = 0; i < c->pList[cov]->stor; i++) {
	if (c->pList[cov]->distr.x2[i]) {
	  bite_printf("Covariate %s[%ld]:\n", c->pList[cov]->name, i);
	  printUse(c->pList[cov]->distr.x2[i]->where);
	}
      }
      break;
    default:
      break;
    }
  }
}

void writeLogFileResp()
{
  long mark = 0, car, i, random;

  for (; mark < d->n; mark++) {
    for (car = 0; car < d->pList[mark]->stor; car++) {
      random = FALSE;
      if (d->pList[mark]->distr.pp.t0[car] ||
	  d->pList[mark]->distr.pp.t1[car])
	random = TRUE;
      else
	for (i = 0; i < d->pList[mark]->distr.pp.npoints[car]; i++)
	  if (d->pList[mark]->distr.pp.points[car][i])
	    random = TRUE;
      if (random) {
	bite_printf("Response %s, observation interval %ld:\n",
	       d->pList[mark]->name, car);
	if (d->pList[mark]->distr.pp.t0[car]) {
	  bite_printf("    t0:\n");
	  printUse(d->pList[mark]->distr.pp.t0[car]->where);
	}
	if (d->pList[mark]->distr.pp.t1[car]) {
	  bite_printf("    t1:\n");
	  printUse(d->pList[mark]->distr.pp.t1[car]->where);
	}
	for (i = 0; i < d->pList[mark]->distr.pp.npoints[car]; i++)
	  if (d->pList[mark]->distr.pp.points[car][i]) {
	    bite_printf("    point %ld:\n", i);
	    printUse(d->pList[mark]->distr.pp.points[car][i]->where);
	  }
      }
    }
  }
}

void writeLogFileMF()
{
  long i = 0, j, mark, k, indic;

  for (; i < f->n; i++)
    for (j = 0; j < f->nFunc[i]; j++) {
      indic = FALSE;
      for (mark = 0; mark < d->n; mark++)
	for (k = 0; k < ind / LONGLEN; k++)
	  indic += (f->fList[i][j]->inUse[mark][k]) ? 1 : 0;
      if (indic) {
	bite_printf("Model function %s(", f->fList[i][0]->name);
	k = 0;
	while (f->fList[i][0]->cov[k][0]) {
	  bite_printf("%s=%g,", f->fList[i][j]->cov[k],
		 f->fList[i][j]->covVal[k]);
	  k++;
	}
	bite_printf(")\n\t");
	for (mark = 0; mark < d->n; mark++) {
	  indic = FALSE;
	  for (k = 0; k < ind / LONGLEN; k++)
	    indic += (f->fList[i][j]->inUse[mark][k]) ? 1 : 0;
	  if (indic)
	    bite_printf("%s, ", d->pList[mark]->name);
	}
	bite_printf("\n");
      }
    }
}
	

void writeLogFile()
{
  /* Print the likelihood contributions of parameters. */

  writeLogFileParam(param);
  writeLogFileVar();
  writeLogFileCov();
  writeLogFileResp();
  writeLogFileMF();
}

long normalApprox(double y1, double y2, double y3, double theta, double eps,
		  double *mu, double *sigma2)
{
  /* Find a normal approximation for a full conditional distribution.
   Input:
   y1, y2, y3 are log-densities at points
   theta-eps, theta, theta+eps.
   Output:
   mean (*mu) and variance (*sigma2). */

  double a = (y1-y2+(y3-y1)/2) / pow(eps, 2.0),
    b = (y3-y1-4*a*theta*eps) / (2*eps);

  if (a >= 0) {
    /* No log-concavity*/
    *mu = 0.0;
    *sigma2 = 1e+10;
    return(FALSE);
  }
  *mu = -b/(2*a);
  *sigma2 = -1/(2*a);
  return(TRUE);
}


#endif
