/* Tommi Harkanen (C) 2000 */
/* Read the file copyright.txt for details */

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <time.h>
#include <string.h>
#include "typedef.h"
#include "sf3.h"
#include "general3.h" 
#include "bit.h"
#include "load.h"
#include "modelf.h"
#include "ftree.h"
#include "bite.tab.h"
#include "prior.h"

/* This module does the calculation of the likelihood. */

/* Definitions of the functions */
double logLikel(mf f, long updJ, long updL, dt d, ff model);

/* Global variables */
extern long max_iter, iter_limit, skip, drawMod, printLevel, ind, nVars,
  nextI, tries, fails, frames, iter, zeroLikel, zeroLI, dumttt,
  lindex[MAX_STACK_SIZE], level;
extern double G_annealing;
extern dt d;
extern ct c;
extern mft f;
extern ff model;

/* Calculate the likelihood of the individual i */
double indLogL(long i, long mark, mf target, double below, double above,
	       dt d, ff model)
{
  long j, k, t, start, stop, null = TRUE;
  double res = 0.0, dum, bot, top, sum, sum1, x, y;
  sf lambda;
  prior ptr;

/* If we are calculating the value of the likelihood with respect to some new 
   parameter, then do the following: */

  if (printLevel > 3)
    bite_printf("%ld: ", i);

  t = 0;
  while (t < d->nCars[mark] && null) {
    if (!(d->pList[mark] &&
	respValue(i, t, 0, T0, d->pList[mark]) ==
	respValue(i, t, 0, T1, d->pList[mark])))
      null = FALSE;
    t++;
  }
  if (null)
    return(0.0);

/* Calculate the intensity function 'lambda'. 
  putSfToFt(i, mark, target, updJ, updL, model->ftList[mark][ind], 
             model->ftList[mark][i], d, f); */
  putSfToFt2(i, mark, target, below, above, model->ftList[mark][0], 
	     model->ftList[mark][0], d, f);
  if (printLevel > 3) {
    inorder(i, model->ftList[mark][0]); 
    bite_printf("\n");
  }
  lambda = calculate(i, model->ftList[mark][0]);
  if (printLevel >= 3)
    printSf(lambda);

/* Then do the calculation of the likelihood. */
  for (t = 0; t < ((d->pList[mark]) ? d->pList[mark]->n2[i] : d->nCars[mark]);
       t++) {
    if (!d->pList[mark] && (d->dList[mark][i][t]->type == DISCR1 || 
			    d->dList[mark][i][t]->type == DISCR2 || 
			    d->dList[mark][i][t]->type == DISCR3)) {
      if (d->dList[mark][i][t]->begin <= lambda->jumpPoints[lambda->n - 1] &&
	  d->dList[mark][i][t]->end >= lambda->jumpPoints[0] &&
	  d->dList[mark][i][t]->name != '\0') {
	switch (d->dList[mark][i][t]->type) {
	case DISCR1:
	  if (d->dList[mark][i][t]->begin < lambda->jumpPoints[0])
	    start = ((lambda->jumpPoints[0] - d->dList[mark][i][t]->begin) /
		     d->dList[mark][i][t]->precision);
	  else
	    start = 0;
	  if (d->dList[mark][i][t]->end > lambda->jumpPoints[lambda->n - 1])
	    stop = d->dList[mark][i][t]->nObs - 
	      ((d->dList[mark][i][t]->end - 
		lambda->jumpPoints[lambda->n - 1]) /
	       d->dList[mark][i][t]->precision);
	  else
	    stop = d->dList[mark][i][t]->nObs;
	  for (j = start; j < stop; j++) {
	    if (printLevel >= 3)
	      bite_printf("(%ld: %ld ", j, d->dList[mark][i][t]->dData[j]);
	    if (d->dList[mark][i][t]->dData[j] >= 0) {
	      dum = intSf(lambda, 
			  d->dList[mark][i][t]->begin + 
			  d->dList[mark][i][t]->precision * j, 
			  d->dList[mark][i][t]->begin + 
			  d->dList[mark][i][t]->precision * (j + 1));
	      if (d->dList[mark][i][t]->formatTop[
		    d->dList[mark][i][t]->dData[j]] != -1) {
		sum = 0.0;
		for (k = d->dList[mark][i][t]->formatBot[
			 d->dList[mark][i][t]->dData[j]];
		     k <= d->dList[mark][i][t]->formatTop[
			  d->dList[mark][i][t]->dData[j]]; k++)
		  sum += exp(k * log(dum)) / factorial(k);
		res += log(sum) - dum;
		if (printLevel >= 3)
		  bite_printf("%g) ", log(sum) - dum);
	      }
	      else {
		sum = 0.0;
		for (k = 0; k < d->dList[mark][i][t]->formatBot[
			        d->dList[mark][i][t]->dData[j]]; k++)
		  sum += exp(k * log(dum)) / factorial(k);
		if (sum * exp(-dum) < 1.0) {
		  res += log(1.0 - sum * exp(-dum));
		  if (printLevel >= 3)
		    bite_printf("%g) ", log(1.0 - sum * exp(-dum)));
		  if (1.0 - sum * exp(-dum) <= 0.0)
		    bite_printf("%g\n", sum * exp(-dum));
		}
		else {
		  sum1 = 0.0;
		  sum = 0.0;
		  k = d->dList[mark][i][t]->formatBot[
			d->dList[mark][i][t]->dData[j]];
		  do {
		    sum += exp(k * log(dum)) / factorial(k);
		    k++;
		    if (sum == sum1)
		      break;
		    sum1 = sum;
		  } while (1 == 1);
		  res += log(sum) - dum;
		  if (printLevel >= 3)
		    bite_printf("%g) ", log(sum) - dum);
		}
	      }
	    }
	  }
	  break;
	case DISCR2:
	  if (d->dList[mark][i][t]->begin < lambda->jumpPoints[0])
	    start = ((lambda->jumpPoints[0] - d->dList[mark][i][t]->begin) /
		     d->dList[mark][i][t]->precision);
	  else
	    start = 0;
	  if (d->dList[mark][i][t]->end > lambda->jumpPoints[lambda->n - 1])
	    stop = d->dList[mark][i][t]->nObs - 
	      ((d->dList[mark][i][t]->end - 
		lambda->jumpPoints[lambda->n - 1]) /
	       d->dList[mark][i][t]->precision);
	  else
	    stop = d->dList[mark][i][t]->nObs;
	  for (j = start; j < stop; j++) {
	    if (printLevel >= 3)
	      bite_printf("(%ld: %ld ", j, d->dList[mark][i][t]->dData[j]);
	    if (d->dList[mark][i][t]->dData[j] >= 0) {
	      dum = intSf(lambda, 
			  d->dList[mark][i][t]->begin + 
			  d->dList[mark][i][t]->precision * j, 
			  d->dList[mark][i][t]->begin + 
			  d->dList[mark][i][t]->precision * (j + 1));
	      if (printLevel >= 3)
		bite_printf("%g ", d->dList[mark][i][t]->dData[j] * LOG(dum) - 
		       LOG(factorial(d->dList[mark][i][t]->dData[j])) - dum);
	      res += d->dList[mark][i][t]->dData[j] * LOG(dum) - 
		LOG(factorial(d->dList[mark][i][t]->dData[j])) - dum;
	    }
	    if (printLevel >= 3)
	      bite_printf(") ");
	  }
	  break;
	case DISCR3:
	  start = 0;
	  if (d->dList[mark][i][t]->begin < lambda->jumpPoints[0])
	    while (start + 2 < d->dList[mark][i][t]->nObs &&
		   d->dList[mark][i][t]->times[start + 1] <
		   lambda->jumpPoints[0])
	      start++;
	  stop = d->dList[mark][i][t]->nObs - 1;
	  if (d->dList[mark][i][t]->end > lambda->jumpPoints[lambda->n - 1])
	    while (stop - 1 >= 0 &&
		   d->dList[mark][i][t]->times[stop - 1] >=
		   lambda->jumpPoints[lambda->n - 1])
	      stop--;
	  for (j = start; j <= stop; j++) {
	    if (printLevel >= 3)
	      bite_printf("(%ld: %ld ", j, d->dList[mark][i][t]->dData[j]);
	    if (d->dList[mark][i][t]->dData[j] >= 0) {
	      dum = intSf(lambda,
			  (!j) ? d->dList[mark][i][t]->begin :
			  d->dList[mark][i][t]->times[j - 1], 
			  (j < d->dList[mark][i][t]->nObs - 1) ?
			  d->dList[mark][i][t]->times[j] : 
			  d->dList[mark][i][t]->end);
	      if (printLevel >= 3)
		bite_printf("%g ", d->dList[mark][i][t]->dData[j] * LOG(dum) - 
		       LOG(factorial(d->dList[mark][i][t]->dData[j])) - dum);
	      res += d->dList[mark][i][t]->dData[j] * LOG(dum) - 
		LOG(factorial(d->dList[mark][i][t]->dData[j])) - dum;
	    }
	    if (printLevel >= 3)
	      bite_printf(") ");
	  }
	  break;
	}
      }
    }
    else {
      double a = bot = (d->pList[mark]) ?
	respValue(i, t, 0, T0, d->pList[mark]) :
	paramValue(i, d->dList[mark][i][t]->pp->t0),
        b = top = (d->pList[mark]) ? respValue(i, t, 0, T1, d->pList[mark]) :
	paramValue(i, d->dList[mark][i][t]->pp->t1);
      /*if ((a < above && b > below && d->dList[mark][0][0]->name != '\0')) {*/
      if (d->dList[mark][0][0]->name != '\0') {
	if (d->dList[mark][0][0]->type == CONT) {
	  if (target) { 
	    if (a < lambda->jumpPoints[0])
	      bot = lambda->jumpPoints[0];
	    if (b > lambda->jumpPoints[lambda->n - 1])
	      top = lambda->jumpPoints[lambda->n - 1];
	  }
	  else {
	    if (below > bot)
	      bot = below;
	    if (above < top)
	      top = above;
	  }
	  if (top < bot) { /* If the sets are disjoint */
	    bot = -100.0;
	    top = -100.0;
	  }
	  if (a > b) {
	    bite_eprintf( "%g<%g\n", a, b);
	    bite_eprintf( "Something wrong with the data! "
		    "(i==%ld, mark==%s)\n", i, d->pList[mark]->name);
	    bite_exit(1);
	  }
	  res -= intSf(lambda, bot, top);
	  if (printLevel >= 3)
	    bite_printf("(%g %g: %g ", bot, top, -intSf(lambda, bot, top));
	  if (d->pList[mark]) {
	    for (j = 0; j < d->pList[mark]->val.pp.npoints[i][t]; j++) {
	      x = respValue(i, t, j, POINTS, d->pList[mark]);
	      if (x > bot && x <= top) {
		y = value(lambda, x);
		if (y == 0 && dumttt) {
		  bite_printf("Indiv %ld: ", i);
		  printSf(lambda); 
		  bite_printf("%g, %g\n", x, y);
		}
		res += LOG(y);
		if (printLevel >= 3)
		  bite_printf("%g: +%g, ", x, log(y));
	      }
	    }
	  }
	  else {
	    ptr = d->dList[mark][i][t]->pp->vector;
	    while (ptr) {
	      x = paramValue(i, ptr);
	      if (x > bot && x <= top) {
		y = value(lambda, x);
		if (y == 0 && dumttt) {
		  bite_printf("Indiv %ld: ", i);
		  printSf(lambda); 
		  bite_printf("%g, %g\n", x, y);
		}
		res += LOG(y);
		if (printLevel >= 3)
		  bite_printf("%g: +%g, ", x, log(y));
	      }
	      ptr = ptr->next;
	    }
	  }
	  if (printLevel >= 3)
	    bite_printf("=%g)", res);
	  if (res < -1e+50 && printLevel > 1) {
	    bite_printf("|");
	  }
/*	    if (!zeroLI) {
	      zeroLikel++;
	      zeroLI = TRUE;
	    }
	    bite_printf("mark=%ld, ind=%ld, bot=%g, top=%g, ", mark, i, bot, top);
	    printSf(lambda);
	    ptr = d->dList[mark][i][t]->pp->vector;
	    while (ptr) {
	      x = paramValue(i, ptr);
	      if (x > bot && x <= top) {
		y = value(lambda, x);
		bite_printf("%g: +%g, ", x, log(y));
	      }
	      ptr = ptr->next;
	    }
	    bite_printf("\n");
	  } */
	}
      }
    }
  }
  if (printLevel >= 3)
    bite_printf("\n");
  return(res * G_annealing); 
} /* end indLogL */ 

double logLikel2(long ii, mf target, double below, double above, dt d,
		 ff model)
{
/* Calculate the value of the logaritm of the likelihood */ 

  long mark, i, j;
  double res = 0.0;

  if (target->nIndiv >= 0) {
    for (mark = 0; mark < d->n; mark++)
      for (i = 0; i < target->nIndiv; i++)
	if (checkBit(i % LONGLEN, target->inUse[mark][i / LONGLEN]))
	  res += indLogL(target->indivVec[i], mark, target,
			 below, above, d, model);
  }
  else {
    for (mark = 0; mark < d->n; mark++) {
      i = (ii < 0) ? 0 : ii;
      j = (ii < 0) ? ind : ii + 1;
      for (; i < j; i++) {
	if (!(i % LONGLEN) && !target->inUse[mark][i / LONGLEN])
	  i += LONGLEN - 1;
	else
	  if (checkBit(i % LONGLEN, target->inUse[mark][i / LONGLEN]))
	    res += indLogL(i, mark, target, below, above, d, model);
      }
    }
  }

  return(res); 
}

double logLikel(mf target, long updJ, long updL, dt d, ff model)
{
/* Calculate the value of the logaritm of the likelihood */ 

  double below, above;

  if ((updJ < 0 && updL < 0) || target->type != SF_ORIG) {
    below = -1.0e90;
    above = 1.0e90;
  }
  else {
    if (updL < 0) {
      below = target->f->f->jumpPoints[updJ - 1];
      above = target->f->f->jumpPoints[updJ + 1];
    }
    else {
      below = target->f->f->jumpPoints[updL];
      above = target->f->f->jumpPoints[updL + 1];
    }
  }
  return logLikel2(-1, target, below, above, d, model);
  /* for (mark = 0; mark < d->n; mark++)
    for (i = 0; i < ind; i++) {
      if (checkBit(i % LONGLEN, target->inUse[mark][i / LONGLEN]))
	res += indLogL(i, mark, target, below, above, d, model);
    }
  return(res); */
}

double logLikel3(long i, use list, ft indiv_likel)
     /* This one for other parameters than model functions. */
{
  long mark, j = 0, dirac = FALSE;
  double res = 0.0, x;
  use ptr = list, f;

  if (!ptr && i >= 0) {
    /* A covariate has probably been used to condition a model function so
       calculate the likelihood of the point process responces --
       just in case. */
    long mark, found;
    for (mark = 0; mark < d->n; mark++) {
      found = findNewMf(i, mark, model->ftList[mark][0]);
      res += (found) ? indLogL(i, mark, NULL, -INFTY, INFTY, d, model) : 0.0;
    }
  }

  while (ptr) {
    if (!iter && printLevel && ind < 100 &&
	(ptr->type != USE_PRIOR ||
	 (ptr->type == USE_PRIOR && ptr->val->x->type == D_DIRAC)))
      printUse(ptr);
    switch (ptr->type) {
    case USE_TABLE:
      for (mark = 0; mark < d->n; mark++)
	if (ptr->val->inUse[mark]) {
	  res += indLogL(i, mark, NULL, -INFTY, INFTY, d, model);
	}
      break;
    case USE_TABLE2:
      res += logLikel2(i, NULL, -INFTY, INFTY, d, model);
      /* for (mark = 0; mark < d->n; mark++)
	for (j = 0; j < ind; j++) {
	  if (checkBit(j % LONGLEN, ptr->val->inUse2[mark][j / LONGLEN]))
	    res += indLogL(j, mark, NULL, -INFTY, INFTY, d, model);
	} */
      break;
    case USE_PRIOR:
      if (!ptr->val->x->fixed) {
	level++;
	lindex[level] = ptr->val->x->index;
	if (ptr->val->x->type != D_MNORM) {
	  x = (ptr->val->x->transform) ? 
	    calculate2(i, ptr->val->x->transform) : paramValue(i, ptr->val->x);
	  res += returnPriorLogDensity(i, ptr->val->x, x) * G_annealing;
	}
	else {
	  res += returnPriorLogDensity(i, ptr->val->x, 0.0) * G_annealing;
	}
	/* res += (ptr->val->x->type != D_MNORM) ?
	   returnPriorLogDensity(i, ptr->val->x, (ptr->val->x->transform) ? 
	   calculate2(i, ptr->val->x->transform) :
	   paramValue(i, ptr->val->x))) :
	   returnPriorLogDensity(i, ptr->val->x, 0.0));
	   if ((ptr->val->x->type != D_MNORM) ?
	   returnPriorLogDensity(i, ptr->val->x, (ptr->val->x->transform) ? 
	   calculate2(i, ptr->val->x->transform) :
	   paramValue(i, ptr->val->x))) :
	   returnPriorLogDensity(i, ptr->val->x, 0.0)) < -1e+80) {
	   bite_printf("%s,%g,%g, ", ptr->val->x->name, ptr->val->x->x->x, res);
	   if (ptr->val->x->type == D_DIRAC)
	   bite_printf("add %g, ", logLikel3(i, ptr->val->x->where));
	   } */
	if (ptr->val->x->type == D_DIRAC)
	  res += 0.0; /* logLikel3(i, ptr->val->x->where); */
	level--;
      }
      break;
    case USE_MF:
      res += logLikel2(i, ptr->val->f, -INFTY, INFTY, d, model);
      break;
    case USE_VAR: /* The parameter has been used in a model function. */
      f = ptr->val->var->where;
      while (f) {
	switch (f->val->f->type) {
	case SF_ORIG:
	  res += LOG(returnMfPriorDensity(i, f->val->f)) * G_annealing;
	  if (res <= -100) {
	    bite_printf("Low prior density found.\n");
	    returnMfPriorDensity(i, f->val->f);
	  }
	  break;
	case SF_TIMELESS:
	  if (f->val->f->f->p->type == D_DIRAC)
	    res += /* logLikel3(i, f->val->f->f->p->where) +*/
	      logLikel2(i, f->val->f, -INFTY, INFTY, d, model);
	  else
	    res += 
	      returnPriorLogDensity(i, f->val->f->f->p,
				     (f->val->f->f->p->transform) ? 
				     calculate2(i, f->val->f->f->p->transform):
				     paramValue(i, f->val->f->f->p))
	      * G_annealing;
	  break;
	}
	f = f->next;
      }
      break;
    case USE_INDPARAM:
      if (ptr->val->indParam.p->type == D_DIRAC ||
	  (ptr->val->indParam.p->ptype ==  PT_VECTOR &&
	   ptr->val->indParam.p->x->vec &&
	   ptr->val->indParam.p->x->vec->type == D_DIRAC)) {
	res += logLikel3(ptr->val->indParam.i, ptr->val->indParam.p->where,
			 ptr->val->indParam.p->indiv_likel);
      }
      break;
    case USE_COV2:
      switch (ptr->val->pvec->type) {
      case COV_CNST2:
	if (ptr->val->pvec->distr.x->type == D_DIRAC)
	  res += 0.0; /*logLikel3(i, ptr->val->pvec->distr.x->where);*/
	else {
	  if (i < 0) {
	    long j;
	    if (ptr->val->pvec->distr.x->indiv_likel) {
	      for (j = 0; j < ind; j++) {
		if (calculate2(j, ptr->val->pvec->distr.x->indiv_likel)) {
		  res += returnPriorLogDensity(j, ptr->val->pvec->distr.x,
					       ptr->val->pvec->val.x[j])
		    * G_annealing;
		  /*bite_printf("%ld, %g\n", j, res);*/
		}
	      }
	    }
	    else {
	      for (j = 0; j < ind; j++)
		res += returnPriorLogDensity(j, ptr->val->pvec->distr.x,
					     ptr->val->pvec->val.x[j])
		  * G_annealing;
	    }
	  }
	  else {
	    if (!ptr->val->pvec->distr.x->indiv_likel ||
		(ptr->val->pvec->distr.x->indiv_likel && 
		 calculate2(i, ptr->val->pvec->distr.x->indiv_likel))) {
	      res += returnPriorLogDensity(i, ptr->val->pvec->distr.x,
					   ptr->val->pvec->val.x[i])
		* G_annealing;
	      /*if (ptr->val->pvec->distr.x->indiv_likel)
		bite_printf("%ld, %g\n", i, res);*/
	    }
	  }
	}
	break;
      case COV_START2:
	for (j = 0; j < ptr->val->pvec->stor; j++)
	  if (ptr->val->pvec->distr.x2[j]) {
	    if (ptr->val->pvec->distr.x2[j]->type == D_DIRAC && !dirac) {
	      res += 0.0; /*logLikel3(i, ptr->val->pvec->distr.x2[j]->where); */
	      dirac = TRUE;
	    }
	    else {
	      if (i < 0) {
		long k;
		level++;
		lindex[level] = j;
		if (ptr->val->pvec->distr.x2[j]->indiv_likel) {
		  for (k = 0; k < ind; k++) {
		    if (calculate2(j, ptr->val->pvec->distr.x2[j]->indiv_likel))
		      res += returnPriorLogDensity(k, ptr->val->pvec->distr.x2[j],
						   ptr->val->pvec->val.x2[k][j])
			* G_annealing;
		  }
		}
		else {
		  for (k = 0; k < ind; k++)
		    res += returnPriorLogDensity(k, ptr->val->pvec->distr.x2[j],
						 ptr->val->pvec->val.x2[k][j])
		      * G_annealing;
		}
		level--;
	      }
	      else {
		level++;
		lindex[level] = j;
		res += returnPriorLogDensity(i, ptr->val->pvec->distr.x2[j],
					     ptr->val->pvec->val.x2[i][j])
		   * G_annealing;
		level--;
	      }
	    }
	  }
	break;
      }
      break;
    }
    ptr = ptr->next;
  }

  return res;
}
