/* Tommi Harkanen (C) 2000 */
/* Read the file copyright.txt for details */

#ifndef MODELFN
#define MODELFN
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include "typedef.h"
#include "sf3.h"
#include "general3.h"
#include "bit.h"
#include "load.h"
#include "prior.h"
#include "ftree.h"
#include "modelf.h"

extern ct c;
extern dt d; /* Point process responce. */
extern long ind;

/* long printLevel = 1;

void main()
{
  long i, ind = 24, dim, elem;
  mf *z;
  covNode *c;
  char **covs, dum;

  covs = (char **) malloc(2 * sizeof(c));
  for (i = 0; i < 2; i++) {
    covs[i] = (char *) malloc(20 * sizeof(dum));
    covs[i][0] = '\0';
  }
  strcat(covs[1], "sex"); 
  strcat(covs[0], "indiv"); 
  c = initCov(245, 3);
  dim = covCars(c);
  loadCov("birth.dat", "birth", ind, COV_START, -999, c);
  loadCov("sex.dat", "sex", ind, COV_CNST, -999, c);
  loadCov("indiv.dat", "indiv", ind, COV_CNST, -999, c);
  z = makeModel("Z", 45, 2, covs, c, ind, &elem);
  bite_printf("%ld\n", elem);
  for (i = 0; i < elem; i++)
    printMf(z[i]);
} */

mf initMf(char *name, double length, double restrict0, long ind,
	  long fixedIndivVec)
{
  long i, j;
  mf x;
  char dum;

  x = mfAlloc();
  x->name[0] = '\0';
  strcat(x->name, name);
  x->internOp = '\0';
  x->monitor = NONE;
  x->wait = 0;

  x->save = NONE; /* This is nonzero, if the values of the function will be */
  x->file = -1;   /* saved to a file during the simulation -- or loaded from */
                  /* a file. */
  x->epsilon = 0.05;

  for (i = 0; i < MAX_COV; i++) {
    /*bite_printf("malloc(%ld).\n", (long) 20 * sizeof(dum));*/
    x->cov[i] = (char *) malloc(20 * sizeof(dum));
    x->cov[i][0] = '\0';
    x->covVal[i] = 0.0;
  }
  x->neighbour_dim = 0;
  x->neighbour_prev = NULL;
  x->neighbour_next = NULL;
  x->type = NONE;
  /*bite_printf("malloc(%ld).\n", (long) (sizeof(fType)));*/
  x->f = (fType *) malloc(sizeof(fType));
  x->f->f = NULL;
  x->nfixjp = 0;
  x->fixjp = NULL;
  x->nfixle = 0;
  x->fixle_x = NULL;
  x->fixle_y = NULL;
  x->levels = NULL;
  x->jumpPoints = NULL;
  /*bite_printf("malloc(%ld).\n", (long) d->n * sizeof(x));*/
  x->inUse = malloc(d->n * sizeof(x));
  if (fixedIndivVec) {
    for (i = 0; i < d->n; i++)
      /*bite_printf("malloc(%ld).\n", (long) (ind / LONGLEN + 1) * sizeof(ind));*/
      x->inUse[i] = NULL;
    x->nIndiv = 0;
    x->indivVec = NULL;
  }
  else {
    x->nIndiv = -1;
    x->indivVec = NULL;
    for (i = 0; i < d->n; i++) {
      x->inUse[i] = (long *) malloc((ind / LONGLEN + 1) * sizeof(ind));
      for (j = 0; j < ind / LONGLEN + 1; j++)
	x->inUse[i][j] = 0;
    }
  }
  x->riskSet = NULL;
  x->length = length;
  x->restrict0 = restrict0;
  x->minPieceLength = 0.0;
/*  x->mu = 0.0; */
  x->prior = -1;
  x->pVal = NULL;
  /*  x->title[0] = '\0'; */
  return(x);
}

mft initMft(long storage)
{
  long i;
  mf dum;
  mft res;

  res = mftAlloc();
  res->n = 0;
  res->storage = storage;
  res->nFunc = (long *) malloc(storage * sizeof(i));
  res->fList = (mf **) malloc(storage * sizeof(dum));
  for (i = 0; i < storage; i++)
    res->fList[i] = NULL;

  return(res);
}

double **initMedian(long elem)
{
  long i, j;
  double x, **res;

  res = (double **) malloc(GP * sizeof(res));
  for (i = 0; i < GP; i++) {
    if (!(res[i] = (double *) malloc( elem * sizeof( x ) ))) {
      bite_eprintf( "Out of Memory -> CRASH!!.\n"
	      "Try to decrease either the number of plots with credibility "
	      "intervals\nor (iter_limit - skip)/thinning.\n");
      bite_exit(1);
    }
    for (j = 0; j < elem; j++)
      res[i][j] = 0.0;
  }

  return(res);
}

sf priorSf0(double aa, double bb, double mu, double alpha0, 
	    double beta0, double alpha, double beta, double restrict0,
	    double minPieceLength)
{
  sf f;

  f = poiProcess(aa, bb, mu, minPieceLength, NULL);
  levelsG(f, alpha0, beta0, alpha, beta, restrict0);
  return(f);
}

sf priorSf1(double aa, double bb, double mu, double a, 
	    double w, double restrict0, double minPieceLength)
{
  sf f;

  f = poiProcess(aa, bb, mu, minPieceLength, NULL);
  levelsPD(f, a, w, restrict0);
  return(f);
}

void my_free(void *dummy)
{
  bite_printf("*");
}

/* void my_realloc(void *dummy, size_t size)
{
  void n;

  bite_printf("+");
  return malloc(size_t size);
} */

mf *makeModel(char *name, double length, double restrict0,
	      labelArray covs, ct c, long ind, long *elem, long fixedIndivVec)
{
  long i, j, j2, k, l, n = 1, *m = NULL, *m_cum = NULL,  /* *m_coord = NULL,  *m_coord2 = NULL, */ *idx_v = NULL;
  double **clist = NULL, *cov_values = NULL /* , *cov_values2 = NULL */;
  mf *res;

  if (covs->n) {
    idx_v = malloc(covs->n * sizeof(i));
    m = malloc(covs->n * sizeof(i));
    cov_values = malloc(covs->n * sizeof(length));
    /* cov_values2 = malloc(covs->n * sizeof(length)); */
    m_cum = malloc(covs->n * sizeof(i));
    /* m_coord = malloc(covs->n * sizeof(i));
       m_coord2 = malloc(covs->n * sizeof(i)); */
    clist = malloc(covs->n * sizeof(clist));

  /* Calculate the number of different values of each covariate (m[]) and the
     number of possible combinations of them (n)
     for (i = 0; i < covs->n; i++) {
     clist[i] = (double *) malloc(ind * sizeof(restrict0));
     clist[i] = covLevels(c, covs->label[i]->label, ind, &m[i], clist[i]);
     n *= m[i];
     } */
    for (i = 0; i < covs->n; i++) {
      k = returnType(covs->label[i]->label, &j, FALSE);
      clist[i] = c->values[j];
      m[i] = c->nval[j];
      n *= m[i];
    }
    m_cum[covs->n - 1] = 1;
    if (covs->n > 1) {
      for (i = covs->n - 2; i >= 0; i--) {
	m_cum[i] = m_cum[i+1] * m[i];
      }
    }
  }

  /* Initialize the model functions */
  res = (mf *) malloc(n * sizeof(res));
  for (i = 0; i < n; i++) {
    res[i] = initMf(name, length, restrict0, ind, fixedIndivVec);
    res[i]->neighbour_dim = covs->n;
    res[i]->neighbour_prev = 
      malloc(covs->n * sizeof(clist));
    res[i]->neighbour_next = 
      malloc(covs->n * sizeof(clist));
    /*    bite_printf("%p:\n", res[i]->f); */
    k = n;
    l = i;
    for (j = 0; j < covs->n; j++) {
      res[i]->neighbour_prev[j] = NULL;
      res[i]->neighbour_next[j] = NULL;
      m_cum[j] = k /= m[j];
      strcat(res[i]->cov[j], covs->label[j]->label);
      res[i]->covVal[j] = clist[j][l / k];
      l = l % k;
    }
  }

  /* Create the links to previous and next elements according to the
     covariate values: */
  for (j = 0; j < covs->n; j++) {
    idx_v[j] = 0;
  }
  for (i = 0; i < n; i++) {
    k = n;
    l = i;
    for (j = 0; j < covs->n; j++) {
      /* Find the coordinates of the i'th element: */
      k /= m[j];
      idx_v[j] = l / k;
      cov_values[j] = clist[j][l / k];
      l = l % k;
    }
    for (j = 0; j < covs->n; j++) {
      if (idx_v[j] > 0) {
	/* If there is a smaller covariate value (neighbour), then
	   create pointer in 'prev': */
	idx_v[j]--;
	k = 0;
	for (j2 = 0; j2 < covs->n; j2++)
	  k += idx_v[j2] * m_cum[j2];
	/* bite_printf("prev. k=%ld, j=%ld\n", k, j); */
	res[i]->neighbour_prev[j] = res[k];
	idx_v[j]++;
      }
      if (idx_v[j] < m[j] - 1) {
	/* If there is a larger covariate value (neighbour), then
	   create pointer in 'next': */
	idx_v[j]++;
	k = 0;
	for (j2 = 0; j2 < covs->n; j2++)
	  k += idx_v[j2] * m_cum[j2];
	/* bite_printf("next, k=%ld, j=%ld\n", k, j); */
	res[i]->neighbour_next[j] = res[k];
	idx_v[j]--;
      }
    }
  }

  /* Free memory:
     for (i = 0; i < covs->n; i++)
     free(clist[i]); */
  if (covs->n) {
    free(clist);
    free(m);
    free(m_cum);
    free(idx_v);
  }

  /* Return values: */
  *elem = n;
  return(res);
  }

/* Check if the covariates & their values are the same as of the model
   function. */
long testCovEq(mf f, labelArray covs)
{
  long m, n, boolean = 1;
  for (m = 0; m < covs->n; m++)
    for (n = 0; n < MAX_COV; n++)
      if (!strcmp(covs->label[m]->label, f->cov[n]))
	if (covs->label[m]->value != f->covVal[n])
	  boolean = 0;

  return boolean;
}

mf *findMf2(labelArray covs, mft f)
{
  mf *res = NULL;
  long i = 0, m, boolean;

  while (i < f->n && !res) {
    boolean = 1;
    for (m = 0; m < covs->n; m++)
      if (strcmp(covs->label[m]->label, f->fList[i][0]->cov[m]))
	boolean = 0;
    if (boolean && f->fList[i][0]->cov[covs->n - 1][0] &&
	!f->fList[i][0]->cov[covs->n][0])
      res = f->fList[i];
    i++;
  }
  return res;
}

void printMf(mf f)
{
  long i;

  bite_printf("Model function %s. Covariate(s):\n", f->name);
  for (i = 0; i < MAX_COV; i++)
    if (f->cov[i][0] != '\0')
      bite_printf("\t%s == %f\n", f->cov[i], f->covVal[i]);
  if (f->pVal) {
    switch (f->prior) {
    case GAMMA:
      bite_printf("alpha0: %f, beta0: %f, alpha: %f, mu: %f\n",
	     calculate2(-1, f->pVal->pGa->alpha0), 
	     calculate2(-1, f->pVal->pGa->beta0),
	     calculate2(-1, f->pVal->pGa->alpha),
	     calculate2(-1, f->pVal->pGa->mu));
      break;
    }
  }
  if (f->f)
    printSf(f->f->f);
  bite_printf("Length: %f\n", f->length);
  bite_printf("\n");
}

void printMf2(mf f)
{
  long j = 0;

  bite_printf("function %s(", f->name);
  while (f->cov[j][0]) {
    if (j)
      bite_printf(", ");
    bite_printf("%s == %g", f->cov[j], f->covVal[j]);
    j++;
  }
  bite_printf(")");
}

long findMf(mft f, char *name, long nc, char **cov, double *values)
{
  long i = 0, j = -1, m, n, boolean = FALSE;

  while (strcmp(f->fList[i][0]->name, name))
    i++;
  boolean = TRUE;
  for (m = 0; m < nc; m++) {
    boolean *= (!strcmp(cov[m], f->fList[i][0]->cov[m])) ? TRUE :  FALSE;
  }
  boolean *= (!f->fList[i][0]->cov[m][0]) ? TRUE :  FALSE;
  if (nc) {
    if (boolean) {
      boolean = FALSE;
      while (!boolean) {
	j++;
	boolean = TRUE;
	for (m = 0; m < nc; m++) {
	  boolean *= (values[m] == f->fList[i][j]->covVal[m]) ? TRUE :  FALSE;
	}
      }
    }
    else {
      boolean = FALSE;
      while (!boolean) {
	j++;
	boolean = TRUE;
	for (m = 0; m < nc; m++) {
	  for (n = 0; n < MAX_COV && f->fList[i][j]->cov[n][0]; n++)
	    if (!strcmp(cov[m], f->fList[i][j]->cov[n]))
	      boolean *= (values[m] == f->fList[i][j]->covVal[n]) ?
		TRUE :  FALSE;
	}
      }
    }
  }
  else
    j++;
  /* return f->fList[i][j]; */
  return j;
}

long findNewMf(long i, long mark, ft ptr)
     /* This is for updating the covariates: if the value of an covariate
	changes, we need to check, whether there is a change in the
	likelihood. */
{
  long res = 0, j, m, n = 0, found = FALSE;
  static double values[MAX_COV];

  if (ptr) {
    if (ptr->left)
      res += findNewMf(i, mark, ptr->left);
    if (ptr->right)
      res += findNewMf(i, mark, ptr->right);
    else {
      while (ptr->f[0]->cov[n][0])
	n++;
      if (n) {
	for (j = 0; j < n; j++) {
	  /* k = returnType(ptr->f[0]->cov[j], &m, FALSE); */
	  values[j] = covValue(i, 0, c->pList[j]);
	}
	j = 0;
	while (!found && j < ptr->storageF) {
	  found = TRUE;
	  for (m = 0; m < n; m++)
	    found *= (values[m] == ptr->f[j]->covVal[m]) ? TRUE :  FALSE;
	  j++;
	}
	j--;
	if (found && ptr->fc[i] != j) {
	  putBitTo0(i % LONGLEN,
		    &ptr->f[ptr->fc[i]]->inUse[mark][i / LONGLEN]);
	  putBitTo1(i % LONGLEN, &ptr->f[j]->inUse[mark][i / LONGLEN]);
	  ptr->fc[i] = j;
	  res = found;
	}
      }
    }
  }
  return res;
}

long findMfO(mft f, char *name, long nc, char **cov, double *values)
{
  long i = 0, j = 0, m, n, boolean = FALSE;

  while (strcmp(f->fList[i][0]->name, name))
    i++;
  if (nc) {
    while (!boolean) {
      boolean = TRUE;
      for (m = 0; m < nc; m++) {
	for (n = 0; n < MAX_COV && f->fList[i][j]->cov[n][0]; n++)
	  if (!strcmp(cov[m], f->fList[i][j]->cov[n]))
	    boolean *= (values[m] == f->fList[i][j]->covVal[n])
	      ? TRUE : FALSE;
      }
      j++;
    }
    return(j - 1);
  }
  else
    return(0);
}

long cmpCovs(mf f, labelArray a, long i)
{
  long m, n, boolean, *ok;

  if (a) { /* Compare the covariates. */
    if (a->n == 0)
      return(TRUE);
    ok = (long *) malloc(a->n * sizeof(m));
    for (m = 0; m < a->n; m++)
      ok[m] = FALSE;
    for (m = 0; m < a->n; m++)
      for (n = 0; n < MAX_COV; n++)
	if (!strcmp(a->label[m]->label, f->cov[n]))
	  if (a->label[m]->value == f->covVal[n])
	    ok[m] = TRUE;
    boolean = ok[0];
    for (m = 1; m < a->n; m++)
      boolean *= ok[m];
    free(ok);
  }
  else { /* Check whether the covariate values of the i'th individual
	    match the values of f. */
    ok = (long *) malloc(c->n * sizeof(m));
    for (m = 0; m < c->n; m++)
      ok[m] = TRUE;
    for (m = 0; m < c->n; m++)
      for (n = 0; n < MAX_COV && f->cov[n][0]; n++)
	if (!strcmp(c->cList[m][0]->name, f->cov[n]))
	  if (((c->pList[m]) ? covValue(i, 0, c->pList[m]) :
	      paramValue(i, c->cList[m][i]->x)) != f->covVal[n])
	    ok[m] = FALSE;
    boolean = ok[0];
    for (m = 1; m < c->n; m++)
      boolean *= ok[m];
    free(ok);
  }

  return(boolean);
}
    
long findInd2(mf f)
{
  /* Find an individual which has the same covariate values as the model
     function f. */
  long cov[MAX_COV], j, j1 = 0, v;
  long i = -1;

  while (f->cov[j1][0])
    j1++;
  /* for (j = 0; j < j1; j++)
     dum = returnType(f->cov[j], &(cov[j]), FALSE); */
  do {
    i++;
    v = TRUE;
    for (j = 0; j < j1; j++)
      v *= (((c->pList[cov[j]]) ? covValue(i, 0, c->pList[cov[j]]) :
	     paramValue(i, c->cList[cov[j]][i]->x)) == f->covVal[j]) ? 1 : 0;
  } while (i < ind && !v);
  if (i == ind) {
    bite_eprintf( "Couldn't find an individual with covariate values ");
    for (j = 0; j < j1; j++)
      bite_eprintf( "%s=%g ", f->cov[j], f->covVal[j]);
    bite_exit(1);
  }
  return i;
}

mf mfAlloc()
{
  return( (mf) malloc( sizeof( jackmf ) ) );
}

mft mftAlloc()
{
  return( (mft) malloc( sizeof( jackmft ) ) );
}

#endif
