/* Tommi Harkanen (C) 2000 */
/* Read the file copyright.txt for details */

#ifndef PRIOR
#define PRIOR
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include "typedef.h"

extern dt d;

pGamma pgAlloc()
{
  return( (pGamma) malloc( sizeof( jackpg ) ) );
}

pGamma initPg()
{
  pGamma x;

  x = pgAlloc();
  x->alpha0 = NULL;
  x->beta0 = NULL;
  x->alpha = NULL;
  x->beta = NULL;
  x->mu = NULL;
  x->mix = 0.5; /* Probability of choosing the narrow uniform prior. */
  return(x);
}

pGamma_lexis pglAlloc()
{
  return( (pGamma_lexis) malloc( sizeof( jackpgl ) ) );
}

pGamma_lexis initPgl()
{
  pGamma_lexis x;

  x = pglAlloc();
  x->alpha0 = NULL;
  x->beta0 = NULL;
  x->alpha = NULL;
  x->lexis_dim = 0;
  x->lexis_weight = NULL;
  x->beta = NULL;
  x->mu = NULL;
  x->mix = 0.5; /* Probability of choosing the narrow uniform prior. */
  return(x);
}

pBrownian initPbm()
{
  pBrownian x = malloc( sizeof( jackbrownian ) );

  x->delta = NULL;
  x->drift = NULL;
  x->sigma2 = NULL;
  x->mix = 0.5; /* Probability of choosing the narrow uniform prior. */
  return(x);
}

pPairdiff ppAlloc()
{
  return( (pPairdiff) malloc( sizeof( jackpp ) ) );
}

pPairdiff initPp(double a, double w)
{
  pPairdiff x;

  x = ppAlloc();
  x->a = a;
  x->w = w;
  return(x);
}

ptl pTlAlloc()
{
  return(malloc( sizeof( jackpTL ) ) );
}

ptl initTl()
{
  ptl x;

  x = pTlAlloc();
  x->p = NULL;
  return(x);
}

pJuha pJuhaAlloc()
{
  return( (pJuha) malloc( sizeof( jackpj ) ) );
}

pJuha initpJuha(double mu, double sigma2, double beta, double lambdaXi,
		double c, double cc, double delta)
{
  pJuha x;

  x = pJuhaAlloc();
  x->mu = mu;
  x->sigma2 = sigma2; /* Variance */
  x->beta = beta;
  x->lambdaXi = lambdaXi;
  x->c = c; /* 0.0 < c < 0.5 */
  x->cc = cc;
  x->delta = delta;
  return(x);
}

variable initVar(char name[20])
{
  variable x;

  x = (variable) malloc(sizeof(jackVariable));
/*  x->name = (char *) malloc(MAX_VARNAME + 1); */
  x->name[0] = '\0';
  strcat(x->name, name);
  x->type = 0;
  x->spec = 0;
  x->x = (vValue *) malloc(sizeof(vValue));
  x->where = NULL;

  return(x);
}

use initUse()
{
  use res;

  res = (use) malloc(sizeof(jackUse));

  res->type = NONE;
  res->val = (useVal *) malloc(sizeof(useVal));
  res->next = NULL;
  res->prev = NULL;
  res->back = NULL; /* Which parameters are pointing back? */

  return res;
}

void addUse(prior where, prior x)
{
  use y = where->where;

  while (y->next)
    y = y->next;
  y->next = initUse();
  y->next->type = USE_PRIOR;
  y->next->val->x = x;
}

use addUse2(use root)
{
  use ptr = root, new = NULL, next;

  new = initUse();
  if (ptr) {
    next = ptr->next;
    ptr->next = new;
    new->next = next;
    if (next)
      new->next->prev = new;
    new->prev = ptr;
  }

  return new;
}

use addUseLast(use root)
{
  use ptr = root, new = NULL;

  new = initUse();
  /* Search for the last nodel ... */
  while (ptr && ptr->next)
    ptr = ptr->next;
  /* And add the new node after that: */
  if (ptr) {
    ptr->next = new;
    new->prev = ptr;
  }

  return new;
}

void printName(prior p)
{
  if (p->up) {
    printName(p->up);
    bite_printf("[%s]", p->name);
  }
  else
    bite_printf("%s", p->name);
}

void printUse(use root)
{
  if (root) {
    long i;
    switch (root->type) {
    case USE_TABLE:
      bite_printf("\tResponse: ");
      for (i = 0; i < d->n; i++)
	if (root->val->inUse[i])
	  bite_printf("%s ", d->dList[i][0][0]->name);
      bite_printf("\n");
      break;
    case USE_PRIOR:
      bite_printf("\tParameter ");
      printName(root->val->x);
      bite_printf("\n");
      break;
    case USE_MF:
      i = 0;
      bite_printf("\tModel function %s(", root->val->f->name);
      while (root->val->f->cov[i][0]) {
	bite_printf("%s=%g,", root->val->f->cov[i], root->val->f->covVal[i]);
	i++;
      }
      bite_printf(")\n");
      break;
    case USE_VAR:
      bite_printf("\tVariable %s.\n", root->val->var->name);
      break;
    case USE_INDPARAM:
      bite_printf("\tIndividual %ld:\n", root->val->indParam.i);
      printUse(root->val->indParam.p->where);
      break;
    case USE_COV2:
      bite_printf("\tUsing covariate %s.\n", root->val->pvec->name);
      break;
    }
    printUse(root->next);
  }
}

void deleteOverlappingUse(use root, use nextroot)
{
  long i, j, found;
  use ptr1=nextroot, ptr2=root, newptr, f;
  /* Check links which have Dirac distribution
     in order to avoid calculating likelihood terms more
     than once.
     Arguments:
     root := root of the use-list of parameter "A".
     nextroot := root of the use-list of another parameter which
     has a Dirac distribution and parameter "A" as an argument.
  */
  if (root == nextroot) {
    /* Parameter "A": */
    
    while (ptr2) {
      switch (ptr2->type) {
      case USE_PRIOR:
	if (ptr2->val->x->type == D_DIRAC)
	  deleteOverlappingUse(root, ptr2->val->x->where);
	break;
      case USE_VAR: /* The parameter has been used in a model function. */
	f = ptr2->val->var->where;
	while (f) {
	  switch (f->val->f->type) {
	  case SF_ORIG:
	    break;
	  case SF_TIMELESS:
	    if (f->val->f->f->p->type == D_DIRAC)
	      deleteOverlappingUse(root, f->val->f->f->p->where);
	    break;
	  }
	  f = f->next;
	}
	break;
      case USE_INDPARAM:
	if (ptr2->val->indParam.p->type == D_DIRAC ||
	    (ptr2->val->indParam.p->ptype ==  PT_VECTOR &&
	     ptr2->val->indParam.p->x->vec &&
	     ptr2->val->indParam.p->x->vec->type == D_DIRAC)) {
	  deleteOverlappingUse(root, ptr2->val->indParam.p->where);
	}
	break;
      case USE_COV2:
	switch (ptr2->val->pvec->type) {
	case COV_CNST2:
	  if (ptr2->val->pvec->distr.x->type == D_DIRAC)
	    deleteOverlappingUse(root, ptr2->val->pvec->distr.x->where);
	  break;
	case COV_START2:
	  for (j = 0; j < ptr2->val->pvec->stor; j++)
	    if (ptr2->val->pvec->distr.x2[j]) {
	      if (ptr2->val->pvec->distr.x2[j]->type == D_DIRAC)
		deleteOverlappingUse(root, ptr2->val->pvec->distr.x2[j]->where);
	    }
	  break;
	}
	break;
      }
      ptr2 = ptr2->next;
    }
  }
  else {
    /* A parameter with Dirac distribution using parameter "A": */
    while (ptr1 != NULL) {
      found = FALSE;
      ptr2 = root;
      while (!found && ptr2 != NULL) {
	switch (ptr1->type) {
	case USE_TABLE:
	  if (ptr2->type == ptr1->type) {
	    for (i = 0; i < d->n; i++)
	      if (ptr1->val->inUse[i] || ptr2->val->inUse[i])
		ptr2->val->inUse[i] = 1;
	    found = TRUE;
	  }
	  break;
	case USE_PRIOR:
	  if (ptr1->val->x->type == D_DIRAC) {
	    deleteOverlappingUse(root, ptr1->val->x->where);
	    found = TRUE;
	  }
	  break;
	case USE_MF:
	  if (ptr1->val->f == ptr2->val->f)
	    found = TRUE;
	  break;
	case USE_VAR:
	  if (ptr1->val->var == ptr2->val->var)
	    found = TRUE;
	  break;
	case USE_INDPARAM:
	  if (ptr1->val->indParam.i == ptr2->val->indParam.i &&
	      ptr1->val->indParam.p == ptr2->val->indParam.p)
	    found = TRUE;
	  break;
	case USE_COV2:
	  switch (ptr2->val->pvec->type) {
	  case COV_CNST2:
	    if (ptr1->val->pvec == ptr2->val->pvec)
	      found = TRUE;
	    break;
	  case COV_START2:
	    if (ptr1->val->pvec == ptr2->val->pvec)
	      found = TRUE;
	    break;
	  }
	  break;
	}
	ptr2 = ptr2->next;
      }
      /* If match not found, then create a new node to list 'root': */
      newptr = addUseLast(root);
      free(newptr->val);
      /* and assign the value of 'ptr1' to the new node: */
      newptr->type = ptr1->type;
      newptr->val = ptr1->val;
      ptr1 = ptr1->next;
    }
  }
}

clusterUpdate initCU(long n, char *name)
{
  char ch;
  clusterUpdate c = malloc(sizeof(ClusterUpdate));

  c->name = malloc((strlen(name) + 1) * sizeof(ch));
  sprintf(c->name, "%s", name);
  c->n = n;
  c->vec = malloc(n * sizeof(c));
  c->new_val = malloc(n * sizeof(c));

  return c;
}

void initCU2(long dim, long nl, clusterUpdate c)
{
  long i, j, k;
  long dum;
  double missing;

  c->nl = nl;
  c->qn_xy = 0;
  c->qn_yx = 0;
  c->dim = NULL;
  c->last = malloc(dim * sizeof(c));
  c->mean = malloc(dim * sizeof(c));
  c->cov = malloc(dim * sizeof(c));
  c->sd = malloc(dim * sizeof(missing));
  c->last_update = malloc(dim * sizeof(dum));
  for (k = 0; k < dim; k++) {
    c->sd[k] = 1.0;
    c->last_update[k] = 0;
    c->last[k] = malloc(c->n * sizeof(missing));
    c->mean[k] = malloc(c->n * sizeof(missing));
    c->cov[k] = malloc(c->n * sizeof(c));
    for (i = 0; i < c->n; i++) {
      c->mean[k][i] = 0.0;
      c->cov[k][i] = malloc(c->n * sizeof(missing));
      for (j = 0; j < c->n; j++)
	c->cov[k][i][j] = 0.0;
      c->cov[k][i][i] = 0.0;
    }
  }
  c->likel = malloc(nl * sizeof(c));
  c->prop_dens_xy = NULL;
  c->prop_dens_yx = NULL;
}

gradientUpdate initGU(long n, char *name)
     /* Gradient updating */
{
  char ch;
  long i;
  double dbl;
  gradientUpdate g = malloc(sizeof(GradientUpdate));

  g->name = malloc((strlen(name) + 1) * sizeof(ch));
  sprintf(g->name, "%s", name);
  g->nLoop = 30; /* Choose an even, positive integer for this */

  g->n = n;
  g->vec = malloc(n * sizeof(g));
  g->nl = 0;
  g->likel = NULL;

  g->lx = malloc((g->nLoop+1) * sizeof(dbl));
  g->lx0 = malloc((g->nLoop+1) * sizeof(dbl));

  g->gr = malloc((g->nLoop+1) * sizeof(g));
  g->values = malloc((g->nLoop+1) * sizeof(g));
  g->values0 = malloc((g->nLoop+1) * sizeof(g));
  for (i = 0; i < g->nLoop + 1; i++) {
    g->gr[i] = malloc(n * sizeof(dbl));
    g->values[i] = malloc(n * sizeof(dbl));
    g->values0[i] = malloc(n * sizeof(dbl));
  }
  g->next = NULL;

  g->mm_p = -1;
  g->mm_mu = 0;
  g->mm_sigma2 = 1;

  return g;
}

prior initPrior(char *name)
{
  prior res;

  res = (prior) malloc(sizeof(jackPrior));

  res->name = malloc(strlen(name) + 1);
  sprintf(res->name, "%s", name);
  res->index = 0;
  res->fixed = FALSE;
  res->wait = 0;
  res->update = TRUE;
  res->burn = 1;

  res->type = NONE;
  res->monitor = 0;

  res->stor = 0;
  res->vector = NULL;

  res->nolikelihood = FALSE;
  res->transform = NULL;
  res->restrict1 = NULL;
  res->restrict2 = NULL;
  res->mix = 0.0;
  res->epsilon = 0.1;
  res->save = 0;
  /*  res->fp = NULL; */
  res->file = -1;
  res->nArgs = NONE;
  res->args = NULL;
  res->where = NULL;
  res->indiv_likel = NULL;
  res->cut = NULL;
  res->ptype = NONE;
  res->x = (pType *) malloc(sizeof(pType));
  /*  res->sample = NULL;
  res->ngt = 0;
  res->gt = NULL;
  res->nlt = 0;
  res->lt = NULL; */
  res->prev = NULL;
  res->next = NULL;
  res->up = NULL;

  return res;
}

void putPrior(prior last, prior x)
{
  prior next;

  x->prev = last;
  if (last) {
    next = last->next;
    x->next = next;
    last->next = x;
    if (next) {
      x->next = next;
      next->prev = x;
    }
  }
}
    

#endif
