/* Tommi Harkanen (C) 2000 */
/* Read the file copyright.txt for details */

#ifndef SF3
#define SF3
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <unistd.h>
#include "typedef.h"
#include "pipefiles.h"

/* Version with with vector 'generating points' in a sf. */

/* #define MAX_JUMPS 20 */

/* typedef struct simpleF *sf;

struct simpleF {
  long storage;
  long n;
  double *genPoints;
  double *jumpPoints;
  double *levels;
} jack; */


/* Prototypes of functions */ 
  sf makeVector(long length, double jp, double le, sf old);
  void addNode(sf f, long i, double jp, double le);
  void addGP(sf f, long i, double gp);
  void delNode(sf f, long i);
  long nElem(sf f); 
  void printSf(sf f);
  void writeSf(long file, sf f, double nextI);
  void readSf(long file, sf f, double nextI);

  sf initSf(double jp, double le, sf old);
  double value(sf f, double x); 
  sf copySf(sf f, sf g);
  sf moveSf(sf f, double delta, sf old);
  void move2Sf(sf f, double delta);
  sf cutSf(sf f, double a, double b, sf old);
  void cut2Sf(sf f, double a, double b);
  sf multSf(sf f1, sf f2, sf old);
  sf addSf(sf f1, sf f2, sf old); 
  sf multSf2(sf f1, sf f2, sf old);
  double intSf(sf f, double a, double b);

  sf sfAlloc();
#ifdef DEBUG_SF3
extern long sf3_print;
extern FILE *debug_sf3;
#endif

void increaseStorage(sf f, long n)
{
  extern long printLevel;
  double esim;
  /*  cholRow *newChol; */

  if (printLevel >= 2) {
    /* bite_printf("Requesting more memory for a simple function: %ld -> %ld.\n",
	   f->storage, n + 5);
    printSf(f); */
    fflush(NULL);
  }
  if (n > 10000) {
    printSf(f);
    bite_eprintf( "Model function too big (%ld > 10000).\n", n);
    bite_exit(1);
  }
  f->storage = n + 5;
  f->genPoints = realloc(f->genPoints, f->storage * sizeof(esim));
  f->jumpPoints = realloc(f->jumpPoints, f->storage * sizeof(esim));
  f->levels = realloc(f->levels, f->storage * sizeof(esim));
  /*    newChol = (cholRow *) malloc(2 * f->storage * sizeof(cholRow));
	for (j = 0; j < f->n; j++) {
	newGP[j] = f->genPoints[j];
	newJP[j] = f->jumpPoints[j];
	newL[j] = f->levels[j];
	newChol[j].diag = f->chol[j].diag;
	newChol[j].offDiag = f->chol[j].offDiag;
	}
	free(f->genPoints);
	free(f->jumpPoints);
	free(f->levels);
	free(f->chol);
	f->genPoints = newGP;
	f->jumpPoints = newJP;
	f->levels = newL;
	f->chol = newChol; */
  if (printLevel >= 2) {
    /* bite_printf("done.\n"); */
    fflush(NULL);
  }
}

sf makeVector(long length, double jp, double le, sf old)
{
  long i;
  sf f;

  if (length > MAX_JUMPS)
    bite_printf("Too many jump points!!!\n");
  f = initSf(jp, le, old);
  for (i = 1; i < length; i++)
    addNode(f, i, jp, le);
  f->n = length;
  return(f);
} 

void addNode(sf f, long i, double jp, double le)
{
  if (f->n + 1 >= f->storage)
    increaseStorage(f, f->n + 1);
#ifdef DEBUG_SF3
  if (sf3_print) {
    fprintf(debug_sf3, "jp = %g, le = %g, i = %ld, f->n = %ld, "
	    "f->storage = %ld\n", jp, le, i, f->n, f->storage);
    fflush(NULL);
  }
#endif
  if (i < f->n) {
    long j;
    for (j = f->n; j > i; j--) {
      f->jumpPoints[j] = f->jumpPoints[j - 1];
      f->levels[j] = f->levels[j - 1];
    }
    f->jumpPoints[i] = jp;
    f->levels[i] = le;
    f->n++;
  }
  else {
    if (i == f->n) {
      f->jumpPoints[i] = jp;
      f->levels[i] = le;
      f->n++;
    }
    else {
      bite_eprintf( "Index %ld out of range {0, ..., %ld}.\n",
	      i, f->n);
      bite_exit(1);
    }
  }
#ifdef DEBUG_SF3
  if (sf3_print) {
    fprintf(debug_sf3, "jp = %g, le = %g, i = %ld, f->n = %ld, "
	    "f->storage = %ld\n", jp, le, i, f->n, f->storage);
    fflush(NULL);
  }
#endif
}

void addGP(sf f, long i, double gp)
{
  if (i < f->n) {
    long j;
    for (j = f->n; j > i; j--)
      f->genPoints[j] = f->genPoints[j - 1];
    f->genPoints[i] = gp;
  }
  else
    f->genPoints[i] = gp;
}

void delNode(sf f, long i)
{
  if (i < f->n - 1) {
    long j;
    for (j = i; j < f->n - 1; j++) {
      f->jumpPoints[j] = f->jumpPoints[j + 1];
      f->levels[j] = f->levels[j + 1];
    }
  }
  f->n--;
}

void delGP(sf f, long i)
{
  if (i < f->n - 1) {
    long j;
    for (j = i; j < f->n - 1; j++)
      f->genPoints[j] = f->genPoints[j + 1];
  }
}

long nElem(sf f)
{
  return(f->n);
}

void printSf(sf f)
{
  long i;

  bite_printf("Values: ");
  if (f->n > 1) {
    for (i = 0; i < f->n - 1; i++)
      bite_printf("(%ld, %g, %g), ", i, f->jumpPoints[i], f->levels[i]);
    bite_printf("(%ld, %g, NA)\n", f->n - 1, f->jumpPoints[f->n - 1]);
  }
  else
    bite_printf("0.\n");
} 

void writeSf(long file, sf f, double nextI)
{
  long i;
  char str[40];

  for (i = 0; i < f->n - 1; i++) {
    sprintf(str, "%g %g\n", f->jumpPoints[i], f->levels[i]);
    writeFileS(file, str);
  }
  if (f->n < 1) {
    sprintf(str, "0 0\n");
    writeFileS(file, str);
  }
  sprintf(str, "%g %g\n\n", f->jumpPoints[f->n - 1], nextI);
  writeFileS(file, str);
} 

void readSf(long file, sf f, double nextI)
{
  double a, b, count = 0;

  f->n = 0;
  do {
    count += 1;
    a = readFileD(file);
    if (count == 1 && a != 0)
      bite_printf("Missed a zero! (a == %g)\n", a);
    b = readFileD(file);
#ifdef DEBUG_SF3
    fprintf(debug_sf3, "a = %g, b = %g\n", a, b);
    fflush(NULL);
#endif
    addNode(f, f->n, a, b);
#ifdef DEBUG_SF3
    if (count == 50) {
      printSf(f);
      bite_printf("ReadSf(%ld) %g (%g,%g) %ld\n", file, count, a, b, f->storage);
      fflush(NULL);
      bite_exit(1);
    }
#endif
  } while (b != nextI);
}

void readSf2(FILE *file, sf f, double nextI)
{
  double a, b;

  /*  while (!feof(fp) && ((c = fgetc(fp)) == ' ' || c == '\t' || 
      c == '\n'));
  if (feof(fp))
    rewind(fp);
  else
    ungetc(c, fp); */
  f->n = 0;
  do {
    fscanf(file, "%lf", &a);
    fscanf(file, "%lf", &b);
    addNode(f, f->n, a, b);
  } while (b != nextI);
}

sf initSf(double jp, double le, sf old)
{
  sf newF;
  double esim;

  if (!old) {
    newF = sfAlloc();
    newF->storage = MAX_JUMPS;
  }
  else
    newF = old;
  newF->n = 1;
  newF->genPoints = (double *)malloc(MAX_JUMPS * sizeof(esim));
  newF->jumpPoints = (double *)malloc(MAX_JUMPS * sizeof(esim));
  newF->jumpPoints[0] = jp;
  newF->levels = (double *)malloc(MAX_JUMPS * sizeof(esim));
  newF->levels[0] = le;
/*  newF->chol = (double *)malloc(MAX_JUMPS * sizeof(cholRow)); */
  return( newF );
}

void delSf(sf old)
{
  free(old->genPoints);
  free(old->jumpPoints);
  free(old->levels);
  free(old);
}

void freeSf(sf f)
{
  if (f) {
    free(f->jumpPoints);
    free(f->levels);
    free(f);
  }
}

double value(sf f, double x)
{
  long i = 0;

  if ( x <= f->jumpPoints[0] || x > f->jumpPoints[f->n - 1])
    return(0.0);
  else {
    while ( x > f->jumpPoints[i + 1] ) 
      i++;
    return(f->levels[i]);
  }
}

sf copySf(sf f, sf g)
{
  long i;

  if (f) {
    if (!g)
      g = initSf(0.0, 0.0, g);
    if (f->n > g->storage)
      increaseStorage(g, f->n);
    g->n = f->n;
    for (i = 0; i < f->n; i++) {
      g->genPoints[i] = f->genPoints[i];
      g->jumpPoints[i] = f->jumpPoints[i];
      g->levels[i] = f->levels[i];
    }
  }
  return g;
}

sf moveSf(sf f, double delta, sf old)
{
  long i;
  sf g;

  if (!old)
    g = initSf(f->jumpPoints[0] + delta, f->levels[0], NULL);
  else
    g = old;
  if (g->storage <= f->n)
    increaseStorage(g, f->n);
  for (i = 0; i < f->n; i++) {
    g->genPoints[i] = f->genPoints[i] + delta;
    g->jumpPoints[i] = f->jumpPoints[i] + delta;
    g->levels[i] = f->levels[i];
  }
  g->n = f->n;
  return(g);
}

void move2Sf(sf f, double delta)
{
  long i;

  for (i = 0; i < f->n; i++) {
    f->genPoints[i] += delta;
    f->jumpPoints[i] += delta;
  }
}

sf cutSf(sf f, double a, double b, sf old)
{
  long i = 0, j = 1;
  sf g;

  if (a < f->jumpPoints[f->n - 1] && b > f->jumpPoints[0]) {
    if (b > f->jumpPoints[f->n - 1])
      b = f->jumpPoints[f->n - 1];
    if (a < f->jumpPoints[0])
      a = f->jumpPoints[0];
    while (f->jumpPoints[i + 1] <= a) 
      i++;
    if (!old) 
      g = initSf(a, f->levels[i++], NULL);
    else {
      g = old;
      g->jumpPoints[0] = a;
      g->levels[0] = f->levels[i];
      g->n = 1;
      i++;
    }
    while (f->jumpPoints[i] < b) {
      addNode(g, j, f->jumpPoints[i], f->levels[i]);
      i++;
      j++;
    }
    addNode(g, j, b, 0.0);
  }
  else {
    if (!old)
      g = initSf(0.0, 0.0, NULL);
    else {
      g = old;
      g->jumpPoints[0] = 0.0;
      g->levels[0] = 0.0;
      g->n = 1;
    }
    /* addNode(g, 1, 0.0, 0.0); */
  }
  return(g);
}  

void cut2Sf(sf f, double a, double b)
{
  long i = 0, j = 1;

  if (a < f->jumpPoints[f->n - 1] && b > f->jumpPoints[0]) {
    if (b > f->jumpPoints[f->n - 1])
      b = f->jumpPoints[f->n - 1];
    if (a < f->jumpPoints[0])
      a = f->jumpPoints[0];
    while (a >= f->jumpPoints[i + 1])
      i++;
    for (j = i; j < f->n - 1; j++) {
      f->jumpPoints[j - i] = f->jumpPoints[j];
      f->levels[j - i] = f->levels[j];
    }
    if (a > f->jumpPoints[0])
      f->jumpPoints[0] = a;
    f->n -= i;
    i = f->n - 1;
    while (f->jumpPoints[i - 1] > b)
      i--;
    f->jumpPoints[i] = b;
    f->n = i + 1;
  }
  else {
    f->jumpPoints[0] = 0.0;
    f->levels[0] = 0.0;
    f->n = 1;
    /* addNode(f, 1, 0.0, 0.0); */
  }
}  

sf multSf(sf fn1, sf fn2, sf old)
{
  double min, max, dumJ = 0;
  long i1 = 0, i2 = 0, j = 1;
  sf res;

  if (fn1->jumpPoints[0] >= fn2->jumpPoints[fn2->n - 1] || 
      fn2->jumpPoints[0] >= fn1->jumpPoints[fn1->n - 1] ||
      fn1->n < 2 || fn2->n < 2) {
    if (!old)
      res = initSf(0.0, 0.0, NULL);
    else {
      res = old;
      res->jumpPoints[0] = 0.0;
      res->levels[0] = 0.0;
      res->n = 1;
    }
    addNode(res, 1, 0.0, 0.0);
  }
  else {
    while (fn1->jumpPoints[i1 + 1] <= fn2->jumpPoints[0]) 
      i1++;
    while (fn2->jumpPoints[i2 + 1] <= fn1->jumpPoints[0]) 
      i2++;
    min = (fn1->jumpPoints[0] > fn2->jumpPoints[0]) ? fn1->jumpPoints[0] :
      fn2->jumpPoints[0];
    max = (fn1->jumpPoints[fn1->n - 1] < fn2->jumpPoints[fn2->n - 1]) ? 
      fn1->jumpPoints[fn1->n - 1] : fn2->jumpPoints[fn2->n - 1];

    if (!old)
      res = initSf(min, fn1->levels[i1] * fn2->levels[i2], NULL);
    else {
      res = old;
      res->jumpPoints[0] = min;
      res->levels[0] = fn1->levels[i1] * fn2->levels[i2];
      res->n = 1;
    }

    while (fn1->jumpPoints[i1 + 1] < max || fn2->jumpPoints[i2 + 1] < max) {
      if (fn1->jumpPoints[i1 + 1] < fn2->jumpPoints[i2 + 1]) {
	dumJ = fn1->jumpPoints[i1 + 1];
        i1++;
      }
      else {
        if (fn1->jumpPoints[i1 + 1] > fn2->jumpPoints[i2 + 1]) {
	  dumJ = fn2->jumpPoints[i2 + 1];
          i2++;
        }
	else {
	  dumJ = fn2->jumpPoints[i2 + 1];
	  i1++;
	  i2++;
	}
      }
      addNode(res, j, dumJ, fn1->levels[i1] * fn2->levels[i2]);
      j++;
    }
    addNode(res, j, max, 0.0);
  }
  return(res);
}

sf addSf(sf f1, sf f2, sf old)
{
  double max, dumJ = 0.0;
  long i, i1 = 0, i2 = 0, i3, j = 1;
  sf fn1, fn2, fn3, res;

/* If one of the functions is NULL, then we simply return the other function */
  if (f1->n < 2 || f2->n < 2) {
    if (!old)
      old = initSf(0.0, 0.0, NULL);
    if (f1->n < 2)
      copySf(f2, old);
    else
      copySf(f1, old);
    res = old;
  }
  else {
/* Order the functions so that 'fn1' starts before 'fn2'. */
    if (f1->jumpPoints[0] > f2->jumpPoints[0]) {
      fn1 = f2;
      fn2 = f1;
    }
    else {
      fn1 = f1;
      fn2 = f2;
    }

/* If the 'positive areas' of 'fn1' and of 'fn2' are disjoint,
   then the sum 'fn1+fn2' is easy. */
    if (fn2->jumpPoints[0] > fn1->jumpPoints[fn1->n - 1]) {
      if (!old)
	res = initSf(fn1->jumpPoints[0], fn1->levels[0], NULL);
      else {
	res = old;
	res->jumpPoints[0] = fn1->jumpPoints[0];
	res->levels[0] = fn1->levels[0];
	res->n = 1;
      }
      for (i = 1; i < fn1->n; i++)
	addNode(res, i, fn1->jumpPoints[i], fn1->levels[i]);
      res->levels[i - 1] = 0.0;
      if (f1->jumpPoints[f1->n - 1] == f2->jumpPoints[0])
	i--;
      for (j = 0; j < fn2->n; j++)
	addNode(res, i + j, fn2->jumpPoints[j], fn2->levels[j]);
    }
/* Otherwise we have to do more work. */
    else {
/* If 'fn1' starts before 'fn2', then copy the beginning of 'fn1'
   to 'res'. */
      i = 1;
      if (fn1->jumpPoints[0] < fn2->jumpPoints[0]) {
	if (!old)
	  res = initSf(fn1->jumpPoints[0], fn1->levels[0], NULL);
	else {
	  res = old;
	  res->jumpPoints[0] = fn1->jumpPoints[0];
	  res->levels[0] = fn1->levels[0];
	  res->n = 1;
	}
	while (fn1->jumpPoints[i1 + 1] < fn2->jumpPoints[0]) {
	  i1++;
	  addNode(res, i, fn1->jumpPoints[i], fn1->levels[i]);
	  i++;
	}
	if (fn1->jumpPoints[i1 + 1] != fn2->jumpPoints[0])
	  addNode(res, i++, fn2->jumpPoints[0], fn1->levels[i1] + 
		  fn2->levels[0]); 
      }
/* Otherwise both functions start at the same point, and so we just
   initialize the function 'res'. */
      else {
	if (!old)
	  res = initSf(fn2->jumpPoints[0], 
		       fn1->levels[0] + fn2->levels[0], NULL);
	else {
	  res = old;
	  res->jumpPoints[0] = fn2->jumpPoints[0];
	  res->levels[0] = fn1->levels[i1] + fn2->levels[i2];
	  res->n = 1;
	}
      }

/* Now we can process the part, where both functions are positive. */
      max = (fn1->jumpPoints[fn1->n - 1] < fn2->jumpPoints[fn2->n - 1]) ? 
	fn1->jumpPoints[fn1->n - 1] : fn2->jumpPoints[fn2->n - 1];
      while (fn1->jumpPoints[i1 + 1] < max || fn2->jumpPoints[i2 + 1] < max) {
	if (fn1->jumpPoints[i1 + 1] < fn2->jumpPoints[i2 + 1]) {
	  dumJ = fn1->jumpPoints[i1 + 1];
	  i1++;
	}
	else {
	  if (fn1->jumpPoints[i1 + 1] > fn2->jumpPoints[i2 + 1]) {
	    dumJ = fn2->jumpPoints[i2 + 1];
	    i2++;
	  }
	  else {
	    dumJ = fn2->jumpPoints[i2 + 1];
	    i1++;
	    i2++;
	  }
	}
	addNode(res, i, dumJ, fn1->levels[i1] + fn2->levels[i2]);
	i++;
      }

/* Finally we can take care of the last part of the positive area. */
      if (fn1->jumpPoints[fn1->n - 1] == max && 
	  fn2->jumpPoints[fn2->n - 1] == max)
	addNode(res, i, max, 0.0);
      else {
	if (fn1->jumpPoints[fn1->n - 1] == max) {
	  fn3 = fn2;
	  i3 = i2;
	}
	else {
	  fn3 = fn1;
	  i3 = i1;	
	}
	if (fn3->jumpPoints[i3 + 1] != max)
	  addNode(res, i++, max, fn3->levels[i3++]);
	else {
	  i3++;
	}
	for (j = i3; j < fn3->n - 1; j++)
	  addNode(res, i++, fn3->jumpPoints[j], fn3->levels[j]);
	addNode(res, i, fn3->jumpPoints[fn3->n - 1], 0.0);
      }
    }
    /* Delete levels (and corresponding jump points) which are eq. to 0. */
    while (res->levels[0] == 0.0 && res->n > 2)
      delNode(res, 0);
    while (res->levels[res->n - 2] == 0.0 && res->n > 2)
      delNode(res, res->n - 2);
  }
  return(res);
}

sf multSf2(sf f1, sf f2, sf old)
{
  double max, dumJ = 0;
  long i, i1 = 0, i2 = 0, i3, j = 1;
  sf fn1, fn2, fn3, res;

/* If one of the functions is NULL, then we simply return the other function */
  if (f1->n < 2 || f2->n < 2) {
    if (!old)
      old = initSf(0.0, 0.0, NULL);
    if (f1->n < 2)
      copySf(f2, old);
    else
      copySf(f1, old);
    res = old;
  }
  else {
/* Order the functions so that 'fn1' starts before 'fn2'. */
    if (f1->jumpPoints[0] > f2->jumpPoints[0]) {
      fn1 = f2;
      fn2 = f1;
    }
    else {
      fn1 = f1;
      fn2 = f2;
    }

/* If the 'positive areas' of 'fn1' and of 'fn2' are disjoint,
   then the sum 'fn1*fn2' is easy. */
    if (fn2->jumpPoints[0] > fn1->jumpPoints[fn1->n - 1]) {
      if (!old)
	res = initSf(fn1->jumpPoints[0], fn1->levels[0], NULL);
      else {
	res = old;
	res->jumpPoints[0] = fn1->jumpPoints[0];
	res->levels[0] = fn1->levels[0];
	res->n = 1;
      }
      for (i = 1; i < fn1->n; i++)
	addNode(res, i, fn1->jumpPoints[i], fn1->levels[i]);
      res->levels[i - 1] = 0.0;
      if (f1->jumpPoints[f1->n - 1] == f2->jumpPoints[0])
	i--;
      for (j = 0; j < fn2->n; j++)
	addNode(res, i + j, fn2->jumpPoints[j], fn2->levels[j]);
    }
/* Otherwise we have to do more work. */
    else {
/* If 'fn1' starts before 'fn2', then copy the beginning of 'fn1'
   to 'res'. */
      i = 1;
      if (fn1->jumpPoints[0] < fn2->jumpPoints[0]) {
	if (!old)
	  res = initSf(fn1->jumpPoints[0], fn1->levels[0], NULL);
	else {
	  res = old;
	  res->jumpPoints[0] = fn1->jumpPoints[0];
	  res->levels[0] = fn1->levels[0];
	  res->n = 1;
	}
	while (fn1->jumpPoints[i1 + 1] < fn2->jumpPoints[0]) {
	  i1++;
	  addNode(res, i, fn1->jumpPoints[i], fn1->levels[i]);
	  i++;
	}
	if (fn1->jumpPoints[i1 + 1] != fn2->jumpPoints[0])
	  addNode(res, i++, fn2->jumpPoints[0], fn1->levels[i1] * 
		  fn2->levels[0]); 
      }
/* Otherwise both functions start at the same point, and so we just
   initialize the function 'res'. */
      else {
	if (!old)
	  res = initSf(fn2->jumpPoints[0], 
		       fn1->levels[0] * fn2->levels[0], NULL);
	else {
	  res = old;
	  res->jumpPoints[0] = fn2->jumpPoints[0];
	  res->levels[0] = fn1->levels[i1] * fn2->levels[i2];
	  res->n = 1;
	}
      }

/* Now we can process the part, where both functions are positive. */
      max = (fn1->jumpPoints[fn1->n - 1] < fn2->jumpPoints[fn2->n - 1]) ? 
	fn1->jumpPoints[fn1->n - 1] : fn2->jumpPoints[fn2->n - 1];
      while (fn1->jumpPoints[i1 + 1] < max || fn2->jumpPoints[i2 + 1] < max) {
	if (fn1->jumpPoints[i1 + 1] < fn2->jumpPoints[i2 + 1]) {
	  dumJ = fn1->jumpPoints[i1 + 1];
	  i1++;
	}
	else {
	  if (fn1->jumpPoints[i1 + 1] > fn2->jumpPoints[i2 + 1]) {
	    dumJ = fn2->jumpPoints[i2 + 1];
	    i2++;
	  }
	  else {
	    dumJ = fn2->jumpPoints[i2 + 1];
	    i1++;
	    i2++;
	  }
	}
	addNode(res, i, dumJ, fn1->levels[i1] * fn2->levels[i2]);
	i++;
      }

/* Finally we can take care of the last part of the positive area. */
      if (fn1->jumpPoints[fn1->n - 1] == max && 
	  fn2->jumpPoints[fn2->n - 1] == max)
	addNode(res, i, max, 0.0);
      else {
	if (fn1->jumpPoints[fn1->n - 1] == max) {
	  fn3 = fn2;
	  i3 = i2;
	}
	else {
	  fn3 = fn1;
	  i3 = i1;	
	}
	if (fn3->jumpPoints[i3 + 1] != max)
	  addNode(res, i++, max, fn3->levels[i3++]);
	else {
	  i3++;
	}
	for (j = i3; j < fn3->n - 1; j++)
	  addNode(res, i++, fn3->jumpPoints[j], fn3->levels[j]);
	addNode(res, i, fn3->jumpPoints[fn3->n - 1], 0.0);
      }
    }
  }
  return(res);
}

void opSf(char op, sf target, sf missile, long init)
{
  static sf f;

  if (init) 
    f = NULL;
  else {
    switch (op) {
    case '*':
      f = multSf(target, missile, f);
      break;
    case '+':
      f = addSf(target, missile, f);
      break;
    case '_': /* RS Changed pykala to _ */
      f = multSf2(target, missile, f);
      break;
    }
    copySf(f, target);
  }
}

double intSf(sf f, double a, double b)
{
  long i = 0;
  double res = 0.0;

  if (a > b) {
    bite_eprintf( "Error when integrating a simple function: ");
    bite_eprintf( "Lower limit is greater than upper.\n");
    printSf(f);
    bite_exit(1);
  }
  if (a <= f->jumpPoints[f->n - 1] && b >= f->jumpPoints[0]) {
    if (a < f->jumpPoints[0])
      a = f->jumpPoints[0];
    if (b > f->jumpPoints[f->n - 1])
      b = f->jumpPoints[f->n - 1];

    while (f->jumpPoints[i + 1] < a) 
      i++;
    if (b <= f->jumpPoints[i + 1])
      res = (b - a) * f->levels[i];
    else {
      res = (f->jumpPoints[i + 1] - a) * f->levels[i];
      if (i < f->n - 1) {
        while (f->jumpPoints[i + 2] < b) {
	  i++;
          res += (f->jumpPoints[i + 1] - f->jumpPoints[i]) * f->levels[i];
	  if (f->jumpPoints[i + 1] < f->jumpPoints[i]) {
	    bite_eprintf( "High risk of error in integrating a sf!!!\n");
	    printSf(f);
	    bite_printf("a==%g b==%g\n", a, b);
	    bite_exit(1);
	  }
        }
      }
      if (b > f->jumpPoints[i + 1])
        res += (b - f->jumpPoints[i + 1]) * f->levels[i + 1];
    }
  }
  return(res);
}

double invIntSf(sf lambda, double lim)
{
  long i;
  double a, b, res; /* Find the point where the cumulative integral of
		       'lambda' exceeds 'lim'. */

  if (lambda->n < 2 || (lambda->n < 2 && lambda->levels[0] == 0.0)) {
    return INFTY;
  }
  else {
    a = lambda->jumpPoints[0];
    b = lambda->jumpPoints[lambda->n - 1];
    if (intSf(lambda, a, b) <= lim)
      res = (lim - intSf(lambda, a, b)) / lambda->levels[lambda->n - 2] +
	lambda->jumpPoints[lambda->n - 1];
    else {
      for (i = 0; i < 15; i++) {
	res = (a + b) / 2.0;
	if (intSf(lambda, lambda->jumpPoints[0], res) < lim)
	  a = res;
	else
	  b = res;
      }
    }
    return res;
  }
}

sf sfAlloc()
{
  return( (sf) malloc( sizeof( jackF ) ) );
}

#endif
