/* Tommi Harkanen (C) 2000 */
/* Read the file copyright.txt for details */

/* #ifndef SPLUSPLOT
#define SPLUSPLOT */
#define _POSIX_SOURCE
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <time.h>
#include <string.h>
#include <unistd.h>
#include "general3.h"
#include "sf3.h"
#include "bit.h"
#include "modelf.h"
#include "ftree.h"
#include "load.h"
#include "bite.tab.h"

#ifndef STANDALONE
#define NOGRAPH 1
#endif

/* Structure for plotting 

typedef struct drawStructure *draw;

In this struct we store information about each frame. How many pictures 
there'll be in that frame (1 <= nPic <= MAX_PICTURES). For each picture 
we determine the following things:
- There can be 3 kinds of pictures:
  1. the mean of estimated stepwise constant functions ('f')
  2. the mean of estimated intensities ('tree')
  3. the estimated posterior density of some scalar parameter ('*pv')
- 'dest' is used when we calculate the intensity function (a technicality):
  You can probably use the routines of the likelihood calculation to get the 
  values of the intensity.
- '*mean' is used to store the values of stepwise constant functions at 
  equally spaced grid points. (The constant GP:=100.)
- If we want to calculate confidence intervals (actually they are 'Bayes 
  probability regions 'bpr'), we need to put the probability to the array 
  'bpr'. (E.g. 90% == 0.9.) Then we need to reserve memory for the matrix 
  of values of the iterations and of the grid points. The matrix is 
  therefore of size GP x (iter_limit - skip), i.e. pretty big, if we are 
  gonna make lots of iterations :-(
- If we are going to put data values to the pictures (especially useful for
  the estimated intensities), we put an '0' to the array 'data', otherwise
  '1'. If we, however, have an scalar parameter to be plotted, then we have
  value '-1'. (I admit, that this is a very unclear way to present this thing.)

struct drawStructure {
  long nPic;
  mf f[MAX_PICTURES];
  ft tree[MAX_PICTURES];
  ft dest[MAX_PICTURES];
  double *mean[MAX_PICTURES];
  double bpr[MAX_PICTURES];
  double **median[MAX_PICTURES];
  double *pv[MAX_PICTURES];
  long data[MAX_PICTURES];
} jackDS; */

/* Routines to plot pictures with Splus */

extern long ind, /* Number of individuals */
  printLevel, /* Not important in this module. */
  frames, /* How many frames we'll draw. */
  skip, /* The length of 'Burn in'-phase. */
  drawMod, /* How many iterations there is between the updates of */
  /* the Splus window. */
  orientation, /* The way the picture is drawn on paper. */
  iter_limit, /* The total number of iterations. */
  thinning, /* Every 'thinning'th iteration will be saved. */
  iter; /* The current iteration step. */
extern draw *plots; /* The vector of elements of 'draw' ('frames' number of 
		       elements). */
extern ct c; /* Covariates. */
extern dt d; /* Point process responce. */
extern ff model;
extern mft f;
static FILE *plot = NULL;

void quitSplus();

/* Before we can define plots, we must initialize the 'draw' structures: */

draw initDraw()
{
  long i;
  draw res;

  res = (draw) malloc(sizeof(jackDS));
  res->nPic = 0;
  for (i = 0; i < MAX_PICTURES; i++) {
    res->drawType[i] = NONE;
    res->f[i] = NULL;
    res->tree[i] = NULL;
    res->dest[i] = NULL;
    res->median[i] = NULL;
    res->mean[i] = NULL;
    res->pv[i] = NULL;
    res->mfName[i] = NULL;
    res->data[i] = 0;
    res->maxVal[i] = 0.0;
    res->minVal[i] = 0.0;
    res->logScale[i] = FALSE;
    res->maxRiskSet[i] = 0;
  }

  return(res);
}

/* Open a pipe, and start an Splus session: */

void startSplus()
{
#ifdef SPLUS
  plot = popen("Splus > /dev/null", "w");
  fprintf(plot, "%s", "openlook()\n");
#endif
#ifdef NOGRAPH
  plot = bite_fopen("splus.S", "w");
#endif
  fprintf(plot, "Cplot3 <- function(title.no, data.points, cint, rset, minVal, maxVal, maxRisk)\n"
"{\n"
"  ##	Plotting simple data files produced by Bite.\n"
"  ##	This function will be called by the C program via\n"
"  ##	pipes (but can be run also in S+).\n"
"\n"
"  par(mar=c(5,4,4,4)+.1)\n"
"  i <- length(data.points)\n"
	  );
  fprintf(plot, "  dims <- switch(i, c(1,1), c(1,2), c(2,2), c(2,2), c(2,3),\n"
"		 c(2,3), c(3,3), c(3,3), c(3,3), c(3,4), c(3,4), c(3,4))\n"
"  zz <- list(x = 0, id = \"\")\n"
"  print(dims)\n"
"  par(mfrow = dims)\n"
"  ##	mtext(side = 3, paste(\"Iteration \", iter), outer = TRUE)\n"
"  \n"
	  );
  fprintf(plot, 
"  for(j in 1:i) {\n"
"    datafile <- paste(\"mean\",title.no, \"_\", j, \".tmp\", sep = \"\")\n"
"    titlefile <- paste(\"title\", title.no, \"_\", j, \".tmp\", sep = \"\")\n"
"    ch <- list(dum=\"\")\n"
"    not.fun <- scan(titlefile, ch, sep=\"\\n\")\n"
"    if (data.points[j] < 0) {\n"
"      if (data.points[j] == -1)\n"
"	plot(density(scan(datafile),n=125),type=\"l\",xlab=\"parameter value\", \n"
	  );
  fprintf(plot, 
"	     ylab=\"density\", main=not.fun$dum[1])\n"
"      if (data.points[j] == -2)\n"
"	hist(scan(datafile), xlab=\"parameter value\", \n"
"	     ylab=\"density\", main=not.fun$dum[1])\n"
"    }\n"
"    else {\n"
	  );
  fprintf(plot, 
"      minim <- minVal[j]\n"
"      maxim <- maxVal[j]\n"
"      if(!cint[j]) {\n"
"	data <- t(matrix(scan(datafile), nrow = 2))\n"
"	##			maxim <- max(data[,2])\n"
"	}\n"
"      else {\n"
"	data <- t(matrix(scan(datafile), nrow = 4))\n"
"	##			maxim <- max(data[,4])\n"
"      }\n"
	  );
  fprintf(plot, 
"      if (minVal[j] > 0) {\n"
"        logscale<-\"y\"\n"
"        ylabel <- \"log f(time)\"\n"
"      }\n"
"      else {\n"
"        logscale<-\"\"\n"
"        ylabel <- \"f(time)\"\n"
"      }\n"
	  );
  fprintf(plot, 
"      if(not.fun$dum[1] != \"\")\n"
"	plot(data[,1], data[,2], type=\"l\", ##xlab=\"time\", ylab=ylabel,\n"
"             xlab=\"\", ylab=\"\", main=not.fun$dum[1], ylim=c(minim,maxim), log=logscale)\n"
"      else\n"
	  );
  fprintf(plot, 
"	plot(data[,1], data[,2], type=\"l\", ##xlab=\"time\", ylab=ylabel,\n"
"	xlab=\"\", ylab=\"\", ylim=c(minim,maxim), log=logscale)\n"
"      mtext(\"time\",side=1,line=3)\n"
"      mtext(ylabel,side=2,line=3)\n"
	  );
  fprintf(plot, 
"      if (data.points[j] == 1) {\n"
"	datafile <- paste(\"data\", title.no, \"_\", j, \".tmp\", sep = \"\")\n"
"	fits <- scan(datafile, zz)\n"
"	if(length(fits$x) > 0) {\n"
"	  dev <- maxVal[j] / 30\n"
"	  for(k in 1:length(fits$x)) {\n"
	  );
  fprintf(plot, 
"	    if (fits$x[k] > data[1,1] && \n"
"		fits$x[k] < data[length(data[,1]),1]) {\n"
"	      if(fits$id[k] != \"*\")\n"
"		text(fits$x[k], minim, fits$id[k])\n"
"	      else {\n"
"		if(length(data[,1])<20)\n"
	  );
  fprintf(plot, 
"		  points(fits$x[k], minim, cex=0.4)\n"
"		else\n"
"		  points(fits$x[k], runif(1,-dev,0), cex=0.4)\n"
"	      }\n"
"	    }\n"
"	  }\n"
"	}\n"
"      }\n"
	  );
  fprintf(plot, 
"      if(cint[j]) {\n"
"	lines(data[,1], data[,3], lty=2)\n"
"	lines(data[,1], data[,4], lty=2)\n"
"      }\n"
"      if (rset[j] == 1) {\n"
"	datafile <- paste(\"risk\", title.no, \"_\", j, \".tmp\", sep = \"\")\n"
	  );
  fprintf(plot, 
"	risk <- t(matrix(scan(datafile), nrow=2))\n"
"	par(new=TRUE)\n"
"	plot(risk,type=\"s\",xlab=\"\",ylab=\"\",lty=3,axes=F,ylim=c(0,maxRisk[j]))\n"
"	##				xlim=c(min(fits$x),max(fits$x)))\n"
"	axis(side=4)\n"
	  );
  fprintf(plot, 
"	mtext(\"Size of the risk set\",side=4,line=3)\n"
"      }\n"
"    }\n"
"  }\n"
"}\n"
);

  fflush(plot);
  if (plot == NULL)
    bite_exit(2);
}

/* Find the maximum value of a simple function. */

void yAxisMax()
{
  long i, j, k, nk = 0, boolean, l1, l2, l3, limit, maxRisk;
  double min, max;
  char **names, dum;

  names = (char **) malloc(MAX_PICTURES * frames * sizeof(names));
  for (i = 0; i < MAX_PICTURES * frames; i++)
    names[i] = NULL;
  for (i = 1; i <= frames; i++) {
    for (j = 1; j <= plots[i]->nPic; j++) {
      boolean = FALSE;
      if (plots[i]->drawType[j] == DRAW_CDF)
	plots[i]->maxVal[j] = 1.0;
      for (k = 0; k < nk; k++)
	if ((plots[i]->drawType[j] == DRAW_MF && 
	    !strcmp(plots[i]->f[j]->name, names[k])) ||
	    (plots[i]->drawType[j] == DRAW_INTENSITY && /* intensity */
	    !strcmp(names[k], "I")) ||
	    (plots[i]->drawType[j] == DRAW_MEAN && /* mean */
	    !strcmp(names[k], "M")))
	  boolean = TRUE;
      if (!boolean) { /* If the maximum hasn't been found for this mf,
			 intensity or mean of mf's yet: */
	switch (plots[i]->drawType[j]) {
	case DRAW_MF:
	  names[nk] = plots[i]->f[j]->name;
	  break;
	case DRAW_INTENSITY:
	  names[nk] = (char *) malloc(2 * sizeof(dum));
	  names[nk][0] = '\0';
	  strcat(names[nk], "I");
	  break;
	case DRAW_MEAN:
	  names[nk] = (char *) malloc(2 * sizeof(dum));
	  names[nk][0] = '\0';
	  strcat(names[nk], "M");
	  break;
	case DRAW_PARAM:
	  names[nk] = (char *) malloc(2 * sizeof(dum));
	  names[nk][0] = '\0';
	  strcat(names[nk], "P");
	  break;
	case DRAW_CDF:
	  names[nk] = (char *) malloc(2 * sizeof(dum));
	  names[nk][0] = '\0';
	  strcat(names[nk], "C");
	  break;
	  
	}
	min = 1.0e80;
	max = -1.0e80;
	maxRisk = 0;
	/* Find the maximum value of the functions to be drawn. */
	for (l1 = i; l1 <= frames; l1++)
	  for (l2 = (l1 == i) ? j : 1; l2 <= plots[l1]->nPic; l2++) {
	    if ((names[nk][0] == 'I' && 
		plots[l1]->drawType[l2] == DRAW_INTENSITY) || 
		(names[nk][0] == 'M' && 
		plots[l1]->drawType[l2] == DRAW_MEAN) ||
		(plots[l1]->drawType[l2] == DRAW_MF &&
		!strcmp(names[nk], plots[l1]->f[l2]->name))) {
	      if (plots[l1]->drawType[l2] == DRAW_MF &&
		  !strcmp(names[nk], plots[l1]->f[l2]->name) &&
		  plots[l1]->maxRiskSet[l2] > maxRisk)
		maxRisk = plots[l1]->maxRiskSet[l2];
	      if (plots[l1]->median[l2]) {
		for (l3 = 0; l3 < GP - 1; l3++) {
		  limit = (long)((double)((iter - skip) / thinning + 1)
				 * (0.5 * plots[l1]->bpr[l2]));
		  if (min > plots[l1]->median[l2][l3][limit])
		    min = plots[l1]->median[l2][l3][limit];
		  limit = (long)((double)((iter - skip) / thinning + 1)
				 * (1.0 - 0.5 * plots[l1]->bpr[l2]));
		  if (max < plots[l1]->median[l2][l3][limit])
		    max = plots[l1]->median[l2][l3][limit];
		}
	      }
	      else
		for (l3 = 0; l3 < GP - 1; l3++) {
		  if (min > plots[l1]->mean[l2][l3])
		    min = plots[l1]->mean[l2][l3];
		  if (max < plots[l1]->mean[l2][l3])
		    max = plots[l1]->mean[l2][l3];
		}
	    }
	  }
	for (l1 = i; l1 <= frames; l1++)
	  for (l2 = (l1 == i) ? j : 1; l2 <= plots[l1]->nPic; l2++)
	    if ((names[nk][0] == 'I' && 
		plots[l1]->drawType[l2] == DRAW_INTENSITY) || 
		(names[nk][0] == 'M' && 
		plots[l1]->drawType[l2] == DRAW_MEAN) ||
		(plots[l1]->drawType[l2] == DRAW_MF &&
		!strcmp(names[nk], plots[l1]->f[l2]->name))) {
	      if (plots[l1]->logScale[l2])
		plots[l1]->minVal[l2] = min;
	      plots[l1]->maxVal[l2] = max;
	      plots[l1]->maxRiskSet[l2] = maxRisk;
	    }
	nk++;
      }
    }
  }
/*  for (i = 1; i <= frames; i++) {
    bite_printf("%ld: ", i);
    for (j = 1; j <= plots[i]->nPic; j++)
      bite_printf("%ld %f\t", j, plots[i]->maxVal[j]);
    bite_printf("\n");
  } */
  for (i = 0; i < nk; i++)
    if (names[i] && (names[i][0] == 'I' || names[i][0] == 'M') &&
	names[i][1] == '\0')
      free(names[i]);
  free(names);
}

/* For determining the size of the risk set we use the following procedure.
   It doesn't (unfortunately) yet check the effect of missing covariates. */

void riskSetSize(long frameno, long pictureno)
{
  FILE *meanp;
  long i, j, n, grid, car, mark, choiceLevel = 0, max = 0;
  double x;
  char fname[15];  
  ft ptr;
  mf target = plots[frameno]->f[pictureno];
  /*  dataNode *data;
  prior ptrsp; */

  if (target && plots[frameno]->riskSet[pictureno]) {
    sprintf(fname, "risk%ld_%ld.tmp", frameno, pictureno);
    if ((meanp = bite_fopen(fname, "w")) == NULL) {
      bite_eprintf("Unable to open file %s ... closing S+\n", fname);
      quitSplus();
      bite_exit(1);
    }
    if (!target->riskSet)
      target->riskSet = (long *) malloc( GP * sizeof(i) );
    for (i = 0; i < GP; i++)
      target->riskSet[i] = 0;
    for (mark = 0; mark < model->n; mark++)
      for (i = 0; i < ind; i++)
	if (checkBit(i % LONGLEN, target->inUse[mark][i / LONGLEN])) {
	  ptr = findFt(model->ftList[mark][0], target, f);
	  if (ptr->choice) {
	    for (j = 0; j < ptr->storageF; j++)
	      if (ptr->f[j] == target)
		choiceLevel = j;
	  }
	  for (grid = 0; grid < GP; grid++) {
	    for (car = 0; car < d->pList[mark]->n2[i]; car++) {
	      for (n = 0; n < ptr->start2->n2[i]; n++) {
		x = covValue(i, n, ptr->start2) +	grid * target->length / GP;
		if (x >= respValue(i, car, 0, T0, d->pList[mark]) && 
		    x < respValue(i, car, 0, T1, d->pList[mark]) &&
		    (!ptr->choice ||
		     (long) value(ptr->choice[i], x) == choiceLevel))
		  target->riskSet[grid]++;
	      }
	    }
	  }

	  /*  for (car = 0; car < d->nCars[mark]; car++) {
	      ptrsp = ptr->start->x->vec;
	      while (ptrsp) {
	      x = ptrsp->x->x + grid * target->length / GP;
	      if (data[car]->type < 2) {
	      if (x >= data[car]->begin && x < data[car]->end &&
	      (!ptr->choice || (ptr->choice && 
	      (long) value(ptr->choice[i], x) == choiceLevel))) {
	      if (data[car]->dData[(long)((x - data[car]->begin) /
	      data[car]->precision)] >= 0)
	      target->riskSet[grid]++;
	      }
	      }
	      else {
	      if (x >= paramValue(i, data[car]->pp->t0) && 
	      x < paramValue(i, data[car]->pp->t1) &&
	      (!ptr->choice || (ptr->choice && 
	      (long) value(ptr->choice[i], x) == choiceLevel)))
	      target->riskSet[grid]++;
	      }
	      ptrsp = ptrsp->next;
	      }
	      } */
	}
    for (i = 1; i < GP - 1; i++) {
      fprintf(meanp, "%g %ld\n",
	      i * target->length / GP,
	      target->riskSet[i]);
      if (plots[frameno]->f[pictureno]->riskSet[i] > max)
	max = plots[frameno]->f[pictureno]->riskSet[i];
    }
    plots[frameno]->maxRiskSet[pictureno] = max;
    fclose(meanp);
  }
}

/* If we want to plot the data to pictures, we need to write data files for
   that ('datax_y.tmp' where x is the frame & y the no. of picture). */

void writeRecord(long cars, dataNode *d, long frameno, long pictureno,
		 double shift)
{
  FILE *meanp;
  long i, j;
  char fname[15], dum[6];
  prior ptr;
 
  fname[0] = '\0';
  strcat(fname, "data");
  intToChar(frameno, dum);
  strcat(fname, dum);
  strcat(fname, "_");
  intToChar(pictureno, dum);
  strcat(fname, dum);
  strcat(fname, ".tmp");
  if ((meanp = bite_fopen(fname, "w")) == NULL) {
    bite_eprintf("Unable to open file %s ... closing S+\n", fname);
    quitSplus();
    bite_exit(1);
  }
  for (i = 0; i < cars; i++) { /* Write the data of all 'car's: */
    if (d[i]->type >= 0 && d[i]->name[0]) {
      if (d[i]->type < 2) { /* if the data is of discrete type (types 0 & 1) */
        for (j = 0; j < d[i]->nObs; j++)
          if (d[i]->dData[j] >= 0) /* if we don't have missing data (i.e.
                                      negative values) then print the position
                                      and the value of the data: */
            fprintf(meanp, "%g %ld\n", (j + 0.5) * d[i]->precision - shift +
                    d[i]->begin, d[i]->dData[j]);
      }
      else { /* else we have continuous data (i.e. type 2) then we just write
                the times of occurrences ('*' means that we print a dot
                instead of a number): */
	ptr = d[i]->pp->vector;
	while (ptr) {
/*        for (j = 0; j < d[i]->nObs; j++) */
          fprintf(meanp, "%g *\n", ptr->x->x - shift);
	  ptr = ptr->next;
	}
      }
    }
  }
  fclose(meanp);
  if (printLevel >= 1)
    bite_printf("Wrote file %s\n", fname);
}

void writeData(long frameno, long pictureno)
{
  FILE *meanp;
  long i, j, n, mark, car, choiceLevel = 0;
  char fname[15];  
  mf target = plots[frameno]->f[pictureno];
  ft ptr;
  /* dataNode *data;
  prior ptrsp, ptrdata; */

  if (target && plots[frameno]->riskSet[pictureno]) {
    sprintf(fname, "data%ld_%ld.tmp", frameno, pictureno);
    if ((meanp = bite_fopen(fname, "w")) == NULL) {
      bite_eprintf("Unable to open file %s ... closing S+\n", fname);
      quitSplus();
      bite_exit(1);
    }
    for (mark = 0; mark < model->n; mark++) {
      ptr = findFt(model->ftList[mark][0], target, f);
      for (i = 0; i < ind; i++) {
	if (checkBit(i % LONGLEN, target->inUse[mark][i / LONGLEN])) {
	  if (ptr->choice) {
	    for (j = 0; j < ptr->storageF; j++)
	      if (ptr->f[j] == target)
		choiceLevel = j;
	  }
	  for (car = 0; car < d->nCars[mark]; car++) {
	    for (n = 0; n < ptr->start2->n2[i]; n++) {
	      for (j = 0; j < d->pList[mark]->val.pp.npoints[i][car]; j++)
		if (!ptr->choice ||
		    value(ptr->choice[i],
			  respValue(i, car, j, POINTS, d->pList[mark])) ==
		    choiceLevel)
		  fprintf(meanp, "%g *\n",
			  respValue(i, car, j, POINTS, d->pList[mark]) -
			  covValue(i, n, ptr->start2));
	    }
	  }

	  /*	data = d->dList[mark][i];
		for (k = 0; k < d->nCars[mark]; k++) {
		ptrsp = ptr->start->x->vec;
		while (ptrsp) {
		for (sp = 0; sp < ptr->start->n; sp++) {
		Write the data of all 'car's:
		if (data[k]->type >= 0 && data[k]->name[0]) {
		if (data[k]->type < 2) { if the data is of discrete type 
		for (j = 0; j < data[k]->nObs; j++)
		if (data[k]->dData[j] >= 0 && (long)
		value(ptr->choice[i], (j + 0.5) * data[k]->precision +
		data[k]->begin) == choiceLevel)
		if we don't have missing data (i.e.
		negative values) then print the position
		and the value of the data: 
		fprintf(meanp, "%g %ld\n", (j + 0.5) * data[k]->precision -
		ptrsp->x->x + data[k]->begin,
		data[k]->dData[j]); 
		}
		else { else we have continuous data (i.e. type 2) then we 
		just write the times of occurrences ('*' means that
		we print a dot instead of a number): 
		for (j = 0; j < data[k]->nObs; j++) 
		ptrdata = data[k]->pp->vector;
		while (ptrdata) {
		if (!ptr->choice || (ptr->choice && (long)
		value(ptr->choice[i], ptrdata->x->x) == choiceLevel))
		fprintf(meanp, "%g *\n", ptrdata->x->x -
		ptrsp->x->x);
		ptrdata = ptrdata->next;
		}
		}
		}
		ptrsp = ptrsp->next;
		} */
	}
      }
    }
    fclose(meanp);
    if (printLevel >= 1)
      bite_printf("Wrote file %s\n", fname);
  }
}

void splusStuff(long iter, long frame, long picture, sf f, double *values,
		double max, double *mean, double bpr, double **median)
{
  FILE *meanp = NULL;
  long j, k, l;
  double x, y, *vector;
  char fname[15];

/* Calculate & update the means & write the values to a file ('meanx_y.tmp' 
   where x is the frame & y the no. of picture), if interation 
   step 'iter' >= 'skip' (M-H algoritm requires this). */
  if (iter >= skip && (iter % thinning == 0 || iter == iter_limit - 1)) {
    /* First, open the file (if necessary): */
    if (iter % drawMod == 0 || iter == iter_limit - 1) {
      sprintf(fname, "mean%ld_%ld.tmp", frame, picture);
      if ((meanp = bite_fopen(fname, "w")) == NULL) {
	bite_eprintf("Unable to open file %s ... closing S+\n", fname);
	quitSplus();
	bite_exit(1);
      }
    }
    for (j = 0; j < GP - 1; j++) {
      /* Next calculate the values of the target stepwise constant function 'f'
	 at the grid points 'x' ('f' may be also intensity): */
      if (f) { /* DRAW_MF or DRAW_INTENSITY or DRAW_CDF */
	x = (f->jumpPoints[f->n - 1] - f->jumpPoints[0]) * (j + 1) / GP +
	  f->jumpPoints[0];
	if (bpr > 0.0)
	  y = value(f, x);
	else
	  y = 1.0 - exp(-intSf(f, f->jumpPoints[0], x));
      }
      else { /* DRAW_MEAN */
	x = max / GP * j;
	y = values[j];
      }
      /* Update the mean, and write to the file, if necessary: */
      mean[j] = (double)((iter - skip) / thinning * mean[j] + y) /
	((iter - skip) / thinning + 1);
      if (iter % drawMod == 0 || iter == iter_limit - 1)
	fprintf(meanp, "%g %g ", x, mean[j]);
      /* If we are calculating the 'bpr''s, then do the following: */
      if (median) {
	vector = median[j];
	/* Keep the array 'vector' in an increasing order: */
	if (iter == skip)
	  vector[0] = y;
	else {
	  k = 0;
	  while (vector[k + 1] < y && k < (iter - skip) / thinning - 1)
	    k++;
	  for (l = (iter - skip) / thinning - 1; l > k; l--)
	    vector[l + 1] = vector[l];
	  if (k == 0) {
	    if (vector[0] < y)
	      vector[1] = y;
	    else {
	      vector[1] = vector[0];
	      vector[0] = y;
	    }
	  }
	  else
	    vector[k + 1] = y;
	}
	/* Write (if necessary) the current limits of the 'bpr' to the file: */
	if (iter % drawMod == 0 || iter == iter_limit - 1) 
	  fprintf(meanp, "%g %g ", 
		  vector[(long)((double)((iter - skip) / thinning) * 0.5 * bpr)],
		  vector[(long)((double)((iter - skip) / thinning + 1) * 
			       (1.0 - 0.5 * bpr))]);
      }
      if (iter % drawMod == 0 || iter == iter_limit - 1)
	fprintf(meanp, "\n");
    }
    if (iter % drawMod == 0 || iter == iter_limit - 1) {
      fclose(meanp);
      if (printLevel > 1)
	bite_printf("Wrote file %s\n", fname);
    }
  }
} /* end splusStuff */

/* This is the function I call from the main program: */

void updateMeans(long iter, mft f)
{
  long i, j, k, l1, l2;
  static double values[GP];
  sf lambda;

  for (i = 1; i <= frames; i++) /* Update the means of the 
				   functions/intensities of each frame ... */
    for (j = 1; j <= plots[i]->nPic; j++) { /* ... and of each picture */
      switch (plots[i]->drawType[j]) {
      case DRAW_MF: /* If the target is a function ... */
	splusStuff(iter, i, j, plots[i]->f[j]->f->f, NULL, 0.0, 
		   plots[i]->mean[j], plots[i]->bpr[j], plots[i]->median[j]);
	break;
      case DRAW_PARAM: /* ... else if the target is a scalar parameter:
	if (iter >= skip) {
	  plots[i]->pv[j]->sample[(iter - skip) / thinning] =
	    paramValue(-1, plots[i]->pv[j]);
	  if (iter % drawMod == 0 || iter == iter_limit - 1) {
	    fname[0] = '\0';
	    strcat(fname, "mean");
	    intToChar(i, dum);
	    strcat(fname, dum);
	    strcat(fname, "_");
	    intToChar(j, dum);
	    strcat(fname, dum);
	    strcat(fname, ".tmp");
	    if ((postp = bite_fopen(fname, "w")) == NULL) {
	      bite_eprintf("Unable to open file %s ... closing S+\n", fname);
	      quitSplus();
	      bite_exit(1);
	    }
	    for (k = 0; k <= (iter - skip) / thinning; k++)
	      fprintf(postp, "%g\n", plots[i]->pv[j]->sample[k]);
	    fclose(postp);
	  }
	} */
	break;
      case DRAW_INTENSITY: /* Otherwise the target is an intensity: */
	/* First copy the leaves of the formula tree to the leaves of the
	   formula tree of the artifical individual: */
	putSfToFt(0, 0, NULL, -1, -1, plots[i]->dest[j],
		  plots[i]->tree[j], NULL, f);
	/* Then calculate the intensity: */
	lambda = calculate(-1, plots[i]->dest[j]);
	/* Finally we can update the means (and Bayesian probability
	   regions): */
	splusStuff(iter, i, j, lambda, NULL, 0.0, plots[i]->mean[j],
		   plots[i]->bpr[j], plots[i]->median[j]);
	break;
      case DRAW_CDF:
	putSfToFt(0, 0, NULL, -1, -1, plots[i]->dest[j],
		  plots[i]->tree[j], NULL, f);
	lambda = calculate(-1, plots[i]->dest[j]);
	splusStuff(iter, i, j, lambda, NULL, 0.0, plots[i]->mean[j],
		   -1.0, NULL);
	break;
      case DRAW_MEAN:
	k = 0;
	while (strcmp(f->fList[k][0]->name, plots[i]->mfName[j]))
	  k++;
	for (l1 = 0; l1 < GP - 1; l1++) {
	  values[l1] = 0.0;
	  for (l2 = 0; l2 < f->nFunc[k]; l2++)
	    values[l1] += value(f->fList[k][l2]->f->f, 
				l1 * f->fList[k][0]->length / GP);
	  values[l1] /= (double) f->nFunc[k];
	}
	splusStuff(iter, i, j, NULL, values, f->fList[k][0]->length,
		   plots[i]->mean[j], plots[i]->bpr[j], plots[i]->median[j]);
	break;
      }
    }
}

/* To plot the frame 'i' on screen we call the following function: */

void plotAll(long i)
{ 
  long j;
  char command[500];

/* The corresponding Splus function is called 'Cplot3'. */

#ifdef NOGRAPH
  if (iter == iter_limit - 1) {
#endif
/* Frame */
#ifdef NOGRAPH
  sprintf(command, "pdf(file=\"splus%ld.pdf\")", i);
  fprintf(plot, "%s\n", command);
#endif
  sprintf(command, "Cplot3(%ld, c(", i);

/* Iteration 
  intToChar(iter, dum1);
  strcat(command, dum1);
  strcat(command, ", "); */

/* Draw data or not (0/1) or posterior density estimate of some scalar
   parameter (-1)? */
  for (j = 1; j <= plots[i]->nPic; j++) {
    if (j > 1)
      sprintf(command, "%s,", command);
    sprintf(command, "%s%ld", command, plots[i]->data[j]);
  }
  sprintf(command, "%s), c(", command);

/* Is there credibility intervals? */
  for (j = 1; j <= plots[i]->nPic; j++) {
    if (j > 1)
      sprintf(command, "%s,", command);
    if (plots[i]->median[j])
      sprintf(command, "%s1", command);
    else
      sprintf(command, "%s0", command);
  }
  sprintf(command, "%s), c(", command);

/* Is there the size of the risk set? */
  for (j = 1; j <= plots[i]->nPic; j++) {
    if (j > 1)
      strcat(command, ",");
    if (plots[i]->riskSet[j])
      sprintf(command, "%s1", command);
    else
      sprintf(command, "%s0", command);
  }
  strcat(command, "), c(");

/* Write the vector of minimum values of the y-axis: */
  for (j = 1; j <= plots[i]->nPic; j++) {
    if (j > 1)
      strcat(command, ",");
    sprintf(command, "%s%g", command, plots[i]->minVal[j]);
  }
  strcat(command, "), c(");

/* Write the vector of maximum values of the y-axis: */
  for (j = 1; j <= plots[i]->nPic; j++) {
    if (j > 1)
      strcat(command, ",");
    sprintf(command, "%s%g", command, plots[i]->maxVal[j]);
  }
  strcat(command, "), c(");

/* Write the vector of maximum values of the risk set size. */
  for (j = 1; j <= plots[i]->nPic; j++) {
    if (j > 1)
      strcat(command, ",");
    sprintf(command, "%s%ld", command, plots[i]->maxRiskSet[j]);
  }
  strcat(command, ")");

/* End the command line: */
  strcat(command, ")");
  fprintf(plot, "%s\n", command);
#ifdef NOGRAPH
  fprintf(plot, "dev.off()\n");
#endif
  fflush(plot);

  /* bite_printf("Got here\n"); */

  /*  if (i > 1)
    bite_printf("%s\n", command); */
#ifdef NOGRAPH
  }
#endif
}

/* This file is quite trivial: the string 'text' will appear on top of the
   picture: */

void writeTitleFile(long frame, long pict, char* text)
{
  FILE *titlep;
  char fname[20], dum[6], title[150];
  long linelength = 25;

  fname[0] = '\0';
  strcat(fname, "title");
  intToChar(frame, dum);
  strcat(fname, dum);
  strcat(fname, "_");
  intToChar(pict, dum);
  strcat(fname, dum);
  strcat(fname, ".tmp");
  if ((titlep = bite_fopen(fname, "w")) == NULL) {
    bite_eprintf("Unable to open file %s\n", fname);
    bite_exit(1);
  }
  if (strlen(text) / linelength) {
    long breakL = linelength - 3, i = 0, j = 0, nextLine = FALSE;
    do {
      while (text[j] && j < breakL) {
	title[i] = text[j];
	j++;
	i++;
      }
      if (!text[j])
	title[i] = text[j];
      if (!text[j])
	break;
      while (text[j] && !nextLine) {
	if (text[j] == ' ') {
	  title[i++] = '\\';
	  title[i++] = 'n';
	  j++;
	  nextLine = TRUE;
	  breakL = j + linelength;
	}
	else {
	  title[i] = text[j];
	  j++;
	  i++;
	}
      }
    } while (1 == 1);
  }
    
  fprintf(titlep, "%s\n", title);
  fclose(titlep);
}

/* This function saves the the current Splus picture into a postscript file 
   called 'splusi.ps', where 'i' is the argument of the function: */

void savePicture(long i)
{
  char command[80];

  sprintf(command, "printgraph(method=\"pdf\", "
	  "file=\"splus%ld.pdf\")", i);
  fprintf(plot, "%s\n", command);
  fflush(plot); 
}

/* At the end we quit Splus. */

void quitSplus()
{
  if (frames)
    fprintf(plot, "q()\n");
#ifdef SPLUS
  if (frames)
    pclose(plot);
#endif
#ifdef NOGRAPH
  fclose(plot);
    #ifdef STANDALONE  
      plot =
        popen("tar cvf splus.tar splus.S {mean,title,risk,data}*tmp >& /dev/null &", "w");
      bite_sleep(4);  
      pclose(plot);
    #endif  
#endif
}

/* #endif */
