/* Tommi Harkanen (C) 2000 */
/* Read the file copyright.txt for details */

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <ctype.h>
#include <time.h>
#include <string.h>
#include "typedef.h"
#include "sf3.h"
#include "general3.h" 
#include "bit.h"
#include "load.h"
#include "prior.h"
#include "modelf.h" 
#include "ftree.h"
#include "likel.h"
#include "pipefiles.h"

extern long max_iter, iter_limit, skip, drawMod, orientation,
           printLevel, ind, nVars, nextI, frames, lineno, iter, thinning;
extern double missing;
extern variable vars[MAX_VAR];
extern prior param;
extern dt d;
extern ct c;
extern pi pred;
extern pi surv;
extern ttt tttplots;
extern mft f;
extern ff model;
extern draw *plots;
extern labelArray labela, miss;
extern argArray a;

void updateTTT(ttt ptr, long init);

void initTTT(char *resp)
{
  /* Warning: this procedure uses a global variable called 'labela'! */
  long i = 0, j, dim = 1, index[MAX_COV], stop;
  char filename[250], str[20];
  ttt res, last = tttplots;

  if (!tttplots)
    updateTTT(NULL, TRUE);
  while (last && last->next)
    last = last->next;
  res = (ttt) malloc(sizeof(jackTTT));
  if (last)
    last->next = res;
  else
    tttplots = res;

  res->responce = findResponce(resp);
  res->ncov = labela->n;
  for (i = 0; i < labela->n; i++) {
    res->cov[i] = findCovariate(labela->label[i]->label);
    dim *= c->nval[res->cov[i]];
    index[i] = 0;
  }
  res->mean = (double **) malloc(dim * sizeof(c));
  res->weight = (double **) malloc(dim * sizeof(c));
  res->weight2 = (double **) malloc(dim * sizeof(c));
  res->w0 = (double *) malloc(dim * sizeof(missing));
  res->stor = (long *) malloc(dim * sizeof(i));
  res->np = (double *) malloc(dim * sizeof(i));
  res->file = (long *) malloc(dim * sizeof(missing));
  for (i = 0; i < dim; i++) {
    res->stor[i] = 50;
    res->mean[i] = (double *) malloc(50 * sizeof(missing));
    res->weight[i] = (double *) malloc(50 * sizeof(missing));
    res->weight2[i] = (double *) malloc(50 * sizeof(missing));
    for (j = 0; j < 50; j++) {
      res->mean[i][j] = 0.0;
      res->weight[i][j] = 0.0;
      res->weight2[i][j] = 0.0;
    }
    sprintf(filename, "ttt.%s.", resp);
    for (j = 0; j < res->ncov; j++) {
      doubleToChar(c->values[res->cov[j]][index[j]], str);
      sprintf(filename, "%s%s=%s.", filename, c->cList[j][0]->name, str);
    }
    j = res->ncov - 1;
    stop = FALSE;
    while (j >= 0 && !stop) {
      index[j]++;
      if (index[j] < c->nval[j])
	stop = TRUE;
      else {
	index[j] = 0;
	j--;
      }
    }
    strcat(filename, "tmp");
    /* res->file[i] = openFile(filename, 'w'); */
  }
}

void updateTTT(ttt ptr, long init)
{
  long i, j, k, l, t, ii, np, class, dim = 1, accept, index[MAX_COV];
  double x, wd;
  prior vector;
  static long nPoints, *indiv;
  static sf sum, lambda, part, dumsf;
  static double *points, *sn, *cum, *w;

  if (init) {
    sum = initSf(0.0, 0.0, NULL);
    lambda = NULL;
    part = NULL;
    dumsf = NULL;
    indiv = malloc(ind * sizeof(i));
    nPoints = 50;
    points = malloc(50 * sizeof(x));
    sn = malloc(50 * sizeof(x));
    cum = malloc(50 * sizeof(x));
    w = malloc(50 * sizeof(x));
  }
  else
    if (iter >= skip && iter % thinning == 0) {
      for (j = 0; j < ptr->ncov; j++)
	dim *= c->nval[ptr->cov[j]];
      for (class = 0; class < dim; class++) {
	k = dim;
	l = class;
	for (j = 0; j < ptr->ncov; j++) {
	  k /= c->nval[ptr->cov[j]];
	  index[j] = l / k;
	  l = l % k;
	}
	sum->n = 0;
	np = 0;
	for (j = 0; j < nPoints; j++)
	  w[j] = 0.0;
	for (i = 0; i < ind; i++) {
	  /* See, if the current individual has the correct combination of
	     covariates. */
	  accept = TRUE;
	  for (j = 0; j < ptr->ncov; j++) {
	    if (paramValue(i, c->cList[ptr->cov[j]][i]->x) !=
		c->values[ptr->cov[j]][index[j]])
	      accept = FALSE;
	  }
	  if (accept) {
	    indiv[i] = TRUE;
	    /* Calculate the intensity function 'lambda'. */
	    putSfToFt2(i, ptr->responce, NULL, -INFTY, INFTY,
		       model->ftList[ptr->responce][0], 
		       model->ftList[ptr->responce][0], d, f);
	    /* Some diagnostics. */
	    /*	    bite_printf("\nIndividual %ld (cov %ld)\n", i, class);
	    inorder(i, model->ftList[ptr->responce][ind]); 
	    bite_printf("\nLambda "); */
	    lambda = calculate(i, model->ftList[ptr->responce][0]);
	    /*	    printSf(lambda); */
	    /* Check all cars. */
	    for (t = 0; t < d->nCars[ptr->responce]; t++) {
	      /* A TTT plot is only for continuous point process data. */
	      if (d->dList[ptr->responce][i][t]->type == CONT) {
		/* First add the part of the intensity to the total
		   intensity. */
		copySf(lambda, part);
		part =
		  cutSf(lambda,
			paramValue(i, d->dList[ptr->responce][i][t]->pp->t0),
			paramValue(i, d->dList[ptr->responce][i][t]->pp->t1),
			part);
		dumsf = addSf(sum, part, dumsf);
		/* bite_printf("part ");
		printSf(part);
		bite_printf("dumsf ");
		printSf(dumsf); */
		copySf(dumsf, sum);
		/* Then add the observed points to the ordered vector. */
		vector = d->dList[ptr->responce][i][t]->pp->vector;
		while (vector) {
		  x = paramValue(i, vector);
		  if (x > 20)
		    bite_printf("%g\n", x);
		  ii = 0;
		  while (ii < np && points[ii] < x)
		    ii++;
		  for (l = np - 1; l >= ii; l--)
		    points[l + 1] = points[l];
		  points[ii] = x;
		  if (np == nPoints - 1) {
		    nPoints *= 2;
		    points = realloc(points, nPoints * sizeof(x));
		    sn = realloc(sn, nPoints * sizeof(x));
		    cum = realloc(cum, nPoints * sizeof(x));
		    w = realloc(w, nPoints * sizeof(x));
		  }
		  np++;
		  if (np == ptr->stor[class] - 1) {
		    ptr->stor[class] *= 2;
		    ptr->mean[class] = 
		      realloc(ptr->mean[class], ptr->stor[class] * sizeof(x));
		    ptr->weight[class] = 
		      realloc(ptr->weight[class],
			      ptr->stor[class] * sizeof(x));
		    ptr->weight2[class] = 
		      realloc(ptr->weight2[class],
			      ptr->stor[class] * sizeof(x));
		  }
		  vector = vector->next;
		}
	      }
	    }
	  }
	  else
	    indiv[i] = FALSE;
	}
	/* Some diagnostics. 
	for (jj = 0; jj < np; jj++)
	  bite_printf("%ld:%f ", jj, points[jj]);
	bite_printf("\nIntegrals: "); */
	/* Finally, calculate the S_n's... */
	/* if (iter == skip)
	  ptr->w0[class] = w; */
	/* Update the importance sampling weights. */
	ptr->np[class] = np;
	for (i = 0; i < ind; i++) {
	  if (indiv[i]) {
	    /* bite_printf("%g: ",
		   paramValue(i, d->dList[ptr->responce][i][0]->pp->t1)); */
	    for (j = 0; j < np; j++) {
	      wd = indLogL(i, ptr->responce, NULL,
			   (j) ? points[j - 1] : -INFTY, INFTY, d, model);
	      w[j] -= (wd < -1.0e50) ? 0.0 : wd;
	      /* if (wd < -1.0e50)
		bite_printf("%ld,%ld:%g\n", i, j, wd); */
	    }
	    /* bite_printf("\n"); */
	  }
	}
	for (j = 0; j <= np; j++) {
	  sn[j] = intSf(sum, (j) ? points[j - 1] : -INFTY,
			(j == np) ? INFTY : points[j]);
	  /* + ((j) ? sn[j - 1] : 0.0); */
	  cum[j] += sn[j];
	  ptr->weight[class][j] += exp(w[j]);
	  ptr->weight2[class][j] += exp(2.0 * w[j]);
	  /* if (w[j] == 0.0 && j < np - 1) {
	    printSf(sum);
	    bite_printf("%ld:%ld:%g:%g:%g:%g:%g\n", j, np, points[j], sn[j], cum[j],
		   ptr->weight[class][j], w[j]);
	  }
	  bite_printf("%ld:%f ", j, sn[j]); */
	}
	/*	bite_printf("\nCumulative: "); */
	/* and update the TTT plot mean (of 'GP' grid points). */
	/* printSf(sum);
	prnD(np,points); */
	for (j = 0; j <= np; j++) {
	  ptr->mean[class][j] += sn[j] * exp(w[j]);
	  x = ptr->mean[class][j];
	  /* if (ptr->mean[class][j] / ptr->weight[class][j] > 15) {
	    printSf(sum);
	    bite_printf("(%ld,%ld):%g,%g,(%g/%g=%g),%g\n",
		   j, np, points[j], sn[j], ptr->mean[class][j],
		   ptr->weight[class][j], ptr->mean[class][j] /
		   ptr->weight[class][j], w[j]);
	    bite_printf("");
	  } */
	}
      }
    }
}

void saveTTT(ttt ptr)
{
  long i, j, dim = 1, index[MAX_COV], stop;
  char filename[250], str[80];
  FILE *fp;

  for (j = 0; j < ptr->ncov; j++) {
    dim *= c->nval[ptr->cov[j]];
    index[j] = 0;
  }
  for (i = 0; i < dim; i++) {
    /* Open the file */
    sprintf(filename, "ttt.%s.", d->dList[ptr->responce][0][0]->name);
    for (j = 0; j < ptr->ncov; j++) {
      doubleToChar(c->values[ptr->cov[j]][index[j]], str);
      sprintf(filename, "%s%s=%s.", filename, c->cList[j][0]->name, str);
    }
    strcat(filename, "tmp");
    fp = bite_fopen(filename, "w");

    j = ptr->ncov - 1;
    stop = FALSE;
    while (j >= 0 && !stop) {
      index[j]++;
      if (index[j] < c->nval[j])
	stop = TRUE;
      else {
	index[j] = 0;
	j--;
      }
    }
    for (j = 0; j <= ptr->np[i]; j++) {
      fprintf(fp, "%g %g\n",
	      ptr->mean[i][j] / (ptr->weight[i][j]),
	      ptr->weight2[i][j] / (iter_limit - skip) * thinning -
	      pow(ptr->weight[i][j] / (iter_limit - skip) * thinning, 2.0));
    }
    fclose(fp);
  }
}
