/* Tommi Harkanen (C) 2000 */
/* Read the file copyright.txt for details */

/* ------- TYPEDEF.H ------- */
/***********************************************/
/*                                             */
/*      Copyright (c) 2000 Tommi Hrknen      */
/*                                             */
/***********************************************/
#ifndef TYPEDEF
#define TYPEDEF

/* popen() and pclose() are not ANSI C standards, so define Posix: 
#define _POSIX_SOURCE */
#define _GNU_SOURCE

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <ctype.h>
#include <string.h> 

/* #define ETA1  For special independence sampling. Usually not defined. */
/* #define HAMMAS If the first jump points of some functions 'h' are fixed. */
/* #define DEBUG_SF3 */
/* #define B_FACTOR */

#define NODEBUG

#define INFTY 1.0e+90 /* A big value. */
#define LONGLEN 32 /* # of bits of 'long'. */

#define MIN(a,b) (((a)<(b)) ? (a) : (b))
#define MAX(a,b) (((a)>(b)) ? (a) : (b))
#define SQR(a) ((sqrarg=(a)) == 0.0 ? 0.0 : sqrarg*sqrarg)
#define LOG(a) (((a) == 0) ? -INFTY : log(a))

#ifndef PI
#define PI 3.1415928
#endif

#define MAX_FILES 200 /* These are for pipes with files */
#define MAX_PROCESSES 80
#define BUFSIZE 100

#define MAX_COMMANDS 500
#define MAX_JUMPS 20
#define MAX_STACK_SIZE 20
#define MAX_COV 10
#define MAX_VAR 10
#define MAX_MARKS 6
#define MAX_CARS 5
#define MAX_FUNCS 10
#define MAX_PICTURES 12
#define MAX_VARNAME 35
#define MAX_STRLEN 40
#define MAX_ARGS 10
#define MAX_MISSING 20
#define MAX_STACK 10
#define EXP_VAL_STOR 10

#define USE_TABLE 1
#define USE_PRIOR 2
#define USE_MF 3
#define USE_VAR 4
#define USE_RESP 5
#define USE_TABLE2 6
#define USE_INDPARAM 7
#define USE_COV2 8

/* Available types of parameter types: */
#define PT_SCALAR 1 /* Type of the parameter */
#define PT_VECTOR 2
#define PT_PPROCESS 3
#define PT_SF 4
#define PT_RESP 5
#define PT_COV 6
#define PT_CONST 7
#define PT_MODELF 8
#define PT_COV2 9
#define PT_RESP2 10
#define PT_MIX 11
#define PT_STRING 12

#define PT_SEMICOL 20
#define PT_SET 21
#define PT_IF 22
#define PT_WHILE 24
#define PT_FOR 25
#define PT_RETURN 26
#define PT_ECHO 27
#define PT_SWITCH 28
#define PT_CUT 29

/* #define PI 3.1415928 */

#define VAR_PRIOR 1

#define NONE 0
#define GAMMA 1
#define PAIRDIFF 2
#define LOGNORM 3
#define MU 4
#define MIDP 5
#define JHEIKK 6
#define TIMELESS 7
#define INCR_GAMMA 8
#define GAMMA2 9
#define DECR_GAMMA 10
#define BROWNIAN 11
#define GAMMA_LEXIS 12

/* Probability distribution functions: */
#define D_DUNIF 1
#define D_BIN 2
#define D_NORM 3
#define D_GAMMA 4
#define D_UNIF 5
#define D_DIRAC 6
#define D_MNORM 7
#define D_DIRICHLET 8
#define D_MULTIN 9
#define D_MIX 10
#define D_DISCR 11
#define D_STUDENT 12
#define D_LAPLACE 13
#define D_POISSON 14

/* Log density functions: distr. funct. + 30: */
#define DD_BIN 32
#define DD_NORM 33
#define DD_GAMMA 34
#define DD_UNIF 35
#define DD_DISCR 41
#define DD_STUDENT 42

#define F_LOG 101 
#define F_EXP 102 
#define F_MIN 103
#define F_MAX 104
#define F_GAMMA 105
#define F_INT 111 
#define F_VALUE 112
#define F_SUM 113 
#define F_PROD 114
#define F_LOGLIKE 115
#define F_INVINT 116
#define F_INDIV 117
#define F_INDEX 118
#define F_OLD 119
#define F_LOGPRIOR 120
#define F_PPPOINT 121
#define F_FLOOR 122
#define F_LOGCONTR 123
#define F_ADDP 130
#define F_DELP 131
#define F_ADDI 132
#define F_DELI 133
#define F_THIS 134
#define F_CUT 135
#define F_PROBIT 136

#define SF_ORIG 1
#define SF_TIMELESS 2

#define GP 100

#define DISCR1 0
#define DISCR2 1
#define CONT 2
#define DISCR3 3

#define FALSE 0
#define TRUE 1

#define SAVEITER 1
#define LOADITER 2

#define RESPONCE -2

#define COV_CNST 1
#define COV_SF 2
#define COV_START 3
#define COV_CNST2 4
#define COV_START2 5

#define DRAW_MF 1
#define DRAW_INTENSITY 2
#define DRAW_PARAM 3
#define DRAW_MEAN 4
#define DRAW_CDF 5

#define CLUSTER_RANDOM_WALK 1
#define CLUSTER_INDEPENDENCE 2
#define GROUP_UPDATE 11

/* What is the type of a formula tree? It is either constant (NONE=0) or: */

#define SCALAR 1 /* Contains only scalar parameters. */
#define INDIVIDUAL 2 /* Contains covariates or responce. */
#define MODELF 3 /* Contains model functions. */

/* Union for lex & yacc */

/* typedef struct pmu *pMu; */

typedef struct pmp *pMp;

typedef struct pG *pGamma;

typedef struct pBm_struct *pBrownian;

typedef struct pP *pPairdiff;

typedef struct pJ *pJuha;

typedef struct pTL *ptl;

typedef struct pln *pLn;

typedef struct uSe *use;

typedef struct modelF *mf;

typedef struct fT *ft;

typedef struct args *argArray;

typedef struct labelS *labels;

typedef struct labelA *labelArray;

typedef struct pointProcess *pProcess;

typedef struct priori *prior;

typedef struct pGL *pGamma_lexis;

int yylex();

typedef union {
  double real;
  long integer;
  char name[MAX_VARNAME];
  char string[MAX_STRLEN];
  ft ptr;
  argArray exprL;
  labels label;
} YYSTYPE;

#define YYSTYPE_IS_DECLARED 1

struct labelS {
  double value;
  char label[MAX_STRLEN];
} jackLabel;

struct labelA{ /* Performs as an associative list: (double, string). */
  long n;
  long stor;
  labels *label;
} jackLabelArray;

struct args {
  long n;
  long stor;
  ft *ptrArray;
} jackArgs;

/* Definition of simple function */

typedef struct {
  double diag;
  double offDiag;
} cholRow;

typedef struct simpleF *sf;

struct simpleF {
  long storage;
  long n;
  double *genPoints;
  double *jumpPoints;
  double *levels;
} jackF;

typedef struct {
  long n;
  char **values;
} charVec;

/* To make parameter vectors more compact w.r.t. memory consumption,
   we introduce the following construct: */
/* typedef struct {
  long **npoints; for [i][car]
  double **t0;
  double **t1;
  double ***points;
} val_struct; Point process responce */

/*typedef struct distr_struct {
  long *npoints;
  prior *t0;
  prior *t1;
  prior **points;
} distr_struct; 

typedef struct {
  long **t0;
  long **t1;
  long ***points;
} fixed_struct; */

typedef struct ParamVector_struct {
  long n; /* Length */
  char *name;
  long stor;
  long *n2; /* # of elements in a vector, or # of cars */
  long type;
  union { /* Values: */
    double *x; /* Scalar covariate */
    double **x2; /* Vector covariate (start point) */
    struct {
      long **nstor; /* for [i][car] */
      long **npoints; /* for [i][car] */
      double **t0;
      double **t1;
      double ***points; /* for [i][car][n] */
    } pp;
  } val;
  union { /* Prior distributions: */
    prior x; /* Scalar covariate */
    prior *x2; /* Vector covariate (start point) */
    struct distr_struct {
      long *npoints;
      prior *t0;
      prior *t1;
      prior **points;
    } pp;
  } distr;
  union { /* Are the values fixed or not: */
    long *x; /* Scalar covariate */
    long **x2; /* Vector covariate (start point) */
    struct {
      long **t0;
      long **t1;
      long ***points;
    } pp;
  } fixed; /* Attribute */
} ParamVector;

typedef ParamVector *paramVector;

/* Structures for responce (data) & for covariates (cov) */

typedef struct dn *dataNode;

struct dn {
  char name[20];
  long ind;
  long car;
  long mark;
  long place;
  long type;
  long storage;
  double precision;
  double begin;
  double end;
  long *formatBot;
  long *formatTop;
  long nObs;
/*  double *cData; */
  long *dData;
  double *times;
  pProcess pp;
} dN;

typedef struct dataTree *dt;

struct dataTree {
  long n;
  long storage;
  long *nCars;
  dataNode ***dList;
  paramVector *pList;
} jackdt;

typedef struct cn *covNode;

struct cn {
  char name[20];
  long ind;
  long type;
  prior x;
/*  sf f;
  prior sp; */
} cN;

typedef struct covTree *ct;

struct covTree {
  long n;
  long storage;
  covNode **cList;
  paramVector *pList;
  long *nval;
  double **values;
  char ***labels;
} jackct;

/* Structure for prediction (i.e. mean of integrals of certain intervals) */

typedef struct prediction *pi;

struct prediction {
  long n;
  long mark;
  ft **bot;
  ft **top;
  double ***sample;
  pi next;
} jackPi;

/* Define some structures for hyperparameters of the priors & for variables. */

typedef union {
  pGamma pGa; /* Gamma */
  ptl pTl; /* Timeless */
  pBrownian pBM; /* Brownian motion */
  pPairdiff pPa;
  pJuha pJ;
  pLn pLogn;
/*  pMu pM; */
  pMp pMidp;
  pGamma_lexis pGL; /* Gamma */
} priorVal;

/* struct pmu {
  ft mu;
} jackpmu; */

struct pmp {
  ft mu;
  ft pNewJp;
  ft pDelJp;
} jackpmp;

struct pG { /* Arjas-Gasbarra gamma prior */
  ft alpha0;
  ft beta0;
  ft alpha;
  ft beta;
  ft mu;
  double mix;
} jackpg;

struct pP { /* Pairwise difference prior */
  double a;
  double w;
} jackpp;

struct pBm_struct { /* Pairwise difference prior */
  ft delta;
  ft drift;
  ft sigma2;
  double mix;
} jackbrownian;

struct pJ { /* Arjas-Heikkinen prior */
  double mu;
  double sigma2; /* Variance */
  double beta;
  double lambdaXi;
  double c; /* 0.0 < c < 0.5 */
  double cc;
  double delta;
} jackpj;

struct pTL { /* Timeless prior */
  prior p;
} jackpTL;

struct pGL { /* Arjas-Gasbarra gamma prior with Lexis addition */
  ft alpha0;
  ft beta0;
  ft alpha;
  long lexis_dim;
  ft *lexis_weight;
  ft beta;
  ft mu;
  double mix;
} jackpgl;

struct pln {
  double beta;
} jackpln;

typedef union {
  priorVal *pv;
} vValue;

typedef struct vari *variable;

struct vari{
  char name[MAX_VARNAME + 1];
  long type;
  long spec;
  vValue *x;
  use where;
} jackVariable;

/* The following ones are for priors as well, but they are supposed
   to be better (at least they are newer :-/ ) */

/* Union for parameter type */

typedef union {
  double x;
  prior vec;
  pProcess pp;
  char *str;
  sf f;
  dataNode **resp;
  covNode *cov;
  paramVector pvec;
  prior p;
  mf m;
  struct {
    double x;
    long type;
  } mix;
} pType;

struct pointProcess {
  prior t0, t1;  /* Endpoints of the obs. interval. */
  prior vector;  /* Observed points. */
} jackPp;

/* struct simpleF {  This is the stochastic version of a simple function. 
  prior ext;      Value of the sf outside of [t0,t1]. Usually 0. 
  pProcess jumpPoints;
  prior levels;
} jackSF; */

typedef union {
  long *inUse;
  long **inUse2;
  /*  pType parameter; */
  struct {
    long i;
    prior p;
  } indParam;
  prior x;
  mf f;
  variable var;
  dataNode resp;
  /* struct { */
  paramVector pvec;
  /*   long *inUse;
  } data; */
} useVal;

struct uSe { /* This struct tells where & what for the prior is for */
  long type;  /* Values in USE_... */
  useVal *val;
  use next;
  use prev;
  use back;
} jackUse;

struct priori {
  char *name;
  long index;
  long fixed;
  long update;
  long wait;
  long burn;
  long type; /* Distribution: values are D_... */
  long monitor; /* '0' means no change, otherwise 'printLevel' while updating
		  the parameter. */

  long stor; /* A storage for a vector */
  prior *vector;

  long nolikelihood;
  ft transform;
  ft restrict1;
  ft restrict2;
  double mix;
  double epsilon;
  long save;
  /*  FILE *fp; */
  long file;
  long nArgs;
  ft *args;
  long ptype;
  pType *x;
  /*  double *sample; */
  use where;
  ft indiv_likel;
  ft cut;
  /*  long ngt;
  double **gt;
  long nlt;
  double **lt; */
  prior prev;
  prior next;
  prior up;
} jackPrior;

/* Structure for a model function: */

typedef union {
  sf f;
  prior p;
} fType;

struct modelF {
  char name[MAX_VARNAME];
  char internOp;
  long monitor; /* '0' means no change, otherwise 'printLevel' while updating
		  the model function. */
  long wait;
  long save;
  long file;
  double epsilon;
  char *cov[MAX_COV];
  double covVal[MAX_COV];
  /* Vectors of pointers to the neighbouring mf's (lexis): */
  long neighbour_dim;
  mf *neighbour_prev;
  mf *neighbour_next;
  long type;
  fType *f;
  long nfixjp;
  double *fixjp;
  long nfixle;
  double *fixle_x;
  double *fixle_y;
  /*  sf f; */
  prior jumpPoints;
  prior levels;
  long **inUse;
  long nIndiv;
  long *indivVec;
  long *riskSet;
  double length;
  double restrict0;
  double minPieceLength;
/*  double mu; */
  long prior;
  priorVal *pVal;
  /*  char title[MAX_STRLEN]; */
} jackmf;

typedef struct modelFTree *mft;

struct modelFTree {
  long n;
  long storage;
  long *nFunc;
  mf **fList;
} jackmft;

/* Structure for the formula tree: */

struct fT {
  long lineno;
  long op;
  long function;
  long nArgs;
  ft *args;
  char *varName;
  long ptype;
  pType *var;
/*  double x; */

  long fieldId;
  long storageF; /* How many mf's there is in the vector 'f'. */
  mf *f;
  long *fc; /* Which 'f' for which individual?
	      (Length == ind, and values in {0,1,...,storageF-1}.) */
  long val;
  long ni; /* Number of indeces. */
  ft *i; /* Index array. */
  /*  long index;
  long *jpIndex; */
  sf *choice;
  long storageG;
  /*  long sp; */
  sf *g;
  prior start;
  paramVector start2;
  prior stop;
  paramVector stop2;
  /*  covNode factor; */
  ft left;
  ft right;
  ft third;
} jackFt;

typedef struct fForest *ff;

struct fForest {
  long n;
  long storage;
  char **name;
  ft **ftList;
} jackfF;

typedef struct {
  char *name;
  char *start;
} fTerm;

/* Structure for missing covariates */

typedef struct {
  long n;
  long *mark;
  long *ind;
  prior p;
  double *value;
} missingCov;

/* Structure for TTT plot: */

typedef struct tttPlot *ttt;

struct tttPlot {
  long responce;
  long ncov;
  long cov[MAX_COV];
  double **mean;
  double **weight;
  double **weight2;
  double *w0;
  double *np;
  long *stor;
  long *file;
  ttt next;
} jackTTT;

/* Structure for plotting */

typedef struct drawStructure *draw;

struct drawStructure {
  long nPic;
  long drawType[MAX_PICTURES]; /* 1...4 as follows: */
  mf f[MAX_PICTURES]; /* 1: for model functions */
  ft tree[MAX_PICTURES]; /* 2: for intensities */
  ft dest[MAX_PICTURES];
  double *mean[MAX_PICTURES]; /* Estimated posterior mean */
  double bpr[MAX_PICTURES]; /* Credibility level of the Bayes prob. reg. */
  double **median[MAX_PICTURES]; /* Storage of the sample of a simple fn. */
  prior pv[MAX_PICTURES]; /* 3: for scalar parameter */
  char *mfName[MAX_PICTURES]; /* 4: mean of model functions */
  long data[MAX_PICTURES]; /* If data points included (boolean) */
  long riskSet[MAX_PICTURES]; /* If risk set size is included (boolean) */
  long logScale[MAX_PICTURES]; /* The range of the y-axis is (0,maxVal) */
  double maxVal[MAX_PICTURES]; /* The range of the y-axis is (0,maxVal) */
  double minVal[MAX_PICTURES];
  /* The range of the log y-axis is (minVal,maxVal) */
  long maxRiskSet[MAX_PICTURES]; /* The maximum of the risk set size */
} jackDS;

/* Structures of posterior expectations. */

typedef struct {
  long n;
  ft *f;
  prior loopParam;
  ft lower;
  ft upper;
  double *mean;
  double *variance;
  double **val;
  long counter;
  ft weight;
  double sumOfWeights;
  double scaleOfWeights;
} PostExp;

/* typedef PostExp *postExp;
PostExp jackPE; */

typedef struct Expectation_struct {
  char name[MAX_VARNAME];
  long type;
  long n;
  long saveVal;
  long printVar;
  char from[MAX_VARNAME]; /* If set, exchange the values of these */
  char to[MAX_VARNAME];   /* parameters before calculating. Note: the values */
                          /* are not reset after updateExpectation. */
  PostExp *pe;            
  struct Expectation_struct *next;
} Expectation;

typedef Expectation *expectation;

/* Structure for cluster updating of scalar parameters. */

typedef union {
  prior p; /* Parameter. */
  struct {
    ft i; /* Individual */
    ft x; /* Covariate or responce. */
  } d;
} clem; /* Cluster element. */

typedef struct ClusterUpdate_struct {
  char *name;
  long type;
  long nLoop;
  prior *loopParam;
  ft *dim;
  ft *lower;
  ft *upper;

  long n;
  ft *vec; /* Parameters to be updated. */
  ft *new_val; /* Proposal values in case of Metropolis updating. */
  double **last;

  long nl; /* The loglikelihood contributions. */
  ft *likel;

  long qn_xy;
  ft *prop_dens_xy; /* Log density of M-H proposal old -> new. */

  long qn_yx;
  ft *prop_dens_yx; /* Log density of M-H proposal new -> old. */

  long nclean;
  ft *clean; /* Restoration of parameter values in case of rejection. */

  double *sd;
  long *last_update;

  double **mean; /* Parameters of the multinormal proposal distribution */
  double ***cov;
  struct ClusterUpdate_struct *next;
} ClusterUpdate;

typedef ClusterUpdate *clusterUpdate;

typedef struct GradientUpdate_struct {
  char *name;
  long nLoop;

  long n;
  ft *vec; /* Parameters to be updated. */
  long nl; /* The loglikelihood contributions. */
  ft *likel;

  double *lx;
  double *lx0;
  double **gr;
  double **values;
  double **values0;

  /* Information for possible search for multimodal maxima: */
  long mm_p; /* Index of the parameter presenting multimodality in *vec */
  double mm_mu;
  double mm_sigma2;

  struct GradientUpdate_struct *next;
} GradientUpdate;

typedef GradientUpdate *gradientUpdate;

extern void bite_exit(int nro);
extern void bite_eprintf( const char* format, ... );
extern void bite_printf( const char* format, ... );
extern FILE *bite_fopen(char *path, char *mode);
extern void bite_srand(unsigned int seed);
extern double bite_rand();

#endif
