/* popen() and pclose() are not ANSI C standards, so define Posix: 
#define _POSIX_SOURCE */
#define _GNU_SOURCE

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <ctype.h>
#include <time.h>
#include <string.h>
#include <unistd.h>
#include "typedef.h"
#include "sf3.h"
#include "general3.h" 
#include "bit.h"
#include "load.h"
#include "prior.h"
#include "modelf.h" 
#include "ftree.h"
#include "splusplot.h"
#include "likel.h"
#include "ttt.h"
#include "bite.tab.c.h"

extern FILE * yyin;
extern FILE * yyout;

/* double returnLexisMfLogPriorDensity(long ind, mf f, 
				    long updL, double start, double stop,
				    double alpha0, double beta0, double alpha,
				    double beta, double *lexis_weight,
				    long children); */

/* Version 1.2 of 'BITE'. */
/* New old things: possibility to have covariates with missing values and
   hierarchical structures for parameters. */

/* Definitions of the functions */
void simulation();
void totalPPLikel();
void updateMissingCov();
void updateMissingCovNormalApprox();
void updateResponce();
void updateParameters(prior ptr);
void updateClusters(clusterUpdate c);
void updateGradient(gradientUpdate g);
void updatePred();
void printPred();
void updateSurv();
/* void updateMeans(); */
void printSurv();
void updateExpectation();
void printExpectation();
void printPostOfMissCov(char *directory);
void update(mf target, dt d, ff model);
prior *makeParamVec(prior param, long *nParam);

/* extern long yyparse(); */

/* Global variables */
FILE *bite_err;
long max_iter = 1, iter_limit = 100, thinning = 1, skip = 25, drawMod = 25,
  printLevel = 1, ind = 1, nVars = 0, nextI = -999, tries = 0, fails = 0,
  frames = 0, cFrame = 0, cPicture = 0, iter = -1, orientation = FALSE, 
  files = 0, lineno = 1, aStack = 0, lStack = 0, zeroLikel, zeroLI,
  maxVars = 10, dumttt = FALSE, nextCar = -222, level = 0, findUse = FALSE,
  lindex[MAX_STACK_SIZE], index0, loadLag = 1, fixedIndivVec = FALSE,
  nParam = 0, maxWait = 0, annealingLength = 0, G_annealing_i, lexis_dim = 0;
double missing = -88.0, *old = NULL, logLmean = 0.0, logLmom2 = 0.0,
  logPPmean = 0.0, logPPmom2 = 0.0, lMean = 0.0, lMom2 = 0.0, lMscale,
  annealingInit = 1.0, G_annealing = 1.0;
/* Define global variables for lexis prior weights in order to 
   avoid memory allocations: */
long lexis_dim;
double *lexis_weight=NULL;
/* Declare hyperparameters as global: */
double alpha0, beta0, alpha, beta, mu, *lexis_alpha = NULL; 
/* "*old" is for storing old parameter values in group updating. */
variable *vars;
argArray a[MAX_STACK];
ct c = NULL; /* Covariates. */
dt d = NULL; /* Point process responce. */
expectation e = NULL; /* (Some) posterior expectations to be calculated. */
mft f = NULL; /* Model functions (the component functions of intensities). */
prior param = NULL, /* Scalar parameters and prototypes. */
  *param_vec = NULL;
clusterUpdate cu = NULL;
gradientUpdate gu = NULL;
pi pred = NULL;
pi surv = NULL;
ttt tttplots = NULL;
draw *plots = NULL;
ff model = NULL;
ft init_comm = NULL;
ft iter_comm = NULL;
labelArray labela[MAX_STACK], miss[MAX_STACK];
#ifdef DEBUG_SF3
long sf3_print = FALSE;
FILE *debug_sf3;
#endif

#ifdef STANDALONE
int main( int argc, char *argv[] )
{
  int inopen, outopen;
  long k = 1, i;
  time_t start;
  FILE *fp;

  inopen=0; outopen=0;
  yyin=stdin; yyout=stdout; bite_err=stderr;
  bite_printf("Copyright (C) Tommi Harkanen 2000.\n");
  bite_printf("Absolutely NO WARRANTY!\n");
  bite_printf("Bite was compiled on " __DATE__ "\n");
  time(&start);
  /* bite_printf("Probit function values %g %g %g.\n",
	 probit(0.0, 0.0, 1.0),
	 probit(-30, 0.0, 1.0),
	 probit(30, 0.0, 1.0));
  bite_printf("Probit function values %g %g %g.\n",
	 probit(-1.0, 0.0, 1.0),
	 probit(-0.001, 0.0, 1.0),
	 probit(-0.007, 0.0, 1.0));
  bite_printf("Probit function values %g %g %g.\n",
	 probit(1.0, 0.0, 1.0),
	 probit(0.001, 0.0, 1.0),
	 probit(0.007, 0.0, 1.0));
  bite_printf("Probit function values %g %g %g.\n",
	 probit(1.0003, 0.0, 1.0),
	 probit(0.0001, 0.0, 1.0),
	 probit(0.0007, 0.0, 1.0));
  bite_printf("Probit function values %g %g %g.\n",
	 probit(-1.0005, 0.0, 1.0),
	 probit(-0.0015, 0.0, 1.0),
	 probit(-0.0025, 0.0, 1.0));
  bite_printf("Probit function values %g %g %g.\n",
	 probit(-1.0005, 0.0, 1.0),
	 probit(-2.0005, 0.0, 1.0),
	 probit(-3.0005, 0.0, 1.0));
  bite_printf("Probit function values %g %g %g.\n",
	 probit(-0.01, 0.0, 1.0),
	 probit(-0.011, 0.0, 1.0),
	 probit(-0.017, 0.0, 1.0));
  bite_printf("Probit function values %g %g %g.\n",
	 probit(-29.9995, 0.0, 1.0),
	 probit(-29.9990, 0.0, 1.0),
	 probit(-29.9895, 0.0, 1.0));
  bite_printf("Probit function values %g %g %g.\n",
	 probit(29.9995, 0.0, 1.0),
	 probit(29.9990, 0.0, 1.0),
	 probit(29.9895, 0.0, 1.0));
	 bite_exit(1); */
  /*  bite_printf("Log-Gamma function values %g %g %g.\n",
	 lgamma(1), lgamma(2), lgamma(3));
  bite_printf("Gamma function values %g %g %g.\n",
	 gamma(1), gamma(2), gamma(3));
	 bite_exit(1); */
  /* if (argc == 1) {
    bite_printf("\nInput the name of the options file: ");
    scanf("%s", &cmdFile);
  }
  else {
    cmdFile[0] = '\0';
    strcat(cmdFile, argv[1]);
  } */
  if (argc >= 2)
    sscanf(argv[1], "%ld", &k);
  else {
    k = start; /* & 0777777; */
    bite_printf("Seed is %ld.\n", k);
  }
  if (argc>=3) /* RS 22.5.2014 */
  	{
  	yyin = bite_fopen(argv[2],"r");
  	if (yyin==NULL)
  		{
  		bite_eprintf("File %s not found!\n",argv[2]);
  		return(-1);
  		}
  	inopen=1;
  	bite_printf("Command file is %s\n",argv[2]);
  	}
    /*    bite_printf("Input seed for the random number generator: ");
  scanf("%ld", &k); */
#else
/*
int main( int argc, char *argv[] )
{
return (yylex());
}
*/

int biteR(int seed, char *infile, char *outfile)
	{
	int inopen, outopen;
	long k = 1, i;
	time_t start;
	FILE *fp;

	inopen=0; outopen=0;
	yyin=NULL; yyout=NULL;
  	time(&start);

  	if (seed) k=(long)seed;
  	else 
		{
		k = start; /* & 0777777; */
		}  
	bite_printf("Seed is %ld.\n", k);

	bite_printf("Opening files %s and %s.\n", infile, outfile);

  	yyin = bite_fopen(infile,"r");
  	if (yyin==NULL)
  		{
  		bite_eprintf("File %s not found!\n",infile);
  		return(-1);
  		}
  	inopen=1;

  	yyout = bite_fopen(outfile,"w");
  	if (yyout==NULL)
  		{
  		bite_eprintf("File %s not found!\n",infile);
  		return(-1);
  		}
  	bite_err=yyout;	
  	outopen=1;

#endif
  
  bite_printf("Entering srand.\n");
  
  bite_srand(k);

  /* Remove this:
  {
    long j;
    double b;
    FILE *fp;
    sf test_f = initSf(0.0, 1, NULL);

    addNode(test_f, 1, 1.0, 5);
    addNode(test_f, 2, 2.0, 1);
    addNode(test_f, 3, 6.0, 0.00);
    fp = bite_fopen("dum.tmp", "w");
    for (j = 0; j < 1000; j++) {
      b = ran_poiPr(test_f, 0, 6);
      fprintf(fp, "%g\n", b);
    }
    fclose(fp);
    bite_exit(1);
  }
  End remove. */

  bite_printf("malloc.\n");

  vars = malloc(maxVars * sizeof(variable));
/*  labela = (labelArray) malloc(sizeof()); */

  bite_printf("Entering opSf.\n");

  opSf(' ', NULL, NULL, TRUE);
  
  bite_printf("Entering initCovTree.\n");
  
  c = initCovTree(MAX_COV);
  d = initDataTree(1);
  f = initMft(MAX_FUNCS);
  model = initFF(MAX_MARKS);
  plots = NULL;
  for (i = 0; i < MAX_STACK; i++)
    a[i] = (argArray) malloc(sizeof(jackArgs));

  yydebug = 0;
  bite_printf("Entering yyparse.\n");
  
  yyparse();
  if (inopen) fclose(yyin);
  if (outopen) fclose(yyout);  
  bite_printf("yyparse finished.\n");
  param_vec = makeParamVec(param, &nParam);

  bite_printf("makeParamVec finished.\n");

  /* printLevel=3; */

  if (printLevel > 2)
    writeLogFile();

  fp = bite_fopen("loglikel.sav", "w");
//  fprintf(fp, "");
  fclose(fp);    
  bite_printf("Entering simulation, printlevel=%d.\n",printLevel);
    
  simulation();

  return 0;
}

prior *makeParamVec(prior param, long *nParam)
{
  long i;
  prior *res = NULL, ptr = param;

  while (ptr) {
    (*nParam)++;
    ptr = ptr->next;
  }
  res = malloc(*nParam * sizeof(param));
  for (i = 0, ptr = param; i < *nParam; i++, ptr = ptr->next)
    res[i] = ptr;
  return(res);
}

void simulation()
{
  long i, j, oldPL = 0, annealing = FALSE, printLevelBackup = printLevel;
  time_t start, present, prev, stop, firstPred = 0, nt1, nt2;
  double aver_time = 0.0;
  ttt t3;

  if (frames)
    startSplus();

/* Test the speed of likelihood routines 
  for (iter = 0; iter < iter_limit; iter++) {
    lx = logLikel(f->fList[0][0], -1, -1, d, model);
    bite_printf("%ld\n", iter);
  }
  time(&stop);
  bite_printf("Likelihood: %f,\ttime: %ld\n", lx, stop - start);
  bite_exit(1); */

/* Iteration */
  for (i = 0; i < f->n; i++)
    for (j = 0; j < f->nFunc[i]; j++)
      findInd(ind, model, f->fList[i][j], f);
  time(&start);
  nt1 = prev = present = start;

  /* {
    FILE *fp = bite_fopen("grid.dat", "w");
    prior p1 = param, p2 = param;
    while (strcmp(p1->name, "beta_op"))
      p1 = p1->next;
    while (strcmp(p2->name, "beta_age"))
      p2 = p2->next;
    for (i = 0; i < 100; i++) {
      bite_printf("%ld\n", i);
      p1->x->x = i / 10.0 - 6.0;
      for (j = 0; j < 100; j++) {
	p2->x->x = j / 10.0 - 6.0;
	fprintf(fp, "%g\n", logLikel3(-1, p1->where));
      }
    }
    bite_printf("\n");
    fclose(fp);
    bite_exit(1);
  } */

  calculate2(-1, init_comm);

  for (iter = 0; iter < iter_limit; iter++) {
#ifdef DEBUG_SF3
    debug_sf3 = bite_fopen("junk.txt", "w");
#endif
    if (iter == skip)
      for (i = 1; i <= frames; i++)
	for (j = 1; j <= plots[i]->nPic; j++) {
	  writeData(i, j);
	  riskSetSize(i, j);
	}
    zeroLikel = 0;
    if (printLevel >= 1)
      bite_printf("\nIteration step %ld:\n", iter);
    if (printLevel >= 2)
      bite_printf("\n");

    if (annealingLength > 0 &&
	((iter >= skip && (iter - skip) % thinning == 1) ||
	 (iter > 100 && iter < skip && iter % thinning == 1))) {
      /* If using constrained annealing, then ... */
      G_annealing_i = 0;
      G_annealing = annealingInit;
      annealing = TRUE;
    }
    else {
      annealing = FALSE;
      G_annealing = 1.0;
    }
    printLevelBackup = printLevel;

    while ((annealing && G_annealing < 1.0) ||
	   (!annealing && G_annealing <= 1.0)) {
      /*if (printLevel >= 1)*/
	bite_printf("Annealing temperature %g.\n", 1/G_annealing);
      if (annealing)
	printLevel = 0;
      for (i = 0; i < f->n; i++) {
	if (printLevel >= 2)
	  totalPPLikel();
	for (j = 0; j < f->nFunc[i]; j++) {
	  if (f->fList[i][j]->monitor) {
	    oldPL = printLevel;
	    printLevel = f->fList[i][j]->monitor;
	  }
	  update(f->fList[i][j], d, model);
	  if (f->fList[i][j]->monitor)
	    printLevel = oldPL;
	  if (f->fList[i][j]->save == SAVEITER && iter >= skip &&
	      iter % thinning == 0) {
	    switch (f->fList[i][j]->type) {
	    case SF_ORIG:
	      writeSf(f->fList[i][j]->file, f->fList[i][j]->f->f, nextI);
	      break;
	    case SF_TIMELESS:
	      break;
	    }
	  }
	}
	time(&nt2);
	if (printLevel >= 1)
	  bite_printf("\tUpdating of %s took %ld seconds.\n",
		 f->fList[i][0]->name, (long)nt2 - nt1);
	nt1 = nt2;
	fflush(NULL);
      }
      updateMissingCov();
      updateMissingCovNormalApprox();
      fflush(NULL);
      time(&nt2);
      if (printLevel >= 1)
	bite_printf("\tUpdating of missing covariates took %ld seconds.\n",
	       (long)nt2 - nt1);
      nt1 = nt2;
      /* {
	 prior grr = param->next->next->next->x->vec->next;
	 long i;
	 double x = 0.0;
	 for (i = 0; i < 240; i++) {
	 x += paramValue(-1, grr);
	 grr = grr->next;
	 }
	 bite_printf("\t\tMean of eta's was %g.\n", x / 240.0);
	 } */
      updateParameters(param);
      fflush(NULL);
      /* {
	 prior grr = param->next->next->next->x->vec->next;
	 long i;
	 double x = 0.0;
	 for (i = 0; i < 240; i++) {
	 x += paramValue(-1, grr);
	 grr = grr->next;
	 }
	 bite_printf("\t\tMean of eta's is %g.\n", x / 240.0);
	 } */
      time(&nt2);
      if (printLevel >= 1)
	bite_printf("\tUpdating of parameters took %ld seconds.\n", (long)nt2 - nt1);
      nt1 = nt2;
      updateResponce();
      fflush(NULL);
      time(&nt2);
      if (printLevel >= 1)
	bite_printf("\tUpdating of missing responces took %ld seconds.\n",
	       (long)nt2 - nt1);
      nt1 = nt2;
      /*printLevel = 2;*/
      updateClusters(cu);
      if (iter >= maxWait + 10 &&
	  (iter < skip / 2 ||
	   iter % 1 == 0))
	updateGradient(gu);
      fflush(NULL);
      /*printLevel = 1;*/
      time(&nt2);
      if (printLevel >= 1)
	bite_printf("\tUpdating of clusters took %ld seconds.\n", (long)nt2 - nt1);
      nt1 = nt2;
      updateMeans(iter, f);
      updatePred();
      updateSurv();
      fflush(NULL);

      /* {
	 double pr1 = 0.0, pr2 = 0.0, mean = 0.0, mean2, old_mean;
	 long i, j, k;
	 for (i = 0; i < ind / 20; i++)
	 mean += f->fList[6][i]->f->f->levels[0];
	 mean /= ind / 20;
	 bite_printf("\tFrailty mean is %g\n", mean);
	 for (i = 0; i < ind / 20; i++) {
	 f->fList[6][i]->f->f->levels[0] /= mean;
	 pr1 += returnMfLogPriorDensity(0, f->fList[6][i]);
	 if (returnMfLogPriorDensity(0, f->fList[6][i]) < -1e+89)
	 bite_printf("\t\t%ld:%g\n", i, f->fList[6][i]->f->f->levels[0]);
	 }
	 for (i = 1; i < 6; i++) {
	 for (j = 0; j < f->nFunc[i]; j++) {
	 for (k = 0; k < f->fList[i][j]->f->f->n - 1; k++)
	 f->fList[i][j]->f->f->levels[k] *= mean;
	 pr1 += returnMfLogPriorDensity(0, f->fList[i][j]);
	 if (returnMfLogPriorDensity(0, f->fList[i][j]) < -1e+89)
	 bite_printf("\t\t%s(%s==%g);%ld,%ld,%ld:%g\n", f->fList[i][j]->name,
	 f->fList[i][j]->cov[0], f->fList[i][j]->covVal[0],
	 i, j, k, f->fList[i][j]->f->f->levels[k]);
	 }
	 }
	 for (i = 0; i < ind / 20; i++)
	 pr2 += f->fList[6][i]->f->f->levels[0];
	 bite_printf("\tFrailty mean is %g, should be 1.\n", pr2 / ind * 20);
	 bite_printf("\tPrior logdensity of Z's and h's is %g\n", pr1);
	 old_mean = 1.0;
	 for (mean2 = 0.5; mean2 <= 1.5; mean2 += 0.1) {
	 pr1 = 0.0;
	 for (i = 0; i < ind / 20; i++) {
	 f->fList[6][i]->f->f->levels[0] *= mean2 / old_mean;
	 pr1 += returnMfLogPriorDensity(0, f->fList[6][i]);
	 }
	 for (i = 1; i < 6; i++) {
	 for (j = 0; j < f->nFunc[i]; j++) {
	 for (k = 0; k < f->fList[i][j]->f->f->n - 1; k++)
	 f->fList[i][j]->f->f->levels[k] /= mean2 / old_mean;
	 pr1 += returnMfLogPriorDensity(0, f->fList[i][j]);
	 }
	 }
	 bite_printf("\tPrior logdensity of Z's and h's is %g when frailty mean is "
	 "%g\n", pr1, mean2);
	 old_mean = mean2;
	 }
	 for (i = 0; i < ind / 20; i++)
	 f->fList[6][i]->f->f->levels[0] *= mean / old_mean;
	 for (i = 1; i < 6; i++) {
	 for (j = 0; j < f->nFunc[i]; j++) {
	 for (k = 0; k < f->fList[i][j]->f->f->n - 1; k++)
	 f->fList[i][j]->f->f->levels[k] /= mean / old_mean;
	 }
	 }
	 } */

      updateExpectation();
      fflush(NULL);
      t3 = tttplots;
      dumttt = TRUE;
      while (t3) {
	updateTTT(t3, FALSE);
	t3 = t3->next;
      }
      dumttt = FALSE;
      /*    if (printLevel >= 1)
	    bite_printf("Poisson likelihood was zero %ld times.\n", zeroLikel); */
      if (frames && iter % drawMod == 0 && iter >= skip) {
	yAxisMax();
	plotAll(1);
      }
      /* {
	 long ii, jj;
	 prior p1 = param, p2;
	 for (ii = 0; ii < 12; ii++)
	 p1 = p1->next;
	 bite_printf("Parameter %s:\n", p1->name);
	 p2 = p1->x->vec;
	 for (ii = 0; ii < 7; ii++, p2 = p2->next)
	 bite_printf("%g, ", p2->x->x);
	 bite_printf("\n");
	 for (ii = 0; ii < 7; ii++)
	 printSf(f->fList[6][ii]->f->f);
	 p1 = p1->next->next;
	 bite_printf("Parameter %s:\n", p1->name);
	 p2 = p1->x->vec;
	 for (ii = 0; ii < 12; ii++) {
	 for (jj = 0; jj < 20; jj++, p2 = p2->next)
	 bite_printf("%3ld, ", (long) p2->x->x);
	 bite_printf("\n");
	 }
	 } */
      totalPPLikel();
      calculate2(-1, iter_comm);
      totalPPLikel();
      fflush(NULL);
      /* {
	 long ii, jj;
	 prior p1 = param, p2;
	 for (ii = 0; ii < 12; ii++)
	 p1 = p1->next;
	 bite_printf("Parameter %s:\n", p1->name);
	 p2 = p1->x->vec;
	 for (ii = 0; ii < 7; ii++, p2 = p2->next)
	 bite_printf("%g, ", p2->x->x);
	 bite_printf("\n");
	 for (ii = 0; ii < 7; ii++)
	 printSf(f->fList[6][ii]->f->f);
	 p1 = p1->next->next;
	 bite_printf("Parameter %s:\n", p1->name);
	 p2 = p1->x->vec;
	 for (ii = 0; ii < 12; ii++) {
	 for (jj = 0; jj < 20; jj++, p2 = p2->next)
	 bite_printf("%3ld, ", (long) p2->x->x);
	 bite_printf("\n");
	 }
	 } */
      time(&nt2);
      if (printLevel >= 1)
	bite_printf("\tExecution of \'iter_commands\' took %ld seconds.\n",
	       (long)nt2 - nt1);
      nt1 = nt2;
      time(&present);
      nt1 = present;
      if (printLevel >= 1) {
	bite_printf("\tThis step took %ld seconds.\n", (long)(present - prev));
	if (iter < 2 * thinning)
	  aver_time = (present - start) / (iter + 1);
	else
	  aver_time = aver_time * 0.9 + (present - prev) * 0.1;
	bite_printf("\tAverage time per iteration has been %g seconds.\n", 
	       (double) (present - start) / (iter + 1));
	stop = present + aver_time * (iter_limit - iter - 1);
	bite_printf("\tPredicted time of completion is %s",
	       ctime(&stop));
	if (start != present && !firstPred)
	  firstPred = stop;
	prev = present;
      }
      totalPPLikel();
      fflush(NULL);
#ifdef DEBUG_SF3
      fclose(debug_sf3);
#endif
      if (annealing)
	G_annealing += (1.0 - annealingInit) / annealingLength;
      else
	G_annealing += 1.0;
    }
    printLevel = printLevelBackup;
  }
  time(&present);

  /* Final output. */
  printPostOfMissCov("");
  printPred();
  printSurv();
  printExpectation();
  t3 = tttplots;
  while (t3) {
    saveTTT(t3);
    t3 = t3->next;
  }
  closeAllFiles();
  iter--;
  yAxisMax();
  for (i = 1; i <= frames; i++) {
    plotAll(i);
#ifdef SPLUS
    savePicture(i);
    sleep(3);
#endif
#ifdef NOGRAPH
    sleep(0);
#endif
  }
  if (printLevel >= 1) {
    bite_printf("\nThe simulation started on %s", ctime(&start));
    bite_printf("Time is now %s", ctime(&present));
    bite_printf("First prediction was %s", ctime(&firstPred));
    fflush(NULL);
  }
  if (frames)
    quitSplus();

  bite_printf("Bite was compiled on " __DATE__ "\n");

  if (1 == 2) { /* Truncate the 'out.txt' file for keeping Matti happy. */
    FILE *plot = NULL;
    plot = popen("head out.txt > /tmp/out.txt ; tail -n 1000 out.txt >> /tmp/out.txt ; mv -f /tmp/out.txt out.txt", "w");
    fflush(plot);
    fclose(plot);
  }
  /*  bite_exit(0); */
} /* end main */

double paramLD(prior root)
{
  double res = 0.0, incr;
  prior ptr = root;

  lindex[level] = 0;
  while (ptr) {
    if (ptr->type) {
      res += incr = (!ptr->fixed) ?
	returnPriorLogDensity(lindex[level], ptr,
			      paramValue(lindex[level], ptr)) : 0.0;
      if (incr < -10000) {
	bite_printf("%s:%g, %g,%g\n", ptr->name, ptr->x->x, incr, res);
	returnPriorLogDensity(lindex[level], ptr, paramValue(lindex[level], ptr));
      }
    }
    else
      if (ptr->ptype == PT_VECTOR) {
	level++;
	res += paramLD(ptr->x->vec);
	level--;
      }
    lindex[level]++;
    ptr = ptr->next;
  }
  return res;
}

void totalPPLikel()
{
  long i, j, mark, cov;
  static long last_iter = 0;
  static double res, resPP, resLL, incr, term; // prev

  if (printLevel > 0) {
    res = 0.0;
    for (mark = 0; mark < d->n; mark++)
      for (i = 0; i < ind; i++) {
	res += incr = indLogL(i, mark, NULL, -INFTY, INFTY, d, model);
	if (incr < -INFTY / 2 && printLevel >= 1 && iter > 30) {
	  bite_printf("(%ld,%ld) ", mark, i);
	  /* dum = printLevel;
	  printLevel = 4;
	  indLogL(i, mark, NULL, -INFTY, INFTY, d, model);
	  printLevel = dum; */
	}
      }
    bite_printf("\tLog Poisson likelihood is %g.\n", res);
    if (iter >= skip && last_iter < iter) {
      last_iter = iter;
      resPP = res;
      logPPmean += res;
      logPPmom2 += pow(res, 2.0);
      if (printLevel > 0 && iter > skip)
	bite_printf("\tAverage of log Poisson likelihood is %g and variance %g.\n",
		    logPPmean / (iter - skip + 1),
		    logPPmom2 / (iter - skip + 1) -
		    pow(logPPmean / (iter - skip + 1), 2.0));
      if (iter == skip)
	lMscale = res;
      lMean += exp(res - lMscale);
      lMom2 += exp(res - lMscale) * exp(res - lMscale);
      /* if (printLevel > 0)
	 bite_printf("\tAverage of Poisson likelihood is %g "
	 "(scaled by exp(%g)) and variance %g.\n",
	 lMean / (iter - skip),
	 lMscale,
	 lMom2 / (iter - skip) -
	 pow(lMean / (iter - skip), 2)); */
      /* } */
      term = paramLD(param);
      /* if (term < -0.5 * INFTY || term > 0.5 * INFTY)
	 bite_eprintf( "Parameters have Inf terms.\n"); */
      res += term;
      for (cov = 0; cov < c->n; cov++)
	if (c->pList[cov]) {
	  switch (c->pList[cov]->type) {
	  case COV_CNST2:
	    if (c->pList[cov]->distr.x)
	      for (i = 0; i < ind; i++)
		res += returnPriorLogDensity(i, c->pList[cov]->distr.x,
					     c->pList[cov]->val.x[i]);
	    break;
	  case COV_START2:
	    for (j = 0; j < c->pList[cov]->stor; j++) {
	      if (c->pList[cov]->distr.x2[j]) {
		level++;
		lindex[level] = j;
		for (i = 0; i < ind; i++)
		  if (c->pList[cov]->n2[i] > j)
		    res += returnPriorLogDensity(i, c->pList[cov]->distr.x2[j],
						 c->pList[cov]->val.x2[i][j]);
		level--;
	      }
	    }
	    break;
	  default:
	    break;
	  }
	}
      for (i = 0; i < f->n; i++)
	for (j = 0; j < f->nFunc[i]; j++) {
	  term = returnMfLogPriorDensity(-1, f->fList[i][j]);
	  /* if (term < -0.5 * INFTY || term > 0.5 * INFTY)
	     bite_eprintf( "Model function %s[%ld] has Inf terms.\n",
	     f->fList[i][j]->name, j); */
	  res += term;
	}
      /* bite_printf("\tLogdensity of posterior is %g.\n", res); */
      if (iter >= skip) {
	FILE *fp;
	resLL = res;
	logLmean += res;
	logLmom2 += pow(res, 2.0);
	fp = bite_fopen("loglikel.sav", "a");
	fprintf(fp, "%g %g\n", resLL, resPP);
	fclose(fp);
      }
    }
    /* if (iter > skip && printLevel > 0)
      bite_printf("\tAverage of logdensity of posterior is %g and variance %g.\n",
	     logLmean / (iter - skip),
	     logLmom2 / (iter - skip) -
	     pow(logLmean / (iter - skip), 2)); */
    /* if (iter && prev - res > 200)
      bite_printf("Loglikelihood got much worse! %g -> %g.\n", prev, res);
    if (iter && prev - res > 20)
      bite_printf("Loglikelihood got worse! %g -> %g.\n", prev, res);
    if (iter && prev - res < -20)
      bite_printf("Loglikelihood got better! %g -> %g.\n", prev, res); 
      prev = res; */
  }
}

void updateNormalApproxCheckCov(prior p, ff model, long i, ct c, mft f)
{
  long mark;
  use ptr;

  ptr = p->where;

  while (ptr) {
    /* switch (ptr->type) {
       case USE_TABLE:
       for (mark = 0; mark < d->n; mark++)
       if (ptr->val->inUse[mark]) {
       #ifndef B_FACTOR
       checkCovs(model->ftList[mark][0], mark, i, c, f);
       ly += indLogL(i, mark, NULL, -INFTY, INFTY,
       d, model);
       #endif
       }
       break;
       case USE_PRIOR:
       ly += returnPriorLogDensity(i, ptr->val->x,
       ptr->val->x->x->x);
       break;
       case USE_VAR:
       break;
       } */
    if (ptr->type == USE_TABLE)
      for (mark = 0; mark < d->n; mark++)
	if (ptr->val->inUse[mark])
	  checkCovs(model->ftList[mark][0], mark, i, c, f);
    ptr = ptr->next;
  }
}

void updateMissingCovNormalApprox()
{
  long cov, i;
  prior ptr;

  /*  printLevel = 4; */
  for (cov = 0; cov < c->n; cov++) { /* Check all covariates and */
    zeroLI = FALSE;
    switch (c->cList[cov][0]->type) {
    case COV_CNST2:
      ptr = c->pList[cov]->distr.x;
      if (ptr && ptr->mix && ptr->update && ptr->save != LOADITER &&
	  ptr->wait <= iter &&
	  (ptr->type == D_NORM || ptr->type == D_STUDENT ||
	   ptr->type == D_LAPLACE) &&
	  ptr->restrict1 == NULL && ptr->restrict2 == NULL) {
	if (printLevel >= 2)
	  bite_printf("Update covariate %s by using normal approximation.\n",
		 c->pList[cov]->name);
	for (i = 0; i < ind; i++) { /* all individuals. */
	  if (!checkBit(i % LONGLEN, c->pList[cov]->fixed.x[i / LONGLEN])) {
	    long l, stop, found=FALSE;
	    double x, old, y, y1, y2, lx, ly, ly1, ly2,
	      mu, sigma2;
	    /*bite_printf("Missing Cov Normal Approx i=%ld\n", i);*/
	    old = x = c->pList[cov]->val.x[i];
	    lx = returnPriorLogDensity(i, ptr, x) * G_annealing;
	    if (ptr->where)
	      lx += logLikel3(i, ptr->where, ptr->indiv_likel);

	    c->pList[cov]->val.x[i] = y1 = x - ptr->epsilon;
	    updateNormalApproxCheckCov(c->pList[cov]->distr.x,
				       model, i, c, f);
	    ly1 = returnPriorLogDensity(i, ptr, y1) * G_annealing;
	    if (ptr->where)
	      ly1 += logLikel3(i, ptr->where, ptr->indiv_likel);

	    c->pList[cov]->val.x[i] = y2 = x + ptr->epsilon;
	    updateNormalApproxCheckCov(c->pList[cov]->distr.x,
				       model, i, c, f);
	    ly2 = returnPriorLogDensity(i, ptr, y2) * G_annealing;
	    if (ptr->where)
	      ly2 += logLikel3(i, ptr->where, ptr->indiv_likel);

	    stop = FALSE;
	    l = 0;
	    while (!stop && l < 10) {
	      /*bite_printf("%ld, %g, %g, %g\n", l, ly1, lx, ly2);*/
	      if (ly1 < lx && lx > ly2)
		/* Maximum is between y1 and y2. */
		stop = TRUE;
	      else {
		if (ly1 < lx && lx < ly2) {
		  /* Move right. */
		  y1 = x;
		  ly1 = lx;
		  x = y2;
		  lx = ly2;
		  y2 += ptr->epsilon;
		  c->pList[cov]->val.x[i] = y2;
		  updateNormalApproxCheckCov(c->pList[cov]->distr.x,
					     model, i, c, f);
		  ly2 = returnPriorLogDensity(i, ptr, y2) * G_annealing;
		  if (ptr->where)
		    ly2 += logLikel3(i, ptr->where, ptr->indiv_likel);
		}
		else {
		  /* Move left. */
		  y2 = x;
		  ly2 = lx;
		  x = y1;
		  lx = ly1;
		  y1 -= ptr->epsilon;
		  c->pList[cov]->val.x[i] = y1;
		  updateNormalApproxCheckCov(c->pList[cov]->distr.x,
					     model, i, c, f);
		  ly1 = returnPriorLogDensity(i, ptr, y1) * G_annealing;
		  if (ptr->where)
		    ly1 += logLikel3(i, ptr->where, ptr->indiv_likel);
		}
	      }
	      l++;
	    }

	    /* Check if the epsilon is too small or too big: */
	    if ((fabs(lx -ly1) > 4 || fabs(lx -ly2) > 4) &&
		ptr->epsilon > 0.00) { /* ... too big. */
	      ptr->epsilon /=
		1.001 + pow((1.0 - iter / iter_limit) / 5.0, 2.0);
	      if (printLevel > 1)
		bite_printf("%s %g\n", c->pList[cov]->name, ptr->epsilon);
	    }
	    if ((fabs(lx -ly1) < 0.5 || fabs(lx -ly2) < 0.5) &&
		ptr->epsilon < 1000) { /* ... too small. */
	      ptr->epsilon *=
		1.001 + pow((1.0 - iter / iter_limit) / 5.0, 2.0);
	      if (printLevel > 1)
		bite_printf("%s %g\n", c->pList[cov]->name, ptr->epsilon);
	    }
	    if (printLevel > 0 && iter == iter_limit - 1)
	      bite_printf("%s %g\n", c->pList[cov]->name, ptr->epsilon);

	    found = FALSE;
	    if (normalApprox(ly1, lx, ly2, x, ptr->epsilon, &mu, &sigma2)) {
	      x = c->pList[cov]->val.x[i] = old;
	      updateNormalApproxCheckCov(c->pList[cov]->distr.x,
					 model, i, c, f);
	      lx = returnPriorLogDensity(i, ptr, x) * G_annealing -
		(- (LOG(2 * PI * sigma2) + pow(x - mu, 2.0) / sigma2) / 2.0);
	      if (ptr->where)
		lx += logLikel3(i, ptr->where, ptr->indiv_likel);

	      y = c->pList[cov]->val.x[i] = ran_norm() * sqrt(sigma2) + mu;
	      updateNormalApproxCheckCov(c->pList[cov]->distr.x,
					 model, i, c, f);
	      ly = returnPriorLogDensity(i, ptr, y) * G_annealing -
		(- (LOG(2 * PI * sigma2) + pow(y - mu, 2.0) / sigma2) / 2.0);
	      if (ptr->where)
		ly += logLikel3(i, ptr->where, ptr->indiv_likel);

	      if (LOG(ran_uni(0.0, 1.0)) < ly - lx) {
		found = TRUE;
		/*bite_printf("Found %s, l=%ld, mu=%g, sigma=%g. (%g --> %g) %g\n",
		  c->pList[cov]->name, l, mu, sqrt(sigma2), old, y, ly - lx);*/
	      }
	    }
	    if (!found) {
	      /*bite_printf("Failed %s, l=%ld, mu=%g, sigma=%g. (%g --> %g) %g\n",
		c->pList[cov]->name, l, mu, sqrt(sigma2), old, x, ly - lx);*/
	      c->pList[cov]->val.x[i] = old;
	      updateNormalApproxCheckCov(c->pList[cov]->distr.x,
					 model, i, c, f);
	    }
	  }
	}
	break;
      }
    }
  }
}

void updateMissingCov()
{
  long cov, i, j, mark, l, found;
  double x, y, lx, ly, alphaMH;
  use ptr, wptr;
  prior list;

/*  printLevel = 4; */
  for (cov = 0; cov < c->n; cov++) { /* Check all covariates and */
    if (iter == 0) {
      switch (c->cList[cov][0]->type) {
      case COV_CNST2:
	if (c->pList[cov]->distr.x) {
	  deleteOverlappingUse(c->pList[cov]->distr.x->where, 
			       c->pList[cov]->distr.x->where);
	  if (printLevel) {
	    bite_printf("Covariate %s:\n", c->pList[cov]->name);
	    printUse(c->pList[cov]->distr.x->where);
	  }
	}
	break;
      case COV_START2:
	for (j = 0; j < c->pList[cov]->stor; j++) {
	  if (c->pList[cov]->distr.x2[j]) {
	    deleteOverlappingUse(c->pList[cov]->distr.x2[j]->where, 
				 c->pList[cov]->distr.x2[j]->where);
	    if (printLevel) {
	      bite_printf("Covariate %s:\n", c->pList[cov]->name);
	      printUse(c->pList[cov]->distr.x2[j]->where);
	    }
	  }
	}
	break;
      }
    }
    for (i = 0; i < ind; i++) { /* all individuals. */
      zeroLI = FALSE;
      switch (c->cList[cov][0]->type) {
      case COV_CNST2:
	if (c->pList[cov]->distr.x && c->pList[cov]->distr.x->update) {
	  if (c->pList[cov]->distr.x->save == LOADITER) {
	    if (iter % loadLag == 0)
	      c->pList[cov]->val.x[i] =
		readFileD(c->pList[cov]->distr.x->file);
	  }
	  else {
	    if (!checkBit(i % LONGLEN, 
			  c->pList[cov]->fixed.x[i / LONGLEN]) &&
		c->pList[cov]->distr.x->type != D_DIRAC &&
		c->pList[cov]->distr.x->wait <= iter) {
	      long unifProp = FALSE;
	      double prx, pry;
	      if (!iter  && printLevel && ind < 100) {
		bite_printf("Update covariate %s, i=%ld.\n", c->pList[cov]->name, i);
		printUse(c->pList[cov]->distr.x->where);
	      }
	      /* 'unifProp == 1' means that the proposal distribution is uniform: */
	      if (c->pList[cov]->distr.x->mix &&
		  c->pList[cov]->distr.x->type != D_BIN &&
		  c->pList[cov]->distr.x->type != D_DUNIF)
		unifProp = (ran_uni(0.0, 1.0) < c->pList[cov]->distr.x->mix) ? 1 : 0;
	      l = 0;
	      while (l < c->pList[cov]->distr.x->burn || l < max_iter) {
		c->pList[cov]->distr.x->x->x = x = c->pList[cov]->val.x[i];
		prx = (unifProp) ?
		  returnPriorLogDensity(i, c->pList[cov]->distr.x, x)
		  * G_annealing : 0.0;
		lx = logLikel3(i, c->pList[cov]->distr.x->where,
			       c->pList[cov]->distr.x->indiv_likel);
		updateNormalApproxCheckCov(c->pList[cov]->distr.x, model, i, c, f);
		if (printLevel >= 2)
		  bite_printf("%s(%ld): L(x)=%g, x=%g ",
			 c->pList[cov]->name, i, lx, x);
		found = FALSE;
		while (!found && l < max_iter) {
		  switch (c->pList[cov]->distr.x->type) {
		  case D_DUNIF:
		  case D_BIN:
		  case D_MULTIN:
		  case D_DISCR:
		    /* Discrete prior */
		    if (unifProp) {
		      double a, b;
		      returnSupport(i, c->pList[cov]->distr.x, &a, &b);
		      c->pList[cov]->distr.x->x->x = c->pList[cov]->val.x[i] = y =
			(int) ran_uni(a, b + 1);
		    }
		    else
		      c->pList[cov]->distr.x->x->x = c->pList[cov]->val.x[i] = y =
			generatePriorValue(i, c->pList[cov]->distr.x);
		    break;
		  default:
		    /* Continuous distribution */
		    c->pList[cov]->distr.x->x->x = c->pList[cov]->val.x[i] = y =
		      (unifProp) ?
		      ran_uni(x - c->pList[cov]->distr.x->epsilon,
			      x + c->pList[cov]->distr.x->epsilon) :
		      generatePriorValue(i, c->pList[cov]->distr.x);
		  }
		  pry = (unifProp) ?
		    returnPriorLogDensity(i, c->pList[cov]->distr.x, y)
		    * G_annealing  : 0.0;
		  if (x == y)
		    ly = lx;
		  else {
		    ly = 0.0;
		    updateNormalApproxCheckCov(c->pList[cov]->distr.x,
					       model, i, c, f);
		    ly = logLikel3(i, c->pList[cov]->distr.x->where,
				   c->pList[cov]->distr.x->indiv_likel);
		  }
		  alphaMH = ly + pry - lx - prx;
		  if (printLevel >= 2) {
		    if (alphaMH < LOG(0.05))
		      bite_printf(".");
		    else
		      if (alphaMH < LOG(0.25))
			bite_printf("+");
		      else 
			bite_printf("*"); 
		  }
		  if (LOG(ran_uni(0.0, 1.0)) < alphaMH) {
		    found = TRUE;
		    if (printLevel >= 2)
		      bite_printf(" L(y)=%g, y=%g\n", ly, y);
		  }
		  l++;
		}
		if (!found) {
		  c->pList[cov]->val.x[i] = x;
		  updateNormalApproxCheckCov(c->pList[cov]->distr.x,
					     model, i, c, f);
		  if (printLevel >= 2)
		    bite_printf("\n");
		}
		l++;
	      }
	      if (c->pList[cov]->distr.x->save == SAVEITER &&
		  iter >= skip && iter % thinning == 0) {
		char str[20];
		str[19] = '\0';
		sprintf(str, "%g\n", c->pList[cov]->val.x[i]);
		writeFileS(c->pList[cov]->distr.x->file, str);
	      }
	      /* bite_printf("\t%g\n", c->pList[cov]->val.x[i]); */
	    }
	  }
	}
	break;
      case COV_START2:
	for (j = 0; j < c->pList[cov]->n2[i]; j++) {
	  if (c->pList[cov]->distr.x2[j] &&
	      c->pList[cov]->distr.x2[j]->update) {
	    if (!iter  && printLevel > 1) {
	      bite_printf("Update startpoint=%s[%ld], i=%ld\n", c->pList[cov]->name, j, i);
	      printUse(c->pList[cov]->distr.x2[j]->where);
	    }
	    list = c->pList[cov]->distr.x2[j];
	    wptr = c->pList[cov]->distr.x2[j]->where;
	    /* Update the start points one by one. */
	    if (!checkBit(j % LONGLEN,
			c->pList[cov]->fixed.x2[i][j / LONGLEN]) &&
		list->type != D_DIRAC) {
	      zeroLI = FALSE;
	      if (list->save == LOADITER) {
		if (iter % loadLag == 0)
		  list->x->x = readFileD(list->file);
	      }
	      else {
		if (!iter  && printLevel && ind < 100)
		  bite_printf("Startpoint %s.\n", c->pList[cov]->name);
		l =0;
		while (l < list->burn || l < max_iter) {
		  x = list->x->x = c->pList[cov]->val.x2[i][j];
		  lx = 0.0;
		  ptr = wptr;
		  while (ptr) {
		    switch (ptr->type) {
		    case USE_TABLE:
		      for (mark = 0; mark < d->n; mark++)
			if (ptr->val->inUse[mark]) {
#ifndef B_FACTOR
			  /*checkCovs(model->ftList[mark][0], mark, i, c, f);*/
			  lx += indLogL(i, mark, NULL, -INFTY, INFTY, d, model);
#endif
			}
		      break;
		    case USE_PRIOR:
		      lx += returnPriorLogDensity(i, ptr->val->x,
						  ptr->val->x->x->x)
			* G_annealing ;
		      break;
		    case USE_VAR:
		      break;
		    }
		    ptr = ptr->next;
		  }
		  if (printLevel >= 2)
		    bite_printf("%s: L(x)=%g, x=%g ", list->name, lx, x);
		  found = FALSE;
		  while (!found && l < max_iter) {
		    c->pList[cov]->val.x2[i][j] = list->x->x = y =
		      generatePriorValue(i, list);
		    if (x == y)
		      ly = lx;
		    else {
		      ly = 0.0;
		      ptr = wptr;
		      while (ptr) {
			switch (ptr->type) {
			case USE_TABLE:
			  for (mark = 0; mark < d->n; mark++)
			    if (ptr->val->inUse[mark]) {
#ifndef B_FACTOR
			      /*checkCovs(model->ftList[mark][0], mark, i, c, f);*/
			      ly +=
				indLogL(i, mark, NULL, -INFTY, INFTY, d, model);
#endif
			    }
			  break;
			case USE_PRIOR:
			  ly += returnPriorLogDensity(i, ptr->val->x,
						      ptr->val->x->x->x)
			    * G_annealing;
			  break;
			case USE_VAR:
			  break;
			}
			ptr = ptr->next;
		      }
		    }
		    alphaMH = ly - lx;
		    if (printLevel >= 2) {
		      if (alphaMH < LOG(0.05))
			bite_printf(".");
		      else
			if (alphaMH < LOG(0.25))
			  bite_printf("+");
			else 
			  bite_printf("*"); 
		    }
		    if (LOG(ran_uni(0.0, 1.0)) < alphaMH) {
		      found = TRUE;
		      if (printLevel >= 2)
			bite_printf(" L(y)=%g, y=%g\n", ly, y);
		    }
		    l++;
		  }
		  if (!found) {
		    c->pList[cov]->val.x2[i][j] = list->x->x = x;
		    ptr = wptr;
		    while (ptr) {
		      switch (ptr->type) {
		      case USE_TABLE:
#ifndef B_FACTOR
			/*for (mark = 0; mark < d->n; mark++)
			  if (ptr->val->inUse[mark])
			  checkCovs(model->ftList[mark][0], mark, i, c, f);*/
#endif
			break;
		      case USE_PRIOR:
			break;
		      case USE_VAR:
			break;
		      }
		      ptr = ptr->next;
		    }
		    if (printLevel >= 2)
		      bite_printf("\n");
		  }
		  l++;
		}
		/* if (list->sample && iter >= skip)
		   list->sample[iter - skip] = paramValue(i, list); */
		if (list->save == SAVEITER && iter >= skip &&
		    iter % thinning == 0) {
		  char str[20];
		  str[19] = '\0';
		  sprintf(str, "%g\n", c->pList[cov]->val.x2[i][j]);
		  writeFileS(list->file, str);
		}
	      }
	    }
	  }
	}
	break;
      }
    }
  }
}

void updateMissingCov1()
{
  long cov, i, j, mark, l, found;
  double x, y, lx, ly, alphaMH;
  use ptr, wptr;
  prior list;

/*  printLevel = 4; */
  for (cov = 0; cov < c->n; cov++) { /* Check all covariates and */
    if (iter == 0) {
      switch (c->cList[cov][0]->type) {
      case COV_CNST2:
	if (c->pList[cov]->distr.x)
	  deleteOverlappingUse(c->pList[cov]->distr.x->where, 
			       c->pList[cov]->distr.x->where);
	break;
      case COV_START2:
	for (j = 0; j < c->pList[cov]->stor; j++)
	  if (c->pList[cov]->distr.x2[j])
	    deleteOverlappingUse(c->pList[cov]->distr.x2[j]->where, 
				 c->pList[cov]->distr.x2[j]->where);
	break;
      }
    }
    for (i = 0; i < ind; i++) { /* all individuals. */
      zeroLI = FALSE;
      switch (c->cList[cov][0]->type) {
      case COV_CNST:
	if (c->cList[cov][i]->x->save == LOADITER) {
	  if (iter % loadLag == 0)
	    c->cList[cov][i]->x->x->x = readFileD(c->cList[cov][i]->x->file);
	}
	else {
	  if (!c->cList[cov][i]->x->fixed &&
	      c->cList[cov][i]->x->type != D_DIRAC) {
	    x = c->cList[cov][i]->x->x->x;
	    lx = 0.0;
	    ptr = c->cList[cov][i]->x->where;
	    while (ptr) {
	      switch (ptr->type) {
	      case USE_TABLE:
		for (mark = 0; mark < d->n; mark++)
		  if (ptr->val->inUse[mark]) {
		    checkCovs(model->ftList[mark][i], mark, i, c, f);
		    lx += indLogL(i, mark, NULL, -INFTY, INFTY, d, model);
		  }
		break;
	      case USE_PRIOR:
		lx += returnPriorLogDensity(i, ptr->val->x, 
					     ptr->val->x->x->x);
		break;
	      case USE_VAR:
		break;
	      }
	      ptr = ptr->next;
	    }
	    if (printLevel >= 2)
	      bite_printf("%s: L(x)=%g, x=%g ", c->cList[cov][i]->x->name, 
		     lx, x);
	    l = 0;
	    found = FALSE;
	    while (!found && l < max_iter) {
	      l++;
	      y = generatePriorValue(i, c->cList[cov][i]->x);
	      if (x == y)
		ly = lx;
	      else {
		c->cList[cov][i]->x->x->x = y;
		ly = 0.0;
		ptr = c->cList[cov][i]->x->where;
		while (ptr) {
		  switch (ptr->type) {
		  case USE_TABLE:
		    for (mark = 0; mark < d->n; mark++)
		      if (ptr->val->inUse[mark]) {
			checkCovs(model->ftList[mark][i], mark, i, c, f);
			ly += indLogL(i, mark, NULL, -INFTY, INFTY, d, model);
		      }
		    break;
		  case USE_PRIOR:
		    ly += returnPriorLogDensity(i, ptr->val->x,
						 ptr->val->x->x->x);
		    break;
		  case USE_VAR:
		    break;
		  }
		  ptr = ptr->next;
		}
	      }
	      alphaMH = ly - lx;
	      if (printLevel >= 2) {
		if (alphaMH < LOG(0.05))
		  bite_printf(".");
		else
		  if (alphaMH < LOG(0.25))
		    bite_printf("+");
		  else 
		    bite_printf("*"); 
	      }
	      if (LOG(ran_uni(0.0, 1.0)) < alphaMH) {
		found = TRUE;
		if (printLevel >= 2)
		  bite_printf(" L(y)=%g, y=%g\n", ly, y);
	      }
	    }
	    if (!found) {
	      c->cList[cov][i]->x->x->x = x;
	      ptr = c->cList[cov][i]->x->where;
	      while (ptr) {
		switch (ptr->type) {
		case USE_TABLE:
		  for (mark = 0; mark < d->n; mark++)
		    if (ptr->val->inUse[mark])
		      checkCovs(model->ftList[mark][i], mark, i, c, f);
		  break;
		case USE_PRIOR:
		  break;
		case USE_VAR:
		  break;
		}
		ptr = ptr->next;
	      }
	      if (printLevel >= 2)
		bite_printf("\n");
	    }
	    /*	    if (c->cList[cov][i]->x->sample && iter >= skip)
		    c->cList[cov][i]->x->sample[iter - skip] = 
		    c->cList[cov][i]->x->x->x; */
	    if (c->cList[cov][i]->x->save == SAVEITER && iter >= skip &&
		iter % thinning == 0) {
	      char str[20];
	      str[19] = '\0';
	      sprintf(str, "%g\n", c->cList[cov][i]->x->x->x);
	      writeFileS(c->cList[cov][i]->x->file, str);
	    }
	  }
	}
	break;
      case COV_CNST2:
	if (c->pList[cov]->distr.x && c->pList[cov]->distr.x->update) {
	  if (c->pList[cov]->distr.x->save == LOADITER) {
	    if (iter % loadLag == 0)
	      c->pList[cov]->val.x[i] =
		readFileD(c->pList[cov]->distr.x->file);
	  }
	  else {
	    if (!checkBit(i % LONGLEN, 
			  c->pList[cov]->fixed.x[i / LONGLEN]) &&
		c->pList[cov]->distr.x->type != D_DIRAC &&
		c->pList[cov]->distr.x->wait <= iter) {
	      long unifProp = FALSE;
	      double prx, pry;
	      if (!iter  && printLevel && ind < 100) {
		bite_printf("Update covariate %s, i=%ld.\n", c->pList[cov]->name, i);
		printUse(c->pList[cov]->distr.x->where);
	      }
	      /* 'unifProp == 1' means that the proposal distribution is uniform: */
	      if (c->pList[cov]->distr.x->mix &&
		  c->pList[cov]->distr.x->type != D_BIN &&
		  c->pList[cov]->distr.x->type != D_DUNIF)
		unifProp = (ran_uni(0.0, 1.0) < c->pList[cov]->distr.x->mix) ? 1 : 0;
	      l = 0;
	      while (l < c->pList[cov]->distr.x->burn || l < max_iter) {
		c->pList[cov]->distr.x->x->x = x = c->pList[cov]->val.x[i];
		prx = (unifProp) ?
		  returnPriorLogDensity(i, c->pList[cov]->distr.x, x) : 0.0;
		lx = logLikel3(i, c->pList[cov]->distr.x->where,
			       c->pList[cov]->distr.x->indiv_likel);
		ptr = c->pList[cov]->distr.x->where;
		while (ptr) {
		  /* switch (ptr->type) {
		     case USE_TABLE:
		     for (mark = 0; mark < d->n; mark++)
		     if (ptr->val->inUse[mark]) {
		     #ifndef B_FACTOR
		     checkCovs(model->ftList[mark][0], mark, i, c, f);
		     lx += indLogL(i, mark, NULL, -INFTY, INFTY, d, model);
		     #endif
		     }
		     break;
		     case USE_PRIOR:
		     lx += returnPriorLogDensity(i, ptr->val->x, 
		     ptr->val->x->x->x);
		     break;
		     case USE_VAR:
		     break;
		     } */
		  if (ptr->type == USE_TABLE)
		    for (mark = 0; mark < d->n; mark++)
		      if (ptr->val->inUse[mark])
			checkCovs(model->ftList[mark][0], mark, i, c, f);
		  ptr = ptr->next;
		}
		if (printLevel >= 2)
		  bite_printf("%s(%ld): L(x)=%g, x=%g ",
			 c->pList[cov]->name, i, lx, x);
		found = FALSE;
		while (!found && l < max_iter) {
		  switch (c->pList[cov]->distr.x->type) {
		  case D_DUNIF:
		  case D_BIN:
		  case D_MULTIN:
		  case D_DISCR:
		    /* Discrete prior */
		    if (unifProp) {
		      double a, b;
		      returnSupport(i, c->pList[cov]->distr.x, &a, &b);
		      c->pList[cov]->distr.x->x->x = c->pList[cov]->val.x[i] = y =
			(int) ran_uni(a, b + 1);
		    }
		    else
		      c->pList[cov]->distr.x->x->x = c->pList[cov]->val.x[i] = y =
			generatePriorValue(i, c->pList[cov]->distr.x);
		    break;
		  default:
		    /* Continuous distribution */
		    c->pList[cov]->distr.x->x->x = c->pList[cov]->val.x[i] = y =
		      (unifProp) ?
		      ran_uni(x - c->pList[cov]->distr.x->epsilon,
			      x + c->pList[cov]->distr.x->epsilon) :
		      generatePriorValue(i, c->pList[cov]->distr.x);
		  }
		  pry = (unifProp) ?
		    returnPriorLogDensity(i, c->pList[cov]->distr.x, y) : 0.0;
		  if (x == y)
		    ly = lx;
		  else {
		    ly = 0.0;
		    ptr = c->pList[cov]->distr.x->where;
		    while (ptr) {
		      /* switch (ptr->type) {
			 case USE_TABLE:
			 for (mark = 0; mark < d->n; mark++)
			 if (ptr->val->inUse[mark]) {
			 #ifndef B_FACTOR
			 checkCovs(model->ftList[mark][0], mark, i, c, f);
			 ly += indLogL(i, mark, NULL, -INFTY, INFTY,
			 d, model);
			 #endif
			 }
			 break;
			 case USE_PRIOR:
			 ly += returnPriorLogDensity(i, ptr->val->x,
			 ptr->val->x->x->x);
			 break;
			 case USE_VAR:
			 break;
			 } */
		      if (ptr->type == USE_TABLE)
			for (mark = 0; mark < d->n; mark++)
			  if (ptr->val->inUse[mark])
			    checkCovs(model->ftList[mark][0], mark, i, c, f);
		      ptr = ptr->next;
		    }
		    ly = logLikel3(i, c->pList[cov]->distr.x->where,
				   c->pList[cov]->distr.x->indiv_likel);
		  }
		  alphaMH = ly + pry - lx - prx;
		  if (printLevel >= 2) {
		    if (alphaMH < LOG(0.05))
		      bite_printf(".");
		    else
		      if (alphaMH < LOG(0.25))
			bite_printf("+");
		      else 
			bite_printf("*"); 
		  }
		  if (LOG(ran_uni(0.0, 1.0)) < alphaMH) {
		    found = TRUE;
		    if (printLevel >= 2)
		      bite_printf(" L(y)=%g, y=%g\n", ly, y);
		  }
		  l++;
		}
		if (!found) {
		  c->pList[cov]->val.x[i] = x;
		  ptr = c->pList[cov]->distr.x->where;
		  while (ptr) {
		    if (ptr->type == USE_TABLE)
		      for (mark = 0; mark < d->n; mark++)
			if (ptr->val->inUse[mark])
			  checkCovs(model->ftList[mark][0], mark, i, c, f);
		    /* switch (ptr->type) {
		       case USE_TABLE:
		       for (mark = 0; mark < d->n; mark++)
		       #ifndef B_FACTOR
		       if (ptr->val->inUse[mark])
		       checkCovs(model->ftList[mark][0], mark, i, c, f);
		       #endif
		       break;
		       case USE_PRIOR:
		       break;
		       case USE_VAR:
		       break;
		       } */
		    ptr = ptr->next;
		  }
		  if (printLevel >= 2)
		    bite_printf("\n");
		}
		l++;
	      }
	      if (c->pList[cov]->distr.x->save == SAVEITER &&
		  iter >= skip && iter % thinning == 0) {
		char str[20];
		str[19] = '\0';
		sprintf(str, "%g\n", c->pList[cov]->val.x[i]);
		writeFileS(c->pList[cov]->distr.x->file, str);
	      }
	      /* bite_printf("\t%g\n", c->pList[cov]->val.x[i]); */
	    }
	  }
	}
	break;
      case COV_START2:
	for (j = 0; j < c->pList[cov]->n2[i]; j++) {
	  if (c->pList[cov]->distr.x2[j] &&
	      c->pList[cov]->distr.x2[j]->update) {
	    if (!iter  && printLevel > 1) {
	      bite_printf("Update startpoint=%s[%ld], i=%ld\n", c->pList[cov]->name, j, i);
	      printUse(c->pList[cov]->distr.x2[j]->where);
	    }
	    list = c->pList[cov]->distr.x2[j];
	    wptr = c->pList[cov]->distr.x2[j]->where;
	    /* Update the start points one by one. */
	    if (!checkBit(j % LONGLEN,
			c->pList[cov]->fixed.x2[i][j / LONGLEN]) &&
		list->type != D_DIRAC) {
	      zeroLI = FALSE;
	      if (list->save == LOADITER) {
		if (iter % loadLag == 0)
		  list->x->x = readFileD(list->file);
	      }
	      else {
		if (!iter  && printLevel && ind < 100)
		  bite_printf("Startpoint %s.\n", c->pList[cov]->name);
		l =0;
		while (l < list->burn || l < max_iter) {
		  x = list->x->x = c->pList[cov]->val.x2[i][j];
		  lx = 0.0;
		  ptr = wptr;
		  while (ptr) {
		    switch (ptr->type) {
		    case USE_TABLE:
		      for (mark = 0; mark < d->n; mark++)
			if (ptr->val->inUse[mark]) {
#ifndef B_FACTOR
			  /*checkCovs(model->ftList[mark][0], mark, i, c, f);*/
			  lx += indLogL(i, mark, NULL, -INFTY, INFTY, d, model);
#endif
			}
		      break;
		    case USE_PRIOR:
		      lx += returnPriorLogDensity(i, ptr->val->x,
						  ptr->val->x->x->x);
		      break;
		    case USE_VAR:
		      break;
		    }
		    ptr = ptr->next;
		  }
		  if (printLevel >= 2)
		    bite_printf("%s: L(x)=%g, x=%g ", list->name, lx, x);
		  found = FALSE;
		  while (!found && l < max_iter) {
		    c->pList[cov]->val.x2[i][j] = list->x->x = y =
		      generatePriorValue(i, list);
		    if (x == y)
		      ly = lx;
		    else {
		      ly = 0.0;
		      ptr = wptr;
		      while (ptr) {
			switch (ptr->type) {
			case USE_TABLE:
			  for (mark = 0; mark < d->n; mark++)
			    if (ptr->val->inUse[mark]) {
#ifndef B_FACTOR
			      /*checkCovs(model->ftList[mark][0], mark, i, c, f);*/
			      ly +=
				indLogL(i, mark, NULL, -INFTY, INFTY, d, model);
#endif
			    }
			  break;
			case USE_PRIOR:
			  ly += returnPriorLogDensity(i, ptr->val->x,
						      ptr->val->x->x->x);
			  break;
			case USE_VAR:
			  break;
			}
			ptr = ptr->next;
		      }
		    }
		    alphaMH = ly - lx;
		    if (printLevel >= 2) {
		      if (alphaMH < LOG(0.05))
			bite_printf(".");
		      else
			if (alphaMH < LOG(0.25))
			  bite_printf("+");
			else 
			  bite_printf("*"); 
		    }
		    if (LOG(ran_uni(0.0, 1.0)) < alphaMH) {
		      found = TRUE;
		      if (printLevel >= 2)
			bite_printf(" L(y)=%g, y=%g\n", ly, y);
		    }
		    l++;
		  }
		  if (!found) {
		    c->pList[cov]->val.x2[i][j] = list->x->x = x;
		    ptr = wptr;
		    while (ptr) {
		      switch (ptr->type) {
		      case USE_TABLE:
#ifndef B_FACTOR
			/*for (mark = 0; mark < d->n; mark++)
			  if (ptr->val->inUse[mark])
			  checkCovs(model->ftList[mark][0], mark, i, c, f);*/
#endif
			break;
		      case USE_PRIOR:
			break;
		      case USE_VAR:
			break;
		      }
		      ptr = ptr->next;
		    }
		    if (printLevel >= 2)
		      bite_printf("\n");
		  }
		  l++;
		}
		/* if (list->sample && iter >= skip)
		   list->sample[iter - skip] = paramValue(i, list); */
		if (list->save == SAVEITER && iter >= skip &&
		    iter % thinning == 0) {
		  char str[20];
		  str[19] = '\0';
		  sprintf(str, "%g\n", c->pList[cov]->val.x2[i][j]);
		  writeFileS(list->file, str);
		}
	      }
	    }
	  }
	}
	break;
      case COV_START:
	list = c->cList[cov][i]->x->x->vec;
	wptr = c->cList[cov][i]->x->where;
	while (list) { /* Update the start points one by one. */
	  if (!list->fixed && list->type != D_DIRAC) {
	    zeroLI = FALSE;
	    if (list->save == LOADITER) {
	      if (iter % loadLag == 0)
		list->x->x = readFileD(list->file);
	    }
	    else {
	      x = list->x->x;
	      lx = 0.0;
	      ptr = wptr;
	      while (ptr) {
		switch (ptr->type) {
		case USE_TABLE:
		  for (mark = 0; mark < d->n; mark++)
#ifndef B_FACTOR
		    if (ptr->val->inUse[mark]) {
		      checkCovs(model->ftList[mark][i], mark, i, c, f);
		      lx += indLogL(i, mark, NULL, -INFTY, INFTY, d, model);
		    }
#endif
		  break;
		case USE_PRIOR:
		  lx += returnPriorLogDensity(i, ptr->val->x,
					       ptr->val->x->x->x);
		  break;
		case USE_VAR:
		  break;
		}
		ptr = ptr->next;
	      }
	      if (printLevel >= 2)
		bite_printf("%s: L(x)=%g, x=%g ", list->name, 
		       lx, x);
	      l = 0;
	      found = FALSE;
	      while (!found && l < max_iter) {
		l++;
		y = generatePriorValue(i, list);
		if (x == y)
		  ly = lx;
		else {
		  list->x->x = y;
		  ly = 0.0;
		  ptr = wptr;
		  while (ptr) {
		    switch (ptr->type) {
		    case USE_TABLE:
		      for (mark = 0; mark < d->n; mark++)
#ifndef B_FACTOR
			if (ptr->val->inUse[mark]) {
			  checkCovs(model->ftList[mark][i], mark, i, c, f);
			  ly += indLogL(i, mark, NULL, -INFTY, INFTY, d, model);
			}
#endif
		      break;
		    case USE_PRIOR:
		      ly += returnPriorLogDensity(i, ptr->val->x,
						   ptr->val->x->x->x);
		      break;
		    case USE_VAR:
		      break;
		    }
		    ptr = ptr->next;
		  }
		}
		alphaMH = ly - lx;
		if (printLevel >= 2) {
		  if (alphaMH < LOG(0.05))
		    bite_printf(".");
		  else
		    if (alphaMH < LOG(0.25))
		      bite_printf("+");
		    else 
		      bite_printf("*"); 
		}
		if (LOG(ran_uni(0.0, 1.0)) < alphaMH) {
		  found = TRUE;
		  if (printLevel >= 2)
		    bite_printf(" L(y)=%g, y=%g\n", ly, y);
		}
	      }
	      if (!found) {
		list->x->x = x;
		ptr = wptr;
		while (ptr) {
		  switch (ptr->type) {
		  case USE_TABLE:
		    for (mark = 0; mark < d->n; mark++)
#ifndef B_FACTOR
		      if (ptr->val->inUse[mark])
			checkCovs(model->ftList[mark][i], mark, i, c, f);
#endif
		    break;
		  case USE_PRIOR:
		    break;
		  case USE_VAR:
		    break;
		  }
		  ptr = ptr->next;
		}
		if (printLevel >= 2)
		  bite_printf("\n");
	      }
	      /* if (list->sample && iter >= skip)
		 list->sample[iter - skip] = paramValue(i, list); */
	      if (list->save == SAVEITER && iter >= skip &&
		  iter % thinning == 0) {
		char str[20];
		str[19] = '\0';
		sprintf(str, "%g\n", paramValue(i, list));
		writeFileS(list->file, str);
	      }
	    }
	  }
	  list = list->next;
	}
	break;
      }
    }
  }
}

void printPostOfMissCov(char *directory)
{
  FILE *fp;
  long i, j, k, nVal, first, fileno;
  char fileName[50], cdum[6];
  double *postProb, *values, dum;

  postProb = (double *) malloc(ind * sizeof(dum));
  values = (double *) malloc(ind * sizeof(dum));
  for (i = 0; i < c->n; i++) {
    fileno = 0;
    switch (c->cList[i][0]->type) {
    case COV_CNST:
      values = covLevels(c, c->cList[i][0]->name, ind, &nVal, values);
      orderDValues(nVal, values); 
      for (k = 0; k < ind; k++) {
	if (!c->cList[i][k]->x->fixed) {
	  fileno++;
	  fileName[0] = '\0';
	  strcat(fileName, directory);
	  strcat(fileName, "misscov.");
	  strcat(fileName, c->cList[i][0]->name);
	  strcat(fileName, ".");
	  intToChar(fileno, cdum);
	  strcat(fileName, cdum);
	  strcat(fileName, ".tmp");
	  if ((fp = bite_fopen(fileName, "w")) == NULL) {
	    bite_eprintf("Unable to open file %s ... closing S+\n", fileName);
	    quitSplus();
	    bite_exit(1);
	  }
	  /*	  for (j = 0; j < iter_limit - skip; j++)
	    fprintf(fp, "%g\n", c->cList[i][k]->x->sample[j]); */
	  fclose(fp);
	}
      }
      fileName[0] = '\0';
      strcat(fileName, directory);
      strcat(fileName, "misscov.");
      strcat(fileName, c->cList[i][0]->name);
      if ((fp = bite_fopen(fileName, "w")) == NULL) {
	bite_eprintf("Unable to open file %s ... closing S+\n", fileName);
	quitSplus();
	bite_exit(1);
      }
      first = TRUE;
      for (k = 0; k < ind; k++) 
	if (!c->cList[i][k]->x->fixed) {
	  if (!first)
	    fprintf(fp, "\\\\\n");
	  for (j = 0; j < nVal; j++)
	    postProb[j] = 0.0;
	  /* for (j = 0; j < iter_limit - skip; j++) {
	    for (l = 0; l < nVal; l++)
	      if (c->cList[i][k]->x->sample[j] == values[l])
		postProb[l] += 1.0;
	  } */
	  fprintf(fp, "%ld ", k + 1);
	  for (j = 0; j < nVal; j++) {
	    postProb[j] /= ((double) (iter_limit - skip));
	    fprintf(fp, "& %4.3f ", postProb[j]);
	  }
	  first = FALSE;
	}
      fprintf(fp, "\n");
/*      bite_printf("File %s was written, covariate values are ", fileName);
      for (j = 0; j < nVal; j++)
	bite_printf("%g, ", values[j]);
      bite_printf("\n"); */
      break;
    }
  }
  free(postProb);
  free(values);
}

/*void updateElement(long mark, long i, long car, long fieldId, prior list)
{
  long l, trys, found;
  double x, y, lx, ly, alphaMH, bot = -INFTY, top = INFTY, pbot, ptop;

  switch (fieldId) {
  case T0:
    list = d->dList[mark][i][car]->pp->t0;
    top = paramValue(i, d->dList[mark][i][car]->pp->t1);
    break;
  case T1:
    bot = paramValue(i, d->dList[mark][i][car]->pp->t0);
    list = d->dList[mark][i][car]->pp->t1;
    if (iter == 100 && paramValue(i, list) < 6.0 && i == 57)
      bite_printf("%g\n", paramValue(i, list));
    break;
  case POINTS:
    bot = paramValue(i, d->dList[mark][i][car]->pp->t0);
    bot = paramValue(i, d->dList[mark][i][car]->pp->t1);
    break;
  }
  x = paramValue(i, list);
  lx = returnPriorLogDensity(i, list, paramValue(i, list));
  lx += indLogL(i, mark, NULL, bot, top, d, model);
  lx += logLikel3(i, list->where, list->indiv_likel);
  if (printLevel >= 2) {
    bite_printf("%s(%ld,%ld,%ld): ", list->name, mark, i, car);
    bite_printf("(prior args=");
    for (l = 0; l < list->nArgs; l++) {
      if (l)
	bite_printf(",");
      bite_printf("%g", calculate2(i, list->args[l]));
    }
    bite_printf(") L(x)=%g, x=%g ", lx, x);
  }

  l = 0;
  found = FALSE;
  while (!found && l < max_iter) {
    ly = 0.0;
    l++;
    trys = 0;
    do {
      y = generatePriorValue(i, list);
      trys++;
    } while ((y < bot || y > top) && trys < max_iter);
    if (trys == max_iter) {
      returnSupport(i, list, &pbot, &ptop);
      y = ran_uni((bot > pbot) ? bot : pbot, (top < ptop) ? top : ptop); 
    }
    if (x == y)
      ly = lx;
    else {
      list->x->x = y;
      ly = returnPriorLogDensity(i, list, paramValue(i, list));
      ly += indLogL(i, mark, NULL, bot, top, d, model);
      ly += logLikel3(i, list->where, list->indiv_likel);
    }
    if (paramValue(i, list) > 1000.0)
      bite_printf(" ");
    alphaMH = ly - lx;
    if (printLevel >= 2) {
      if (alphaMH < LOG(0.05))
	bite_printf(".");
      else
	if (alphaMH < LOG(0.25))
	  bite_printf("+");
	else 
	  bite_printf("*"); 
    }
    if (LOG(ran_uni(0.0, 1.0)) < alphaMH) {
      found = TRUE;
      if (printLevel >= 2)
	bite_printf(" L(y)=%g, y=%g", ly, y);
    }
  }
  if (!found)
    list->x->x = x;
  if (printLevel >= 2)
    bite_printf("\n");
#ifdef ETA1
    double x;
    sf lambda;
    putSfToFt2(i, mark, NULL, -INFTY, INFTY, model->ftList[mark][ind], 
	       model->ftList[mark][i], d, f);
    lambda = calculate(i, model->ftList[mark][ind]);
    do {
      x = ran_poiPr(lambda, 0.0);
    } while (x > 1e+10);
    list->x->x = x;
  }
#endif
  if (list->save == SAVEITER && iter >= skip &&
      iter % thinning == 0) {
    char str[20];
    str[19] = '\0';
    sprintf(str, "%g\n", paramValue(i, list));
    writeFileS(list->file, str);
  }
 */

void updateElement2(long mark, long i, long car, long j, long fieldId)
{
  long l, trys, found;
  double x, y, lx, ly, alphaMH, bot = -INFTY, top = INFTY, pbot, ptop,
    *adr = NULL;
  prior list = NULL;

  switch (fieldId) {
  case T0:
    if (!iter  && printLevel && ind < 100)
      bite_printf("Update %s[%ld]->t0.\n", d->pList[mark]->name, mark);
    list = d->pList[mark]->distr.pp.t0[car];
    adr = &d->pList[mark]->val.pp.t0[i][car];
    top = respValue(i, car, j, T1, d->pList[mark]);
    break;
  case T1:
    if (!iter  && printLevel && ind < 100)
      bite_printf("Update %s[%ld]->t1.\n", d->pList[mark]->name, mark);
    bot = respValue(i, car, j, T0, d->pList[mark]);
    list = d->pList[mark]->distr.pp.t1[car];
    adr = &d->pList[mark]->val.pp.t1[i][car];
    break;
  case POINTS:
    if (!iter  && printLevel && ind < 100)
      bite_printf("Update %s[%ld]->points[%ld].\n", d->pList[mark]->name, mark, j);
    bot = respValue(i, car, j, T0, d->pList[mark]);
    top = respValue(i, car, j, T1, d->pList[mark]);
    list = d->pList[mark]->distr.pp.points[car][j];
    adr = &d->pList[mark]->val.pp.points[i][car][j];
    break;
  }
  l =0;
  while (l < list->burn || l < max_iter) {
    x = respValue(i, car, j, fieldId, d->pList[mark]);
    lx = returnPriorLogDensity(i, list, x);
    lx += indLogL(i, mark, NULL, bot, top, d, model);
    if (!iter  && printLevel && ind < 100)
      bite_printf("\tResponse %s[%ld] from %g to %g.\n", d->dList[mark][0][0]->name, mark,
	     bot, top);
    lx += (list->nolikelihood) ? 0.0 : logLikel3(i, list->where,
						 list->indiv_likel);
    /* if (lx < -INFTY / 2.0 && iter > 20)
       bite_printf("%ld,%ld\n", mark, i); */
    if (printLevel >= 2) {
      bite_printf("%s(%ld,%ld,%ld): ", list->name, mark, i, car);
      bite_printf("(prior args=");
      for (l = 0; l < list->nArgs; l++) {
	if (l)
	  bite_printf(",");
	bite_printf("%g", calculate2(i, list->args[l]));
      }
      bite_printf(") L(x)=%g, x=%g ", lx, x);
    }

    found = FALSE;
    while (!found && l < max_iter) {
      ly = 0.0;
      trys = 0;
      do {
	y = generatePriorValue(i, list);
	trys++;
      } while ((y < bot || y > top) && trys < max_iter + 1);
      if (trys == max_iter + 1) {
	returnSupport(i, list, &pbot, &ptop);
	y = ran_uni((bot > pbot) ? bot : pbot, (top < ptop) ? top : ptop); 
	/* Don't tell anyone about this! */
      }
      if (x == y)
	ly = lx;
      else {
	*adr = y;
	ly = returnPriorLogDensity(i, list, y);
	ly += indLogL(i, mark, NULL, bot, top, d, model);
	ly += (list->nolikelihood) ? 0.0 : logLikel3(i, list->where,
						     list->indiv_likel);
      }
      alphaMH = ly - lx;
      if (printLevel >= 2) {
	if (alphaMH < LOG(0.05))
	  bite_printf(".");
	else
	  if (alphaMH < LOG(0.25))
	    bite_printf("+");
	  else 
	    bite_printf("*");
      }
      if (LOG(ran_uni(0.0, 1.0)) < alphaMH) {
	found = TRUE;
	if (printLevel >= 2)
	  bite_printf(" L(y)=%g, y=%g", ly, y);
	/* if (printLevel > 1 && lx - ly > 2.5)
	   bite_printf(" L(x)=%g,  x=%g -> L(y)=%g, y=%g\n", lx, x, ly, y); */
      }
      l++;
    }
    if (!found)
      *adr = x;
    if (printLevel >= 2)
      bite_printf("\n");
    l++;
  }
#ifdef ETA1
  { /* Independence sampling of eruption times. */
    double x;
    sf lambda;
    putSfToFt2(i, mark, NULL, -INFTY, INFTY, model->ftList[mark][0], 
	       model->ftList[mark][0], d, f);
    lambda = calculate(i, model->ftList[mark][0]);
    do { /* until finite value */
      x = ran_poiPr(lambda, 0.0);
    } while (x > 1e+10);
    d->pList[mark]->val.pp.t1[i][car] = x;
  }
#endif
  /* if (list->sample && iter >= skip)
     list->sample[iter - skip] = paramValue(i, list); */
  if (list->save == SAVEITER && iter >= skip &&
      iter % thinning == 0) {
    char str[20];
    str[19] = '\0';
    sprintf(str, "%g\n", *adr);
    writeFileS(list->file, str);
  }
}

void updateResponce()
{
  long mark, i, car, j;

  for (mark = 0; mark < d->n; mark++) { /* Check all responces, */
    for (i = 0; i < ind; i++) { /* all individuals and */
      /* if (i == 491)
	bite_printf("i:%ld\n", i); */
      for (car = 0; car < d->nCars[mark]; car++) {
	if (d->pList[mark]) {
	  switch (d->dList[mark][0][0]->type) {
	  case CONT:
	    zeroLI = FALSE;
	    if (d->pList[mark]->distr.pp.t0[car] &&
		d->pList[mark]->distr.pp.t0[car]->update &&
		!d->pList[mark]->fixed.pp.t0[i][car] &&
		d->pList[mark]->distr.pp.t0[car]->wait <= iter &&
		d->pList[mark]->distr.pp.t0[car]->type != D_DIRAC)
	      updateElement2(mark, i, car, -1, T0);
	    if (d->pList[mark]->distr.pp.t1[car] &&
		d->pList[mark]->distr.pp.t1[car]->update &&
		!d->pList[mark]->fixed.pp.t1[i][car] &&
		d->pList[mark]->distr.pp.t1[car]->wait <= iter &&
		d->pList[mark]->distr.pp.t1[car]->type != D_DIRAC)
	      updateElement2(mark, i, car, -1, T1);
	    for (j = 0; j < d->pList[mark]->val.pp.npoints[i][car]; j++) {
	      zeroLI = FALSE;
	      if (d->pList[mark]->distr.pp.points[car][j] &&
		  d->pList[mark]->distr.pp.points[car][j]->update &&
		  !checkBit(j % LONGLEN,
			    d->pList[mark]->fixed.pp.points[i][car]
			    [j / LONGLEN]) &&
		  d->pList[mark]->distr.pp.points[car][j]->wait <= iter &&
		  d->pList[mark]->distr.pp.points[car][j]->type != D_DIRAC)
		updateElement2(mark, i, car, j, POINTS);
	    }
	    break;
	  }
	}
	/*	else {
	  if (d->dList[mark][i][car]) {
	    node = d->dList[mark][i][car];
	    switch (node->type) {
	    case CONT:
	      zeroLI = FALSE;
	      if (!node->pp->t0->fixed && node->pp->t0->type != D_DIRAC)
		updateElement(mark, i, car, T0, NULL);
	      if (!node->pp->t1->fixed && node->pp->t1->type != D_DIRAC)
		updateElement(mark, i, car, T1, NULL);
	      list = node->pp->vector;
	      while (list) {
		zeroLI = FALSE;
		if (!list->fixed && list->type != D_DIRAC)
		  updateElement(mark, i, car, POINTS, list);
		list = list->next;
	      }
	      break;
	    }
	  }
	} */
      }
    }
  }
}

void updateParameter(prior ptr, long coord)
{
  long old_type = NONE, i = 0, idx, j, l, found, unifProp = FALSE, trys;
  double x, x2 = -INFTY, y = -INFTY, mu = 0.0, sigma = 0.0, lx, ly, prx,
    pry, alphaMH;
  prior p = NULL, p2 = NULL;
  use list;

  if (coord >= 0) {
    /* p = ptr->vector[coord]; */
    p = ptr->x->vec; /* Updating a multivariate parameter. */
    for (j = 0; j < coord; j++)
      p = p->next;
  }

  if (ptr->save == LOADITER) {
    if (iter % loadLag == 0) {
      if (coord >= 0)
	p->x->x = readFileD(ptr->file);
      else
	ptr->x->x = readFileD(ptr->file);
    }
  }
  else {
    if (ptr->update) {
      if (!iter  && printLevel && ind < 100) {
	bite_printf("Update parameter ");
	printName(ptr);
	bite_printf(".\n");
      }
      zeroLI = FALSE;
      /* 'unifProp == 1' means that the proposal distribution is uniform: */
      if (ptr->mix && ptr->type != D_BIN && ptr->type != D_DUNIF)
	unifProp = (ran_uni(0.0, 1.0) < ptr->mix) ? 1 : 0;
      i = 0;
      while (i < ptr->burn || i < max_iter) {
	if (coord >= 0) {
	  x = paramValue(lindex[level], p);
	  switch (ptr->type) {
	  case D_DIRICHLET:
	    do {
	      idx = (int) ran_uni(0.0, ptr->nArgs);
	    } while (idx == coord);
	    p2 = ptr->x->vec;
	    for (j = 0; j < idx; j++)
	      p2 = p2->next;
	    x2 = paramValue(lindex[level], p2);
	    break;
	  case D_MNORM:
	    mu = calculate2(lindex[level], ptr->args[coord]);
	    sigma = sqrt(calculate2(lindex[level], ptr->args[2 + coord * 3]));
	    break;
	  }
	}
	else {
	  if (ptr->type == D_MIX) {
	    x = ptr->x->mix.x;
	    old_type = ptr->x->mix.type;
	  }
	  else
	    x = ptr->x->x;
	}
	prx = (unifProp || coord >= 0) ?
	  returnPriorLogDensity(lindex[level], ptr, x) * G_annealing : 0.0;
	if (coord >= 0)
	  prx -= (unifProp) ? 0.0 :
	    ((ptr->type == D_MNORM) ? LOG(dens_norm(x, mu, sigma * sigma)) : 0.0);
	if (ptr->where || !p)
	  lx = logLikel3(-1, ptr->where, ptr->indiv_likel);
	else
	  lx = logLikel3(-1, p->where, p->indiv_likel);
	if (printLevel >= 2) {
	  printName(ptr);
	  bite_printf(" (prior args=");
	  for (l = 0; l < ptr->nArgs; l++) {
	    if (l)
	      bite_printf(",");
	    if (ptr->ptype == PT_MIX && l % 2 == 0) {
	      long ll = 0;
	      switch (ptr->args[l]->var->p->type) {
	      case D_GAMMA:
		bite_printf("gamma(");
		break;
	      case D_DIRAC:
		bite_printf("dirac(");
		break;
	      case D_NORM:
		bite_printf("normal(");
		break;
	      default:
		bite_printf("xxx(");
	      }
	      for (;ll < ptr->args[l]->var->p->nArgs; ll++)
		bite_printf("%g,", calculate2(ind, ptr->args[l]->var->p->args[ll]));
	      bite_printf(")");
	    }
	    else
	      bite_printf("%g", calculate2(ind, ptr->args[l]));
	  }
	  bite_printf(") L(x)=%g, prx=%g, x=%g ", lx, prx, x);
	}
	/* if (coord >= 0) {
	   for (l = -10; l < 10; l++) {
	   y = p->x->x = (double) 0.25 * l;
	   bite_printf("%g:%g,%g\t", 0.25 * l,logLikel3(lindex[level], ptr->where),
	   returnPriorLogDensity(lindex[level], ptr, x));
	   }
	   bite_printf("\n");
	   } */
	found = FALSE;
	while (!found && i < max_iter) {
	  trys = 0;
	  do {
	    if (coord >= 0) {
	      switch (ptr->type) {
	      case D_MNORM:
		y = p->x->x = (unifProp) ? 
		  ran_uni(x - ptr->epsilon, x + ptr->epsilon) :
		  mu + ran_norm() * sigma;
		break;
	      case D_DIRICHLET:
		y = p->x->x = ran_uni(0.0, x + x2);
		p2->x->x = x + x2 - y;
		break;
	      }
	    }
	    else {
	      switch (ptr->type) {
	      case D_DUNIF:
	      case D_BIN:
	      case D_MULTIN:
	      case D_DISCR:
		/* Discrete prior */
		if (unifProp) {
		  double a, b;
		  returnSupport(-1, ptr, &a, &b);
		  y = (int) ran_uni(a, b + 1);
		}
		else
		  y = generatePriorValue(lindex[level], ptr);
		break;
	      default:
		/* Continuous distribution */
		y = (unifProp) ?
		  ran_uni(x - ptr->epsilon, x + ptr->epsilon) :
		  generatePriorValue(lindex[level], ptr);
	      }
	      y = (returnPriorLogDensity(lindex[level], ptr, y) > -INFTY) ? y : x;
	      if (ptr->type != D_MIX)
		ptr->x->x = y;
	      else
		ptr->x->mix.x = y;
	    }
	    trys++;
	  } while(returnPriorLogDensity(lindex[level], ptr, y) <= -INFTY &&
		  trys < max_iter);
	  pry = (unifProp || coord >= 0) ?
	    returnPriorLogDensity(lindex[level], ptr, y) * G_annealing : 0.0;
	  if (coord >= 0)
	    pry -= (unifProp) ? 0.0 :
	      ((ptr->type == D_MNORM) ? LOG(dens_norm(y, mu, sigma * sigma)) : 0.0);

	  list = ptr->where;
	  while (list) {
#ifndef B_FACTOR
	    if (list->type == USE_INDPARAM &&
		list->val->indParam.p->where &&
		list->val->indParam.p->where->type == USE_TABLE) {
	      long mark;
	      for (mark = 0; mark < d->n; mark++)
#ifndef B_FACTOR
		if (list->val->indParam.p->where->val->inUse[mark])
		  checkCovs(model->ftList[mark][0], mark,
			    list->val->indParam.i, c, f);
#endif
	    }
#endif
	    list = list->next;
	  }

	  /* if (!strcmp(ptr->name, "tau"))
	     bite_printf("x=%g, lx=%g, prx=%g => y=%g, pry=%g\n",
	     x, lx, prx, y, pry); */
	  if (ptr->where || !p)
	    ly = logLikel3(-1, ptr->where, ptr->indiv_likel);
	  else
	    ly = logLikel3(-1, p->where, p->indiv_likel);
	  /* ly = logLikel3(lindex[level], ptr->where);
	     if (!strcmp(ptr->name, "tau") || printLevel >= 3) {
	     long i, j = 0;
	     prior pptr;
	     bite_printf("x=%g, lx=%g, prx=%g => y=%g, ly=%g, pry=%g\n",
	     x, lx, prx, y, ly, pry);
	     pptr = param->next->next->x->vec;
	     for (i = 0; i < 241; i++)
	     if (pptr->x->x <= 4 || pptr->x->x >= 20) {
	     j++;
	     pptr = pptr->next;
	     }
	     for (i = 1; i < 60; i++) {
	     ptr->x->x = (double) i / 10;
	     bite_printf("%3.1f %10g  ", (double) i / 10,
	     logLikel3(lindex[level], ptr->where));
	     if (!(i + 1) % 5)
	     bite_printf("\n");
	     }
	     bite_printf("");
	     } */
	  alphaMH = ly + pry - lx - prx;
	  if (printLevel >= 2) {
	    bite_printf("(%g)", y);
	    if (alphaMH < LOG(0.05))
	      bite_printf(".");
	    else
	      if (alphaMH < LOG(0.25))
		bite_printf("+");
	      else 
		bite_printf("*"); 
	  }
	  if (LOG(ran_uni(0.0, 1.0)) < alphaMH) {
	    found = TRUE;
	    if (printLevel >= 2)
	      bite_printf(" L(y)=%g, pry=%g, y=%g\n", ly, pry, y);
	    if (printLevel > 0 && ly - lx > 100.0) {
	      bite_printf("Parameter ");
	      printName(ptr);
	      bite_printf(" improved a lot %g --> %g (%g --> %g).\n",
		     x, y, lx, ly);
	    }
	  }
	  i++;
	}
	if (!found) {
	  if (coord >= 0) {
	    p->x->x = x;
	    if (p2)
	      p2->x->x = x2;
	  }
	  else {
	    if (ptr->type == D_MIX) {
	      ptr->x->mix.x = x;
	      ptr->x->mix.type = old_type;
	    }
	    else
	      ptr->x->x = x;
	  }
	  if (printLevel >= 2)
	    bite_printf("\n");
	  /* Make sure that the model functions have correct support info. */
	  list = ptr->where;
	  while (list) {
#ifndef B_FACTOR
	    if (list->type == USE_INDPARAM &&
		list->val->indParam.p->where->type == USE_TABLE) {
	      long mark;
	      for (mark = 0; mark < d->n; mark++)
		if (list->val->indParam.p->where->val->inUse[mark])
		  checkCovs(model->ftList[mark][0], mark, list->val->indParam.i,
			    c, f);
	    }
#endif
	    list = list->next;
	  }
	}
	i++;
      }
    }
    if (ptr->save == SAVEITER && !p && iter >= skip && iter % thinning == 0) {
      char str[40];
      sprintf(str, "%g\n", (coord >= 0) ? p->x->x : ptr->x->x);
      writeFileS(ptr->file, str);
    }
#ifdef ETA1
    if (ptr->name[0] == '1')
      ptr->x->x = generatePriorValue(lindex[level], ptr);
#endif
    /* if (iter >= skip && ptr->sample)
       ptr->sample[iter - skip] = ptr->x->x; */
    
  }
}

void updateNormalApproxCheck(prior ptr)
{
  use list;

  list = ptr->where;
  while (list) {
#ifndef B_FACTOR
    if (list->type == USE_INDPARAM &&
	list->val->indParam.p->where &&
	list->val->indParam.p->where->type == USE_TABLE) {
      long mark;
      for (mark = 0; mark < d->n; mark++)
#ifndef B_FACTOR
	if (list->val->indParam.p->where->val->inUse[mark])
	  checkCovs(model->ftList[mark][0], mark,
		    list->val->indParam.i, c, f);
#endif
    }
#endif
    list = list->next;
  }
}

void updateParameterNormalApprox(prior ptr)
{
  long i = 0, stop, found=FALSE;
  double x, old, y, y1, y2, lx, ly, ly1, ly2, mu, sigma2;

  if (ptr->mix && ptr->update && ptr->save != LOADITER && ptr->wait <= iter &&
      (ptr->type == D_NORM || ptr->type == D_STUDENT || ptr->type == D_LAPLACE) &&
      ptr->restrict1 == NULL && ptr->restrict2 == NULL) {
    zeroLI = FALSE;
    old = x = ptr->x->x;
    lx = returnPriorLogDensity(lindex[level], ptr, x) * G_annealing;
    if (ptr->where)
      lx += logLikel3(-1, ptr->where, ptr->indiv_likel);

    ptr->x->x = y1 = x - ptr->epsilon;
    updateNormalApproxCheck(ptr);
    ly1 = returnPriorLogDensity(lindex[level], ptr, y1) * G_annealing;
    if (ptr->where)
      ly1 += logLikel3(-1, ptr->where, ptr->indiv_likel);

    ptr->x->x = y2 = x + ptr->epsilon;
    updateNormalApproxCheck(ptr);
    ly2 = returnPriorLogDensity(lindex[level], ptr, y2) * G_annealing;
    if (ptr->where)
      ly2 += logLikel3(-1, ptr->where, ptr->indiv_likel);

    stop = FALSE;
    i = 0;
    while (!stop && i < 10) {
      if (ly1 < lx && lx > ly2)
	/* Maximum is between y1 and y2. */
	stop = TRUE;
      else {
	if (ly1 < lx && lx < ly2) {
	  /* Move right. */
	  y1 = x;
	  ly1 = lx;
	  x = y2;
	  lx = ly2;
	  y2 += ptr->epsilon;
	  ptr->x->x = y2;
	  updateNormalApproxCheck(ptr);
	  ly2 = returnPriorLogDensity(lindex[level], ptr, y2) * G_annealing;
	  if (ptr->where)
	    ly2 += logLikel3(-1, ptr->where, ptr->indiv_likel);
	}
	else {
	  /* Move left. */
	  y2 = x;
	  ly2 = lx;
	  x = y1;
	  lx = ly1;
	  y1 -= ptr->epsilon;
	  ptr->x->x = y1;
	  updateNormalApproxCheck(ptr);
	  ly1 = returnPriorLogDensity(lindex[level], ptr, y1) * G_annealing;
	  if (ptr->where)
	    ly1 += logLikel3(-1, ptr->where, ptr->indiv_likel);
	}
      }
      i++;
    }
    /* Find maximum more accurately */
    /*y1b = y1;
    ly1b = ly1;
    y2b = y2;
    ly2b = ly2;
    i = 0;
    while (i < 5) {
      y1b = y1;
      ly1b = ly1;
      y2b = y2;
      ly2b = ly2;
      ptr->x->x = y1 = (x + y1) / 2;
      updateNormalApproxCheck(ptr);
      ly1 = returnPriorLogDensity(lindex[level], ptr, y1);
      if (ptr->where)
	ly1 += logLikel3(-1, ptr->where, ptr->indiv_likel);
      ptr->x->x = y2 = (x + y2) / 2;
      updateNormalApproxCheck(ptr);
      ly2 = returnPriorLogDensity(lindex[level], ptr, y2);
      if (ptr->where)
	ly2 += logLikel3(-1, ptr->where, ptr->indiv_likel);
      if (ly1 > ly2) {
	Move left. 
	y1 = y1b;
	y2 = x;
	x = y1;
      }
      else {
	Move right. 
	y2 = y2b;
	y1 = x;
	x = y2;
      }
      i++;
    }
    ptr->x->x = y1 = x - ptr->epsilon;
    updateNormalApproxCheck(ptr);
    ly1 = returnPriorLogDensity(lindex[level], ptr, y1);
    if (ptr->where)
      ly1 += logLikel3(-1, ptr->where, ptr->indiv_likel);
    ptr->x->x = y2 = x + ptr->epsilon;
    updateNormalApproxCheck(ptr);
    ly2 = returnPriorLogDensity(lindex[level], ptr, y2);
    if (ptr->where)
      ly2 += logLikel3(-1, ptr->where, ptr->indiv_likel);*/

    /* Check if the epsilon is too small or too big: */
    if ((fabs(lx -ly1) > 4 || fabs(lx -ly2) > 4) &&
	ptr->epsilon > 0.001) { /* ... too big. */
      ptr->epsilon /=
	1.001 + pow((1.0 - iter / iter_limit) / 5.0, 2.0);
      if (printLevel > 1) {
	printName(ptr);
	bite_printf(" %g\n", ptr->epsilon);
      }
    }
    if ((fabs(lx -ly1) < 0.5 || fabs(lx -ly2) < 0.5) &&
	ptr->epsilon < 1000) { /* ... too small. */
      ptr->epsilon *=
	1.001 + pow((1.0 - iter / iter_limit) / 5.0, 2.0);
      if (printLevel > 1) {
	printName(ptr);
	bite_printf(" %g\n", ptr->epsilon);
      }
    }
    if (printLevel > 0 && iter == iter_limit - 1) {
      printName(ptr);
      bite_printf(" %g\n", ptr->epsilon);
    }

    if (normalApprox(ly1, lx, ly2, x, ptr->epsilon, &mu, &sigma2)) {
      x = ptr->x->x = old;
      updateNormalApproxCheck(ptr);
      lx = returnPriorLogDensity(lindex[level], ptr, x) * G_annealing -
	(- (LOG(2 * PI * sigma2) + pow(x - mu, 2.0) / sigma2) / 2.0);
      if (ptr->where)
	lx += logLikel3(-1, ptr->where, ptr->indiv_likel);

      y = ptr->x->x = ran_norm() * sqrt(sigma2) + mu;
      updateNormalApproxCheck(ptr);
      ly = returnPriorLogDensity(lindex[level], ptr, y) * G_annealing -
	(- (LOG(2 * PI * sigma2) + pow(y - mu, 2.0) / sigma2) / 2.0);
      if (ptr->where)
	ly += logLikel3(-1, ptr->where, ptr->indiv_likel);

      if (LOG(ran_uni(0.0, 1.0)) < ly - lx) {
	found = TRUE;
	/*bite_printf("Found %s, i=%ld, mu=%g, sigma=%g. (%g --> %g) %g\n",
	  ptr->name, i, mu, sqrt(sigma2), old, y, ly - lx);*/
      }
    }
    if (!found) {
      /*bite_printf("Failed %s, i=%ld, mu=%g, sigma=%g. (%g --> %g) %g\n",
	ptr->name, i, mu, sqrt(sigma2), old, x, ly - lx);*/
      ptr->x->x = old;
      updateNormalApproxCheck(ptr);
    }
  }
}

void updateParameters(prior ptr)
{
  long i;

  level++;
  lindex[level] = 0;
  while (ptr) {
    if (strcmp(ptr->name, "buns") == 0)
      bite_printf(" ");
    if (!ptr->fixed && ptr->wait <= iter) {
      zeroLI = FALSE;
      switch (ptr->ptype) {
      case PT_SCALAR:
      case PT_MIX:
      case PT_CONST:
	updateParameter(ptr, -1);
	if (iter > ptr->wait)
	  updateParameterNormalApprox(ptr);
	break;
      case PT_VECTOR:
	if (ptr->type) {
	  long nc = 0;
	  prior p = ptr->x->vec;
	  switch (ptr->type) {
	  case D_DIRICHLET:
	    nc = ptr->nArgs;
	    break;
	  case D_MNORM:
	    nc = (long) (sqrt((double) ptr->nArgs + 0.25) - 0.5);
	    break;
	  }
	  if (ptr->save == LOADITER) {
	    if (iter % loadLag == 0) {
	      for (i = 0; i < nc; i++, p = p->next)
		p->x->x = readFileD(ptr->file);
	    }
	  }
	  else {
	    for (i = 0; i < nc; i++)
	      updateParameter(ptr, i);
	    if (ptr->save == SAVEITER && iter >= skip &&
		iter % thinning == 0) {
	      char str[40];
	      for (i = 0; i < nc; i++, p = p->next) {
		sprintf(str, "%g\n", p->x->x);
		writeFileS(ptr->file, str);
	      }
	    }
	  }
	}
	else {
	  if (ptr->where && !ptr->x->vec->where) {
	    prior p = ptr->x->vec;
	    while (p) {
	      if (!p->where)
		p->where = ptr->where;
	      p = p->next;
	    }
	  }
	  updateParameters(ptr->x->vec);
	}
	break;
      }
    }
    ptr = ptr->next;
    lindex[level]++;
  }
  level--;
}

double inner(long n, double *x, double *y)
{
  long i;
  double res = 0.0;
  for (i = 0; i < n; i++)
    res += x[i] * y[i];
  return(res);
}

void updateGradient(gradientUpdate g)
{
  long j, k, kl, l, found, maxSteps = 10, df = 3, fwd;
  double lx = 0.0, ly = 0.0, ly1, ly2, ly_far, length = 1.0, dist = 0.0, step = 1.0,
    mu, sigma2, y, qx, qy, mm_x = 0.0;
  while (g) {
    if (printLevel >= 1)
      bite_printf("Updating group of parameters %s with gradient method.\n",
	     g->name);
    /* Backup the parameter values: */
    for (k = 0; k < g->n; k++) {
      g->values[0][k] = calculate2(-1, g->vec[k]);
    }
    if (iter == maxWait + 10 || iter % 10 == 0) {
      /* Update the gradient */
      if (g->mm_p >= 0) {
	mm_x = sqrt(g->mm_sigma2) * ran_norm() + g->mm_mu;
	/*setValue(-1, g->vec[g->mm_p], g->values[0][g->mm_p]);*/
	if (printLevel >= 1) {
	  bite_printf("\tMultimodal parameter %s (=%g) has been set to value %g.\n",
		 g->vec[g->mm_p]->varName, g->values[0][g->mm_p], mm_x);
	  fflush(NULL);
	}
      }
      for (j = 0; j < g->nLoop && length > 0.0; j++) {
	if (printLevel >= 2) {
	  bite_printf("\tStep %ld.\n", j);
	  fflush(NULL);
	}
	dist = 0.0;
	step = 1.0;
	if (g->mm_p >= 0 && j < 2 * g->nLoop / 3) {
	  g->values[j][g->mm_p] = (j < g->nLoop / 3) ?
	    g->values[0][g->mm_p] + (j) / (g->nLoop / 3.0) *
	    (mm_x - g->values[0][g->mm_p]) : mm_x;
	  setValue(-1, g->vec[g->mm_p], g->values[j][g->mm_p]);
	}
	/* Calculate the gradient: */
	for (k = 0; k < g->n; k++) {
	  if (g->mm_p != k || j >= 2 * g->nLoop / 3) {
	    if (g->vec[k]->function == F_INDIV)
	      lx = logPrior(calculate2(-1, g->vec[k]->args[1]),
			    g->vec[k]->args[0], NULL, NULL) +
		logContrib(calculate2(-1, g->vec[k]->args[1]),
			   g->vec[k]->args[0]);
	    else
	      lx = logPrior(-1, g->vec[k], NULL, NULL) +
		logContrib(-1, g->vec[k]);
	    setValue(-1, g->vec[k], g->values[j][k] + 0.0001);
	    if (g->vec[k]->function == F_INDIV)
	      ly = logPrior(calculate2(-1, g->vec[k]->args[1]),
			    g->vec[k]->args[0], NULL, NULL) +
		logContrib(calculate2(-1, g->vec[k]->args[1]),
			   g->vec[k]->args[0]);
	    else
	      ly = logPrior(-1, g->vec[k], NULL, NULL) +
		logContrib(-1, g->vec[k]);
	    g->gr[j][k] = (ly - lx) / 0.0001;
	    setValue(-1, g->vec[k], g->values[j][k]);
	  }
	  else {
	    g->gr[j][k] = 0.0;
	  }
	}
	lx = 0.0;
	for (kl = 0; kl < g->nl; kl++)
	  lx += calculate2(-1, g->likel[kl]);
	g->lx[j] = lx;
	if (printLevel >= 2) {
	  bite_printf("\tlx=%g.\n", lx);
	  fflush(NULL);
	}
	length = sqrt(inner(g->n, g->gr[j], g->gr[j]));
	if (length > 0) {
	  /* Normalize the length of the gradient: */
	  for (k = 0; k < g->n; k++)
	    g->gr[j][k] /= length;
	  /* Find the maximum along that gradient: */
	  for (k = 0; k < g->n; k++)
	    setValue(-1, g->vec[k], g->values[j][k] - g->gr[j][k]);
	  ly1 = 0.0;
	  for (kl = 0; kl < g->nl; kl++)
	    ly1 += calculate2(-1, g->likel[kl]);

	  for (k = 0; k < g->n; k++)
	    setValue(-1, g->vec[k], g->values[j][k] + g->gr[j][k]);
	  ly2 = 0.0;
	  for (kl = 0; kl < g->nl; kl++)
	    ly2 += calculate2(-1, g->likel[kl]);

	  if (length < 0.1 && !(ly1 < lx && lx > ly2)) {
	    for (k = 0; k < g->n; k++)
	      setValue(-1, g->vec[k], g->values[j][k] + (dist + 10 * step)
		       * g->gr[j][k]);
	    ly_far = 0.0;
	    for (kl = 0; kl < g->nl; kl++)
	      ly_far += calculate2(-1, g->likel[kl]);
	    if (ly_far > ly2) {
	      step *= 10.0;
	      ly2 = ly_far;

	      for (k = 0; k < g->n; k++)
		setValue(-1, g->vec[k],
			 g->values[j][k] - (dist - step) * g->gr[j][k]);
	      ly1 = 0.0;
	      for (kl = 0; kl < g->nl; kl++)
		ly1 += calculate2(-1, g->likel[kl]);
	    }
	  }
	  l = 0;
	  while (!(ly1 < lx && lx > ly2) && l < maxSteps) {
	    if (ly1 < lx && lx < ly2) {
	      dist += step;
	      ly1 = lx;
	      lx = ly2;
	      for (k = 0; k < g->n; k++)
		setValue(-1, g->vec[k],
			 g->values[j][k] + (dist + step) * g->gr[j][k]);
	      ly2 = 0.0;
	      for (kl = 0; kl < g->nl; kl++)
		ly2 += calculate2(-1, g->likel[kl]);
	    }
	    else {
	      if (ly1 < lx && lx < ly2) {
		dist -= step;
		ly2 = lx;
		lx = ly1;
		for (k = 0; k < g->n; k++)
		  setValue(-1, g->vec[k],
			   g->values[j][k] - (dist - step) * g->gr[j][k]);
		ly1 = 0.0;
		for (kl = 0; kl < g->nl; kl++)
		  ly1 += calculate2(-1, g->likel[kl]);
	      }
	    }
	    l++;
	  }
	  if (normalApprox(ly1, lx, ly2, dist, step, &mu, &sigma2)) {
	    for (k = 0; k < g->n; k++)
	      g->values[j+1][k] = g->values[j][k] + mu * g->gr[j][k];
	    if (sqrt(sigma2) < 0.4)
	      step /= 2.0;
	  }
	  else
	    for (k = 0; k < g->n; k++)
	      g->values[j+1][k] = g->values[j][k] + dist * g->gr[j][k];
	}
	else
	  for (k = 0; k < g->n; k++)
	    g->values[j+1][k] = g->values[j][k];
	for (k = 0; k < g->n; k++) {
	  setValue(-1, g->vec[k], g->values[j+1][k]);
	}
      }
      /* Set the gradient equal to the difference of nLoop-1'th and nLoop/2'th
	 position: */
      for (k = 0; k < g->n; k++)
	g->gr[g->nLoop][k] =
	  /*g->values[g->nLoop][k] - g->values[g->nLoop / 2][k];*/
	  (g->values[g->nLoop][k] - g->values[0][k]);
      length = sqrt(inner(g->n, g->gr[g->nLoop], g->gr[g->nLoop]));
      dist = length;
      step = 1.0;
      if (length > 0) {
	/* Normalize the length of the gradient: */
	for (k = 0; k < g->n; k++)
	  g->gr[g->nLoop][k] /= length;

	/* Find the maximum along that gradient: */
	for (k = 0; k < g->n; k++)
	  setValue(-1, g->vec[k], g->values[0][k] + dist * g->gr[g->nLoop][k]);
	lx = 0.0;
	for (kl = 0; kl < g->nl; kl++)
	  lx += calculate2(-1, g->likel[kl]);
	g->lx[g->nLoop] = lx;
	if (printLevel >= 2) {
	  bite_printf("\tlx=%g.\n", lx);
	  fflush(NULL);
	}
      }
    }
    else  {
      length = 1.0;
      dist = 0.0;
      step = 1.0;
      lx = 0.0;
      for (kl = 0; kl < g->nl; kl++)
	lx += calculate2(-1, g->likel[kl]);
      g->lx[0] = lx;
    }

    /* Find the maximum along the gradient: */
    for (k = 0; k < g->n; k++)
      setValue(-1, g->vec[k], g->values[0][k] +
	       (dist - step) * g->gr[g->nLoop][k]);
    ly1 = 0.0;
    for (kl = 0; kl < g->nl; kl++)
      ly1 += calculate2(-1, g->likel[kl]);

    for (k = 0; k < g->n; k++)
      setValue(-1, g->vec[k], g->values[0][k] +
	       (dist + step) * g->gr[g->nLoop][k]);
    ly2 = 0.0;
    for (kl = 0; kl < g->nl; kl++)
      ly2 += calculate2(-1, g->likel[kl]);

    if (printLevel >= 2) {
      bite_printf("\t(ly1=%g, lx=%g, ly2=%g)\n", ly1, lx, ly2);
      fflush(NULL);
    }
    fwd = (ly1 < ly2) ? TRUE : FALSE;
    l = 0;
    while (!(ly1 < lx && lx > ly2) && l < maxSteps) {
      for (k = 0; k < g->n; k++)
	setValue(-1, g->vec[k],
		 g->values[0][k] +
		 (dist + (fwd ? 2.0 : -2.0) * step) *
		 g->gr[g->nLoop][k]);
      ly_far = 0.0;
      for (kl = 0; kl < g->nl; kl++)
	ly_far += calculate2(-1, g->likel[kl]);
      if (ly_far > ly1 || ly_far > ly2) {
	step *= 2.0;
	if (fwd)
	  ly2 = ly_far;
	else
	  ly1 = ly_far;

	for (k = 0; k < g->n; k++)
	  setValue(-1, g->vec[k],
		   g->values[0][k] +
		   (dist - (fwd ? 1.0 : -1.0) * step) * g->gr[g->nLoop][k]);
	ly = 0.0;
	for (kl = 0; kl < g->nl; kl++)
	  ly += calculate2(-1, g->likel[kl]);
	if (fwd)
	  ly1 = ly;
	else
	  ly2 = ly;
      }
      else
	l = maxSteps;
      if (printLevel >= 2) {
	bite_printf("\tFlatness (ly1=%g, lx=%g, ly2=%g)\n", ly1, lx, ly2);
	fflush(NULL);
      }
      l++;
    }
    step /= 4.0;
    l = 0;
    while (!(ly1 < lx && lx > ly2) && l < maxSteps) {
      if (ly1 < lx && lx < ly2) {
	if (printLevel >= 2) {
	  bite_printf("\t\tdist=%g + step=%g\n", dist, step);
	  fflush(NULL);
	}
	dist += step;
	ly1 = lx;
	lx = ly2;
	for (k = 0; k < g->n; k++)
	  setValue(-1, g->vec[k],
		   g->values[0][k] + (dist + step) * g->gr[g->nLoop][k]);
	ly2 = 0.0;
	for (kl = 0; kl < g->nl; kl++)
	  ly2 += calculate2(-1, g->likel[kl]);
      }
      else {
	if (ly1 > lx && lx > ly2) {
	  if (printLevel >= 2) {
	    bite_printf("\t\tdist=%g - step=%g\n", dist, step);
	    fflush(NULL);
	  }
	  dist -= step;
	  ly2 = lx;
	  lx = ly1;
	  for (k = 0; k < g->n; k++)
	    setValue(-1, g->vec[k],
		     g->values[0][k] + (dist - step) * g->gr[g->nLoop][k]);
	  ly1 = 0.0;
	  for (kl = 0; kl < g->nl; kl++)
	    ly1 += calculate2(-1, g->likel[kl]);
	}
      }
      if (printLevel >= 2) {
	bite_printf("\tl=%ld (dist=%g, step=%g,ly1=%g, lx=%g, ly2=%g)\n",
	       l, dist, step, ly1, lx, ly2);
	fflush(NULL);
      }
      l++;
    }
    if (printLevel >= 1) {
      bite_printf("\t(ly1=%g, lx=%g, ly2=%g)\n", ly1, lx, ly2);
      /*if (ly1 < 0.5 * -INFTY)
	for (kl = 0; kl < g->nl; kl++)
	  if (calculate2(-1, g->likel[kl]) < 0.5 * -INFTY)
	    bite_printf("-INFTY detected on line %ld.\n", g->likel[kl]->lineno);
      if (ly2 < 0.5 * -INFTY)
	for (kl = 0; kl < g->nl; kl++)
	  if (calculate2(-1, g->likel[kl]) < 0.5 * -INFTY)
	  bite_printf("-INFTY detected on line %ld.\n", g->likel[kl]->lineno);*/
      fflush(NULL);
    }
    found = FALSE;
    if (normalApprox(ly1, lx, ly2, dist, step, &mu, &sigma2)) {
      if (printLevel >= 1 && g->mm_p >= 0) {
	bite_printf("\tMultimodal parameter %s had value %g.\n",
	       g->vec[g->mm_p]->varName,
	       calculate2(-1, g->vec[g->mm_p]));
	fflush(NULL);
      }
      if (ly1 < 0.5 * -INFTY || ly2 < 0.5 * -INFTY)
	sigma2 = 0.01;
      if (pow(fabs(mu) / 1.96, 2.0) > sigma2) {
	if (printLevel >= 1) {
	  bite_printf("\tsigma2==%g --> %g\n",
		 sigma2, pow(fabs(mu) / 1.96, 2.0));
	  fflush(NULL);
	}
	sigma2 = sqrt(sigma2 * pow(fabs(mu) / 1.96, 2.0));
      }
      /* Normal proposal:
	 y = ran_norm() * sqrt(sigma2) + mu; */
      /* Student proposal: */
      y = mu + sqrt(sigma2) * ran_norm() * sqrt(df) /
	ran_gamma(df / 2.0, 0.5);
      qx = log(1.0 + pow(-mu, 2.0) / (sigma2) / df) * (-(df+1.0)/2.0);
      qy = log(1.0 + pow(y - mu, 2.0) / (sigma2) / df) * (-(df+1.0)/2.0);
      bite_printf("\t(dist=%g, step=%g, y=%g, mu=%g, sigma2=%g, "
	     "qx=%g, qy=%g)\n",
	     dist, step, y, mu, sigma2, qx, qy);
      fflush(NULL);
      for (k = 0; k < g->n; k++)
	setValue(-1, g->vec[k], g->values[0][k]
		 + y * g->gr[g->nLoop][k]);
      if (printLevel >= 1 && g->mm_p >= 0) {
	bite_printf("\tMultimodal parameter %s has value %g.\n",
	       g->vec[g->mm_p]->varName,
	       calculate2(-1, g->vec[g->mm_p]));
	fflush(NULL);
      }
      ly = 0.0;
      for (kl = 0; kl < g->nl; kl++)
	ly += calculate2(-1, g->likel[kl]);
      if (LOG(ran_uni(0.0, 1.0)) < ly - g->lx[0] + qx - qy) {
	found = TRUE;
	if (printLevel >= 1) {
	  bite_printf("\tGradient update succeeded. "
		 "(dist=%g, step=%g, y=%g, mu=%g, sigma2=%g, "
		 "lx=%g, ly=%g, qx=%g, qy=%g)\n",
		 dist, step, y, mu, sigma2, g->lx[0], ly, qx, qy);
	  fflush(NULL);
	}
	if (printLevel >= 1 && g->mm_p >= 0) {
	  bite_printf("\tMultimodal parameter %s has new value %g.\n",
		 g->vec[g->mm_p]->varName,
		 calculate2(-1, g->vec[g->mm_p]));
	  fflush(NULL);
	}
      }
      else {
	if (printLevel >= 1) {
	  bite_printf("\tGradient update failed. "
		 "(dist=%g, step=%g, y=%g, mu=%g, sigma2=%g, "
		 "lx=%g, ly=%g, qx=%g, qy=%g)\n",
		 dist, step, y, mu, sigma2, g->lx[0], ly, qx, qy);
	  fflush(NULL);
	}
      }
    }
    if (!found) {
      /* ... then restore the original values: */
      for (k = 0; k < g->n; k++)
	setValue(-1, g->vec[k], g->values[0][k]);
    }
    g = g->next;
  }
}

long findAddr(clusterUpdate c, double **addr, prior *pl)
{
  long i, cars, n, m, wait = FALSE;

  /* Find the addresses of the parameter values: */
  for (i = 0; i < c->n; i++) {
    if (c->vec[i]->function == F_INDIV) { /* A responce. */
      n = calculate2(-1, c->vec[i]->args[1]);
      if (n >= ind || n < 0) {
	bite_eprintf( "%ld: Not so many individuals.\n", n);
	bite_exit(1);
      }
      pl[i] = NULL;
      switch (c->vec[i]->args[0]->ptype) {
      case PT_RESP2:
	cars = calculate2(-1, c->vec[i]->args[0]->i[0]);
	if (cars >= c->vec[i]->args[0]->var->pvec->n2[n]) {
	  bite_eprintf( "%ld: Not so many observation intervals for "
		  "individual %ld.\n", cars, n);
	  bite_exit(1);
	}
	switch (c->vec[i]->args[0]->fieldId) {
	case T0:
	  wait =
	    (c->vec[i]->args[0]->var->pvec->distr.pp.t0[cars]->wait > iter ||
	     c->vec[i]->args[0]->var->pvec->fixed.pp.t0[n][cars]) ? TRUE : wait;
	  addr[i] = &c->vec[i]->args[0]->var->pvec->val.pp.t0[n][cars];
	  break;
	case T1:
	  wait =
	    (c->vec[i]->args[0]->var->pvec->distr.pp.t1[cars]->wait > iter ||
	     c->vec[i]->args[0]->var->pvec->fixed.pp.t0[n][cars]) ? TRUE : wait;
	  addr[i] = &c->vec[i]->args[0]->var->pvec->val.pp.t1[n][cars];
	  break;
	case POINTS:
	  m = calculate2(-1, c->vec[i]->args[0]->i[1]);
	  if (m >= c->vec[i]->args[0]->var->pvec->n2[n]) {
	    bite_eprintf( "%ld: Not so many event times on the "
		    "observation interval %ld for individual %ld.\n",
		    m, cars, n);
	    bite_exit(1);
	  }
	  wait =
	    (c->vec[i]->args[0]->var->pvec->distr.pp.points[cars][m]->wait >
	     iter ||
	     checkBit(m % LONGLEN,
		      c->vec[i]->args[0]->var->pvec->fixed.pp.points[n][cars]
		      [m / LONGLEN])) ? TRUE : wait;
	  addr[i] = &c->vec[i]->args[0]->var->pvec->val.pp.points[n][cars][m];
	  break;
	default:
	  bite_eprintf( "%ld: unknown type.\n",
		  c->vec[i]->args[0]->fieldId);
	}
	break;
      case PT_COV2:
	switch (c->vec[i]->args[0]->var->pvec->type) {
	case COV_CNST2:
	  wait = (c->vec[i]->args[0]->var->pvec->distr.x->wait > iter ||
		  checkBit(n % LONGLEN, 
			   c->vec[i]->args[0]->var->pvec->fixed.x[n / LONGLEN])) ?
	    TRUE : wait;
	  addr[i] = &c->vec[i]->args[0]->var->pvec->val.x[n];
	  break;
	case COV_START2:
	  m = calculate2(-1, c->vec[i]->args[0]->i[1]);
	  if (m >= c->vec[i]->args[0]->var->pvec->n2[n]) {
	    bite_eprintf( "%ld: Not so many start points %s "
		    "for individual %ld.\n", m,
		    c->vec[i]->args[0]->var->pvec->name, n);
	    bite_exit(1);
	  }
	  wait = (c->vec[i]->args[0]->var->pvec->distr.x2[m]->wait > iter ||
		  checkBit(m % LONGLEN, 
			   c->vec[i]->args[0]->var->pvec->fixed.x2[n][m / LONGLEN])) ?
	    TRUE : wait;
	  addr[i] = &c->vec[i]->args[0]->var->pvec->val.x2[n][m];
	  break;
	default:
	  bite_eprintf( "%ld: unknown type.\n",
		  c->vec[i]->args[0]->ptype);
	}
      }
    }
    else { /* A parameter. */
      if (c->vec[i]->ptype == PT_VECTOR) {
	prior p = c->vec[i]->var->p->x->vec;
	n = 0;
	m = calculate2(-1, c->vec[i]->i[0]);
	/* while (n < m) {
	  p = p->next;
	  n++;
	} */
	p = p->up->vector[m];
	addr[i] = &(p->x->x);
	wait = (p->wait > iter) ? TRUE : wait;
	pl[i] = p;
      }
      else { /* A scalar. */
	addr[i] = &(c->vec[i]->var->p->x->x);
	wait = (c->vec[i]->var->p->wait > iter) ? TRUE : wait;
	pl[i] = c->vec[i]->var->p;
      }
    }
  }
  return wait;
}

double groupLikel(clusterUpdate c, double **addr, prior *pl)
{
  long i, n, m, mark;
  double lx = 0.0;

  for (i = 0; i < c->n; i++) {
    if (c->vec[i]->function == F_INDIV) {
      long n = calculate2(-1, c->vec[i]->args[1]);
      lx += returnPriorLogDensity(n, c->vec[i]->args[0]->var->
				  pvec->distr.pp.t1
				  [(long) calculate2(-1, c->vec[i]->args[0]->i[0])],
				  *addr[i]);
    }
    else /* A parameter. */
      lx += returnPriorLogDensity(-1, pl[i], paramValue(-1, pl[i]));
  }
  if (lx > -0.5 * INFTY) {
    for (i = 0; i < c->nl; i++) {
      if (c->likel[i]->function == F_INDIV) { /* A responce. */
	n = calculate2(-1, c->likel[i]->args[1]);
	switch (c->likel[i]->args[0]->ptype) {
	case PT_RESP2:
	  mark = 0;
	  while (c->likel[i]->args[0]->var->pvec != d->pList[mark])
	    mark++;
	  lx += indLogL(n, mark, NULL, -INFTY, INFTY, d, model);
	  /* cars = calculate2(-1, c->likel[i]->args[0]->i[0]);
	     switch (c->likel[i]->args[0]->fieldId) {
	     case T0:
	     addr[i] = &c->likel[i]->args[0]->var->pvec->val.pp.t0[n][cars];
	     break;
	     case T1:
	     addr[i] = &c->likel[i]->args[0]->var->pvec->val.pp.t1[n][cars];
	     break;
	     case POINTS:
	     m = calculate2(-1, c->likel[i]->args[0]->i[1]);
	     addr[i] = &c->likel[i]->args[0]->var->pvec->val.pp.points[n][cars][m];
	     break;
	     default:
	     bite_eprintf( "%ld: unknown type.\n",
	     c->likel[i]->args[0]->fieldId);
	     } */
	  break;
	case PT_COV2:
	  switch (c->likel[i]->args[0]->var->pvec->type) {
	  case COV_CNST2:
	    addr[i] = &c->likel[i]->args[0]->var->pvec->val.x[n];
	    break;
	  case COV_START2:
	    m = calculate2(-1, c->likel[i]->args[0]->i[1]);
	    addr[i] = &c->likel[i]->args[0]->var->pvec->val.x2[n][m];
	    break;
	  default:
	    bite_eprintf( "%ld: unknown type.\n",
		    c->likel[i]->args[0]->ptype);
	  }
	}
      }
      else { /* A parameter. */
	if (c->likel[i]->ptype == PT_VECTOR) {
	  prior p = c->likel[i]->var->p->x->vec;
	  n = 0;
	  m = calculate2(-1, c->likel[i]->i[0]);
	  /* while (n < m) {
	     p = p->next;
	     n++;
	     } */
	  p = p->up->vector[m];
	  lx += logLikel3(-1, p->where, p->indiv_likel);
	}
	else /* A scalar. */
	  lx += logLikel3(-1, c->likel[i]->var->p->where,
			  c->likel[i]->var->p->indiv_likel);
      }
    }
  }
  return lx;
}

void updateGroups(clusterUpdate c)
{
  long i, found = FALSE, n, stop = FALSE, *dims;
  double lx, ly, qx = 0.0, qy = 0.0, alphaMH;

  static long stor = 10;
  static double **addr = NULL;
  static prior *pl;

  if (!addr) {
    addr = malloc(10 * sizeof(a));
    pl = malloc(10 * sizeof(a));
  }
  dims = malloc(c->nLoop * sizeof(i));
  if (stor < c->n) {
    old = realloc(old, c->n * sizeof(a));
    addr = realloc(addr, c->n * sizeof(a));
    pl = realloc(pl, c->n * sizeof(a));
    stor = c->n;
  }

  for (i = 0; i < c->nLoop; i++) {
    c->loopParam[i]->x->x = calculate2(-1, c->lower[i]);
    dims[i] = 0;
  }
  while (!stop) {
    findAddr(c, addr, pl);

    lx = groupLikel(c, addr, pl);

    found = FALSE;
    /* Generate proposal values: */
    for (i = 0; i < c->n; i++) {
      old[i] = *addr[i];
      n = (c->vec[i]->function == F_INDIV) ?
	calculate2(-1, c->vec[i]->args[1]) : -1;
      *addr[i] = calculate2(n, c->new_val[i]);
    }
    /* Calculate proposal logdensities, and generate the proposal values: */
    for (i = 0; i < c->qn_xy; i++)
      qy += calculate2(-1, c->prop_dens_xy[i]);

    ly = groupLikel(c, addr, pl);

    /* for (i = 0; i < c->n; i++) {
      double dum;
      dum = old[i];
      old[i] = *addr[i];
      *addr[i] = dum;
    } */

    /* Calculate the proposal logdensity of the reverse move. */
    for (i = 0; i < c->qn_yx; i++)
      qx += calculate2(-1, c->prop_dens_yx[i]);

    if (printLevel >= 2) {
      bite_printf("Updating group %s (", c->name);
      for (i = 0; i < c->nLoop; i++)
	bite_printf("%s=%g, ", c->loopParam[i]->name, c->loopParam[i]->x->x);
      bite_printf(")");
      for (i = 0; i < c->n; i++) {
	bite_printf("%s", (c->vec[i]->function == F_INDIV) ?
	       c->vec[i]->args[0]->varName : c->vec[i]->varName);
	if (c->vec[i]->function != F_INDIV && c->vec[i]->ni)
	  bite_printf("[%g]", calculate2(-1, c->vec[i]->i[0]));
	bite_printf(",=(%g->%g),", old[i], *addr[i]);
      }
      bite_printf(") on one step.\n(lx=%g) --> (ly=%g)\n", lx, ly);
    }
    alphaMH = ly - qy - (lx - qx);
    if (LOG(ran_uni(0.0, 1.0)) >= alphaMH) {
      /* Reject the proposal
	 bite_printf("\n"); */
    }
    else {
      found = TRUE;
    }
    if (printLevel >= 2) {
      if (found)
	bite_printf("and the proposal was accepted.\n");
      else
	bite_printf("but the proposed value was rejected.\n");
    }
    if (found) {
    }
    else { /* then restore the old values. */
      for (i = 0; i < c->nclean; i++)
	calculate2(-1, c->clean[i]);
      for (i = 0; i < c->n; i++)
	*addr[i] = old[i];
    }
    i = c->nLoop;
    while (i > 0) {
      i--;
      dims[i]++;
      if (dims[i] == calculate2(-1, c->dim[i])) {
	c->loopParam[i]->x->x = calculate2(-1, c->lower[i]);
	dims[i] = 0;
      }
      else {
	c->loopParam[i]->x->x +=
	  (calculate2(-1, c->upper[i]) - calculate2(-1, c->lower[i])) /
	  calculate2(-1, c->dim[i]);
	break;
      }
    }
    stop = TRUE;
    for (i = 0; i < c->nLoop; i++)
      stop *= (dims[i] == 0);
  }
  free(dims);
}

void updateClusters(clusterUpdate c)
{
  long i, j, l = 0, found = FALSE, dim;
  static long stor = 10;
  double lx, ly, alphaMH;
  static double **a, **m, *std_norm, **addr = NULL;
  static prior *pl;

  if (!old)
    old = malloc(stor * sizeof(missing));
  if (!addr) {
    std_norm = malloc(stor * sizeof(missing));
    a = malloc(stor * sizeof(a));
    m = malloc(stor * sizeof(a));
    for (i = 0; i < stor; i++) {
      a[i] = malloc(stor * sizeof(missing));
      m[i] = malloc(stor * sizeof(missing));
    }
    addr = malloc(stor * sizeof(a));
    pl = malloc(stor * sizeof(a));
    /* prop_d = malloc(stor * sizeof(missing));
    prop_l = malloc(stor * sizeof(missing)); */
  }
  while (c) {
    if (c->n > stor) {
      old = realloc(old, c->n * sizeof(missing));
      std_norm = realloc(std_norm, c->n * sizeof(missing));
      a = realloc(a, c->n * sizeof(a));
      m = realloc(m, c->n * sizeof(a));
      for (i = 0; i < stor; i++) {
	a[i] = realloc(a[i], c->n * sizeof(missing));
	m[i] = realloc(m[i], c->n * sizeof(missing));
      }
      for (i = stor; i < c->n; i++) {
	a[i] = malloc(c->n * sizeof(missing));
	m[i] = malloc(c->n * sizeof(missing));
      }
      addr = realloc(addr, c->n * sizeof(a));
      pl = realloc(pl, c->n * sizeof(a));
      stor = c->n;
    }
    if (c->type == GROUP_UPDATE)
      updateGroups(c);
    else {
      for (dim = 0; dim < calculate2(-1, c->dim[0]); dim++) {
	/* printLevel = 1; */
	c->loopParam[0]->x->x =
	  (calculate2(-1, c->upper[0]) - calculate2(-1, c->lower[0])) /
	  calculate2(-1, c->dim[0]) * dim + calculate2(-1, c->lower[0]);
	if (!findAddr(c, addr, pl)) {
	  for (i = 0; i < c->n; i++)
	    old[i] = *addr[i];
	  lx = groupLikel(c, addr, pl);

	  if (printLevel > 2) {
	    bite_printf("Updating cluster %s (%s=%g, sd=%g) (", c->name,
		   c->loopParam[0]->name, c->loopParam[0]->x->x, c->sd[dim]);
	    for (i = 0; i < c->n; i++)
	      bite_printf("%s=(%g->%g),", (c->vec[i]->function == F_INDIV) ?
		     c->vec[i]->args[0]->varName : c->vec[i]->varName,
		     c->last[dim][i], old[i]);
	    bite_printf(") on one step. (lx=%g) -->\n", lx);
	  }
	  if (iter >= 0) {
	    for (i = 0; i < c->n; i++) {
	      for (j = 0; j < c->n; j++)
		a[i][j] = 0.0;
	      a[0][i] = a[i][0] = 0.009;
	      a[i][i] = 0.01;
	    }
	  }
	  else {
	    for (i = 0; i < c->n; i++)
	      for (j = 0; j < c->n; j++)
		a[i][j] =
		  (c->cov[dim][i][j] - c->mean[dim][i] * c->mean[dim][j]) /
		  c->sd[dim];
	    if (printLevel > 2) {
	      bite_printf("Mean vector: ");
	      for (i = 0; i < c->n; i++)
		bite_printf("%g, ", c->mean[dim][i]);
	      bite_printf("\nCovariance matrix of the proposal:\n");
	      printMatrix(c->n, a);
	    }
	  }
	  if (c->type == CLUSTER_INDEPENDENCE)
	    for (i = 0; i < c->n; i++)
	      for (j = 0; j < c->n; j++)
		lx += (old[i] - c->mean[dim][i]) * (old[j] - c->mean[dim][j]) *
		  a[i][j] / 2.0;
	  cholesky(c->n, a, m);
	  while (!(m[c->n-1][c->n-1] < INFTY && m[c->n-1][c->n-1] > -INFTY)) {
	    for (i = 0; i < c->n; i++)
	      a[i][i] += 0.01;
	    cholesky(c->n, a, m);
	  }
	  if (printLevel > 2) {
	    bite_printf("Cholesky decomposition:\n");
	    printMatrix(c->n, m);
	  }
	  found = FALSE;
	  l = 0;
	  while (!found && l < max_iter + 4) {
	    for (i = 0; i < c->n; i++)
	      std_norm[i] = ran_norm();
	    for (i = 0; i < c->n; i++) {
	      *addr[i] = (c->type == CLUSTER_RANDOM_WALK) ?
		old[i] : c->mean[dim][i];
	      for (j = 0; j < c->n; j++)
		*addr[i] += std_norm[j] * m[i][j];
	    }
	    ly = groupLikel(c, addr, pl);
	    /* for (i = 0; i < c->n; i++) {
	      if (c->vec[i]->function == F_INDIV) {
		long n = calculate2(-1, c->vec[i]->args[1]);
		ly += returnPriorLogDensity(n, c->vec[i]->args[0]->var->pvec->distr.pp.t1
					    [(long) calculate2(-1, c->vec[i]->args[0]->i[0])], *addr[i]);
	      }
	      else A parameter.
		ly += returnPriorLogDensity(-1, pl[i], paramValue(-1, pl[i]));
	    }
	    if (ly > -0.5 * INFTY) {
	      If the prior density at the proposal is zero, then no likelihood
		 is necessary.
	      for (i = 0; i < c->nl; i++) {
		if (c->likel[i]->function == F_INDIV) { A responce.
		  long n = calculate2(-1, c->likel[i]->args[1]), mark = 0;
		  while (c->likel[i]->args[0]->var->pvec != d->pList[mark])
		    mark++;
		  ly += indLogL(n, mark, NULL, -INFTY, INFTY, d, model);
		}
		else
		  ly += logLikel3(-1, pl[i]->where);
	      }
	    }
	    else
	      ly = lx - INFTY; */
	    if (c->type == CLUSTER_INDEPENDENCE)
	      for (i = 0; i < c->n; i++)
		for (j = 0; j < c->n; j++)
		  ly += (*addr[i] - c->mean[dim][i]) *
		    (*addr[j] - c->mean[dim][j]) * a[i][j] / 2.0;
	    if (printLevel > 2) {
	      bite_printf("(");
	      for (i = 0; i < c->n; i++)
		bite_printf("%s=%g,", (c->vec[i]->function == F_INDIV) ?
		       c->vec[i]->args[0]->varName : c->vec[i]->varName, *addr[i]);
	      bite_printf(") (ly=%g) ", ly);
	    }
	    alphaMH = ly - lx;
	    if (LOG(ran_uni(0.0, 1.0)) >= alphaMH) {
	      /* Reject the proposal
		 bite_printf("\n"); */
	    }
	    else {
	      found = TRUE;
	      c->last_update[dim] = iter;
	    }
	    l++;
	  }
	  if (printLevel > 2) {
	    if (found)
	      bite_printf("and the proposal was accepted.\n");
	    else
	      bite_printf("but the proposed value was rejected.\n");
	  }
	  if (printLevel > 2) {
	    bite_printf("\nCorrelation matrix of the proposal:\n");
	    for (i = 0; i < c->n; i++)
	      for (j = 0; j < c->n; j++)
		m[i][j] = a[i][j] / (sqrt(a[i][i]) * sqrt(a[j][j]));
	    printMatrix(c->n, m);
	  }
	  if (!found)
	    for (i = 0; i < c->n; i++)
	      *addr[i] = old[i];
	  for (i = 0; i < c->n; i++) {
	    old[i] = (c->type == CLUSTER_RANDOM_WALK) ?
	      *addr[i] - old[i] : *addr[i];
	    c->mean[dim][i] = ((double) iter * c->mean[dim][i] + old[i]) /
	      (iter + 1.0);
	  }
	  for (i = 0; i < c->n; i++) {
	    c->last[dim][i] = *addr[i];
	    for (j = 0; j < c->n; j++) {
	      c->cov[dim][i][j] = 
		((double) iter * c->cov[dim][i][j] + old[i] * old[j]) /
		(iter + 1);
	      /* c->cov[dim][i][j] = 
		 ((double) iter * c->cov[dim][i][j] +
		 ((*addr[i] - ((c->type == CLUSTER_RANDOM_WALK) ?
		 old[i] : 0.0))
		 * (*addr[j] - ((c->type == CLUSTER_RANDOM_WALK) ?
		 old[j] : 0.0)))) / (iter + 1); */
	    }
	  }

	  if (c->type == CLUSTER_RANDOM_WALK) {
	    if (iter - c->last_update[dim] > 50) {
	      if (printLevel > 2)
		bite_printf("Shrinking the covariance matrix.\n");
	      for (i = 0; i < c->n; i++) {
		c->mean[dim][i] /= 1.5;
		for (j = 0; j < c->n; j++)
		  c->cov[dim][i][j] /= 2.25;
	      }
	      c->sd[dim] /= 2.0;
	      c->last_update[dim] = iter;
	    }
	    if (iter - c->last_update[dim] == 0 &&
		ran_uni(0.0, 1.0) > 0.99) {
	      if (printLevel > 2)
		bite_printf("Expanding the covariance matrix.\n");
	      for (i = 0; i < c->n; i++) {
		c->mean[dim][i] *= 1.5;
		for (j = 0; j < c->n; j++)
		  c->cov[dim][i][j] *= 2.25;
	      }
	      c->sd[dim] *= 2.0;
	      c->last_update[dim] = iter;
	    }
	  }
	  /* printLevel = 1; */
	}
      }
    }
    c = c->next;
  }
}

void updatePred()
{
  long i, j;
  pi ptr;
  sf lambda;

  if (pred && iter >= skip && iter % thinning == 0) {
    ptr = pred->next;
    while (ptr) {
      for (i = 0; i < ind; i++) {
	/* Calculate the intensity function 'lambda'. */
	putSfToFt(i, ptr->mark, NULL, -1, -1, model->ftList[ptr->mark][ind], 
		  model->ftList[ptr->mark][i], d, f);
	if (printLevel > 3) {
	  inorder(i, model->ftList[ptr->mark][ind]); 
	  bite_printf("\n");
	}
	lambda = calculate(i, model->ftList[ptr->mark][ind]);
	if (printLevel > 3)
	  printSf(lambda); 
	for (j = 0; j < ptr->n; j++) {
	  ptr->sample[j][i][(iter - skip) / thinning] =
	    intSf(lambda, calculate2(i, ptr->bot[j][i]), 
		  calculate2(i, ptr->top[j][i]));
	}
      }
      ptr = ptr->next;
    }
  }
}

void printPred()
{
  FILE *fp;
  long i, j, k;
  char fileName[40], cdum[6];
  pi ptr;

  if (pred) {
    ptr = pred->next;
    for (i = 0; i < ind; i++) {
      for (j = 0; j < ptr->n; j++) {
	fileName[0] = '\0';
	strcat(fileName, model->name[ptr->mark]);
	strcat(fileName, ".");
	intToChar(i, cdum);
	strcat(fileName, cdum);
	strcat(fileName, ".");
	intToChar(j, cdum);
	strcat(fileName, cdum);
	strcat(fileName, ".pred");
	fp = bite_fopen(fileName, "w");
	for (k = 0; k < iter_limit - skip; k++)
	  fprintf(fp, "%g\n", ptr->sample[j][i][k]);
	fclose(fp);
      }
    }
    ptr = ptr->next;
  }
}

void updateSurv()
{
  long i, j;
  double bot, top;
  pi ptr;
  sf lambda;

  if (surv && iter >= skip) {
    ptr = surv->next;
    while (ptr) {
      for (i = 0; i < ind; i++) {
/* Calculate the intensity function 'lambda'. */
	putSfToFt(i, ptr->mark, NULL, -1, -1, model->ftList[ptr->mark][ind], 
		  model->ftList[ptr->mark][i], d, f);
	if (printLevel > 3) {
	  inorder(i, model->ftList[ptr->mark][ind]); 
	  bite_printf("\n");
	}
	lambda = calculate(i, model->ftList[ptr->mark][ind]);
	if (printLevel > 3)
	  printSf(lambda);
	bot = calculate2(i, ptr->bot[0][i]);
	top = calculate2(i, ptr->top[0][i]);
	for (j = 0; j < GP; j++) {
	  /* 'F bar' is saved to ptr->sample[0][i][0,1,...,GP - 1]. */
	  ptr->sample[0][i][j] = (ptr->sample[0][i][j] *
				  ((iter - skip) / thinning) +
	    exp(-intSf(lambda, 0, bot + j * (top - bot) / GP))) / 
	      ((iter - skip) / thinning + 1);
	  /* 'f' is saved to ptr->sample[0][i][GP,GP + 1,...,2 * GP - 1]. */
	  ptr->sample[0][i][j + GP] = 
	    (ptr->sample[0][i][j + GP] * ((iter - skip) / thinning) +
	     value(lambda, bot + j * (top - bot) / GP) * 
	     exp(-intSf(lambda, 0, bot + j * (top - bot) / GP))) / 
	       ((iter - skip) / thinning + 1);
	}
      }
      ptr = ptr->next;
    }
  }
}

void printSurv()
{
  FILE *fp;
  long i, j;
  double bot, top;
  char fileName[40], cdum[6];
  pi ptr;

  if (surv) {
    ptr = surv->next;
    for (i = 0; i < ind; i++) {
      bot = calculate2(i, ptr->bot[0][i]);
      top = calculate2(i, ptr->top[0][i]);
      fileName[0] = '\0';
      strcat(fileName, model->name[ptr->mark]);
      strcat(fileName, ".");
      intToChar(i, cdum);
      strcat(fileName, cdum);
      strcat(fileName, ".surv");
      fp = bite_fopen(fileName, "w");
      for (j = 0; j < GP; j++)
	fprintf(fp, "%g %g\n", bot + j * (top - bot) / GP,
		ptr->sample[0][i][j + GP] / ptr->sample[0][i][j]);
      fclose(fp);
    }
    ptr = ptr->next;
  }
}

void exchangeValues(char *from, char *to)
{
  long i, i_from, i_to, j;
  sf fn;

  if ((i = returnType(from, &i_from, FALSE)) != returnType(to, &i_to, FALSE)) {
    bite_eprintf( "Incompatible data types for %s and %s.\n", from, to);
    bite_exit(1);
  }

  switch (i) {
  case PT_MODELF:
    if (f->fList[i_to][0]->type == SF_ORIG) {
      if (f->nFunc[i_from] != f->nFunc[i_to]) {
	bite_eprintf( "Inequal lengths of %s and %s.\n", from, to);
	bite_exit(1);
      }
      fn = initSf(0.0, 0.0, NULL);
      for (j = 0; j < f->nFunc[i_from]; j++) {
	copySf(f->fList[i_to][j]->f->f, fn);
	copySf(f->fList[i_from][j]->f->f, f->fList[i_to][j]->f->f);
	copySf(fn, f->fList[i_from][j]->f->f);
      }
      delSf(fn);
    }
    break;
  default:
    bite_eprintf(
	    "This datatype of \'exchangeValues\' is not implemented yet.\n");
  }
}

void updateExpectation()
{
  long i, j;
  double weight, val;
  expectation ptr = e;

  while (ptr && iter >= skip && (iter - skip) % thinning == 0) {
    /* Check if there is need for exchanging parameter values: */
    if (ptr->from[0])
      exchangeValues(ptr->from, ptr->to);

    /* Update the expectations: */
    if (printLevel >= 2)
      bite_printf("Calculating expectation %s.\n", ptr->name);
    if (ptr->pe[0].n > 1) { /* If a vector valued expectation. */
      double bot, top;
      for (i = 0; i < ptr->n; i++) {
	bot = calculate2(i, ptr->pe[i].lower);
	top = calculate2(i, ptr->pe[i].upper);
	weight = (ptr->pe[i].weight) ? calculate2(i, ptr->pe[i].weight) : 0.0;
	if (iter == skip)
	  ptr->pe[i].scaleOfWeights = weight;
	ptr->pe[i].sumOfWeights += exp(weight - ptr->pe[i].scaleOfWeights);

	for (j = 0; j < ptr->pe[i].n; j++) {
	  ptr->pe[i].loopParam->x->x = (top - bot) / ptr->pe[i].n * j + bot;
	  val = calculate2(i, ptr->pe[i].f[0]);
	  if (ptr->saveVal)
	    ptr->pe[i].val[j][ptr->pe[i].counter] = val;
	  ptr->pe[i].mean[j] += val * 
	     exp(weight - ptr->pe[i].scaleOfWeights);
	  ptr->pe[i].variance[j] += pow(val, 2.0) * 
	     exp(weight - ptr->pe[i].scaleOfWeights);
	}
	if (ptr->saveVal)
	  ptr->pe[i].counter++;
      }
    }
    else { /* Else a scalar expectation. */
      for (i = 0; i < ptr->n; i++) {
	weight = (ptr->pe[i].weight) ? calculate2(i, ptr->pe[i].weight) : 0.0;
	if (iter == skip)
	  ptr->pe[i].scaleOfWeights = 0.0;
	ptr->pe[i].sumOfWeights += exp(weight - ptr->pe[i].scaleOfWeights);

#ifdef DEBUG
#endif
	val = calculate2(i, ptr->pe[i].f[0]);
	if (ptr->saveVal) {
	  ptr->pe[i].val[0][ptr->pe[i].counter] = val;
	  ptr->pe[i].counter++;
	}
	ptr->pe[i].mean[0] +=
	   val * exp(weight - ptr->pe[i].scaleOfWeights);
	ptr->pe[i].variance[0] +=
	  pow(val, 2.0) * exp(weight - ptr->pe[i].scaleOfWeights);
      }
    }

    if (ptr->saveVal && ptr->pe[0].counter == EXP_VAL_STOR) {
      /* Append the buffers to the files: */
      long k;
      FILE *fp;
      char fileName[40];
      if (ptr->pe[0].n == 1) { /* A scalar expectation */
	sprintf(fileName, "%s.sav", ptr->name);
	fp = bite_fopen(fileName, "a");
	for (k = 0; k < ptr->pe[0].counter; k++) {
	  for (j = 0; j < ptr->n; j++)
	    fprintf(fp, "%g ", ptr->pe[j].val[0][k]);
	  fprintf(fp, "\n");
	}
	fclose(fp);
      }
      else { /* A vector valued expectation. */
	for (i = 0; i < ptr->n; i++) {
	  sprintf(fileName, "%s.%ld.sav", ptr->name, i);
	  fp = bite_fopen(fileName, "a");
	  for (k = 0; k < ptr->pe[i].counter; k++) {
	    for (j = 0; j < ptr->pe[i].n; j++)
	      fprintf(fp, "%g ", ptr->pe[i].val[j][k]);
	    fprintf(fp, "\n");
	  }
	  fclose(fp);
	}
      }
      for (i = 0; i < ptr->n; i++)
	ptr->pe[i].counter = 0;
    }

    /* Check if there is need for exchanging parameter values: */
    if (ptr->from[0])
      exchangeValues(ptr->from, ptr->to);
    ptr = ptr->next;
  }
}

void printExpectation()
{
  FILE *fp;
  long i, j;
  double w;
  char fileName[40];
  expectation ptr = e;

  while (ptr) {
    if (ptr->pe[0].n == 1) { /* A scalar expectation */
      sprintf(fileName, "%s.tmp", ptr->name);
      fp = bite_fopen(fileName, "w");
      for (j = 0; j < ptr->n; j++) {
	w = ptr->pe[j].sumOfWeights; /* / ((iter_limit - skip) / thinning); */
	fprintf(fp, "%g ", ptr->pe[j].mean[0] / w);
	if (ptr->printVar)
	  fprintf(fp, "%g ",
		  ptr->pe[j].variance[0] / w - pow(ptr->pe[j].mean[0] / w, 2));
	fprintf(fp, "\n");
      }
      fclose(fp);
    }
    else { /* A vector valued expectation. */
      double bot, top;
      for (i = 0; i < ptr->n; i++) {
	sprintf(fileName, "%s.%ld.tmp", ptr->name, i);
	fp = bite_fopen(fileName, "w");
        bot = calculate2(i, ptr->pe[i].lower);
        top = calculate2(i, ptr->pe[i].upper);
	w = ptr->pe[i].sumOfWeights; /* / ((iter_limit - skip) / thinning); */
	for (j = 0; j < ptr->pe[i].n; j++) {
	  fprintf(fp, "%g ", ptr->pe[i].mean[j] / w);
	  if (ptr->printVar)
	    fprintf(fp, "%g ", ptr->pe[i].variance[j] / w -
		    pow(ptr->pe[i].mean[j] / w, 2));
	  fprintf(fp, "\n");
	}
	fclose(fp);
      }
    }
    if (ptr->saveVal) {
      /* Append the buffers to the files: */
      long k;
      if (ptr->pe[0].n == 1) { /* A scalar expectation */
	sprintf(fileName, "%s.sav", ptr->name);
	fp = bite_fopen(fileName, "a");
	for (k = 0; k < ptr->pe[0].counter; k++) {
	  for (j = 0; j < ptr->n; j++)
	    fprintf(fp, "%g ", ptr->pe[j].val[0][k]);
	  fprintf(fp, "\n");
	}
	fclose(fp);
      }
      else { /* A vector valued expectation. */
	for (i = 0; i < ptr->n; i++) {
	  sprintf(fileName, "%s.%ld.sav", ptr->name, i);
	  fp = bite_fopen(fileName, "a");
	  for (k = 0; k < ptr->pe[i].counter; k++) {
	    for (j = 0; j < ptr->pe[i].n; j++)
	      fprintf(fp, "%g ", ptr->pe[i].val[j][k]);
	    fprintf(fp, "\n");
	  }
	  fclose(fp);
	}
      }
      for (i = 0; i < ptr->n; i++)
	ptr->pe[i].counter = 0;
    }
    ptr = ptr->next;
  }
}

void levelDens(long priorProp, double epsilon, double x, mf target,
	       long updL, double *qx, double *prx)
{
  long j, k = 0;
  /* static long lexis_dim;
     static double *lexis_weight = NULL; */
  if (updL >= target->f->f->n - 1) {
    *qx = *prx = 0.0;
  } else {
    switch (target->prior) {
    case GAMMA:
      if (updL == 0) {
	if (!priorProp)
	  *qx = 0;
	else
	  *qx = log_dens_gamma(x, alpha0, beta0);
	/* (alpha0 - 1) * LOG(x) - beta0 * x; */
	*prx = log_dens_gamma(x, alpha0, beta0);
	/* *prx = (alpha0 - 1.0) * LOG(x) - beta0 * x; */
	/* if (mu && target->f->f->n > 2)
	 *prx -= alpha * (target->f->f->levels[1] / x + LOG(x)); */
      }
      else {
	if (!priorProp)
	  *qx = 0;
	else
	  *qx = log_dens_gamma(x, alpha * beta, alpha / target->f->f->levels[updL - 1]);
	/* (alpha * beta - 1) * LOG(x) - alpha * x /
	   target->f->f->levels[updL - 1]; */
	*prx = log_dens_gamma(x, alpha * beta, alpha / target->f->f->levels[updL - 1]);
	/* -LOG(x) - alpha * (x / target->f->f->levels[updL - 1] + 
	   ((updL < target->f->f->n - 2) ?
	   target->f->f->levels[updL + 1] / x : 0.0)); */
      }
      break;
    case GAMMA_LEXIS:
      for (j = 0; j < target->neighbour_dim; j++) {
	if (target->neighbour_prev[j])
	  k++;
      }
      if (k) {
	/* long i;
	double alpha0 = calculate2(ind, target->pVal->pGL->alpha0),
	  beta0 = calculate2(ind, target->pVal->pGL->beta0),
	  alpha = calculate2(ind, target->pVal->pGL->alpha),
	  beta = calculate2(ind, target->pVal->pGL->beta); */
	/* if (lexis_weight == NULL) {
	  lexis_dim = target->pVal->pGL->lexis_dim;
	  lexis_weight = malloc(lexis_dim * sizeof(mu));
	}
	if (lexis_dim < target->pVal->pGL->lexis_dim) {
	  lexis_dim = target->pVal->pGL->lexis_dim;
	  lexis_weight = realloc(lexis_weight, lexis_dim * sizeof(mu));
	}
	for (i = 0; i < target->pVal->pGL->lexis_dim; i++)
	lexis_weight[i] = calculate2(ind, target->pVal->pGL->lexis_weight[i]); */
	*prx = 0.0;
	for (j = 0; j < target->neighbour_dim; j++) {
	  if (target->neighbour_next[j])
	    *prx += returnLexisMfLogPriorDensity(ind, target->neighbour_next[j], -1,
						 (double) target->f->f->jumpPoints[updL],
						 (double) target->f->f->jumpPoints[updL + 1],
						 alpha0, beta0, alpha, beta, lexis_weight,
						 FALSE);
	}
	if (!priorProp)
	  *qx = 0;
	else
	  *qx = (updL == 0) ? log_dens_gamma(x, alpha0, beta0) :
	    log_dens_gamma(x, alpha * beta, alpha / target->f->f->levels[updL - 1]);
      } else {
	/* The base model function: no previous model functions,
	   so apply the Arjas-Gasbarra prior. */
	if (updL == 0) {
	  if (!priorProp)
	    *qx = 0;
	  else
	    *qx = log_dens_gamma(x, alpha0, beta0);
	  *prx = log_dens_gamma(x, alpha0, beta0);
	}
	else {
	  if (!priorProp)
	    *qx = 0;
	  else
	    *qx = log_dens_gamma(x, alpha * beta, alpha / target->f->f->levels[updL - 1]);
	  *prx = log_dens_gamma(x, alpha * beta, alpha / target->f->f->levels[updL - 1]);
	}
      }
      break;
    case GAMMA2: /* The weakness of the bind to the previous level depends
		    on the length T_{updL+1} - T_{updL} */
      {
	double alphaS = alpha / (target->f->f->jumpPoints[updL + 1] -
				 target->f->f->jumpPoints[updL]) + 1;
	if (updL == 0) {
	  double alpha0S = alpha0 / (target->f->f->jumpPoints[updL + 1] -
				     target->f->f->jumpPoints[updL]),
	    beta0S = beta0 / (target->f->f->jumpPoints[updL + 1] -
			      target->f->f->jumpPoints[updL]);
	  if (!priorProp)
	    *qx = 0;
	  else
	    *qx = alpha0S * LOG(x) - beta0S * x;
	  *prx = (alpha0S - 1.0) * LOG(x) - beta0S * x;
	  if (mu && target->f->f->n > 2)
	    *prx -= alphaS * (target->f->f->levels[1] / x + LOG(x));
	}
	else {
	  if (!priorProp)
	    *qx = 0;
	  else
	    *qx = alphaS * LOG(x) - alphaS * x / target->f->f->levels[updL - 1];
	  *prx = -LOG(x) - alphaS *
	    (x / target->f->f->levels[updL - 1] + 
	     ((updL < target->f->f->n - 2) ?
	      target->f->f->levels[updL + 1] / x : 0.0));
	}
      }
      break;
    case INCR_GAMMA:
      if (updL == 0) {
	*qx = (updL < target->f->f->n - 2) ?
	  -LOG(target->f->f->levels[updL + 1]) : 0.0;
	*prx = (updL < target->f->f->n - 2) ?
	  /* LOG(dens_gamma(x, alpha0, beta0)) */
	  (alpha0 - 1.0) * LOG(x) - beta0 * x : 0.0;
      }
      else {
	*qx = (updL < target->f->f->n - 2) ?
	  -LOG(target->f->f->levels[updL + 1] -
	       target->f->f->levels[updL - 1]) : 0.0;
	*prx = (updL < target->f->f->n - 2) ?
	  /*LOG(dens_gamma(x - target->f->f->levels[updL - 1], alpha,
	    beta)) */
	  (alpha - 1.0) * LOG(x - target->f->f->levels[updL - 1]) -
	  beta * (x - target->f->f->levels[updL - 1]) : 0.0;
      }
      *prx += (mu && updL < target->f->f->n - 2) ?
	/* LOG(dens_gamma(target->f->f->levels[updL + 1] - x,
	   alpha, beta)) */
	(alpha - 1.0) * LOG(target->f->f->levels[updL + 1] - x) -
	beta * (target->f->f->levels[updL + 1] - x) : 0.0;
      break;
    case DECR_GAMMA:
      if (updL == 0) {
	*qx = 0;
	*prx = (updL < target->f->f->n - 2) ?
	  LOG(dens_gamma(x, alpha0, beta0)) : 0.0;
      }
      else {
	*qx = 0;
	*prx = (updL < target->f->f->n - 2) ?
	  LOG((target->f->f->levels[updL - 1] > target->f->f->levels[updL]) /
	      target->f->f->levels[updL - 1]) : 0.0;
      }
      *prx += (mu && updL < target->f->f->n - 2) ?
	LOG((target->f->f->levels[updL + 1] < target->f->f->levels[updL]) /
	    target->f->f->levels[updL]) : 0.0;
      break;
    }
  }
}

double generatePropLevel(long priorProp, double epsilon,
			 double x, mf target, long updL)
{
  double y = -INFTY;

  switch (target->prior) {
  case GAMMA:
    y = (!priorProp) ?
      ((x > epsilon) ?
       ran_uni(x - epsilon, x + epsilon) : ran_uni(0.0, 2 * epsilon)) :
	 ((updL == 0) ? ran_gamma(alpha0, beta0) :
	  ran_gamma(alpha * beta, alpha / target->f->f->levels[updL - 1]));
    y = (y < 1e-6) ? 1e-6 : y;
    break;
  case GAMMA_LEXIS:
    y = (!priorProp) ?
      ((x > epsilon) ?
       ran_uni(x - epsilon, x + epsilon) : ran_uni(0.0, 2 * epsilon)) :
	 ((updL == 0) ? ran_gamma(alpha0, beta0) :
	  ran_gamma(alpha * beta, alpha / target->f->f->levels[updL - 1]));
    y = (y < 1e-6) ? 1e-6 : y;
    break;
  case GAMMA2:
    {
      double alphaS = alpha / (target->f->f->jumpPoints[updL + 1] -
			       target->f->f->jumpPoints[updL]) + 1,
	alpha0S = alpha0 / (target->f->f->jumpPoints[updL + 1] -
			    target->f->f->jumpPoints[updL]),
	beta0S = beta0 / (target->f->f->jumpPoints[updL + 1] -
			  target->f->f->jumpPoints[updL]);
      y = (!priorProp) ?
	((x > epsilon) ?
	 ran_uni(x - epsilon, x + epsilon) : ran_uni(0.0, 2 * epsilon)) :
	((updL == 0) ? ran_gamma(alpha0S, beta0S) :
	 ran_gamma(alphaS, alphaS / target->f->f->levels[updL - 1]));
    }
    break; 
  case INCR_GAMMA:
    y = (updL == 0) ?
      ((updL < target->f->f->n - 2) ?
       ran_uni(0.0, target->f->f->levels[updL + 1]) :
       ran_gamma(alpha0, beta0)) :
	 ((updL < target->f->f->n - 2) ?
	  ran_uni(target->f->f->levels[updL - 1],
		  target->f->f->levels[updL + 1]) :
	  ran_gamma(alpha, beta) + target->f->f->levels[updL - 1]);
    if (updL && y == target->f->f->levels[updL - 1])
      y *= 1.0001;
    if (updL < target->f->f->n - 2 && y >= target->f->f->levels[updL + 1])
      y = (target->f->f->levels[updL + 1] +
	   ((updL) ? target->f->f->levels[updL - 1] : 0.0)) / 2.0;
    break; 
  case DECR_GAMMA:
    if (priorProp) {
      if (updL == 0) {
	int i = 0;
	while (updL < target->f->f->n - 2 &&
	       (y = ran_gamma(alpha0, beta0)) < target->f->f->levels[1] &&
	       i < 10)
	  i++;
      }
      else
	y = ran_uni(target->f->f->levels[updL + 1],
		    target->f->f->levels[updL - 1]);
    }
    else {
      if (updL == 0)
	y = (x > target->f->f->levels[1] + epsilon) ?
	  ran_uni(x - epsilon, x + epsilon) :
	  ran_uni(target->f->f->levels[1], target->f->f->levels[1] +
		  2 * epsilon);
      else
	y = (x > target->f->f->levels[updL + 1] + epsilon &&
	     x < target->f->f->levels[updL - 1] + epsilon) ?
	  ran_uni(x - epsilon, x + epsilon) :
	  ran_uni(target->f->f->levels[updL + 1],
		  target->f->f->levels[updL - 1]);
    }
    break;
  }
  return (y < 1.0e-6) ? 1.0e-6 : y;
}

void updateJ( mf target, long updJ, dt d, ff model )
{
  long j, found = FALSE, priorProp = 0;
  double x, y, lx, ly, prx = 0.0, pry = 0.0, /* qx = 0.0, qy = 0.0, */
    u, alphaMH, eps, a = 0.0, b = 0.0;

#ifdef HAMMAS
  if (updJ > 2) {
#endif
    if (ran_uni(0.0, 1.0) < target->pVal->pGa->mix)
      priorProp = 1; /* Use prior as proposal. */
    a = target->f->f->jumpPoints[updJ - 1] + target->minPieceLength;
    b = target->f->f->jumpPoints[updJ + 1] - target->minPieceLength;
    eps = (b - a) / 20;
    /* First choose the correct pointers of the functions we are updating: */
    lx = logLikel(target, updJ, -1, d, model);
    x = target->f->f->jumpPoints[updJ];
    if (target->prior == GAMMA2 || target->prior == GAMMA_LEXIS) {
      /* alpha0 = calculate2(-1, target->pVal->pGa->alpha0);
      beta0 = calculate2(-1, target->pVal->pGa->beta0);
      alpha = calculate2(-1, target->pVal->pGa->alpha);
      beta = calculate2(-1, target->pVal->pGa->beta);
      mu = calculate2(-1, target->pVal->pGa->mu); */
      /* levelDens(TRUE, 0.0, x, target, updJ - 1, &qx, &prx);
      lx += prx;
      levelDens(TRUE, 0.0, x, target, updJ, &qx, &prx);
      lx += prx; */
      prx = returnLexisMfLogPriorDensity(ind, target, -1,
					 (double) target->f->f->jumpPoints[updJ - 1],
					 (double) target->f->f->jumpPoints[updJ + 1],
					 alpha0, beta0, alpha, beta, lexis_weight,
					 FALSE);
      for (j = 0; j < target->neighbour_dim; j++) {
	if (target->neighbour_next[j])
	  prx += returnLexisMfLogPriorDensity(ind, target->neighbour_next[j], -1,
					      (double) target->f->f->jumpPoints[updJ - 1],
					      (double) target->f->f->jumpPoints[updJ + 1],
					      alpha0, beta0, alpha, beta, lexis_weight,
					      FALSE);
      }
    }
    if (printLevel >= 2)
      bite_printf("L(x)=%g, x=%g ", lx, x);
    /* If not the last (real) jump point, then ... */
    if (updJ < target->f->f->n - 1 && updJ > 0 && a < b) {
      long j_max = 0; /* Oli 'j' */
      while (!found && j_max < max_iter) {
	j_max++;
	if (priorProp)
	  y = ran_uni(a, b); 
	else {
	  if (x < a + eps)
	    y = ran_uni(a, a + 2 * eps);
	  else {
	    if (x > b - eps)
	      y = ran_uni(b - 2 * eps, b);
	    else
	      y = ran_uni(x - eps, x + eps);
	  }
	}
	target->f->f->jumpPoints[updJ] = y;
	ly = logLikel(target, updJ, -1, d, model);
	if (target->prior == GAMMA2 || target->prior == GAMMA_LEXIS) {
	  /* levelDens(TRUE, 0.0, y, target, updJ - 1, &qy, &pry);
	  ly += pry;
	  levelDens(TRUE, 0.0, y, target, updJ, &qy, &pry);
	  ly += pry; */	  
	  pry = returnLexisMfLogPriorDensity(ind, target, -1,
					     target->f->f->jumpPoints[updJ - 1],
					     target->f->f->jumpPoints[updJ + 1],
					     alpha0, beta0, alpha, beta, lexis_weight,
					     FALSE);
	  for (j = 0; j < target->neighbour_dim; j++) {
	    if (target->neighbour_next[j])
	      pry += returnLexisMfLogPriorDensity(ind, target->neighbour_next[j], -1,
						  target->f->f->jumpPoints[updJ - 1],
						  target->f->f->jumpPoints[updJ + 1],
						  alpha0, beta0, alpha, beta, lexis_weight,
						  FALSE);
	  }
	}
	alphaMH = ly + pry - lx - pry;
	u = ran_uni(0.0, 1.0);
	if (printLevel >= 2) {
	  if (alphaMH < LOG(0.05))
	    bite_printf(".");
	  else
	    if (alphaMH < LOG(0.25))
	      bite_printf("+");
	    else 
	      bite_printf("*"); 
	}
      
	if (LOG(u) < alphaMH) {
	  found = TRUE;
	  if (printLevel >= 2)
	    bite_printf(" L(y)=%g, y=%g ", ly, y);
	  target->f->f->jumpPoints[updJ] = y;
	}
      }
      if (!found) {
	target->f->f->jumpPoints[updJ] = x;
	fails++;
      }
    }
#ifdef HAMMAS
  }
#endif
/*  if (printLevel >= 3) {
    bite_printf("\nJump point: ");
    for (j = 0; j < 100; j++) 
      bite_printf(" %g: %g, ", 
	     (double)j / 100 * 
	     (target->f->f->jumpPoints[updJ + 1] - 
	      target->f->f->jumpPoints[updJ - 1])
	     + target->f->f->jumpPoints[updJ - 1],
	     logLikel(target, updJ, -1, d, model));
    bite_printf("\n");
  } */
} /* end updateJ */

void updateL( mf target, long updL, dt d, ff model )
{
  long i, j, found = FALSE, priorProp = FALSE;
  double x, y = 0.0, lx = 0.0, ly = 0.0, prx = 0.0, pry = 0.0, prx0, pry0,
    qx = 0.0, qy = 0.0, qx0, qy0, alphaMH, epsilon = target->epsilon;

  /* if 'priorProp' != 0, then we use the prior as proposal. */
  if (ran_uni(0.0, 1.0) < target->pVal->pGa->mix)
    priorProp = TRUE;
  if (target->prior == GAMMA) {
    if (alpha < 0.1)
      priorProp = FALSE;
  }
  else
    priorProp = FALSE;
  if (target->restrict0 < 0.0 || updL > 0) {
    /* Calculate the values of the prior density of the old level 'x' ... */
    x = target->f->f->levels[updL];
    /* ... and likelihood */
    lx = logLikel(target, -1, updL, d, model);
    /* if (!lx && !updL && target->name[0] != 'f')
      bite_printf("Zero likel!\n"); */
    if (target->prior == GAMMA_LEXIS) {
      prx = returnLexisMfLogPriorDensity(ind, target, -1,
					 target->f->f->jumpPoints[updL],
					 target->f->f->jumpPoints[updL + 1],
					 alpha0, beta0, alpha, beta, lexis_weight,
					 FALSE);
      for (j = 0; j < target->neighbour_dim; j++) {
	if (target->neighbour_next[j])
	  prx += returnLexisMfLogPriorDensity(ind, target->neighbour_next[j], -1,
					      target->f->f->jumpPoints[updL],
					      target->f->f->jumpPoints[updL + 1],
					      alpha0, beta0, alpha, beta, lexis_weight,
					      FALSE);
      }
      if (priorProp)
	qx = (updL) ? 
	  log_dens_gamma(target->f->f->levels[updL],
			 alpha * beta, alpha / target->f->f->levels[updL - 1]) :
	  log_dens_gamma(target->f->f->levels[updL],
			 alpha0, beta0);
    } else {
      /* Calculate prior and proposal densities, prx and qx, for the level and the next: */
      levelDens(priorProp, epsilon, x, target, updL, &qx, &prx);
      /* lx += prx - qx; */
      /* levelDens(priorProp, epsilon, x, target, updL + 1, &qx0, &prx0); */
      levelDens(priorProp, epsilon, target->f->f->levels[updL + 1], target, updL + 1, &qx0, &prx0);
      /* lx += prx; */
      prx += prx0;
    }
    if (printLevel >= 2)
      bite_printf("L(x)=%g, prx=%g, qx=%g, x=%g ", lx, prx, qx, x);
    i = 0;
    /* M-H algoritm ('y' is the proposal): */
    while (!found && i < max_iter) {
      i++;
      target->f->f->levels[updL] = y =
	generatePropLevel(priorProp, epsilon, x, target, updL);
      if (printLevel >= 2)
	bite_printf(" proposal=%g ", y);
      ly = logLikel(target, -1, updL, d, model);
      if (target->prior == GAMMA_LEXIS) {
	pry = returnLexisMfLogPriorDensity(ind, target, -1,
					   target->f->f->jumpPoints[updL],
					   target->f->f->jumpPoints[updL + 1],
					   alpha0, beta0, alpha, beta, lexis_weight,
					   FALSE);
	for (j = 0; j < target->neighbour_dim; j++) {
	  if (target->neighbour_next[j])
	    pry += returnLexisMfLogPriorDensity(ind, target->neighbour_next[j], -1,
						target->f->f->jumpPoints[updL],
						target->f->f->jumpPoints[updL + 1],
						alpha0, beta0, alpha, beta, lexis_weight,
						FALSE);
	}
	if (priorProp)
	  qy = (updL) ? 
	    log_dens_gamma(target->f->f->levels[updL],
			   alpha * beta, alpha / target->f->f->levels[updL - 1]) :
	    log_dens_gamma(target->f->f->levels[updL],
			   alpha0, beta0);
	/* alphaMH = (ly + pry - qy) - (lx + prx - qx); */
      } else {
	levelDens(priorProp, epsilon, y, target, updL, &qy, &pry);
	/* lx += pry - qy; */
	/* levelDens(priorProp, epsilon, x, target, updL + 1, &qy0, &pry0); */
	levelDens(priorProp, epsilon, target->f->f->levels[updL + 1], target, updL + 1, &qy0, &pry0);
	/* lx += pry; */
	pry += pry0;
	/* alphaMH = ly - lx; */
      }

      alphaMH = (ly + pry - qy) - (lx + prx - qx);
      if (printLevel >= 2) {
	if (alphaMH < LOG(0.05))
	  bite_printf(".");
	else {
	  if (alphaMH < LOG(0.25))
	    bite_printf("+");
	  else {
	    bite_printf("*");
	    if (ly - lx < -10)
	      bite_printf("(%g,%g -> %g,%g) ", prx,qx, pry,qy);
	  }
	}
      }

      if (LOG(ran_uni(0.0, 1.0)) < alphaMH) { 
	found = TRUE;
	if (printLevel >= 2)
	  bite_printf(" L(y)=%g, pry=%g, qy=%g, y=%g ", ly, pry, qy, y);
	target->f->f->levels[updL] = y;
      } 
    }
    if (!found) {
      if (printLevel >= 2)
	bite_printf(" [L(y)=%g, pry=%g, y=%g] ", ly, pry, y);
      target->f->f->levels[updL] = x;
      fails++;
    }
    /* if (target->prior == INCR_GAMMA && updL == 0) {
      if (target->covVal[0] == 14 ||
	  target->covVal[0] == 9)
	target->f->f->levels[0] =
	  (target->f->f->n > 2 && target->f->f->levels[1] < 2.0) ?
	  target->f->f->levels[1] : 2.0;
      else
	target->f->f->levels[updL] = 0.0;
    } */
  }
} /* end updateL */

double logGamma2Prior(sf f, long a, long b, double alpha0, double beta0,
		      double alpha)
{
  long i;
  double res = 0.0;

  for (i = a; i <= b && i < f->n - 1; i++)
    res +=
      (i) ?
      LOG(dens_gamma(f->levels[i],
		     alpha / (f->jumpPoints[i + 1] - f->jumpPoints[i]) + 1.0,
		     (alpha / (f->jumpPoints[i + 1] - f->jumpPoints[i]) + 1.0)
		     / f->levels[i - 1])) :
    LOG(dens_gamma(f->levels[i],
		   alpha0 / (f->jumpPoints[i + 1] - f->jumpPoints[i]),
		   beta0 / (f->jumpPoints[i + 1] - f->jumpPoints[i])));
  return res;
}

void addOrDeleteJp(mf target, dt d, ff model)
{
  long j, k;
  double lx, ly, u, alphaMH, propJP, propL = 0.0,
    qx, qy, prx = 0.0, pry = 0.0, bk;
  /* static long lexis_dim;
     static double *lexis_weight = NULL; */

  /*  printLevel = 4; */
  switch (target->prior) {
  case GAMMA2:
    alpha0 = calculate2(-1, target->pVal->pGa->alpha0);
    beta0 = calculate2(-1, target->pVal->pGa->beta0);
  case GAMMA:
    alpha = calculate2(-1, target->pVal->pGa->alpha);
    beta = calculate2(-1, target->pVal->pGa->beta);
    mu = calculate2(-1, target->pVal->pGa->mu);
    break;
  case INCR_GAMMA:
    alpha = calculate2(-1, target->pVal->pGa->alpha);
    beta = calculate2(-1, target->pVal->pGa->beta);
    mu = calculate2(-1, target->pVal->pGa->mu);
    break;
  case DECR_GAMMA:
    alpha0 = calculate2(-1, target->pVal->pGa->alpha0);
    beta0 = calculate2(-1, target->pVal->pGa->beta0);
    mu = calculate2(-1, target->pVal->pGa->mu);
    break;
  case GAMMA_LEXIS:
    /* if (lexis_weight == NULL) {
      lexis_dim = target->pVal->pGL->lexis_dim;
      lexis_weight = malloc(lexis_dim * sizeof(mu));
    }
    if (lexis_dim < target->pVal->pGL->lexis_dim) {
      lexis_dim = target->pVal->pGL->lexis_dim;
      lexis_weight = realloc(lexis_weight, lexis_dim * sizeof(mu));
    }
    for (j = 0; j < target->pVal->pGL->lexis_dim; j++)
      lexis_weight[j] = 
      calculate2(-1, target->pVal->pGL->lexis_weight[j]); */
    /* alpha0 = calculate2(-1, target->pVal->pGL->alpha0);
    beta0 = calculate2(-1, target->pVal->pGL->beta0);
    alpha = calculate2(-1, target->pVal->pGL->alpha);
    beta = calculate2(-1, target->pVal->pGL->beta);
    mu = calculate2(-1, target->pVal->pGL->mu); */
    break;
  }
  u = ran_uni(0.0, 1.0);
  bk = (target->f->f->n > 2.0 * target->length * mu) ? 0.5 : 0.5;
  if (ran_uni(0.0, 1.0) < bk) { /* Try to add a new jump point. */
    /* Choose the interval where the new jp should be added: */
    k = (long) ran_uni(0.0, (double)(target->f->f->n - 1));
    if (target->f->f->jumpPoints[k + 1] - target->f->f->jumpPoints[k] >
	2.0 * target->minPieceLength) {
      /* If the length of the interval is longer than twice the minimum length of interval: */
#ifdef HAMMAS
      if (updJ > 2) {
#endif
	qx = 0.0;
	propJP = ran_uni(target->f->f->jumpPoints[k] + target->minPieceLength,
			 target->f->f->jumpPoints[k + 1] -
			 target->minPieceLength);
	if (printLevel >= 2)
	  bite_printf("Trying to add a new jump point %g on (%g,%g)\n\tand ",
		 propJP, target->f->f->jumpPoints[k],
		 target->f->f->jumpPoints[k + 1]);
	if (printLevel >= 3) {
	  bite_printf("Target before adding is ");
	  printSf(target->f->f);
	  bite_printf("and likelihood is ");
	}
	lx = logLikel(target, -1, k, d, model);
	/* qx = LOG(target->f->f->n - 1); */
	switch (target->prior) {
	case GAMMA:
	  prx = log_dens_gamma(target->f->f->levels[k + 1], alpha * beta,
			       alpha / target->f->f->levels[k]);
	  propL = ran_gamma(alpha * beta, alpha / target->f->f->levels[k]);
	  break;
	case GAMMA_LEXIS:
	  /* levelDens(TRUE, 0.0, 0.0, target, k, &qx, &prx);
	     lx += prx - qx; */
	  prx = returnLexisMfLogPriorDensity(ind, target, -1,
					     target->f->f->jumpPoints[k],
					     target->f->f->jumpPoints[k + 1],
					     alpha0, beta0, alpha, beta, lexis_weight,
					     FALSE);
	  for (j = 0; j < target->neighbour_dim; j++) {
	    if (target->neighbour_next[j])
	      prx += returnLexisMfLogPriorDensity(ind, target->neighbour_next[j], -1,
						  target->f->f->jumpPoints[k],
						  target->f->f->jumpPoints[k + 1],
						  alpha0, beta0, alpha, beta, lexis_weight,
						  FALSE);
	  }
	  propL = ran_gamma(alpha * beta, alpha / target->f->f->levels[k]);
	  qx = 0.0;
	  break;
	case GAMMA2:
	  prx = logGamma2Prior(target->f->f, k, k + 1, alpha0, beta0, alpha);

	  LOG(dens_gamma(target->f->f->levels[k + 1],
			 alpha / (target->f->f->jumpPoints[k + 2] -
				  target->f->f->jumpPoints[k + 1]),
			 alpha / (target->f->f->jumpPoints[k + 2] -
				  target->f->f->jumpPoints[k + 1]) /
			 target->f->f->levels[k]));
	  propL = ran_gamma(alpha / (target->f->f->jumpPoints[k + 1] -
				     target->f->f->jumpPoints[k]) + 1.0,
			    (alpha / (target->f->f->jumpPoints[k + 1] -
				      target->f->f->jumpPoints[k]) + 1.0) /
			    target->f->f->levels[k]);
	  break;
	case INCR_GAMMA:
	  propL = (k < target->f->f->n - 2) ?
	    ran_uni(target->f->f->levels[k], target->f->f->levels[k + 1]) :
	      target->f->f->levels[k] + ran_gamma(alpha, beta);
	  if (propL == target->f->f->levels[k])
	    propL *= 1.0001;
	  /* target->f->f->levels[k] +
	     (target->f->f->levels[k + 1] - target->f->f->levels[k]) *
	     pow((propJP - target->f->f->jumpPoints[k]) /
	     (target->f->f->jumpPoints[k + 1] - target->f->f->jumpPoints[k]),
	     xi); */
	  prx = (alpha - 1.0) * LOG(target->f->f->levels[k + 1] - 
				    target->f->f->levels[k]) -
	    beta * (target->f->f->levels[k + 1] - 
		    target->f->f->levels[k]);
	  /* LOG(dens_gamma(target->f->f->levels[k + 1] -
			 target->f->f->levels[k], alpha, beta)); */
	  break;
	case DECR_GAMMA:
	  propL = ran_uni(target->f->f->levels[k + 1],
			  target->f->f->levels[k]);
	  prx = LOG(1.0 / (target->f->f->levels[k + 1] -
			   target->f->f->levels[k]));
	  break;
	}
	propL = (propL < 1e-6) ? 1e-6 : propL;
	if (printLevel >= 2)
	  bite_printf("a level %g on (%g,%g):\n\t", propL,
		 (target->prior == GAMMA) ? 0.0 : target->f->f->levels[k],
		 (target->prior == GAMMA) ? INFTY : target->f->f->levels[k + 1]);
	/* Add the new jump point and the new level at the k+1'th interval: */
	addNode(target->f->f, k + 1, propJP, propL);
	/* Uniform density of the jump point proposal: */
	qy = -LOG(target->f->f->jumpPoints[k + 2] -
		  target->f->f->jumpPoints[k]);
	/*    printSf(target->f->f); */
	if (printLevel >= 3) {
	  bite_printf("\nTarget after adding is ");
	  printSf(target->f->f);
	  bite_printf("and likelihood is ");
	}
	ly = logLikel(target, k + 1, -1, d, model);
	switch (target->prior) {
	case GAMMA:
	  pry = log_dens_gamma(propL, alpha * beta, alpha / target->f->f->levels[k]) +
	    log_dens_gamma(target->f->f->levels[k + 2], alpha * beta, alpha / propL);
	  qy += log_dens_gamma(propL, alpha * beta, alpha / target->f->f->levels[k]);
	  break;
	case GAMMA_LEXIS:
	  /* levelDens(TRUE, 0.0, 0.0, target, k, &qy, &pry);
	  ly += pry - qy;
	  levelDens(TRUE, 0.0, 0.0, target, k + 1, &qy, &pry);
	  ly += pry - qy; */
	  pry = returnLexisMfLogPriorDensity(ind, target, -1,
					     target->f->f->jumpPoints[k],
					     target->f->f->jumpPoints[k + 2],
					     alpha0, beta0, alpha, beta, lexis_weight,
					     FALSE);
	  for (j = 0; j < target->neighbour_dim; j++) {
	    if (target->neighbour_next[j])
	      pry += returnLexisMfLogPriorDensity(ind, target->neighbour_next[j], -1,
						  target->f->f->jumpPoints[k],
						  target->f->f->jumpPoints[k + /* 1 */ 2],
						  alpha0, beta0, alpha, beta, lexis_weight,
						  FALSE);
	  }
	  /* Proposal density is the density of the new level: */
	  qy += log_dens_gamma(propL, alpha * beta, alpha / target->f->f->levels[k]);
	  break;
	case GAMMA2:
	  pry = logGamma2Prior(target->f->f, k, k + 2, alpha0, beta0, alpha);

	  /* LOG(dens_gamma(propL, alpha /
			 (target->f->f->jumpPoints[k + 2] -
			  target->f->f->jumpPoints[k + 1]),
			 alpha / (target->f->f->jumpPoints[k + 2] -
				  target->f->f->jumpPoints[k + 1]) /
			 target->f->f->levels[k])) +
	    LOG(dens_gamma(target->f->f->levels[k + 2],
			   alpha / (target->f->f->jumpPoints[k + 2] -
				    target->f->f->jumpPoints[k + 1]),
			   alpha / (target->f->f->jumpPoints[k + 2] -
			   target->f->f->jumpPoints[k + 1]) / propL)); */
	  qy += LOG(dens_gamma(propL,
			       alpha / (target->f->f->jumpPoints[k + 2] -
					target->f->f->jumpPoints[k]) + 1.0,
			       (alpha / (target->f->f->jumpPoints[k + 2] -
					 target->f->f->jumpPoints[k]) + 1.0) /
			       target->f->f->levels[k]));
	  break;
	case INCR_GAMMA:
	  pry = 
	    (alpha - 1.0) * LOG(propL - target->f->f->levels[k]) -
	    beta * (propL - target->f->f->levels[k]) +
	    /* LOG(dens_gamma(propL - target->f->f->levels[k], alpha, beta)) */
	    (alpha - 1.0) * LOG(target->f->f->levels[k + 2] - propL) -
	    beta * (target->f->f->levels[k + 2] - propL);
	  /* LOG(dens_gamma(target->f->f->levels[k + 2] - propL, alpha, beta)); */
	  qy += LOG(1.0 / (target->f->f->levels[k + 2] -
			   target->f->f->levels[k]));
	  break;
	case DECR_GAMMA:
	  pry =
	    LOG((target->f->f->levels[k] > target->f->f->levels[k + 1]) /
		target->f->f->levels[k]) +
	    LOG((target->f->f->levels[k + 2] < target->f->f->levels[k + 1]) /
		target->f->f->levels[k + 1]);
	  qy += LOG(1.0 / (target->f->f->levels[k + 2] -
			   target->f->f->levels[k]));
	  break;
	}
	alphaMH = LOG(mu) - target->minPieceLength * mu + ly + pry - qy
	  - lx - prx + qx + LOG((1 - bk) / bk);
	if (printLevel >= 2) {
	  if (alphaMH < LOG(0.05))
	    bite_printf(". ");
	  else {
	    if (alphaMH < LOG(0.25))
	      bite_printf("+ ");
	    else 
	      bite_printf("* "); 
	  }
	}
	if (LOG(u) > alphaMH) { /* Reject the proposal. */
	  delNode(target->f->f, k + 1);
	  if (printLevel >= 2)
	    bite_printf("Failed.");
	}
	else {
	  if (printLevel >= 2)
	    bite_printf("Succeeded.");
	}
	if (printLevel >= 2)
	  bite_printf(" (lx=%g, ly=%g, alphaMH=%g.)\n", lx, ly, alphaMH);
#ifdef HAMMAS
      }
#endif
    }
  }
  else
    if (target->f->f->n > 2) { /* Try to delete a point. */
      k = (long) ran_uni(0.0, (double)(target->f->f->n - 2));
#ifdef HAMMAS
      if (updJ > 2) {
#endif
	propJP = target->f->f->jumpPoints[k + 1];
	propL = target->f->f->levels[k + 1];
	ly = logLikel(target, k + 1, -1, d, model);
	qy = -LOG(target->f->f->jumpPoints[k + 2] -
		  target->f->f->jumpPoints[k]);
	switch (target->prior) {
	case GAMMA:
	  pry =
	    log_dens_gamma(target->f->f->levels[k + 1],
			   alpha * beta, alpha / target->f->f->levels[k]) +
	    log_dens_gamma(target->f->f->levels[k + 2],
			   alpha * beta, alpha / target->f->f->levels[k + 1]);
	  qy += log_dens_gamma(propL, alpha * beta, alpha / target->f->f->levels[k]);
	  break;
	case GAMMA_LEXIS:
	  /* levelDens(TRUE, 0.0, 0.0, target, k, &qy, &pry);
	  ly += pry - qy;
	  levelDens(TRUE, 0.0, 0.0, target, k + 1, &qy, &pry);
	  ly += pry - qy; */
	  
	  pry = returnLexisMfLogPriorDensity(ind, target, -1,
					     target->f->f->jumpPoints[k],
					     target->f->f->jumpPoints[k + 2],
					     alpha0, beta0, alpha, beta, lexis_weight,
					     FALSE);
	  for (j = 0; j < target->neighbour_dim; j++) {
	    if (target->neighbour_next[j])
	      pry += returnLexisMfLogPriorDensity(ind, target->neighbour_next[j], -1,
						  target->f->f->jumpPoints[k],
						  target->f->f->jumpPoints[k + /* 1 */ 2],
						  alpha0, beta0, alpha, beta, lexis_weight,
						  FALSE);
	  }
	  qy += log_dens_gamma(target->f->f->levels[k + 1],
			       alpha * beta, alpha / target->f->f->levels[k]);
	  break;
	case GAMMA2:
	  pry = logGamma2Prior(target->f->f, k, k + 2, alpha0, beta0, alpha);

	  /* LOG(dens_gamma(propL, alpha /
			   (target->f->f->jumpPoints[k + 2] -
			    target->f->f->jumpPoints[k + 1]),
			   alpha / (target->f->f->jumpPoints[k + 2] -
				    target->f->f->jumpPoints[k + 1]) /
			   target->f->f->levels[k])) +
	    LOG(dens_gamma(target->f->f->levels[k + 2],
			   alpha / (target->f->f->jumpPoints[k + 2] -
				    target->f->f->jumpPoints[k + 1]), 
			   (alpha / (target->f->f->jumpPoints[k + 2] -
				    target->f->f->jumpPoints[k + 1]) + 1.0) /
				    target->f->f->levels[k + 1])); */
	  qy += LOG(dens_gamma(propL, alpha /
			       (target->f->f->jumpPoints[k + 2] -
				target->f->f->jumpPoints[k]) + 1.0,
			       (alpha / (target->f->f->jumpPoints[k + 2] -
					 target->f->f->jumpPoints[k]) + 1.0) /
			       target->f->f->levels[k]));
	  break;
	case INCR_GAMMA:
	  pry = 
	    (alpha - 1.0) * LOG(propL - target->f->f->levels[k]) -
	    beta * (propL - target->f->f->levels[k]) +
	    /* LOG(dens_gamma(propL - target->f->f->levels[k], alpha, beta)) */
	    (alpha - 1.0) * LOG(target->f->f->levels[k + 2] - propL) -
	    beta * (target->f->f->levels[k + 2] - propL);
	  /* LOG(dens_gamma(target->f->f->levels[k + 2] - propL, alpha, beta)); */
	  qy += LOG(1.0 / (target->f->f->levels[k + 2] -
			   target->f->f->levels[k]));
	  break;
	case DECR_GAMMA:
	  pry =
	    LOG((target->f->f->levels[k] > target->f->f->levels[k + 1]) /
		target->f->f->levels[k]) +
	    LOG((target->f->f->levels[k + 2] < target->f->f->levels[k + 1]) /
		target->f->f->levels[k + 1]);
	  qy += LOG(1.0 / (target->f->f->levels[k + 2] -
			   target->f->f->levels[k]));
	  break;
	}
	/*      printSf(target->f->f); */
	/* Remove jump point k+1 and the corresponding level: */
	delNode(target->f->f, k + 1);
	if (printLevel >= 2)
	  bite_printf("Trying to delete a jump point %g on (%g,%g)\n\tand a level "
		 "%g on (%g,%g):\n\t", propJP, target->f->f->jumpPoints[k],
		 target->f->f->jumpPoints[k + 1], propL,
		 (target->prior == GAMMA) ? 0.0 : target->f->f->levels[k],
		 (target->prior == GAMMA) ? INFTY :
		 target->f->f->levels[k + 1]);
	/*    printSf(target->f->f); */
	lx = logLikel(target, -1, k, d, model);
	qx = 0.0;
	switch (target->prior) {
	case GAMMA:
	  /* bite_printf("%g\n", dens_gamma(target->f->f->levels[k + 1], alpha,
	     alpha / target->f->f->levels[k])); */
	  prx = log_dens_gamma(target->f->f->levels[k + 1],
			       alpha * beta, alpha / target->f->f->levels[k]);
	  break;
	case GAMMA_LEXIS:
	  /* levelDens(TRUE, 0.0, 0.0, target, k, &qx, &prx);
	  lx += prx - qx; */
	  prx = returnLexisMfLogPriorDensity(ind, target, -1,
					     target->f->f->jumpPoints[k],
					     target->f->f->jumpPoints[k + 1],
					     alpha0, beta0, alpha, beta, lexis_weight,
					     FALSE);
	  for (j = 0; j < target->neighbour_dim; j++) {
	    if (target->neighbour_next[j])
	      prx += returnLexisMfLogPriorDensity(ind, target->neighbour_next[j], -1,
						  target->f->f->jumpPoints[k],
						  target->f->f->jumpPoints[k + 1],
						  alpha0, beta0, alpha, beta, lexis_weight,
						  FALSE);
	  }
	  /* prx = */ qx = 0;
	  break;
	case GAMMA2:
	  prx = logGamma2Prior(target->f->f, k, k + 1, alpha0, beta0, alpha);

	  /* LOG(dens_gamma(target->f->f->levels[k + 1],
			       alpha / (target->f->f->jumpPoints[k + 2] -
					target->f->f->jumpPoints[k + 1]),
			       alpha / (target->f->f->jumpPoints[k + 2] -
					target->f->f->jumpPoints[k + 1]) /
			       target->f->f->levels[k])); */
	  break;
	case INCR_GAMMA:
	  prx = (alpha - 1.0) * LOG(target->f->f->levels[k + 1] - 
				    target->f->f->levels[k]) -
	    beta * (target->f->f->levels[k + 1] - 
		    target->f->f->levels[k]);
	  /* LOG(dens_gamma(target->f->f->levels[k + 1] - 
			       target->f->f->levels[k], alpha, beta)); */
	  break;
	case DECR_GAMMA:
	  prx = LOG(1.0 / (target->f->f->levels[k + 1] -
			   target->f->f->levels[k]));
	  break;
	}
	alphaMH = LOG(mu) - target->minPieceLength * mu + ly + pry - qy
	  - lx - prx + qx + LOG((1 - bk) / bk);
	if (printLevel >= 3)
	  bite_printf("alphaMH %g = LOG(mu %g) - target->minPieceLength %g * mu %g\n"
		      "+ ly %g + pry %g - qy %g\n - lx %g - prx %g + qx %g\n"
		      "+ LOG((1 - bk %g) / bk %g)\n",
		      alphaMH, mu, target->minPieceLength, mu, ly, pry, qy,
		      lx, prx, qx, bk, bk);
	if (printLevel >= 2) {
	  if (-alphaMH < LOG(0.05))
	    bite_printf(". ");
	  else {
	    if (-alphaMH < LOG(0.25))
	      bite_printf("+ ");
	    else 
	      bite_printf("* "); 
	  }
	}
	if (LOG(u) > -alphaMH) { /* Reject the proposal. */
	  addNode(target->f->f, k + 1, propJP, propL);
	  if (printLevel >= 2)
	    bite_printf("Failed.");
	}
	else {
	  if (printLevel >= 2)
	    bite_printf("Succeeded.");
	}
	if (printLevel >= 2)
	  bite_printf(" (lx=%g, ly=%g.)\n", ly, lx);
#ifdef HAMMAS
      }
#endif
    }
  /*  printLevel = pl; */
}

void update(mf target, dt d, ff model)
{
  long i, j, n;
  /* double mu, alpha, beta; */

#ifdef DEBUG_SF3
  sf3_print = TRUE;
#endif
  if (target->wait > iter)
    return;
  if (printLevel >= 2) {
    bite_printf("%s(", target->name);
    if (target->cov[0][0]) {
      i = 0;
      while (target->cov[i][0]) {
	if (i)
	  bite_printf(", ");
	bite_printf("%s == %g", target->cov[i], target->covVal[i]);
	i++;
      }
    }
    bite_printf(")\n");
    fflush(NULL);
  }
  if (target->save == LOADITER) {
    if (iter % loadLag == 0) {
      readSf(target->file, target->f->f, nextI);
      if (printLevel >= 2)
	printSf(target->f->f);
    }
  }
  else {
    n = 0;
    /* if (target->prior == BROWNIAN)
       updateBrownian(target, d, model);
       else { */
    for (j = 0; j < d->n; j++)
      for (i = 0; i <= ind / LONGLEN; i++)
	n += target->inUse[j][i];
    if (!n) { /* No data => independence sampling to improve
		 importance sampling. */
      if (target->type == SF_ORIG) {
	target->f->f = 
	  poiProcess(0.0, target->length,
		     calculate2(-1, target->pVal->pGa->mu),
		     target->minPieceLength, target->f->f);
	switch (target->prior) {
	case GAMMA:
	  levelsG(target->f->f, 
		  calculate2(-1, target->pVal->pGa->alpha0),
		  calculate2(-1, target->pVal->pGa->beta0),
		  calculate2(-1, target->pVal->pGa->alpha),
		  calculate2(-1, target->pVal->pGa->beta),
		  target->restrict0);
	  break;
	case GAMMA_LEXIS:
	  if (lexis_weight == NULL) {
	    lexis_dim = target->pVal->pGL->lexis_dim;
	    lexis_weight = malloc(lexis_dim * sizeof(mu));
	  }
	  if (lexis_dim < target->pVal->pGL->lexis_dim) {
	    lexis_dim = target->pVal->pGL->lexis_dim;
	    lexis_weight = realloc(lexis_weight, lexis_dim * sizeof(mu));
	  }
	  for (j = 0; j < target->pVal->pGL->lexis_dim; j++)
	    lexis_weight[j] = 
	      calculate2(-1, target->pVal->pGL->lexis_weight[j]);
	  levelsGL(target, 
		   target->f->f, 
		   calculate2(-1, target->pVal->pGL->alpha0),
		   calculate2(-1, target->pVal->pGL->beta0),
		   calculate2(-1, target->pVal->pGL->alpha),
		   calculate2(-1, target->pVal->pGL->beta),
		   lexis_weight);
	  break;
	case GAMMA2:
	  levelsG2(target->f->f, 
		   calculate2(-1, target->pVal->pGa->alpha0),
		   calculate2(-1, target->pVal->pGa->beta0),
		   calculate2(-1, target->pVal->pGa->alpha),
		   target->restrict0);
	  break;
	case JHEIKK:
	  break;
	case INCR_GAMMA:
	  levelsIG(target->f->f, 
		   calculate2(-1, target->pVal->pGa->alpha0),
		   calculate2(-1, target->pVal->pGa->beta0),
		   calculate2(-1, target->pVal->pGa->alpha),
		   calculate2(-1, target->pVal->pGa->beta),
		   target->restrict0);
	  break;
	case DECR_GAMMA:
	  levelsDG(target->f->f, 
		   calculate2(-1, target->pVal->pGa->alpha0),
		   calculate2(-1, target->pVal->pGa->beta0),
		   target->restrict0);
	  break;
	}
      }
    }
    else {
      switch (target->prior) {
      case GAMMA:
      case GAMMA2:
      case INCR_GAMMA:
      case DECR_GAMMA:
	alpha0 = calculate2(-1, target->pVal->pGa->alpha0);
	beta0 = calculate2(-1, target->pVal->pGa->beta0);
	alpha = calculate2(-1, target->pVal->pGa->alpha);
	beta = calculate2(-1, target->pVal->pGa->beta);
	mu = calculate2(-1, target->pVal->pGa->mu);
      case GAMMA_LEXIS:
	if (target->prior == GAMMA_LEXIS) {
	  if (lexis_weight == NULL) {
	    lexis_dim = target->pVal->pGL->lexis_dim;
	    lexis_weight = malloc(lexis_dim * sizeof(mu));
	  }
	  if (lexis_dim < target->pVal->pGL->lexis_dim) {
	    lexis_dim = target->pVal->pGL->lexis_dim;
	    lexis_weight = realloc(lexis_weight, lexis_dim * sizeof(mu));
	  }
	  for (j = 0; j < target->pVal->pGL->lexis_dim; j++)
	    lexis_weight[j] = 
	      calculate2(-1, target->pVal->pGL->lexis_weight[j]);
	  alpha0 = calculate2(-1, target->pVal->pGL->alpha0);
	  beta0 = calculate2(-1, target->pVal->pGL->beta0);
	  alpha = calculate2(-1, target->pVal->pGL->alpha);
	  beta = calculate2(-1, target->pVal->pGL->beta);
	  mu = calculate2(-1, target->pVal->pGL->mu);
	}
	if (printLevel >= 2)
	  bite_printf("Level 0: ");
	zeroLI = FALSE;
	if (target->prior == INCR_GAMMA)
	  target->f->f->levels[target->f->f->n - 1] =
	    target->f->f->levels[target->f->f->n - 2] +
	    ran_gamma(alpha, beta);
	updateL(target, 0, d, model);
	if (printLevel >= 2)
	  bite_printf("\n");
	/*  if (!strcmp(target->name, "f0"))
	    printLevel = 2; */
	for (j = 1; j < target->f->f->n - 1; j++) {
	  zeroLI = FALSE;
	  if (printLevel >= 2)
	    bite_printf("Jump point %ld: ", j); 
	  if (mu && !target->fixjp) {
	    updateJ(target, j, d, model);
	    tries++;
	    if (printLevel >= 2)
	      bite_printf("\n");
	      }
	  if (printLevel >= 2) 
	    bite_printf("Level %ld: ", j);
	  /*    if (iter == 3 && target == f->fList[2][70] && j == 5)
		printLevel = 4; */
	  zeroLI = FALSE;
	  updateL(target, j, d, model);
	  tries++;
	  if (printLevel >= 2)
	    bite_printf("\n");
	}
	zeroLI = FALSE;
	/* if (target->f->f->n == 2 && mu) {
	   if (printLevel >= 2)
	   bite_printf("Trying to add a real jump point ");
	   switch (target->prior) {
	   case GAMMA:
	   addNode(target->f->f, target->f->f->n - 1, 
	   ran_uni(target->f->f->jumpPoints[0], 
	   target->f->f->jumpPoints[1]), 
	   ran_gamma(alpha, 
	   alpha / target->f->f->levels[0]));
	   break;
	   case INCR_GAMMA:
	   addNode(target->f->f, target->f->f->n - 1, 
	   ran_uni(target->f->f->jumpPoints[0],
	   target->f->f->jumpPoints[1]), 
	   ran_uni(target->f->f->levels[0],
	   target->f->f->levels[1]));
	   break;
	   }
	   zeroLI = FALSE;
	   if (target->f->f->jumpPoints[1] >= target->f->f->jumpPoints[2]) {
	   target->f->f->jumpPoints[1] = target->f->f->jumpPoints[2];
	   target->f->f->n--;
	   if (printLevel >= 2)
	   bite_printf(" ... failed\n");
	   }
	   else
	   updateLast(target, target->f->f->n - 2, d, model);
	   }
	   else {
	   if (mu)
	   updateLast(target, target->f->f->n - 2, d, model);
	   updateL(target, target->f->f->n - 2, d, model);
	   } */
	/* Try to add or delete a jump point on some interval 
	   (T_{k-1},T_{k+1}): */
	zeroLI = FALSE;
	if (mu && !target->fixjp)
	   addOrDeleteJp(target, d, model);
	/*  if (!strcmp(target->name, "f0"))
	    printSf(target->f->f);
	    printLevel = 1; */
	break;
      }
    }
    /* } */
  }
#ifdef DEBUG_SF3
  sf3_print = FALSE;
#endif
} /* end update */
